unit Compdef;

interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons;

type
  Complex = Record     {define a complex number}
  x : Single;          {real part}
  y : Single;          {imaginary part}
end;

type
  pComplex = ^Complex; {pointer to a complex number}

type
  DComplex = Record    {a double precision complex}
  x : Double;
  y : Double;
end;


function Cplx(x,y : single): complex;
function Csub(za,zb: Complex): Complex;
function Cadd(za,zb: Complex): Complex;
function C3add(za,zb,zc: Complex): Complex;
function Cmult(za,zb: Complex): Complex;
function Conj(za: Complex): Complex;
function Cdiv(za,zb: Complex): Complex;
function Cabs(za: Complex): Real;
function Cinv(za: Complex): Complex;
function Crmult(xx: single; zz : complex): complex;
function Cangle(zz: Complex): single;
function Csqrt(cpu : complex): Complex;
function C3root1(cpu : complex): Complex;
function C3root2(cpu : complex): Complex;
function C3root3(cpu : complex): Complex;

function Dcplx(x,y : double): Dcomplex;
function Dsub(za,zb: DComplex): DComplex;
function Dadd(za,zb: DComplex): DComplex;
function D3add(za,zb,zc: DComplex): DComplex;
function Dmult(za,zb: DComplex): DComplex;
function Ddiv(za,zb: DComplex): DComplex;
function Dabs(za: DComplex): Double;
function Drmult(xx: double; zz : Dcomplex): Dcomplex;
function Dangle(zz: DComplex): double;
function Dsqrt(cpu : Dcomplex): DComplex;
function DConj(za: DComplex): DComplex;
function CtoD(z: complex): DComplex;     {single precision complex to double}
function DtoC(z: Dcomplex): Complex;
function D3root1(cpu : Dcomplex): DComplex;
function D3root2(cpu : Dcomplex): DComplex;
function D3root3(cpu : Dcomplex): DComplex;
function DPolarToCplx(magn,angl: double): DComplex;

procedure PhaseToSeq(Pa,Pb,Pc : complex; var Ps,Ns,Zs : complex);

implementation

function Cplx(x,y : single): complex;
{combines two reals into one complex}
var
  cpu : complex;
begin
  cpu.x := x;
  cpu.y := y;
  Cplx := cpu;
end;

function Cadd(za,zb: Complex): Complex;
{adds two complex numbers}
begin
  try
    Cadd.x := za.x+zb.x;
    Cadd.y := za.y+zb.y;
  finally
  end;
end;

function C3add(za,zb,zc: Complex): Complex;
{adds three complex numbers}
begin
  try
    C3add.x := za.x+zb.x+zc.x;
    C3add.y := za.y+zb.y+zc.y;
  finally
  end;
end;

function Csub(za,zb: Complex): Complex;
{subtracts two complex numbers}
begin
  try
    Csub.x := za.x-zb.x;
    Csub.y := za.y-zb.y;
  finally
  end;
end;

function Cmult(za,zb: Complex): Complex;
{multiplies two complex numbers}
const
  XL = 1.e35;
var
  dpu,dpv : double;
begin
  try
    dpu := za.x*zb.x-za.y*zb.y;
    dpv := za.y*zb.x+za.x*zb.y;
    if (abs(dpu) < XL) then begin
      Cmult.x := dpu;
    end else begin
      if (dpu > 0) then Cmult.x := XL
      else              Cmult.x := -XL;
    end;
    if (abs(dpv) < XL) then begin
      Cmult.y := dpv;
    end else begin
      if (dpv > 0) then Cmult.y := XL
      else              Cmult.y := -XL;
    end;
(*
    Cmult.x := za.x*zb.x-za.y*zb.y;
    Cmult.y := za.y*zb.x+za.x*zb.y;
*)
  finally
  end;
end;

function Conj(za: Complex): Complex;
{returns the complex conjugate of a complex number}
begin
  Conj.x :=  za.x;
  Conj.y := -za.y;
end;

function Cdiv(za,zb: Complex): Complex;
{divides two complex numbers, if the divisor is not zero}
const
  XL = 1.e35;  {large number}
  XS = 1.0/XL; {small number}
var
  apu,apv,apw : double;
  cpu         : Complex;
begin
  try
  if (abs(zb.x) > XS) or (abs(zb.y) > XS) then begin
    apu := zb.x*zb.x+zb.y*zb.y;
    cpu := Cmult(za,Conj(zb));
    apv := cpu.x/apu;
    apw := cpu.y/apu;
    if (abs(apv) < XL) then Cdiv.x := apv
    else begin
      if apv > 0 then Cdiv.x :=  XL
      else            Cdiv.x := -XL;
    end;
    if (abs(apw) < XL) then Cdiv.y := apw
    else begin
      if (apw > 0) then Cdiv.y :=  XL
      else              Cdiv.y := -XL;
    end;
(*
    Cdiv.x := cpu.x/apu;
    Cdiv.y := cpu.y/apu;
*)
  end else begin
    Cdiv.x := 0;
    Cdiv.y := 0;
  end;
  finally
  end;
end;

function Cinv(za: Complex): Complex;
{calculates 1/za}
var
  cone : complex;  {complex one}
begin
  cone.x := 1.0;
  cone.y := 0;
  Cinv := Cdiv(cone,za);
end;

function Cabs(za: Complex): Real;
{calculates the absolute value of a complex variable}
const
  XL = 1.e35;         {large number}
var
  dpu,dpv  : double;  {temporary}
begin
  try
    dpu := abs(za.x);
    dpv := abs(za.y);
    if (dpu > XL) or (dpv > XL) or (dpu*dpu > XL) or (dpv*dpv > XL) or
       (dpu*dpu+dpv*dpv > XL) then begin
       Cabs :=XL;
    end else begin
      Cabs := sqrt(za.x*za.x+za.y*za.y);
    end;
  finally
  end;
end;

function Crmult(xx: single; zz : complex): complex;
{multiply the complex with a real}
begin
  Crmult.x := xx*zz.x;
  Crmult.y := xx*zz.y;
end;

function Cangle(zz: Complex): single;
{calculate the polar angle of a complex variable}
const
  pi=3.14159265;
var
  apu : single;
begin
  if (zz.x > 0) then begin
    Cangle := ArcTan(zz.y/zz.x);
  end else if (zz.x < 0) then begin
    apu := ArcTan(abs(zz.y/zz.x));
    if (zz.y > 0)      then Cangle := pi-apu
    else if (zz.y < 0) then Cangle := apu-pi
    else                    Cangle := pi;
  end else begin
    if (zz.y > 0)      then Cangle := 0.5*pi
    else if (zz.y < 0) then Cangle := -0.5*pi
    else                    Cangle := 0;
  end;
end;

function Csqrt(cpu : complex): Complex;
{take the square root of a complex variable}
var
  apu,apv : single;
  cpw     : Complex;
begin
  apu := Cabs(cpu);
  apv := Cangle(cpu);
  cpw.x := sqrt(apu)*cos(0.5*apv);
  cpw.y := sqrt(apu)*sin(0.5*apv);
  Csqrt := cpw;
end;

function C3root1(cpu : complex): Complex;
{take the cubic root of a complex variable}
var
  apu,apv : single;
  cpw     : Complex;
begin
  apu := ln(Cabs(cpu));
  apu := exp(apu/3.0);
  apv := Cangle(cpu);
  cpw.x := apu*cos(apv/3.0);
  cpw.y := apu*sin(apv/3.0);
  C3root1 := cpw;
end;

function C3root2(cpu : complex): Complex;
{take the cubic root of a complex variable}
const
  pi23 = 2.0943951; {2*pi/3}
var
  apu,apv : single;
  cpw     : Complex;
begin
  apu := ln(Cabs(cpu));
  apu := exp(apu/3.0);
  apv := Cangle(cpu);
  cpw.x := apu*cos(apv/3.0+pi23);
  cpw.y := apu*sin(apv/3.0+pi23);
  C3root2 := cpw;
end;

function C3root3(cpu : complex): Complex;
{take the cubic root of a complex variable}
const
  pi23 = 2.0943951; {2*pi/3}
var
  apu,apv : single;
  cpw     : Complex;
begin
  apu := ln(Cabs(cpu));
  apu := exp(apu/3.0);
  apv := Cangle(cpu);
  cpw.x := apu*cos(apv/3.0-pi23);
  cpw.y := apu*sin(apv/3.0-pi23);
  C3root3 := cpw;
end;

function Dcplx(x,y : double): Dcomplex;
{combines two reals into one complex}
var
  cpu : dcomplex;
begin
  cpu.x := x;
  cpu.y := y;
  DCplx := cpu;
end;

function DConj(za: DComplex): DComplex;
{returns the complex conjugate of a complex number}
begin
  DConj.x :=  za.x;
  DConj.y := -za.y;
end;

function Dadd(za,zb: DComplex): DComplex;
{adds two complex numbers}
begin
  try
    Dadd.x := za.x+zb.x;
    Dadd.y := za.y+zb.y;
  finally
  end;
end;

function Dsub(za,zb: DComplex): DComplex;
{subtracts two complex numbers}
begin
  try
    Dsub.x := za.x-zb.x;
    Dsub.y := za.y-zb.y;
  finally
  end;
end;

function D3add(za,zb,zc: DComplex): DComplex;
{adds three complex numbers}
begin
  try
    D3add.x := za.x+zb.x+zc.x;
    D3add.y := za.y+zb.y+zc.y;
  finally
  end;
end;

function Dmult(za,zb: DComplex): DComplex;
{multiplies two complex numbers}
begin
  try
    Dmult.x := za.x*zb.x-za.y*zb.y;
    Dmult.y := za.y*zb.x+za.x*zb.y;
  finally
  end;
end;

function Ddiv(za,zb: DComplex): DComplex;
{divides two complex numbers, if the divisor is not zero}
var
  apu : double;
  cpu : DComplex;
begin
  try
  if (zb.x <> 0) or (zb.y <> 0) then begin
    apu := zb.x*zb.x+zb.y*zb.y;
    cpu := Dmult(za,DConj(zb));
    Ddiv.x := cpu.x/apu;
    Ddiv.y := cpu.y/apu;
  end else begin
    Ddiv.x := 0;
    Ddiv.y := 0;
  end;
  finally
  end;
end;

function CtoD(z: complex): DComplex;
{single precision complex to double precision complex}
begin
  CtoD.x := z.x;
  CtoD.y := z.y;
end;

function DtoC(z: Dcomplex): Complex;
{double precision complex to single precision complex}
begin
  DtoC.x := z.x;
  DtoC.y := z.y;
end;

function Drmult(xx: double; zz : Dcomplex): Dcomplex;
{multiply the complex with a real}
begin
  Drmult.x := xx*zz.x;
  Drmult.y := xx*zz.y;
end;

function Dabs(za: DComplex): Double;
{calculates the absolute value of a complex variable}
const
  XL = 1.e400;         {large number}
var
  dpu,dpv  : double;  {temporary}
begin
  try
    dpu := abs(za.x);
    dpv := abs(za.y);
    if (dpu > XL) or (dpv > XL) or (dpu*dpu > XL) or (dpv*dpv > XL) or
       (dpu*dpu+dpv*dpv > XL) then begin
       Dabs := XL;
    end else begin
      Dabs := sqrt(za.x*za.x+za.y*za.y);
    end;
  finally
  end;
end;

function Dangle(zz: DComplex): double;
{calculate the polar angle of a complex variable}
const
  pi=3.141592654;
var
  apu : double;
begin
  if (zz.x > 0) then begin
    Dangle := ArcTan(zz.y/zz.x);
  end else if (zz.x < 0) then begin
    apu := ArcTan(abs(zz.y/zz.x));
    if (zz.y > 0)      then Dangle := pi-apu
    else if (zz.y < 0) then Dangle := apu-pi
    else                    Dangle := pi;
  end else begin
    if (zz.y > 0)      then Dangle := 0.5*pi
    else if (zz.y < 0) then Dangle := -0.5*pi
    else                    Dangle := 0;
  end;
end;

function Dsqrt(cpu : Dcomplex): DComplex;
{take the square root of a complex variable}
var
  apu,apv : double;
  cpw     : DComplex;
begin
  apu := Dabs(cpu);
  apv := Dangle(cpu);
  cpw.x := sqrt(apu)*cos(0.5*apv);
  cpw.y := sqrt(apu)*sin(0.5*apv);
  Dsqrt := cpw;
end;

function D3root1(cpu : Dcomplex): DComplex;
{take the cubic root of a complex variable}
var
  apu,apv : double;
  cpw     : DComplex;
begin
  apu := ln(Dabs(cpu));
  apu := exp(apu/3.0);
  apv := Dangle(cpu);
  cpw.x := apu*cos(apv/3.0);
  cpw.y := apu*sin(apv/3.0);
  D3root1 := cpw;
end;

function D3root2(cpu : Dcomplex): DComplex;
{take the cubic root of a complex variable}
const
  pi23 = 2.0943951; {2*pi/3}
var
  apu,apv : double;
  cpw     : DComplex;
begin
  apu := ln(Dabs(cpu));
  apu := exp(apu/3.0);
  apv := Dangle(cpu);
  cpw.x := apu*cos(apv/3.0+pi23);
  cpw.y := apu*sin(apv/3.0+pi23);
  D3root2 := cpw;
end;

function D3root3(cpu : Dcomplex): DComplex;
{take the cubic root of a complex variable}
const
  pi23 = 2.0943951; {2*pi/3}
var
  apu,apv : double;
  cpw     : DComplex;
begin
  apu := ln(Dabs(cpu));
  apu := exp(apu/3.0);
  apv := Dangle(cpu);
  cpw.x := apu*cos(apv/3.0-pi23);
  cpw.y := apu*sin(apv/3.0-pi23);
  D3root3 := cpw;
end;

function DPolarToCplx(magn,angl: double): DComplex;
{polar to complex
 magn  magnitude
 angl  phase angle, in radians}
var
  dpu : Dcomplex; {temporary}
begin
  dpu.x := magn*cos(angl);
  dpu.y := magn*sin(angl);
  DPolarToCplx := dpu;
end;
(**)
procedure PhaseToSeq(Pa,Pb,Pc : complex; var Ps,Ns,Zs : complex);
{converts the phase variables to symmetrical components
 Ps   pos. seq. component
 Ns   neg. seq. component
 Zs   zero seq. component
 Pa,Pb,Pc  the phase variables
}
const
  eifii  : complex = (x : -0.5; y: 0.866025404); {exp(i fii), phase difference}
  emifii : complex = (x : -0.5; y:-0.866025404); {exp(-i fii) phase difference}
  one3rd = 0.333333333;
var
  cpu  : complex;  {temporary}
begin
  cpu := Cadd(Pa,Cadd(Pb,Pc));
  Zs  := Crmult(one3rd,cpu);
  cpu := Cadd(Cmult(eifii,Pb),Cmult(emifii,Pc));
  Ps  := Crmult(one3rd,Cadd(cpu,Pa));
  cpu := Cadd(Cmult(emifii,Pb),Cmult(eifii,Pc));
  Ns  := Crmult(one3rd,Cadd(cpu,Pa));
end;
(**)

procedure DPhaseToSeq(Pa,Pb,Pc : complex; var Ps,Ns,Zs : complex);
{converts the phase variables to symmetrical components
 double precision is used internally
 Ps   pos. seq. component
 Ns   neg. seq. component
 Zs   zero seq. component
 Pa,Pb,Pc  the phase variables
}
const
  eifii  : Dcomplex = (x : -0.5; y: 0.866025404); {exp(i fii), phase difference}
  emifii : Dcomplex = (x : -0.5; y:-0.866025404); {exp(-i fii) phase difference}
  one3rd = 0.333333333;
var
  cpu  : Dcomplex;  {temporary}
  Da,Db,Dc : Dcomplex;
begin
  Da.x := Pa.x;
  Da.y := Pa.y;
  Db.x := Pb.x;
  Db.y := Pb.y;
  Dc.x := Pc.x;
  Dc.y := Pc.y;
  cpu := Dadd(Da,Dadd(Db,Dc));
  cpu  := Drmult(one3rd,cpu);
  Zs.x := cpu.x;
  Zs.y := cpu.y;
  cpu := Dadd(Dmult(eifii,Db),Dmult(emifii,Dc));
  cpu  := Drmult(one3rd,Dadd(cpu,Da));
  Ps.x := cpu.x;
  Ps.y := cpu.y;
  cpu := Dadd(Dmult(emifii,Db),Dmult(eifii,Dc));
  cpu  := Drmult(one3rd,Dadd(cpu,Da));
  Ns.x := cpu.x;
  Ns.y := cpu.y;
end;

end.
