{ > < }
unit PQEquation;
{for PGQ-relay equations that are given in a "human understandable" form}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Parser, utildef, imagopas;

type
  TPQEquForm = class(TForm)
    EquParser: TMathParser;
    procedure EquParserGetVar(Sender: TObject; VarName: string;
      var Value: Extended; var Found: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure calculate(equation: string; var temps: single);
  public
    { Public declarations }
    procedure CreateEquationObjects;    
  end;

type
  TNameAndValue = record
  name  : string;
  value : single;
end;

{class for the equations and related operations}
type
  TpPQEquation = ^TPQEquation;
  TPQEquation = class(TObject)
  procedure Initialize;
  procedure PutEquation(equ: string);
  procedure PutEquationName(name: string);
  procedure GetEquationName(var name: string);
  procedure GetEquation(var equ: string);
  procedure PutNameAndValue(name: string; value: single; ii : integer);
  procedure GetValue(name: string; var value: single);
  procedure GetVarByName(VarName: string; var Value: Extended; var Found: Boolean);
  procedure PutValueByIndex(indx : integer; valu : single);
  procedure CalculateTime(Iss, ktm, tadd, curr : single; var temps : single);
  procedure CalculateCurrent(Iss, ktm, tadd, temps : single; igcur: boolean; var curr : single);
  procedure SetEquivalenceIndex(expla, tindx: string);
private
 eEquation  : string;                        {the equation in "computer understandable" form}
 eName      : string;                        {name (description?) of the equation}
 eNameValue : array[1..20] of TNameAndValue; {parameter names and values. 20 should be large enough}
 eCurrIndx : integer;     {index for I/Iset variable}
 eMultIndx : integer;     {index for time multiplier}
 eAddeIndx : integer;     {index for time adder}
end;

var
  PQEquForm: TPQEquForm;

implementation

{$R *.DFM}
uses
  drawmenu, Equation, routines;

procedure TPQEquation.PutEquation(equ: string);
begin
  eEquation := equ;
end;

procedure TPQEquation.PutEquationName(name: string);
begin
  eName := name;
end;

procedure TPQEquation.GetEquationName(var name: string);
begin
  name := eName;
end;

procedure TPQEquation.GetEquation(var equ: string);
begin
  equ := eEquation;
end;

procedure TPQEquation.PutNameAndValue(name: string; value: single; ii: integer);
begin
  if (ii >= low(eNameValue)) and (ii <= high(eNameValue)) then begin
    eNameValue[ii].name := name;
    eNameValue[ii].value := value;
  end;
end;

procedure TPQEquation.GetValue(name: string; var value: single);
{get the value of the variable (parameter) with the given name}
var
  found : boolean;  {true, when variable found}
  ii    : integer;  {index}
  val   : single;
begin
  found := false;
  val := 1;
  for ii := low(eNameValue) to high(eNameValue) do begin
    if (eNameValue[ii].name = name) then begin  {name found?}
      val := eNameValue[ii].value;
      found := true;
      break;
    end;
  end;
  {get value, if name found, else just some value}
  if (found) then value := val else value := 1;
end;

procedure TPQEquation.PutValueByIndex(indx : integer; valu : single);
{put the given value to the variable of given index
 indx  the index
 valu  the value}
begin
  if (indx >= low(eNameValue)) and (indx <= high(eNameValue)) then begin
    eNameValue[indx].value := valu;
  end;
end;

procedure TPQEquation.GetVarByName(VarName: string; var value: Extended; var Found: Boolean);
{get the value of the variable given by name, needed for the parser
 No difference between upper and lower case
 VarName  name of the given variable
 value    the value of the variable
 found    true, when variable found}
var
  ii      : integer;
  nameUp  : string;                {name of variable in upper case}
  namval  : TNameAndValue;         {name of a variable and the corresponding value}
  VarNamU : string;               {variable name in upper case}
begin
  value := 0;                     {initialize}
  found := false;                 {initialize}
  VarNamU := uppercase(VarName);  {compare names in upper case}
  for ii := 1 to high(eNameValue) do begin  {study the whole array}
    namval := eNameValue[ii];
    if (namval.name = '') then begin    {no more names and values}
      break;
    end else begin
      nameUp := uppercase(namval.name); {change to upper case for comparison}
      if (nameUp = VarNamU) then begin  {name found}
        value := namval.value;
        found := true;
        break;
      end;
    end;
  end;
end;

procedure TPQEquation.CalculateTime(Iss, ktm, tadd, curr : single; var temps : single);
{calculate the activation time corresponding to the given current
 Iss     relay param : starting current  (set current)
 ktm     relay param : time multiplier
 tadd    relay param: time adder
 curr    the given current
 temps   the calculated time}
const
  id : integer = 0;   {for testing}
var
  apu      : single;  {temporary}
  equation : string;  {equation in computer readable form}
begin
//showmessage(inttostr(eCurrIndx)+'  '+inttostr(eMultIndx)+'  '+inttostr(eAddeIndx));
//showmessage(floattostr(Iss)+'  '+floattostr(ktm)+'  '+floattostr(tadd)+'  '+floattostr(curr));
  {Iset/curr}
  if (Iss > 0) and (curr > Iss) then begin
    apu := curr/Iss;
    if (eCurrIndx >= 0) then
      PutValueByIndex(eCurrIndx,apu);    {relative current as variable for calculation}
    {time multiplier}
    if (eMultIndx >= 0) then
      PutValueByIndex(eMultIndx,ktm);
    {time adder}
    if (eAddeIndx >= 0) then
      PutValueByIndex(eAddeIndx,tadd);
    GetEquation(equation);               {get the equat.for time from current}
    {parse (every time !?) and calculate}
    PQEquForm.calculate(equation, temps);
//showmessage(floattostr(curr)+'  '+floattostr(temps));
  end else begin                 {not reasonable current or set current}
    temps := -1;
  end;
end;

procedure TPQEquation.CalculateCurrent(Iss, ktm, tadd, temps : single; igcur: boolean; var curr : single);
{calculate the current corresponding to the given activation time
 The calculation is done iteratively (must be...)
 Function to be solved: (given time) - (calculated time) = 0, F = t-f(c) = 0
 Iss     relay param : starting current  (set current)
 ktm     relay param : time multiplier
 tadd    relay param: time adder
 temps   the given time
 igcur   when true, curr is used as initial guess
 curr    the calculated current}
const
  cmax = 2.5e5;       {max value of current = some very large value}
  minc = 1.001;       {multiplier for minimum current}
  niter0 = 25;        {max. number of iterations}
  niter1 = 12;        {max. no of iterations}
  id : integer = 0;   {for testing}
var
  apu      : single;   {temporary}
  equation : string;   {equation in computer readable form}
  fm,fp    : single;   {latest negative, positive values of the function}
  fx       : single;   {function value}
  ibrc     : integer;  {index for brackets}
  iter     : integer;  {iteration counter}
  niter    : integer;  {max. no of iterations}
  xm,xp    : single;   {values of the unknown for latest negative, positive function}
  xx       : single;   {unknown = relative current}
begin
  if (Iss > 0) and (ktm > 0) and (temps > 0) then begin
    {time multiplier}
    if (eMultIndx >= 0) then PutValueByIndex(eMultIndx,ktm);
    {time adder}
    if (eAddeIndx >= 0) then PutValueByIndex(eAddeIndx,Tadd);
    GetEquation(equation);                  {get the equat.for time from current}
    {initialize brackets to unreasonable values}
    xm := -1; xp := -1;
    fm := 1; fp := -1;
    {find brackets}
    for ibrc := 1 to 2 do begin
      if (ibrc = 1) then begin
        xx := minc;
      end else begin
        if (igcur) and (curr > 0) then       {use old value as initial guess}
          xx := curr/Iss
        else
          xx := cmax/Iss;
      end;
      {Iset/curr is assumed to be (must be) the first variable}
      if (eCurrIndx >= 0) then PutValueByIndex(eCurrIndx,xx);   {relative current as variable for calculation}
      {parse (every time !?) and calculate}
      PQEquForm.calculate(equation, apu);    {calculate time for the given relative current}
      fx := temps-apu;                       {function = given time - calculated time}
      {found pos or neg. bracket}
      if (fx >= 0) then begin                {bracket with positive function value found}
        xp := xx;
        fp := fx;
      end else begin                         {bracket with negative function value found}
        xm := xx;
        fm := fx;
      end;
    end;
    {Brackets found?}
    if (fm <= 0) and (fp >= 0) then begin
      {iterate. Fewer iterations if a (good?) initial guess}
      if (igcur) and (curr > 0) then niter := niter1 else niter := niter0;
      for iter:=1 to niter do begin
//      {regula falsi}
//      apu := fm-fp;
//      if (abs(apu) > 1.e-6) then begin
//        xx := (fm*xp-fp*xm)/apu           {new guess for position of fault}
//      end else begin
//        xx := 0.5*(xp+xm);
//      end;
        {no,bisection works better here}
        xx := 0.5*(xp+xm);
        if (eCurrIndx >= 0) then PutValueByIndex(eCurrIndx,xx);   {relative current as variable for calculation}
        {parse (every time !?) and calculate}
        PQEquForm.calculate(equation, apu);    {calculate time for the given relative current}
        fx := temps-apu;                       {function = given time - calculated time}
        {replace upper or lower bracket}
        if (fx > 0) then begin                 {update brakets}
          fp := fx;
          xp := xx;
        end else begin
          fm := fx;
          xm := xx;
        end;
        {convergence check here!!!}
      end;
      {solution}
      curr := Iss*0.5*(xm+xp);
    end else begin                             {bracket(s) not found}
      if (xm < 0) then curr := Iss else curr := cmax;
//    curr := 1;                 {temporary, find better !!}
    end;
  end else begin                 {not reasonable current or set current}
    curr := 1;                   {just some value...??}
  end;
  inc(id);
end;

procedure TPQEquation.SetEquivalenceIndex(expla, tindx: string);
{set the index for an equivalent variable (variables in the equation and parameters
 in PQrelays)
 expla  explanation for the variable(parameter)
 tindx   the index as string}
var
  indx  : integer;  {index of explanation}
begin
  {Old values of indices ????}
  {index should be integer}
  if (IsInteger(tindx,indx)) then begin          {in drawmenu}
    {is integer, but which explanation}
    if (expla = EquationForm.CurrentLabel.caption) then begin
      eCurrIndx := indx+1;      {+1, because eNameValue starts at 1, but indx at 0 (!?)}
    end else if (expla = EquationForm.MultiLabel.caption) then begin
      eMultIndx := indx+1;
    end else if (expla = EquationForm.AdderLabel.caption) then begin
      eAddeIndx := indx+1;
    end;
  end;
end;

procedure TPQEquation.Initialize;
{intialize (some) variables}
var
  ii : integer;
begin
  eEquation  := '';
  eName      := '';
  for ii := low(eNameValue)to high(eNameValue) do begin
    eNameValue[ii].name := '';
    eNameValue[ii].value := 1;
  end;
  eCurrIndx := -1;     {index for I/Iset variable}
  eMultIndx := -1;     {index for time multiplier}
  eAddeIndx := -1;
end;

procedure TPQEquForm.calculate(equation: string; var temps: single);
{parse the equation and calculate the value to temps}
begin
  with EquParser do begin
    ParseString := equation;
    Parse;
    if ParseError then begin
      temps := -1.5;
    end else
      temps := ParseValue;
  end;
end;

procedure TPQEquForm.EquParserGetVar(Sender: TObject; VarName: string;
  var Value: Extended; var Found: Boolean);
{try to use values given in the grid. No difference between upper and lower case }
var
  pEqu   : TpPQEquation; {object for equation}
begin
  {get pointer to the equation object. Put to tag in PQTimeOrCurr of Relcurr.pas}
  pEqu := TpPQEquation(EquParser.tag);
  {get the value of the given variable}
  pEqu^.GetVarByName(VarName, value, Found);
end;

procedure TPQEquForm.CreateEquationObjects;
{create objects for the PQ-equations. Get parameters etc. from the "raw" list
 of equations
 Called in FormCreate of Equation.pas }

var
  apu     : single;         {temporary}
  cnt     : integer;        {count of ..}
  cnt2    : integer;        {count divided by 2}
  err     : integer;        {error flag}
  ident   : integer;        {identifier of data}
  iEqu    : integer;        {index}
  ii      : integer;        {index}
  ival    : integer;        {index of value}
  kend    : integer;        {index of last studied char, when converting text to numerical}
  line    : string;         {one line of text}
  nEqu    : integer;        {number of equations}
  PartStr : TStringList;
  pEqu    : TpPQEquation;   {temporary, for creating}
  pStrL   : TpStringList;   {pointers to strings in raw list}
  pParts  : TpStringList;   {list for separating part-strings (words) from text}
  str     : string;         {temporary}
  value   : single;         {value of a variable}
  varnam  : string;         {name of a variable}
  varval  : string;         {value of a variable, as text}
begin
  PartStr := TStringList.create;
  SetComp.Equ_List.clear;             {clear, just to be sure}
  nEqu := SetComp.EquRawList.count;   {number of equations}
  if (nEqu > 0) then begin
    for iEqu := 0 to nEqu-1 do begin
      new(pEqu);                            {reserve space for the equation}
      pEqu^ := TPQEquation.Create;          {create the equation object}
      pEqu^.initialize;
      SetComp.Equ_List.add(pEqu);
      {get main properties from the list}
      pStrL := SetComp.EquRawList[iEqu];          {pointer to raw-data of equationas lines of text}
      cnt := pStrL^.count;                        {lines in raw data}
      ival := 0;                                  {initialize index of variable}
      if (cnt >= 4) then begin                    {actually, even 4 is too little!}
        cnt2 := cnt div 2;                        {number of items}
        for ii := 0 to cnt2-1 do begin            {each identifier and item}
          line := pStrL^[2*ii];                   {identifier of data}
          if (IsInteger(line,ident)) then begin   {data is an integer = identifier?}
            line := pStrL^[2*ii+1];               {data as string}
            {only data of types 12 (variables), 13 (name) and 14 (equation) are needed here}
            if (ident = 12) then begin            {variables}
              GetPartStrings(line, longint(@PartStr));  {get the name and value separately, as stings}
              {there should be 2 strings}
              if (PartStr.count >= 2) then begin
                varnam := PartStr[0];                  {pick the name as string}
                varval := PartStr[1];                  {and the value, as string}
                {convert value from text to numerical}
                NextValueFromLine(varval, 1, kend, apu , str, err); {value}
                if (err = 0) then value := apu else value := 1;
                {name and value to equation-object}
                inc(ival);
                pEqu^.PutNameAndValue(varnam, value,ival);
              end;
            end else if (ident = 13) then begin   {name of equation}
              pEqu^.PutEquationName(line);
            end else if (ident = 14) then begin   {equation itself}
              pEqu^.PutEquation(line);
            end else if (ident = 15) then begin   {equivalencies between equation and PQrelays}
              GetPartStrings(line, longint(@PartStr));  {get the name and value separately, as stings}
              {there should be 2 strings}
              if (PartStr.count >= 2) then begin
                varnam := PartStr[0];                {pick the explanation as string}
                varval := PartStr[1];                {and the index, as string}
                pEqu^.SetEquivalenceIndex(varnam, varval); {put the index}
              end;
            end;
          end;
        end;
      end;
    end;
  end;
  PartStr.clear;
  PartStr.free;
end;



procedure TPQEquForm.FormCreate(Sender: TObject);
begin
//  {create the objects for working with the equations}
//  CreateEquationObjects;
  {add flag that form created}
  Imagoo.ReadyEdit.text := Imagoo.ReadyEdit.text+'p';  
end;

end.
