unit IEC;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Skemadef, Utildef, Buttons,
  Nodes,Elplex,Compdef,Skemproc,RelCurr, Menus , Cmatmul , Imagodef,
  Distance, Twoshort, matmul , Shortcir, Shortcmp, Earthflt, Polygon,
  emtp ;

procedure IECShort(IpSet : longint);
procedure IECTwoPhase(ipSet : longint);
procedure IECEarthFault(ipSet : longint);
procedure IECLineLineEarth(ipSet : longint);
function  OpenLineFault(ipSet : longint; OneOrTwo: string): boolean;

implementation

uses
  Imagopas;

const
  sqrt3 = 1.7320508;

var
  pZmat   : pComplex;    {pointer to the zero sequence matrix}
  pIndexZ : pInteger;    {pointer to array keeping track on row exchanges}

procedure PutShortCurrent(pSet : TpCompSet; pRhs : pComplex; nnod : integer);
{find the short circuit(s) and save the short circuit current
 pSet    pointer to the set of components, etc.
 pRhs    pointer to the results vector ("right hand side")
 nnod    no of nodes = dim. of rhs
 }
var
  errc    : integer;     {error flag}
  Icx     : complex;     {fault current}
  jcomp   : integer;     {counts components}
  ModNam  : string;      {name (type) of the component}
{ n2sc    : integer;     {number of short circuits}
  pComp   : TpComp;      {pointer to components}
begin
  pComp := nil;                              {initialize pointer to components}
  errc  := 0;                                {initialize error flag}
  jcomp := 0;
{ n2sc := 0;                                 {initialize number of short circuits}
  while (errc = 0) do begin                  {scan all components}
    pSet^.NextComp(pComp,errc,jcomp,true);   {get pointer to next component}
    if (errc = 0) then begin                 {component found}
      pComp^.GetModName(ModNam);             {get the name (type) of the component}
      if (ModNam = gshort) then begin        {is it a short}
        GetVolt(pComp,pRhs,2,nnod,Icx);      {get current = "voltage" of second node}
        pComp^.putZpar(Icx.x,Icx.y,0,3);     {save the current}
{       inc(n2sc);                           {increment count of short circ.s}
      end;
    end;
  end;
end;

procedure SolveEqFNet(ipSet : longint);
{construct the (admittance?) matrix and solves voltages and currents in the net
 at the "equivalent frequency", for the peak current calculation (done later)
 ipSet = pointer to the set of components, as integer
 }
 var
   cpu    : complex;     {temporary}
   Dodd   : single;      {keeps track on permutations}
   errcom : boolean;     {error flag}
   freq   : single;      {the frequency of the net}
   i      : integer;     {index, or pointer}
   nnod   : integer;     {number of nodes}
   pSet   : TpCompSet;   {pointer to the set of components}
   Sngl   : Integer;     {flag for singular matrix}
   Tout   : Complex;     {test output from matrix decomposition}
 begin
   {determine nodes in any case = "Click" button "NodeBut"on form "Imagoo"}
   Imagoo.NodeButClick(nil{Imagoo.NodeBut});
   pSet := tpCompSet(ipSet);        {pointer to the set of components}
   nnod := pSet^.appl_nn;           {number of nodes}
   freq := pSet^.appl_hz;           {frequency}
   if (nnod > 0) then begin         {there are nodes}
     {reserve and initialize memory for matrices}
     ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
     {construct the extended admittance matrix}
     ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, IECPeak,false, errcom);
     {solve voltages, but only if no unknown components}
     if (not(errcom)) then begin
       try
         CLUdecomp(pAdmat,pWs,nnod,pIndex,Dodd,Sngl,Tout); {decompose first}
         {back substitution, note result OVERWRITES rhs !!}
         CLUbacksub(pAdmat,nnod,pIndex,pRhs);               {backsubstitute}
         PutShortCurrent(pSet, pRhs, nnod );  {save short circuit current(s)}
       finally
       end;
     end else begin                   {there was an unknown component}
       for i:=1 to nnod do begin
         cpu.x := 0;                  {complex zero}
         cpu.y := 0;
         putCvec(pRhs,i,nnod,cpu);    {put zeroes as results}
       end;
     end;



     {free the reserved memory for matrices}
     FreeMat(nnod);
   end;
 end;


procedure IECShort(ipSet : longint);
{calculate the short circuit currents according to IEC 60909
ipSet : pointer to the "common" as integer }
var
  nAsyMot : integer;     {number of asynchronous motors}
  nCurrT  : integer;     {number of current transformers}
  nFault  : integer;     {number of short circuits}
  nGenMot : integer;     {number of generators and motors}
  nQrel   : integer;     {number of PGQ-relays, electromech.relays and recloser rel.}
  nSource : integer;     {number of (constant) sources}
  nnod    : integer;     {number of nodes}
  nRelay  : integer;     {number of relays}
  pCurrT  : TpCurrT;     {pointer to object for current transformers}
  pComp   : TpComp;      {pointer to a relay}
  pRelay  : TpRelay;     {pointer to the relays}
  pSet    : TpCompSet;   {pointer to the set of comp. etc}
begin
  pSet := TpCompSet(ipSet);      {pointer to the set, from integer}
  pSet^.Deactivate;              {deactivate evtl. active comp. and contr}
  SolveEqFNet(ipSet);            {solve net at equiv.freq, for the peak current}
  SolveNet(ipSet,false, IECFault);
  nnod := pSet^.appl_nn;         {number of nodes}
  {continue only if there is a network}
  if (nnod > 0) then begin
    {process and show results - - - - - - - -}
    {preparations for activation time . . . .}
    pSet^.GetMemForCurrTrAndRelays;        {memory for curr.tr+relays, if there are any}
    {count components (some already known, some not needed, but all in one routine)}
    CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nQRel);
    {if there are current transformers, create object for operations with current trafos}
    CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
    {if there are relays, create object for operations with relays}
    CreateRelays(ipSet, nRelay,nQrel, pRelay); {normal and PQ-relay}
    {find the current trafos for the relays}
    if (pRelay <> nil) then begin
//    pRelay^.FindCurrTrafos(pCurrT);   {NOT needed, works only with "simple" relays, in shortcmp.pas}
      pRelay^.GroupActivation;         {in shortcmp.pas}
      pSet^.UpdateResultBoxes(2);      {update secondary boxes, when all values known}
    end;
    {end of preparations for activation time, but release memory and objects at the end}
    {show results, if graph visible}
    if (RelayCurr.visible) {and (RelayCurr.RelayChar.visible)} then begin
      pComp := TpComp(RelayCharact.tag);   {pointer to the relay}
      Relay_Show(pComp,'real');
    end;
    if (DistRel.visible) then DistanceRelayShow;
    if (PolyRel.visible) then PolygonRelayShow;
    {update also the eventual result boxes}
    pSet^.UpdateResultBoxes(1);
    {release memory, free objects, etc (from activation time)}
    if (nRelay > 0) then begin
      pRelay^.Destr;                 {destroy object for relays}
      pRelay^.Free;
      dispose(pRelay);
      pRelay := nil;
    end;
    if (nCurrT > 0) then begin
      pCurrT^.Destr;                 {destroy object for current trafos}
      pCurrT^.Free;
      dispose(pCurrT);
      pCurrT := nil;
    end;
    pSet^.FreeMemForCurrTrAndRelays;
    {end of release of objects from activation time}
  end;
end;

procedure IECTwoPhase(ipSet : longint);
{calculate line to line short circuit according to IEC 60909
 ipSet   pointer to the "common" as integer}
{ The principle seems to be the following:
  get nominal Ua-Ub  (multiplied by the c-factor)
  change Ua by dUa, and Ub by dUb = Ua+dUa - Ub (so that Ua+dUa - (Ub+dUb) = 0)
         and Uc by -(Ua+Ub) (so that Ua+dUa + Ub+dUb + Uc+dUc = 0)
  find dUa so that Ia+Ib = 0 at the fault, i.e. to or from the fault
        (and, consequently Ic = 0)
  do this in three phases 1) dUa = 0, 2) dUa = some reasonable value,
        3) solve the final dUa from the results of these two calculations}
var
  Bsum        : complex;     {constant for Ia+Ib = Csum*dUa+Bsum}
  Csum        : complex;     {coeff.   for Ia+Ib = Csum*dUa+Bsum}
  cpu         : complex;     {temporary}
  dodd,doddN  : single;      {keep track of permutations}
  dUa,dUb,dUc : complex;     {(guessed) voltage changes}
  errcom      : boolean;     {error flag}
  er2         : integer;     {error flag}
  freq        : single;      {frequency}
  Ip,Im,Iz    : complex;     {pos,neg,zero seq. currents}
  Ia,Ib,Ic    : complex;     {fault currents}
  ii          : integer;     {index}
  jc          : integer;     {current node}
  nnod        : integer;     {number of nodes}
  nGenMot     : integer;     {number of synch.machines}
  nRelay      : integer;     {number of relays}
  nFault      : integer;     {number of faults}
  nCurrT      : integer;     {number of current trafos}
  nAsyMot     : integer;     {number of asynchronous motors}
  nSource     : integer;     {number of sources (feeders)}
  nPQrel      : integer;     {number of combined relays}
  pComp       : TpComp;      {pointer to a component}
  pCurrT      : TpCurrT;     {pointer to objectg for current trafos}
  pRelay      : TpRelay;     {pointer to object for relays}
  pSet        : TpCompSet;   {pointer to the common}
  pTwosc      : TpComp;      {pointer to the short circuit}
  pVrhs       : pComplex;    {temporary right hand side}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  Uab         : complex;     {Ua-Ub (*C-factor)}
  Vp,Vn,Vz    : complex;     {sequence voltages}
  Vnom        : single;      {nominal voltage at fault, * C-factor}
begin
  SolveEqFNet(ipSet);   {for peak current, but is this correct ??}
  pSet := TpCompSet(ipSet);          {integer to pointer}
  freq := pSet^.appl_Hz;
{ Imagoo.NodeButClick(nil);NotNeeded, done above {determine nodes, get number of nodes}
  pTwosc := nil;                     {initialize pointer to short circuit}
  FindShort(pSet, pTwosc);           {find two phase short(s), in Twoshort.pas}
  {solve the network only, if there is a short !}
  if (pTwosc <> nil) then begin        {two phase short found}
    nnod := pSet^.appl_nn;             {copy number of nodes}
    {count the number of generators, etc. Needed below}
    CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nPQrel); {in shortcir.pas}
    pSet^.GetMemForCurrTrAndRelays;    {memory for lists of curr.tr and relays (if exist)}
    {reserve and initialize memory for matrices}
    ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
    {reserve memory for the neg.seq matrix}
    getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pNmat,nnod);
    getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexN, nnod);
    getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
    clrCrow(pVrhs,1,nnod);
    {clear rhs, it is not (yet) needed}
    clrCrow(pWs,1,nnod);
    ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, IECFault,
               false, errcom);
    {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
    PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex,Dodd,Sngl,Tout);     {decompose, pos.seq}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg.seq}
        {create object for relays, if any exists (for display + checking of activation}
        CreateRelays(ipSet, nRelay, nPQrel, pRelay);
        {if there are current transformers, create object for operations with current trafos}
        CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
        {find the current trafos for the relays, if there are relays & curr.trafos}
        if (pRelay <>nil) then pRelay^.FindCurrTrafos(pCurrT);  {in shortcmp.pas}

        {solve the voltage at the fault . . . . . . . . . . . . . .}
        Vnom := Cabs(pSet^.IEC_Volt);         {nominal voltage *C-factor}
        if (Vnom < 1) then Vnom := 1;         {just to be sure}
        Uab := cplx(1.5*Vnom,0.5*sqrt3*Vnom); {nominal Ua-Ub *C-factor}
        {guess a voltage change}
        for ii := 1 to 3 do begin             {first guessed dUa, then calculated}
          if (ii = 1) then dUa := cplx(0,0)          {first zero..}
          else if (ii = 2) then dUa := cplx(Vnom,0)  {..then some value,..}
          else dUa := Crmult(-1,Cdiv(Bsum,Csum));    {...finally solved value}
          dUb := Cadd(Uab,dUa);
          dUc := Crmult(-1,Cadd(dUa,dUb));
          PhaseToSeq(dUa,dUb,dUc,Vp,Vn,Vz);     {to sequence voltages}
          pTwosc^.getNode(2,jc,er2);            {get current "node"}
          clrCrow(pRhs,1,nnod);
          putCvec(pRhs,jc,nnod,Vp);             {fault = source in IEC, voltage to rhs}
          CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
          getCvec(pRhs,jc,nnod,Ip);             {get pos.seq.current at fault}
          clrCrow(pVrhs,1,nnod);
          putCvec(pVrhs,jc,nnod,Vn);            {fault = source in IEC, voltage to rhs}
          CLUbacksub(pNmat,nnod,pIndexN,pVrhs); {backsubstitute = solve neg.seq.net}
          getCvec(pVrhs,jc,nnod,Im);            {get neg.seq.current at fault}
          Iz := cplx(0,0);                      {no zero seq.current}
          SeqToPhase(Ip,Im,Iz,Ia,Ib,Ic);        {phase currents}
          if (ii = 1) then begin                {solve constant}
            Bsum := Cadd(Ia,Ib)
          end else if (ii = 2) then begin       {solve multiplier}
            Csum := Cdiv(Csub(Cadd(Ia,Ib),Bsum),dUa);
          end;
        end;

        {if there are relays, save the neg seq.current here, pos seq. later, but clear first}
        if (pCurrT <> nil) then begin
          pCurrT^.ClearTempVar;
          pCurrT^.SaveComplex(pVrhs, nnod, 2,{-}1); {save neg.seq. as set number 2}
          pCurrT^.SaveComplex(pRhs , nnod, 1,{-}1); {save pos.seq. as set number 1}
          pCurrT^.PhaseMaxOrSum(1,pSet);            {actually, for special CT's}
        end;                                      {... in Shortcmp.pas}
        if (pSet^.nMeasZ > 0) then begin
          {if there are distance relays, save voltages}
          pSet^.SaveDistVoltTemp(pVrhs, nnod, 2);
          pSet^.SaveDistVoltTemp(pRhs , nnod, 1);     {save voltages of distance relays}
        end;
        {save initial neg sec. currents etc for all components (pos.seq. later}
        ClearAllVar(pSet);                        {clear initially}
        ResultsToComp(pSet, nnod, freq, pVrhs,2,ForBoth); {neg.seq to comp.}
        CopyAllVar(pSet,1,2);                             {copy to 2nd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pRhs,1,ForBoth) ; {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);         {in twoshort.pas} {from seq to phase, all var, all comp.}
        if (pCurrT <> nil) then pCurrT^.SumToOutVar;      {sum current trafos ( = 0 !)}
        PolarToRelaysAndBuses(pSet);
        pSet^.UpdateResultBoxes(1);                          {update the eventual result boxes}
        {show also the current through the current trafos, if allowed}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
        ShowImpedances(pSet);                             {show the measured impedances, skemproc.pas}
        if (RelayCurr.visible)  then begin
          pComp := TpComp(RelayCharact.tag);              {pointer to the relay}
          Relay_Show(pComp,'real');
        end;
        {check and show activation (tripping) of relays}
        if (pRelay <> nil) then begin
          pRelay^.GroupActivation;         {in shortcmp.pas}
          pSet^.UpdateResultBoxes(2);      {update secondary boxes, when all values known}
        end;
        {show also distance relay char. and impedance, if visible}
        if (DistRel.visible) then DistanceRelayShow;
      finally
        {destroy object for current trafos, if created}
        if (nCurrT > 0) then begin
          pCurrT^.Destr;                 {destroy object for current trafors}
          pCurrT^.Free;
          dispose(pCurrT);
          pCurrT := nil;
        end;
        {destroy object for relays, if created}
        if (pRelay <> nil) then begin
          pRelay^.Destr;                 {destroy object for relays}
          pRelay^.Free;
          dispose(pRelay);
          pRelay := nil;
        end;
      end;
    end else begin                             {there was an unknown component}
      show_message('unknown component');
    end;
    FreeMat(nnod);                             {free pAdmat, pRhs, pWs, pIndex}
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    pSet^.FreeMemForCurrTrAndRelays;
    Freemem(pVrhs,nnod*SizeOf(complex));
  end else begin
    show_message('no short circuit! ');
  end;
end;

procedure IECEarthFault(ipSet : longint);
{calculate line to line short circuit according to IEC 60909
 ipSet   pointer to the "common" as integer}
var
  aZ0,aZ1,aZ2 : single;      {abs.value (magnitude) of the sequence impedances}
  aZth,aZThtr : single;      {magnitude of the Theven impedance}
  aZTh2tr     : single;      {magnitude of the neg.seq. Theven imped.}
  Csum        : complex;     {coeff.   for Ua = Csum*Ia}
  cpu,cpx     : complex;     {temporary}
  cpv,cpw     : complex;
  dodd,doddN  : single;      {keep track of permutations}
  doddZ       : single;      {keep track of permutations}
  errcom      : boolean;     {error flag}
  er2         : integer;     {error flag}
  freq        : single;      {frequency}
  fZ0,fZ1,fZ2 : single;      {phase of sequence impedances}
  fZth,fZthtr : single;      {phase of Theven impedance}
  fZTh2tr     : single;      {phase of the neg.seq. Theven imped.}
  Ip,Im,Iz    : complex;     {pos,neg,zero seq. currents}
  Ia,Ib,Ic    : complex;     {fault currents}
  I3,I2,I1    : single;
  ii          : integer;     {index}
  jc          : integer;     {current node}
  ju          : integer;     {voltage node}
  mOhm        : single;      {1000, if results in milliOhms, else 1}
  mOhmTxt     : string;      {text for milliohms, or ohms}
  nnod        : integer;     {number of nodes}
  nGenMot     : integer;     {number of synch.machines}
  nRelay      : integer;     {number of relays}
  nFault      : integer;     {number of faults}
  nCurrT      : integer;     {number of current trafos}
  nAsyMot     : integer;     {number of asynchronous motors}
  nSource     : integer;     {number of sources (feeders)}
  nPQrel      : integer;     {number of combined relays}
  pComp       : TpComp;      {pointer to a component}
  pCurrT      : TpCurrT;     {pointer to objectg for current trafos}
  pRelay      : TpRelay;     {pointer to object for relays}
  pSet        : TpCompSet;   {pointer to the common}
  pEarth      : TpComp;      {pointer to the short circuit}
  pVrhs       : pComplex;    {temporary right hand side}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  SnglZ       : Integer;     {flag for singular matrix}
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  ToutZ       : complex;     {test output}
  Va,Vb,Vc    : complex;     {phase voltages}
  Vp,Vn,Vz    : complex;     {sequence voltages}
  Vnom        : complex;     {nominal voltage at fault, * C-factor}
  ZTh         : complex;     {Theven-impedance without fault = normal impedances used, not subtr.}
  ZThtr       : complex;     {Theven-impedance with transient reactances}
  Zth2tr      : complex;     {Thnven-neg.seq.impedance with transient reactances}
begin
  SolveEqFNet(ipSet);                {for peak current}
  pSet := TpCompSet(ipSet);          {integer to pointer}
  freq := pSet^.appl_Hz;
{ Imagoo.NodeButClick(nil);          {determine nodes, get number of nodes}
  pEarth := nil;                     {initialize pointer to short circuit}
  FindShort(pSet, pEarth);           {find two phase short(s), in Twoshort.pas}
  {solve the network only, if there is a short !}
  if (pEarth <> nil) then begin        {two phase short found}
    nnod := pSet^.appl_nn;             {copy number of nodes}
    {count the number of generators, etc. Needed below}
    CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nPQrel); {in shortcir.pas}
    pSet^.GetMemForCurrTrAndRelays;    {memory for lists of curr.tr and relays (if exist)}
    {reserve and initialize memory for matrices}
    ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
    {reserve memory for the neg.seq matrix}
    getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pNmat,nnod);
    getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexN, nnod);
    getmem(pZmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pZmat,nnod);
    getmem(pIndexZ,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexZ, nnod);
    getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
    clrCrow(pVrhs,1,nnod);
    {clear rhs, it is not (yet) needed}
    clrCrow(pWs,1,nnod);
    {if Theven, calculate Theven (pos.seq) imped. with "normal" impedances}
    if (pSet^.appl_but = 15) then begin
      ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, NoFaultTheven, IECEarth,
                 false{IsInitState}, errcom);
      CLUdecomp(pAdmat,pWs,nnod,pIndex ,Dodd ,Sngl ,Tout);  {decompose, pos. seq}
      clrCrow(pRhs,1,nnod);
      cpv := cplx(1000,0);                  {"test"current at fault}
      pEarth^.getNode(1,ju,er2);            {get voltage node}
      pEarth^.getNode(2,jc,er2);            {get current "node"}
      putCvec(pRhs,jc,nnod,cpv);            {fault = source in IEC, current to rhs}
      CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
      getCvec(pRhs,ju,nnod,cpu);            {get pos.seq.voltage at fault}
      cpu := cplx(-cpu.x,-cpu.y);           {must change sign}
      ZTh := Cdiv(cpu,cpv);
      {Theven impedance with transient reactances and without as.machines - - - - - -}
      clrCrow(pWs,1,nnod);
      clrCrow(pRhs,1,nnod);
      clearCmat(pAdmat,nnod);
      ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, TransientTheven, IECEarth,
                 false{IsInitState}, errcom);
      {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
      clearCmat(pNmat,nnod);
      PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
      {decompose}
      CLUdecomp(pAdmat,pWs,nnod,pIndex ,Dodd ,Sngl ,Tout);  {decompose, pos. seq}
      CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg. seq}
      {continue with transient Th.impedance}
      clrCrow(pRhs,1,nnod);
      cpv := cplx(1000,0);                  {"test"current at fault}
      pEarth^.getNode(1,ju,er2);            {get voltage node}
      pEarth^.getNode(2,jc,er2);            {get current "node"}
      putCvec(pRhs,jc,nnod,cpv);            {fault = source in IEC, current to rhs}
      CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
      getCvec(pRhs,ju,nnod,cpu);            {get pos.seq.voltage at fault}
      cpu := cplx(-cpu.x,-cpu.y);           {must change sign}
      ZThtr := Cdiv(cpu,cpv);
      {calculate also negative sequence impedance, when asynch.machines neglected}
      clrCrow(pVrhs,1,nnod);
      Im := cplx(1000,0);                   {current to be injected}
      putCvec(pVrhs,jc,nnod,Im);            {fault = source in IEC, current to rhs}
      CLUbacksub(pNmat,nnod,pIndexN,pVrhs); {backsubstitute = solve neg.seq.net}
      getCvec(pVrhs,ju,nnod,Vn);            {get neg.seq.voltage at fault}
      Zth2tr := Cdiv(Vn,Im);
      Zth2tr.x := -Zth2tr.x; Zth2tr.y := -Zth2tr.y; 
    end else begin
      ZTh := cplx(1e6,1e6);                 {just some value, not used}
      ZThtr := cplx(1e6,1e6);
      ZTh2tr := cplx(1e6,1e6);
    end;
    {..Now, proceed with the earth fault or the Theven imped. with the fault}
    clrCrow(pWs,1,nnod);
    clrCrow(pRhs,1,nnod);
    clearCmat(pAdmat,nnod);
    ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, IECEarth,
               false{IsInitState}, errcom);
    {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
    PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
    {copy the matrix to zero seq. matrix, convert to correspond to zero seq}
    PosToZeroMat(ipSet, pAdmat,pZmat, freq, 0.0, nnod);
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex ,Dodd ,Sngl ,Tout);  {decompose, pos. seq}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg. seq}
        CLUdecomp(pZmat ,pWs,nnod,pIndexZ,DoddZ,SnglZ,ToutZ); {decompose, zero seq}
        {create object for relays, if any exists (for display + checking of activation}
        CreateRelays(ipSet, nRelay, nPQrel, pRelay);
        {if there are current transformers, create object for operations with current trafos}
        CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
        {find the current trafos for the relays, if there are relays & curr.trafos}
        if (pRelay <>nil) then pRelay^.FindCurrTrafos(pCurrT);   {shortcmp.pas}
        {solve the current at the fault . . . . . . . . . . . . . .}
        Vnom := pSet^.IEC_Volt;                     {nominal voltage *C-factor}
        if (Cabs(Vnom) < 1) then Vnom := cplx(1,0); {just to be sure}
        {guess the fault current}
        pEarth^.getNode(1,ju,er2);              {get voltage node}
        pEarth^.getNode(2,jc,er2);              {get current "node"}
        for ii := 1 to 2 do begin               {first guessed Ia, then calculated}
          if (ii = 1) then Ia := Vnom           {first some value ..}
          else Ia := Cdiv(Vnom,Csum);           {...then solved value}
          Ib := cplx(0,0);
          Ic := cplx(0,0);
          PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);        {to sequence currents}
          clrCrow(pRhs,1,nnod);
          putCvec(pRhs,jc,nnod,Ip);             {fault = source in IEC, current to rhs}
          CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
          getCvec(pRhs,ju,nnod,Vp);             {get pos.seq.voltage at fault}
          clrCrow(pVrhs,1,nnod);
          putCvec(pVrhs,jc,nnod,Im);            {fault = source in IEC, current to rhs}
          CLUbacksub(pNmat,nnod,pIndexN,pVrhs); {backsubstitute = solve neg.seq.net}
          getCvec(pVrhs,ju,nnod,Vn);            {get neg.seq.voltage at fault}
          clrCrow(pWs,1,nnod);
          putCvec(pWs,jc,nnod,Iz);              {fault = source in IEC, current to rhs}
          CLUbacksub(pZmat,nnod,pIndexZ,pWs);   {backsubstitute = solve zero seq.net}
          getCvec(pWs,ju,nnod,Vz);              {get zero seq.voltage at fault}
          SeqToPhase(Vp,Vn,Vz,Va,Vb,Vc);        {phase voltages}
          if (ii = 1) then Csum := Cdiv(Va,Ia); {solve multiplier}
          if (ii = 1) and (pSet^.appl_but = 15) then begin {calculating impedances}
            if (Imagoo.mOhmBut.down) then begin
              mOhm := 1000;
              mOhmTxt := ' milliOhms';
            end else begin
              mOhm := 1;  {results in milliOhm}
              mOhmTxt := ' Ohms';
            end;
            {sequence impedances}
            cpx := cplx(-Vp.x,-Vp.y);
            cpu := Cdiv(cpx,Ip);
            if (abs(cpu.x) < 0.0005) then cpu.x := 0;
            if (abs(cpu.y) < 0.0005) then cpu.y := 0;
            cpx := cplx(-Vn.x,-Vn.y);
            cpv := Cdiv(cpx,Im);
            if (abs(cpv.x) < 0.0005) then cpv.x := 0;
            if (abs(cpv.y) < 0.0005) then cpv.y := 0;
            cpx := cplx(-Vz.x,-Vz.y);
            cpw := Cdiv(cpx,Iz);
            if (abs(cpw.x) < 0.0005) then cpw.x := 0;
            if (abs(cpw.y) < 0.0005) then cpw.y := 0;
            pSet^.Seq_Z0 := cpw;
            pSet^.Seq_Z2 := cpv;
            pSet^.Seq_Z1 := cpu;
            {calculate magnitude and phase}
            aZ0 := Cabs(cpw);
            aZ1 := Cabs(cpu);
            aZ2 := Cabs(cpv);
            fZ0 := Cangle(cpw)*180/pi;
            fZ1 := Cangle(cpu)*180/pi;
            fZ2 := Cangle(cpv)*180/pi;
            aZTh := Cabs(Zth);
            fZTh := Cangle(Zth)*180/pi;
            aZThtr := Cabs(Zthtr);
            fZThtr := Cangle(Zthtr)*180/pi;
            aZTh2tr := Cabs(Zth2tr);
            fZTh2tr := Cangle(Zth2tr)*180/pi;
            Imagoo.OutMemo.clear;
            Imagoo.OutMemo.lines.add('(X) Sequence impedances ');
            Imagoo.OutMemo.lines.add('Thevenin impedance ');
            Imagoo.OutMemo.lines.add('    '+pSet^.Fault_Pos+mOhmTxt);
            Imagoo.OutMemo.lines.add('  ZT = '+sixsignedchar(mOhm*ZTh.x)+'  '+sixsignedchar(mOhm*ZTh.y)+'i ;'+
                                     ' |ZT| = '+sixsignedchar(mOhm*aZTh)+'    = '+sixcharonedecimal(fZTh)+'');
            Imagoo.OutMemo.lines.add('- - - -');
            Imagoo.OutMemo.lines.add('Thevenin transient impedance ');
            Imagoo.OutMemo.lines.add('    '+pSet^.Fault_Pos+mOhmTxt);
            Imagoo.OutMemo.lines.add('  ZT = '+sixsignedchar(mOhm*ZThtr.x)+'  '+sixsignedchar(mOhm*ZThtr.y)+'i ;'+
                                     ' |ZT| = '+sixsignedchar(mOhm*aZThtr)+'    = '+sixcharonedecimal(fZThtr)+'');
            Imagoo.OutMemo.lines.add('  Z2 = '+sixsignedchar(mOhm*ZTh2tr.x)+'  '+sixsignedchar(mOhm*ZTh2tr.y)+'i ;'+
                                     ' |Z2| = '+sixsignedchar(mOhm*aZTh2tr)+'    = '+sixcharonedecimal(fZTh2tr)+'');
            Imagoo.OutMemo.lines.add('  Z0 = '+sixsignedchar(mOhm*cpw.x)+'  '+sixsignedchar(mOhm*cpw.y)+'i ;'+
                                     ' |Z0| = '+sixsignedchar(mOhm*aZ0)+'    = '+sixcharonedecimal(fZ0)+'');
            Imagoo.OutMemo.lines.add('- - - -');
            Imagoo.OutMemo.lines.add('Sequence (subtransient) impedances ');
            Imagoo.OutMemo.lines.add('    '+pSet^.Fault_Pos+mOhmTxt);
            Imagoo.OutMemo.lines.add('  Z1 = '+sixsignedchar(mOhm*cpu.x)+'  '+sixsignedchar(mOhm*cpu.y)+'i ;'+
                                     ' |Z1| = '+sixsignedchar(mOhm*aZ1)+'    = '+sixcharonedecimal(fZ1)+'');
            Imagoo.OutMemo.lines.add('  Z2 = '+sixsignedchar(mOhm*cpv.x)+'  '+sixsignedchar(mOhm*cpv.y)+'i ;'+
                                     ' |Z2| = '+sixsignedchar(mOhm*aZ2)+'    = '+sixcharonedecimal(fZ2)+'');
            Imagoo.OutMemo.lines.add('  Z0 = '+sixsignedchar(mOhm*cpw.x)+'  '+sixsignedchar(mOhm*cpw.y)+'i ;'+
                                     ' |Z0| = '+sixsignedchar(mOhm*aZ0)+'    = '+sixcharonedecimal(fZ0)+'');
            Imagoo.OutMemo.lines.add(' ');
            Imagoo.OutMemo.lines.add('Thevenin = normal reactances used');
            Imagoo.OutMemo.lines.add('Th.transient = transient reactances used');
            Imagoo.OutMemo.lines.add('Sequence = subtransient reactances used');
            Imagoo.OutMemo.width := 440;
            Imagoo.OutMemo.height := 350;
            Imagoo.OutMemo.visible := true;
            {sequence fault currents}
            { I3 := Cabs(Vnom)/Cabs(cpu);
              I2 := sqrt(3)*Cabs(Vnom)/Cabs(Cadd(cpu,cpv));
              I1 := 3*Cabs(Vnom)/Cabs(Cadd(cpu,Cadd(cpv,cpw)));
           {  showmessage(floattostr(I3)+'  '+floattostr(I2)+'  '+floattostr(I1))}
          end;
        end;
        {if there are relays, save the neg seq.current here, pos seq. later, but clear first}
        if (pCurrT <> nil) then begin
          pCurrT^.ClearTempVar;
          pCurrT^.SaveComplex(pWs  , nnod, 3,{-}1); {save zero seq. as set number 3}
          pCurrT^.SaveComplex(pVrhs, nnod, 2,{-}1); {save neg. seq. as set number 2}
          pCurrT^.SaveComplex(pRhs , nnod, 1,{-}1); {save pos. seq. as set number 1}
          pCurrT^.PhaseMaxOrSum(1, pSet);           {actually, for special CT's}
        end;                                        {... in Shortcmp.pas}
        if (pSet^.nMeasZ > 0) then begin
          {if there are distance relays, save voltages}
          pSet^.SaveDistVoltTemp(pWs  , nnod, 3);
          pSet^.SaveDistVoltTemp(pVrhs, nnod, 2);
          pSet^.SaveDistVoltTemp(pRhs , nnod, 1);     {save voltages of distance relays}
        end;
        {save initial neg sec. currents etc for all components (pos.seq. later}
        ClearAllVar(pSet);                        {clear initially}
        ResultsToComp(pSet, nnod, freq, pWs,3,ForBoth);   {zero seq to comp.}
        CopyAllVar(pSet,1,3);                             {copy to 3rd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pVrhs,2,ForBoth); {neg. seq to comp.}
        CopyAllVar(pSet,1,2);                             {copy to 2nd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pRhs,1,ForBoth) ; {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);        {in twoshort.pas}  {from seq to phase, all var, all comp.}
        if (pCurrT <> nil) then pCurrT^.SumToOutVar;      {sum current trafos ( = 0 !)}
        PolarToRelaysAndBuses(pSet);
        pSet^.UpdateResultBoxes(1);                       {update the eventual result boxes}
        {show also the current through the current trafos, if allowed}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
        ShowImpedances(pSet);                             {show the measured impedances, skemproc.pas}
        if (RelayCurr.visible)  then begin
          pComp := TpComp(RelayCharact.tag);              {pointer to the relay}
          Relay_Show(pComp,'real');
        end;
        {check and show activation (tripping) of relays...}
        {...but not when impedance calculation}
        if (pRelay <> nil) and not(pSet^.appl_but=15) then begin
          pRelay^.GroupActivation;         {in shortcmp.pas}
          pSet^.UpdateResultBoxes(2);      {update secondary boxes, when all values known}
        end;
        {show also distance relay char. and impedance, if visible}
        if (DistRel.visible) then DistanceRelayShow;
      finally
        {destroy object for current trafos, if created}
        if (nCurrT > 0) then begin
          pCurrT^.Destr;                 {destroy object for current trafors}
          pCurrT^.Free;
          dispose(pCurrT);
          pCurrT := nil;
        end;
        {destroy object for relays, if created}
        if (pRelay <> nil) then begin
          pRelay^.Destr;                 {destroy object for relays}
          pRelay^.Free;
          dispose(pRelay);
          pRelay := nil;
        end;
      end;
    end else begin                             {there was an unknown component}
      show_message('unknown component');
    end;
    FreeMat(nnod);                             {free pAdmat, pRhs, pWs, pIndex}
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    Freemem(pZmat,nnod*nnod*SizeOf(complex));  {free matrix for zero seq.}
    Freemem(pIndexZ,nnod*Sizeof(integer));     {free index array, zero seq}
    pSet^.FreeMemForCurrTrAndRelays;
    Freemem(pVrhs,nnod*SizeOf(complex));
  end else begin
    show_message('no short circuit! ');
  end;
end;

procedure IECLineLineEarth(ipSet : longint);
{calculate line to line to earth short circuit according to IEC 60909
 ipSet   pointer to the "common" as integer}
var
  Bsum,Csum   : complex;     {coeff.   for Ic = Csum*dUa+Bsum}
  cpu         : complex;     {temporary}
  dodd,doddN  : single;      {keep track of permutations}
  doddZ       : single;      {keep track of permutations}
  errcom      : boolean;     {error flag}
  er2         : integer;     {error flag}
  freq        : single;      {frequency}
  Ip,Im,Iz    : complex;     {pos,neg,zero seq. currents}
  Ia,Ib,Ic    : complex;     {fault currents}
  ii          : integer;     {index}
  jc          : integer;     {current node}
  ju          : integer;     {voltage node}
  nnod        : integer;     {number of nodes}
  nGenMot     : integer;     {number of synch.machines}
  nRelay      : integer;     {number of relays}
  nFault      : integer;     {number of faults}
  nCurrT      : integer;     {number of current trafos}
  nAsyMot     : integer;     {number of asynchronous motors}
  nSource     : integer;     {number of sources (feeders)}
  nPQrel      : integer;     {number of combined relays}
  pComp       : TpComp;      {pointer to a component}
  pCurrT      : TpCurrT;     {pointer to objectg for current trafos}
  pRelay      : TpRelay;     {pointer to object for relays}
  pSet        : TpCompSet;   {pointer to the common}
  pEarth      : TpComp;      {pointer to the short circuit}
  pVrhs       : pComplex;    {temporary right hand side}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  SnglZ       : Integer;     {flag for singular matrix}
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  ToutZ       : complex;     {test output}
  Va,Vb,Vc    : complex;     {phase voltages}
  Vp,Vn,Vz    : complex;     {sequence voltages}
  Vnom        : complex;     {nominal voltage at fault, * C-factor}
begin
  SolveEqFNet(ipSet);                {for peak current}
  pSet := TpCompSet(ipSet);          {integer to pointer}
  freq := pSet^.appl_Hz;
{ Imagoo.NodeButClick(nil);          {determine nodes, get number of nodes}
  pEarth := nil;                     {initialize pointer to short circuit}
  FindShort(pSet, pEarth);           {find two phase short(s), in Twoshort.pas}
  {solve the network only, if there is a short !}
  if (pEarth <> nil) then begin        {two phase short found}
    nnod := pSet^.appl_nn;             {copy number of nodes}
    {count the number of generators, etc. Needed below}
    CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nPQrel); {in shortcir.pas}
    pSet^.GetMemForCurrTrAndRelays;    {memory for lists of curr.tr and relays (if exist)}
    {reserve and initialize memory for matrices}
    ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
    {reserve memory for the neg.seq matrix}
    getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pNmat,nnod);
    getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexN, nnod);
    getmem(pZmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pZmat,nnod);
    getmem(pIndexZ,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexZ, nnod);
    getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
    clrCrow(pVrhs,1,nnod);
    {clear rhs, it is not (yet) needed}
    clrCrow(pWs,1,nnod);
    ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, IECFault,
               false{IsInitState}, errcom);
    {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
    PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
    {copy the matrix to zero seq. matrix, convert to correspond to zero seq}
    PosToZeroMat(ipSet, pAdmat,pZmat, freq, 0.0, nnod);
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex ,Dodd ,Sngl ,Tout);  {decompose, pos. seq}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg. seq}
        CLUdecomp(pZmat ,pWs,nnod,pIndexZ,DoddZ,SnglZ,ToutZ); {decompose, zero seq}
        {create object for relays, if any exists (for display + checking of activation}
        CreateRelays(ipSet, nRelay, nPQrel, pRelay);
        {if there are current transformers, create object for operations with current trafos}
        CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
        {find the current trafos for the relays, if there are relays & curr.trafos}
        if (pRelay <>nil) then pRelay^.FindCurrTrafos(pCurrT); {shortcmp.pas}
        {solve the current at the fault . . . . . . . . . . . . . .}
        Vnom := pSet^.IEC_Volt;                     {nominal voltage *C-factor (see Elplex.pas)}
        if (Cabs(Vnom) < 1) then Vnom := cplx(1,0); {just to be sure}
        {guess the voltage at the unfaulted phase}
        pEarth^.getNode(1,ju,er2);              {get voltage node}
        pEarth^.getNode(2,jc,er2);              {get current "node"}
        Va := Vnom;                             {constant voltages at phase a and b}
        Vb := Cmult(Va,cplx(-0.5,-0.5*sqrt3));
        for ii := 1 to 3 do begin               {first guessed , then calculated}
          if      (ii = 1) then Vc := cplx(0,0)  {first zero voltage ..}
          else if (ii = 2) then Vc := Crmult(-1,Cadd(Va,Vb))  {.. then guessed value..}
          else Vc := Crmult(-1,Cdiv(Bsum,Csum)); {...finally the solved value}
          PhaseToSeq(Va,Vb,Vc,Vp,Vn,Vz);        {to sequence currents}
          clrCrow(pRhs,1,nnod);
          putCvec(pRhs,jc,nnod,Vp);             {fault = source in IEC, voltage to rhs}
          CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
          getCvec(pRhs,jc,nnod,Ip);             {get pos.seq.current at fault}
          clrCrow(pVrhs,1,nnod);
          putCvec(pVrhs,jc,nnod,Vn);            {fault = source in IEC, voltage to rhs}
          CLUbacksub(pNmat,nnod,pIndexN,pVrhs); {backsubstitute = solve neg.seq.net}
          getCvec(pVrhs,jc,nnod,Im);            {get neg.seq.current at fault}
          clrCrow(pWs,1,nnod);
          putCvec(pWs,jc,nnod,Vz);              {fault = source in IEC, current to rhs}
          CLUbacksub(pZmat,nnod,pIndexZ,pWs);   {backsubstitute = solve zero seq.net}
          getCvec(pWs,jc,nnod,Iz);              {get zero seq.voltage at fault}
          SeqToPhase(Ip,Im,Iz,Ia,Ib,Ic);        {phase voltages}
          if (ii = 1) then Bsum := Ic
          else if (ii = 2) then Csum := Cdiv(Csub(Ic,Bsum),Vc);
        end;
        {if there are relays, save the neg seq.current here, pos seq. later, but clear first}
        if (pCurrT <> nil) then begin
          pCurrT^.ClearTempVar;
          pCurrT^.SaveComplex(pWs  , nnod, 3,{-}1); {save zero seq. as set number 3}
          pCurrT^.SaveComplex(pVrhs, nnod, 2,{-}1); {save neg. seq. as set number 2}
          pCurrT^.SaveComplex(pRhs , nnod, 1,{-}1); {save pos. seq. as set number 1}
          pCurrT^.PhaseMaxOrSum(1,pSet);            {actually, for special CT's}
        end;                                        {... in Shortcmp.pas}
        if (pSet^.nMeasZ > 0) then begin
          {if there are distance relays, save voltages}
          pSet^.SaveDistVoltTemp(pWs  , nnod, 3);
          pSet^.SaveDistVoltTemp(pVrhs, nnod, 2);
          pSet^.SaveDistVoltTemp(pRhs , nnod, 1);     {save voltages of distance relays}
        end;
        {save initial neg sec. currents etc for all components (pos.seq. later}
        ClearAllVar(pSet);                        {clear initially}
        ResultsToComp(pSet, nnod, freq, pWs,3,ForBoth);   {zero seq to comp.}
        CopyAllVar(pSet,1,3);                             {copy to 3rd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pVrhs,2,ForBoth); {neg. seq to comp.}
        CopyAllVar(pSet,1,2);                             {copy to 2nd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pRhs,1,ForBoth) ; {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);         {in twoshort.pas} {from seq to phase, all var, all comp.}
        if (pCurrT <> nil) then pCurrT^.SumToOutVar;      {sum current trafos ( = 0 !)}
        PolarToRelaysAndBuses(pSet);
        pSet^.UpdateResultBoxes(1);                       {update the eventual result boxes}
        {show also the current through the current trafos, if allowed}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
        ShowImpedances(pSet);                             {show the measured impedances, skemproc.pas}
        if (RelayCurr.visible)  then begin
          pComp := TpComp(RelayCharact.tag);              {pointer to the relay}
          Relay_Show(pComp,'real');
        end;
        {check and show activation (tripping) of relays}
        if (pRelay <> nil) then begin
          pRelay^.GroupActivation;         {in shortcmp.pas}
          pSet^.UpdateResultBoxes(2);      {update secondary boxes, when all values known}
        end;
        {show also distance relay char. and impedance, if visible}
        if (DistRel.visible) then DistanceRelayShow;
      finally
        {destroy object for current trafos, if created}
        if (nCurrT > 0) then begin
          pCurrT^.Destr;                 {destroy object for current trafors}
          pCurrT^.Free;
          dispose(pCurrT);
          pCurrT := nil;
        end;
        {destroy object for relays, if created}
        if (pRelay <> nil) then begin
          pRelay^.Destr;                 {destroy object for relays}
          pRelay^.Free;
          dispose(pRelay);
          pRelay := nil;
        end;
      end;
    end else begin                             {there was an unknown component}
      show_message('unknown component');
    end;
    FreeMat(nnod);                             {free pAdmat, pRhs, pWs, pIndex}
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    Freemem(pZmat,nnod*nnod*SizeOf(complex));  {free matrix for zero seq.}
    Freemem(pIndexZ,nnod*Sizeof(integer));     {free index array, zero seq}
    pSet^.FreeMemForCurrTrAndRelays;
    Freemem(pVrhs,nnod*SizeOf(complex));
  end else begin
    show_message('no short circuit! ');
  end;
end;
{ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - }

procedure FindOpenLine(pSet : TpCompSet; var pOpenL : TpComp);
{find the short circuit(s)
 pSet    pointer to the set of components, etc.
 pOpenL  pointer to the open-line component
 NOTE, only one accepted
 }
var
  errc    : integer;     {error flag}
  jcomp   : integer;     {counts components}
  ModNam  : string;      {name (type) of the component}
  nOL     : integer;     {number of open lines}
  pComp   : TpComp;      {pointer to components}
begin
  pComp := nil;                              {initialize pointer to components}
  errc  := 0;                                {initialize error flag}
  jcomp := 0;
  nOL := 0;                                  {initialize number of short circuits}
  while (errc = 0) do begin                  {scan all components}
    pSet^.NextComp(pComp,errc,jcomp,true);   {get pointer to next component}
    if (errc = 0) then begin                 {component found}
      pComp^.GetModName(ModNam);             {get the name (type) of the component}
      if (ModNam = gOpenL) then begin        {is it an open line}
        inc(nOL);                            {increment count of open lines}
        if (nOL = 1) then begin              {first short}
          pOpenL := pComp;
        end else begin
          pOpenL := nil;                     {only one fault allowed}
        end;
      end;
    end;
  end;
  if (nOL = 0) then show_message('No open lines')
  else if (nOL > 1) then show_message('Too many open lines');
end;

procedure TwoLineVoltages(ii: integer; Ka,Kb,Kp,Ku,Kd : array of complex; Vtst: single;
                      var Va,Vb,Vp,Vu,Vd: complex);
{set the test or determine the final voltages at the upstream and downstream ends
 of the open-line component, when one line open
 ii    loop index
 Ka,.. coefficient vectors
 Vtst  the magnitude of the test voltage
 Va,.. the voltages
     Va     = phase a, same voltage at up- and downstream ends
     Vp, Vp = phase b, up- and downstream voltages
     Vu, Vd = phase c, up- and downstream voltages}
const
  nn   = 5;      {dimension of the equations}
var
  dodd        : single;      {keep track of permutations}
  jj          : integer;     {index}
  paa,pbb,pcc : pComplex;    {pointers to the coeff.matrix and rhs, and working space}
  pci         : pInteger;    {pointer to space for row exchanges}
  Sngl        : Integer;     {flag for singular matrix}
  Tout        : Complex;     {test output from matrix decomposition}
  aa          : array[1..nn*nn] of complex;  {coeff.matrix for solving the voltages}
  bb          : array[1..nn] of complex;     {rhs for solving the voltages}
  cc          : array[1..nn] of complex;     {working space}
  ci          : array[1..nn] of integer;     {for exchanges of row}
begin
  paa := @aa[1];
  pbb := @bb[1];
  pcc := @cc[1];
  pci := @ci[1];
  {determine voltages}
  if (ii = 0) then begin
    Va := cplx(0,0);                    {first time = no voltage}
    Vb := cplx(0,0);
    Vp := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 1) then begin
    Va := cplx(Vtst,0);
    Vb := cplx(0,0);
    Vp := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 2) then begin
    Va := cplx(0,0);
    Vb := cplx(Vtst,0);
    Vp := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 3) then begin
    Va := cplx(0,0);
    Vb := cplx(0,0);
    Vp := cplx(Vtst,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 4) then begin
    Va := cplx(0,0);
    Vb := cplx(0,0);
    Vp := cplx(0,0);
    Vu := cplx(Vtst,0);
    Vd := cplx(0,0);
  end else if (ii = 5) then begin
    Va := cplx(0,0);
    Vb := cplx(0,0);
    Vp := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(Vtst,0);
  end else begin                        {last time, solve the current}
    {solve the currents so that upstream and downstream currents are the same..
    {..for phases a and b, and zero for phase c}
    clearCmat(paa,nn);                {clear coeff.matrix..}
    clearCvec(pbb,nn);                {..and rhs}
    {construct rhs}
    putCvec(pbb,1,nn,Ka[0]);
    putCvec(pbb,2,nn,Kb[0]);
    putCvec(pbb,3,nn,Kp[0]);
    putCvec(pbb,4,nn,Ku[0]);
    putCvec(pbb,5,nn,Kd[0]);
    {construct coeff.matrix}
    for jj := 1 to nn do begin
      putCmat(paa,1,jj,nn,nn,Ka[jj]);
      putCmat(paa,2,jj,nn,nn,Kb[jj]);
      putCmat(paa,3,jj,nn,nn,Kp[jj]);
      putCmat(paa,4,jj,nn,nn,Ku[jj]);
      putCmat(paa,5,jj,nn,nn,Kd[jj]);
    end;
    {decompose, clear first}
    clearIvec(pci, nn);
    clrCrow(pcc,1,nn);
    CLUdecomp(paa,pcc,nn,pci,Dodd,Sngl,Tout); {decompose}
    CLUbacksub(paa,nn,pci,pbb);
    {pick voltages}
    Va := bb[1];
    Vb := bb[2];
    Vp := bb[3];
    Vu := bb[4];
    Vd := bb[5];
  end;
end;

procedure OneLineVoltages(ii: integer; Ka,Kb,Ku,Kd : array of complex; Vtst: single;
                      var Va,Vb,Vu,Vd: complex);
{set the test or determine the final voltages at the upstream and downstream ends
 of the open-line component, when one line open
 ii    loop index
 Ka,.. coefficient vectors
 Vtst  the magnitude of the test voltage
 Va,.. the voltages
     Va, Vb = phase a and b, same voltage at up- and downstream ends
     Vu, Vd = phase c, up- and downstream voltages}
const
  nn   = 4;      {dimension of the equations}
var
  dodd        : single;      {keep track of permutations}
  jj          : integer;     {index}
  paa,pbb,pcc : pComplex;    {pointers to the coeff.matrix and rhs, and working space}
  pci         : pInteger;    {pointer to space for row exchanges}
  Sngl        : Integer;     {flag for singular matrix}
  Tout        : Complex;     {test output from matrix decomposition}
  aa          : array[1..nn*nn] of complex;  {coeff.matrix for solving the voltages}
  bb          : array[1..nn] of complex;     {rhs for solving the voltages}
  cc          : array[1..nn] of complex;     {working space}
  ci          : array[1..nn] of integer;     {for exchanges of row}
begin
  paa := @aa[1];
  pbb := @bb[1];
  pcc := @cc[1];
  pci := @ci[1];
  {determine voltages}
  if (ii = 0) then begin
    Va := cplx(0,0);                    {first time = no voltage}
    Vb := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 1) then begin
    Va := cplx(Vtst,0);
    Vb := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 2) then begin
    Va := cplx(0,0);
    Vb := cplx(Vtst,0);
    Vu := cplx(0,0);
    Vd := cplx(0,0);
  end else if (ii = 3) then begin
    Va := cplx(0,0);
    Vb := cplx(0,0);
    Vu := cplx(Vtst,0);
    Vd := cplx(0,0);
  end else if (ii = 4) then begin
    Va := cplx(0,0);
    Vb := cplx(0,0);
    Vu := cplx(0,0);
    Vd := cplx(Vtst,0);
  end else begin                        {last time, solve the current}
    {solve the currents so that upstream and downstream currents are the same..
    {..for phases a and b, and zero for phase c}
    clearCmat(paa,nn);                {clear coeff.matrix..}
    clearCvec(pbb,nn);                {..and rhs}
    {construct rhs}
    putCvec(pbb,1,nn,Ka[0]);
    putCvec(pbb,2,nn,Kb[0]);
    putCvec(pbb,3,nn,Ku[0]);
    putCvec(pbb,4,nn,Kd[0]);
    {construct coeff.matrix}
    for jj := 1 to nn do begin
      putCmat(paa,1,jj,nn,nn,Ka[jj]);
      putCmat(paa,2,jj,nn,nn,Kb[jj]);
      putCmat(paa,3,jj,nn,nn,Ku[jj]);
      putCmat(paa,4,jj,nn,nn,Kd[jj]);
    end;
    {decompose, clear first}
    clearIvec(pci, nn);
    clrCrow(pcc,1,nn);
    CLUdecomp(paa,pcc,nn,pci,Dodd,Sngl,Tout); {decompose}
    CLUbacksub(paa,nn,pci,pbb);
    {pick voltages}
    Va := bb[1];
    Vb := bb[2];
    Vu := bb[3];
    Vd := bb[4];
//showmessage('a  '+comptostr(Va));
//showmessage('b  '+comptostr(Vb));
//showmessage('u  '+comptostr(Vu));
//showmessage('d  '+comptostr(Vd));


  end;
end;

procedure OpenCurrents(pAdmat,pNmat,pZmat,pPs,pNs,pWs: pComplex;
                       pIndex,pIndexN,pIndexZ: pInteger;
                       nnod,iu,id : integer;
                       Vpu,Vnu,Vzu,Vpd,Vnd,Vzd: complex;
                   var Iau,Ibu,Icu,Iad,Ibd,Icd: complex);
{solve the currents into and out of both ends of the open line-component,
 when the upstream and downstream voltages are given
 pAdmat,pNmat,pZmat the decomposed pos,neg,zero seq. admittance matrices
 pIndex,pIndexN,pIndexZ  vector for row exhanges
 nnod   dimension of the matrices
 iu,id  indices of the upstream and downstream nodes
 Vpu..  up-, and downstream pos,neg,zero seq. voltages
 Iau..  up- and downstream pos,neg,zero seq. currents}
var
  Ipu,Imu,Izu : complex; {upstream seq. currents}
  Ipd,Imd,Izd : complex; {downstream seq. currents}
begin
  putCvec(pPs,iu,nnod,Vpu);             {upstream voltage to rhs}
  putCvec(pPs,id,nnod,Vpd);             {downstream voltage to rhs}
  CLUbacksub(pAdmat,nnod,pIndex,pPs);   {backsubstitute = solve pos.seq.net}
  getCvec(pPs,iu,nnod,Ipu);             {upstream current}
  getCvec(pPs,id,nnod,Ipd);
  {negative seq. next - - - - - - - - - -}
  clrCrow(pNs,1,nnod);                  {no other sources in neg, zero seq.}
  putCvec(pNs,iu,nnod,Vnu);             {upstream voltage}
  putCvec(pNs,id,nnod,Vnd);             {downstream voltage}
  CLUbacksub(pNmat,nnod,pIndexN,pNs);   {backsubstitute = solve neg.seq.net}
  getCvec(pNs,iu,nnod,Imu);
  getCvec(pNs,id,nnod,Imd);             {downstream current}
  {finally zero sequence}
  clrCrow(pWs,1,nnod);
  putCvec(pWs,iu,nnod,Vzu);             {upstream zero seq. voltage}
  putCvec(pWs,id,nnod,Vzd);             {downstream zero seq. voltage}
  CLUbacksub(pZmat,nnod,pIndexZ,pWs);   {backsubstitute = solve zero seq.net}
  getCvec(pWs,iu,nnod,Izu);
  getCvec(pWs,id,nnod,Izd);
  SeqToPhase(Ipu,Imu,Izu,Iau,Ibu,Icu);  {upstream phase currents}
  SeqToPhase(Ipd,Imd,Izd,Iad,Ibd,Icd);  {downstream phase currents}
end;

function OpenLineFault(ipSet : longint; OneOrTwo: string): boolean;
{solve the steady state network in the case of an open-line fault
 ipSet     pointer to the "common" as integer
 OneOrTwo  'one' = phase c open, 'two' = phases b and c open }
const
  nn   = 5;       {max.dimension of equations}
  Vtst = 1000.0;  {test voltage}
var
  dodd,doddN  : single;      {keep track of permutations}
  doddZ       : single;      {keep track of permutations}
  errcom      : boolean;     {error flag}
  er2         : integer;     {error flag}
  freq        : single;      {frequency}
  Iau,Ibu,Icu : complex;     {upstream currents}
  Iad,Ibd,Icd : complex;     {downstream currents}
  Ipd,Ipu     : complex;     {pos.seq. currents}
  id,iu       : integer;     {up- and downstream current nodes}
  ii,jj       : integer;     {index}
  jd,ju       : integer;     {up-, downstream voltage nodes}
  nnod        : integer;     {number of nodes}
  nGenMot     : integer;     {number of synch.machines}
  nRelay      : integer;     {number of relays}
  nFault      : integer;     {number of faults}
  ncalc       : integer;     {number of calculations}
  nCurrT      : integer;     {number of current trafos}
  nAsyMot     : integer;     {number of asynchronous motors}
  nSource     : integer;     {number of sources (feeders)}
  nPQrel      : integer;     {number of combined relays}
  pCurrT      : TpCurrT;     {pointer to object for current trafos}
  pOpenL      : TpComp;      {pointer to the open line fault}
  pRelay      : TpRelay;     {pointer to object for relays}
  pSet        : TpCompSet;   {pointer to the common}
  pPs,pNs     : pComplex;    {temporary right hand sides}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  SnglZ       : Integer;     {flag for singular matrix}
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  ToutZ       : complex;     {test output}
  Va,Vb,Vu,Vd : complex;     {test voltages}
  Vp          : complex;     {test voltage}
  Vau,Vbu,Vcu : complex;     {upstream phase voltages}
  Vad,Vbd,Vcd : complex;     {downstream phase voltages}
  Vpu,Vnu,Vzu : complex;     {upstream sequence voltages}
  Vpd,Vnd,Vzd : complex;     {downstream sequence voltages}
  Ka,Kb,Kp,Ku,Kd : array[0..nn] of complex;     {coeff.for current from voltage}
begin
  pSet := TpCompSet(ipSet);          {integer to pointer}
  freq := pSet^.appl_Hz;
  pOpenL := nil;                       {initialize pointer to the open line}
  FindOpenLine(pSet, pOpenL);          {find the open line fault(s)}
  Imagoo.NodeButClick(nil);            {determine nodes, get number of nodes (needed?) }
  {solve the network only, if there is exactly one open line fault}
  if (pOpenL <> nil) then begin        {two phase short found}
    nnod := pSet^.appl_nn;             {copy number of nodes}
    {count the number of generators, etc. Needed below}
    CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nPQrel); {in shortcir.pas}
    pSet^.GetMemForCurrTrAndRelays;    {memory for lists of curr.tr and relays (if exist)}
    {reserve and initialize memory for matrices}
    ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
    {reserve memory for the neg.seq matrix}
    getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pNmat,nnod);
    getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexN, nnod);
    getmem(pZmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pZmat,nnod);
    getmem(pIndexZ,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexZ, nnod);
    getmem(pPs,nnod*SizeOf(complex));           {for temporary right hand side}
    clrCrow(pPs,1,nnod);
    getmem(pNs,nnod*SizeOf(complex));           {for temporary right hand side}
    clrCrow(pNs,1,nnod);
    clrCrow(pWs,1,nnod);
    ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, OpenLine,
               Imagoo.InitialStateBut.down, errcom);
    {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
    PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
    {copy the matrix to zero seq. matrix, convert to correspond to zero seq}
    PosToZeroMat(ipSet, pAdmat,pZmat, freq, 0.0, nnod);
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex ,Dodd ,Sngl ,Tout);  {decompose, pos. seq}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg. seq}
        CLUdecomp(pZmat ,pWs,nnod,pIndexZ,DoddZ,SnglZ,ToutZ); {decompose, zero seq}
        {create object for relays, if any exists (for display + checking of activation}
        CreateRelays(ipSet, nRelay, nPQrel, pRelay);
        {if there are current transformers, create object for operations with current trafos}
        CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
        {find the current trafos for the relays, if there are relays & curr.trafos}
        if (pRelay <>nil) then pRelay^.FindCurrTrafos(pCurrT);   {shortcmp.pas}
        {solve the voltages at the open line . . . . . . . . . . . . . . . . .}
        {When one broken line:
         1: voltage in all phases = 0
         2: voltage in phase a <> 0 (up- and downstream), others = 0
         3: voltage in phase b <> 0 (up- and downstream), others = 0
         4: voltage in phase c <> 0, only upstream, others = 0
         5: voltage in phase c <> 0, only downstream, others = 0
         6: deduce voltages in all phases, so that up- and downstream currents
            are equal in phases a and b, and zero in phase c
         When two broken lines: ...?... not yet}
        {if node = ground ??? voltages set to zero... }
        pOpenL^.getNode(1,ju,er2);              {get upstream voltage node}
        pOpenL^.getNode(2,jd,er2);              {get downstream voltage node}
        pOpenL^.getNode(3,iu,er2);              {get upstream current "node"}
        pOpenL^.getNode(4,id,er2);              {get downstream current "node"}
        if (OneOrTwo = 'one') then ncalc := 5 else ncalc := 6;  {note from 0 to ncalc}
        for ii := 0 to ncalc do begin           {first guessed currents, then calculated}
          copyCvec(pRhs,pPs,nnod);              {copy to temporary rhs. Must not destroy rhs}
          {different voltages at each round}
          if (OneOrTwo = 'one') then
           OneLineVoltages(ii, Ka,Kb,Ku,Kd, Vtst, Va,Vb,Vu,Vd)
          else
           TwoLineVoltages(ii, Ka,Kb,Kp,Ku,Kd, Vtst, Va,Vb,Vp,Vu,Vd);
          {solve net at different voltages}
          if (ii = 0) then begin                  {at zero currents}
            {sequence components not needed at first time}
            CLUbacksub(pAdmat,nnod,pIndex,pPs);   {backsubstitute = solve pos.seq.net}
            if (iu > 0) then getCvec(pPs,iu,nnod,Ipu)
            else Vpu := cplx(0,0);                {get pos.seq. upstream current }
            if (id > 0) then getCvec(pPs,id,nnod,Ipd)
            else Vpd := cplx(0,0);                {get pos.seq. downstream current }
            {phase currents}
            Iau := Ipu;
            Iad := Ipd;
            Ibu := Cmult(emifii,Ipu);             {phase shift -120 degrees}
            Ibd := Cmult(emifii,Ipd);
            Icu := Cmult(eifii,Ipu);              {phase shift +120 degrees}
            Icd := Cmult(eifii,Ipd);
          end else begin
            if (OneOrTwo = 'one') then begin
              PhaseToSeq(Va,Vb,Vu,Vpu,Vnu,Vzu);     {to upstream sequence voltages}
              PhaseToSeq(Va,Vb,Vd,Vpd,Vnd,Vzd);     {to downstream sequence voltages}
            end else begin
              PhaseToSeq(Va,Vb,Vu,Vpu,Vnu,Vzu);     {to upstream sequence voltages}
              PhaseToSeq(Va,Vp,Vd,Vpd,Vnd,Vzd);     {to downstream sequence voltages}
            end;
            {solve the currents}
            OpenCurrents(pAdmat,pNmat,pZmat,pPs,pNs,pWs,
                         pIndex,pIndexN,pIndexZ,
                         nnod,iu,id,
                         Vpu,Vnu,Vzu,Vpd,Vnd,Vzd,
                         Iau,Ibu,Icu,Iad,Ibd,Icd);
          end;
          {calculate coefficients, when test voltages }
          if (OneOrTwo = 'one') then begin           {one open line}
            if (ii = 0) then begin                   {constant term}
              Ka[ii] := Csub(Iad,Iau);
              Kb[ii] := Csub(Ibd,Ibu);
              Ku[ii] := Cplx(-Icu.x,-Icu.y);
              Kd[ii] := Cplx(-Icd.x,-Icd.y);
            end else if (ii < ncalc) then begin      {coefficients}
              Ka[ii] := Crmult(1/Vtst,Cadd(Csub(Iau,Iad),Ka[0]));
              Kb[ii] := Crmult(1/Vtst,Cadd(Csub(Ibu,Ibd),Kb[0]));
              Ku[ii] := Crmult(1/Vtst,Cadd(Icu,Ku[0]));
              Kd[ii] := Crmult(1/Vtst,Cadd(Icd,Kd[0]));
            end;
          end else begin                             {two open lines}
            if (ii = 0) then begin                   {constant term}
              Ka[ii] := Csub(Iad,Iau);
              Kb[ii] := Cplx(-Ibu.x,-Ibu.y);
              Kp[ii] := Cplx(-Ibd.x,-Ibd.y);
              Ku[ii] := Cplx(-Icu.x,-Icu.y);
              Kd[ii] := Cplx(-Icd.x,-Icd.y);
            end else if (ii < ncalc) then begin      {coefficients}
              Ka[ii] := Crmult(1/Vtst,Cadd(Csub(Iau,Iad),Ka[0]));
              Kb[ii] := Crmult(1/Vtst,Cadd(Ibu,Kb[0]));
              Kp[ii] := Crmult(1/Vtst,Cadd(Ibd,Kp[0]));
              Ku[ii] := Crmult(1/Vtst,Cadd(Icu,Ku[0]));
              Kd[ii] := Crmult(1/Vtst,Cadd(Icd,Kd[0]));
            end;
          end;
        end;
        {the solution is now in pPs, pNs,pWs . . . . ., process it . . . . . }
        {save initial neg sec. currents etc for all components (pos.seq. later}
        ClearAllVar(pSet);                                {clear initially}
        ResultsToComp(pSet, nnod, freq, pWs,3,ForBoth);   {zero seq to comp.}
        CopyAllVar(pSet,1,3);                             {copy to 3rd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pNs,2,ForBoth);   {neg. seq to comp.}
        CopyAllVar(pSet,1,2);                             {copy to 2nd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pPs,1,ForBoth) ;  {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);        {in twoshort.pas}  {from seq to phase, all var, all comp.}
        PolarToRelaysAndBuses(pSet);
        pSet^.UpdateResultBoxes(1);                       {update the eventual result boxes}
        {show also the current through the current trafos, if allowed}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
      finally
        {destroy object for current trafos, if created}
        if (nCurrT > 0) then begin
          pCurrT^.Destr;                 {destroy object for current trafors}
          pCurrT^.Free;
          dispose(pCurrT);
          pCurrT := nil;
        end;
        {destroy object for relays, if created}
        if (pRelay <> nil) then begin
          pRelay^.Destr;                 {destroy object for relays}
          pRelay^.Free;
          dispose(pRelay);
          pRelay := nil;
        end;

      end;
    end else begin                             {there was an unknown component}
      show_message('unknown component');
    end;
    FreeMat(nnod);                             {free pAdmat, pRhs, pWs, pIndex}
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    Freemem(pZmat,nnod*nnod*SizeOf(complex));  {free matrix for zero seq.}
    Freemem(pIndexZ,nnod*Sizeof(integer));     {free index array, zero seq}
    pSet^.FreeMemForCurrTrAndRelays;
    Freemem(pPs,nnod*SizeOf(complex));
    Freemem(pNs,nnod*SizeOf(complex));
  end else begin
    {no fault, no calculation}
  end;
  OpenLineFault := (pOpenL <> nil);     {one fault = calculation done}
end;
(*
function OpenLineFault(ipSet : longint; OneOrTwo: string): boolean;
{solve the steady state network in the case of an open-line fault
 ipSet     pointer to the "common" as integer
 OneOrTwo  'one' = phase c open, 'two' = phases b and c open }
const
  Itst = 1000.0;  {test current}
var
  a11,a12,b1  : complex;     {coefficients for solving the current}
  a21,a22,b2  : complex;     {coeff..}
  aZ0,aZ1,aZ2 : single;      {abs.value (magnitude) of the sequence impedances}
  Csum        : complex;     {coeff.   for Ua = Csum*Ia}
  cpu,cpx     : complex;     {temporary}
  cpv,cpw     : complex;
  det         : complex;     {determinant}
  dodd,doddN  : single;      {keep track of permutations}
  doddZ       : single;      {keep track of permutations}
  errcom      : boolean;     {error flag}
  er2         : integer;     {error flag}
  freq        : single;      {frequency}
  fZ0,fZ1,fZ2 : single;      {phase of sequence impedances}
  Ip,Im,Iz    : complex;     {pos,neg,zero seq. currents}
  Ia,Ib,Ic    : complex;     {fault currents}
  I3,I2,I1    : single;
  ii          : integer;     {index}
  jc          : integer;     {current node}
  jd,ju       : integer;     {up-, downstream voltage nodes}
  nnod        : integer;     {number of nodes}
  nGenMot     : integer;     {number of synch.machines}
  nRelay      : integer;     {number of relays}
  nFault      : integer;     {number of faults}
  ncalc       : integer;     {number of calculations}
  nCurrT      : integer;     {number of current trafos}
  nAsyMot     : integer;     {number of asynchronous motors}
  nSource     : integer;     {number of sources (feeders)}
  nPQrel      : integer;     {number of combined relays}
  pComp       : TpComp;      {pointer to a component}
  pCurrT      : TpCurrT;     {pointer to objectg for current trafos}
  pOpenL      : TpComp;      {pointer to the open line fault}
  pRelay      : TpRelay;     {pointer to object for relays}
  pSet        : TpCompSet;   {pointer to the common}
  pPs,pNs     : pComplex;    {temporary right hand sides}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  SnglZ       : Integer;     {flag for singular matrix}
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  ToutZ       : complex;     {test output}
  Vau,Vbu,Vcu : complex;     {upstream phase voltages}
  Vad,Vbd,Vcd : complex;     {downstream phase voltages}
  Vpu,Vnu,Vzu : complex;     {upstream sequence voltages}
  Vpd,Vnd,Vzd : complex;     {downstream sequence voltages}
  Vnom        : complex;     {nominal voltage at fault, * C-factor}
  Kau,Kbu     : array[1..3] of complex;  {upstream coeff.for solving the current}
  Kad,Kbd     : array[1..3] of complex;  {downstream coeff.}
begin
  pSet := TpCompSet(ipSet);          {integer to pointer}
  freq := pSet^.appl_Hz;
  pOpenL := nil;                     {initialize pointer to the open line}
  FindOpenLine(pSet, pOpenL);        {find the open line fault(s)}
  Imagoo.NodeButClick(nil);          {determine nodes, get number of nodes (needed?) }
  {solve the network only, if there is exactly one open line fault}
  if (pOpenL <> nil) then begin        {two phase short found}
    nnod := pSet^.appl_nn;             {copy number of nodes}
    {count the number of generators, etc. Needed below}
    CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nPQrel); {in shortcir.pas}
    pSet^.GetMemForCurrTrAndRelays;    {memory for lists of curr.tr and relays (if exist)}
    {reserve and initialize memory for matrices}
    ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
    {reserve memory for the neg.seq matrix}
    getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pNmat,nnod);
    getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexN, nnod);
    getmem(pZmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
    clearCmat(pZmat,nnod);
    getmem(pIndexZ,nnod*Sizeof(integer));       {for index array, neg seq.}
    clearIvec(pIndexZ, nnod);
    getmem(pPs,nnod*SizeOf(complex));           {for temporary right hand side}
    clrCrow(pPs,1,nnod);
    getmem(pNs,nnod*SizeOf(complex));           {for temporary right hand side}
    clrCrow(pNs,1,nnod);
    clrCrow(pWs,1,nnod);
    ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, OpenLine,
{->}           {false}Imagoo.InitialStateBut.down, errcom);
    {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
    PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
    {copy the matrix to zero seq. matrix, convert to correspond to zero seq}
    PosToZeroMat(ipSet, pAdmat,pZmat, freq, 0.0, nnod);
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex ,Dodd ,Sngl ,Tout);  {decompose, pos. seq}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg. seq}
        CLUdecomp(pZmat ,pWs,nnod,pIndexZ,DoddZ,SnglZ,ToutZ); {decompose, zero seq}
        {create object for relays, if any exists (for display + checking of activation}
        CreateRelays(ipSet, nRelay, nPQrel, pRelay);
        {if there are current transformers, create object for operations with current trafos}
        CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
        {find the current trafos for the relays, if there are relays & curr.trafos}
        if (pRelay <>nil) then pRelay^.FindCurrTrafos(pCurrT);   {shortcmp.pas}
        {solve the currents at the open line . . . . . . . . . . . . . . . . .}
        {When one broken line:
         1: current in all phases = 0
         2: current in phase a <> 0, others = 0
         3: current in phase b <> 0, others = 0
         4: deduce currents in phases a and b so that voltages on both sides are
         equal in phases a and b
         When two broken lines: phases 1,2, and 4, phase 3 not needed}
        {if node = ground ??? voltages set to zero... }
        pOpenL^.getNode(1,ju,er2);              {get upstream voltage node}
        pOpenL^.getNode(2,jd,er2);              {get downstream voltage node}
        pOpenL^.getNode(3,jc,er2);              {get current "node"}
        if (OneOrTwo = 'one') then ncalc := 4 else ncalc := 3;
        for ii := 1 to ncalc do begin           {first guessed currents, then calculated}
          copyCvec(pRhs,pPs,nnod);              {copy to temporary rhs. Must not destroy rhs}
          {different currents at each round}
          if (ii = 1) then begin
            Ia := cplx(0,0);                    {first time = no current}
            Ib := cplx(0,0);
            Ic := cplx(0,0);
          end else if (ii = 2) then begin
            Ia := cplx(Itst,0);
            Ib := cplx(0,0);
            Ic := cplx(0,0);
          end else if (ii = {3}ncalc-1) then begin
            Ia := cplx(-Itst{0},0);
            Ib := cplx(Itst,0);
            Ic := cplx(0,0);
          end else begin                        {last time, solve the current}
            {solve the currents so that upstream and downstream voltages are the same}
            if (OneOrTwo = 'one') then begin    {one open line, must solve two currents}
              a11 := Csub(Kau[2],Kad[2]);
              a12 := Csub(Kau[3],Kad[3]);
              b1  := Csub(Kad[1],Kau[1]);
              a21 := Csub(Kbu[2],Kbd[2]);
              a22 := Csub(Kbu[3],Kbd[3]);
              b2  := Csub(Kbd[1],Kbu[1]);
              det := Csub(Cmult(a11,a22),Cmult(a12,a21));  {determinant}
              if (Cabs(det) > 1.e-6) then begin     {nonzero determinant}
                cpu := Csub(Cmult(a22,b1),Cmult(a12,b2));
                cpv := Csub(Cmult(a11,b2),Cmult(a21,b1));
                Ib := Cdiv(cpv,det);
                Ia := Cdiv(cpu,det);
Ia := Csub(Ia,Ib);

                Ic := cplx(0,0);
              end else begin
                Ia := cplx(0,0);                {error case = no current}
                Ib := cplx(0,0);
                Ic := cplx(0,0);
              end;
            end else begin                      {two broken lines, one current solved}
              cpu := Csub(Kau[2],Kad[2]);
              cpv := Csub(Kad[1],Kau[1]);
              if (Cabs(cpu) > 1.e-6) then Ia := Cdiv(cpv,cpu) else Ia := cplx(0,0);
              Ib := cplx(0,0);
              Ic := cplx(0,0);
            end;
          end;
          {solve net at different currents}
          if (ii = 1) then begin                  {at zero currents}
            {sequence components not needed at first time}
            CLUbacksub(pAdmat,nnod,pIndex,pPs);   {backsubstitute = solve pos.seq.net}
            if (ju > 0) then getCvec(pPs,ju,nnod,Vpu)
            else Vpu := cplx(0,0);                {get pos.seq. upstream voltage }
            if (jd > 0) then getCvec(pPs,jd,nnod,Vpd)
            else Vpd := cplx(0,0);                {get pos.seq. downstream voltage }
            {phase voltages}
            Vau := Vpu;
            Vad := Vpd;
            Vbu := Cmult(emifii,Vpu);              {phase shift -120 degrees}
            Vbd := Cmult(emifii,Vpd);
          end else begin
            PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);        {to sequence currents}
            putCvec(pPs,jc,nnod,Ip);              {current to rhs}
            CLUbacksub(pAdmat,nnod,pIndex,pPs);   {backsubstitute = solve pos.seq.net}
            if (ju > 0) then getCvec(pPs,ju,nnod,Vpu)
            else Vpu := cplx(0,0);                {get pos.seq. upstream voltage }
            if (jd > 0) then getCvec(pPs,jd,nnod,Vpd)
            else Vpd := cplx(0,0);                {get pos.seq. downstream voltage }
            {negative seq. next}
            clrCrow(pNs,1,nnod);                  {no other sources in neg, zero seq.}
            putCvec(pNs,jc,nnod,Im);              {fault = current to rhs}
            CLUbacksub(pNmat,nnod,pIndexN,pNs);   {backsubstitute = solve neg.seq.net}
            if (ju > 0) then getCvec(pNs,ju,nnod,Vnu)
            else Vnu := cplx(0,0);                {get neg.seq. upstr.voltage at fault}
            if (jd > 0) then getCvec(pNs,jd,nnod,Vnd)
            else Vnd := cplx(0,0);                {get neg.seq. downstr.voltage at fault}
            {finally zero sequence}
            clrCrow(pWs,1,nnod);
            putCvec(pWs,jc,nnod,Iz);              {fault = current to rhs}
            CLUbacksub(pZmat,nnod,pIndexZ,pWs);   {backsubstitute = solve zero seq.net}
            if (ju > 0) then getCvec(pWs,ju,nnod,Vzu)
            else Vzu := cplx(0,0);                {get zero seq. upstream voltage at fault}
            if (jd > 0) then getCvec(pWs,jd,nnod,Vzd)
            else Vzd := cplx(0,0);                {get zero seq. upstream voltage at fault}
            SeqToPhase(Vpu,Vnu,Vzu,Vau,Vbu,Vcu);  {upstream phase voltages}
            SeqToPhase(Vpd,Vnd,Vzd,Vad,Vbd,Vcd);  {downstream phase voltages}
//showmessage(inttostr(ii)+' a '+comptostr(Vau)+'        '+comptostr(Vad));
//showmessage(inttostr(ii)+' b '+comptostr(Vbu)+'        '+comptostr(Vbd));
//showmessage(inttostr(ii)+' c '+comptostr(Vcu)+'        '+comptostr(Vcd));
          end;
          {calculate coefficients, if test currents }
          if (ii = 1) then begin
            Kau[ii] := Vau;                     {constant term}
            Kbu[ii] := Vbu;
            Kad[ii] := Vad;                     {constant term}
            Kbd[ii] := Vbd;
          end else if (ii <= {3}ncalc-1) then begin      {coefficients}
            Kau[ii] := Crmult(1/Itst,Csub(Vau,Kau[1]));
            Kbu[ii] := Crmult(1/Itst,Csub(Vbu,Kbu[1]));
            Kad[ii] := Crmult(1/Itst,Csub(Vad,Kad[1]));
            Kbd[ii] := Crmult(1/Itst,Csub(Vbd,Kbd[1]));
          end;
        end;
        {the solution is now in pPs, pNs,pWs . . . . ., process it . . . . . }
        {save initial neg sec. currents etc for all components (pos.seq. later}
        ClearAllVar(pSet);                                {clear initially}
        ResultsToComp(pSet, nnod, freq, pWs,3,ForBoth);   {zero seq to comp.}
        CopyAllVar(pSet,1,3);                             {copy to 3rd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pNs,2,ForBoth);   {neg. seq to comp.}
        CopyAllVar(pSet,1,2);                             {copy to 2nd set of var., in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pPs,1,ForBoth) ;  {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);        {in twoshort.pas}  {from seq to phase, all var, all comp.}
        PolarToRelaysAndBuses(pSet);
        pSet^.UpdateResultBoxes(1);                       {update the eventual result boxes}
        {show also the current through the current trafos, if allowed}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
      finally
        {destroy object for current trafos, if created}
        if (nCurrT > 0) then begin
          pCurrT^.Destr;                 {destroy object for current trafors}
          pCurrT^.Free;
          dispose(pCurrT);
          pCurrT := nil;
        end;
        {destroy object for relays, if created}
        if (pRelay <> nil) then begin
          pRelay^.Destr;                 {destroy object for relays}
          pRelay^.Free;
          dispose(pRelay);
          pRelay := nil;
        end;

      end;
    end else begin                             {there was an unknown component}
      show_message('unknown component');
    end;
    FreeMat(nnod);                             {free pAdmat, pRhs, pWs, pIndex}
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    Freemem(pZmat,nnod*nnod*SizeOf(complex));  {free matrix for zero seq.}
    Freemem(pIndexZ,nnod*Sizeof(integer));     {free index array, zero seq}
    pSet^.FreeMemForCurrTrAndRelays;
    Freemem(pPs,nnod*SizeOf(complex));
    Freemem(pNs,nnod*SizeOf(complex));
  end else begin
    {no fault, no calculation}
  end;
  OpenLineFault := (pOpenL <> nil);     {one fault = calculation done}
end;
*)

end.
