unit Impmark;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Grids, Routines, marker, StdCtrls, Buttons, Skemadef,distance,
  Utildef,Polygon,Compdef;

type
  TImpedanceForm = class(TForm)
    ImpedGrid: TStringGrid;
    TypePanel: TPanel;
    TypeImage: TImage;
    MarkerLabel: TLabel;
    AddBut: TButton;
    DeleteBut: TButton;
    CloseBut: TButton;
    MarkerList: TListBox;
    MsgBut: TBitBtn;
    ModifyBut: TButton;
    PhaseRadBut: TRadioButton;
    ResidualRadBut: TRadioButton;
    BothRadBut: TRadioButton;
    procedure ImpedGridSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure TypeImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure CloseButClick(Sender: TObject);
    procedure AddButClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure DeleteButClick(Sender: TObject);
    procedure SaveMarkerData;
    procedure ImpedGridKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MsgButClick(Sender: TObject);
    procedure MarkerListClick(Sender: TObject);
    procedure ConstructMarkerLine(var line: string; var ertot: integer);
    procedure ModifyButClick(Sender: TObject);
    procedure InitializeGrid;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure RemoveMarkerData(txin : string; iMark,nMark: integer; var txut : string);  
procedure CopyMarkerData(tag : longint; point : boolean; pList : TpListBox);
procedure GetNumericalString(line: string; var txb: string);

var
  ImpedanceForm: TImpedanceForm;

implementation
uses
  Skemproc, Imagopas;
{$R *.DFM}

const
  ActRow : integer = 0;     {index of the active row}
  NameRow = 0;              {row for the name of the marker}
  ResiRow = 1;              {row for the resistance value of the marker}
  ReacRow = 2;              {row for the reactance value of the marker}
  MagnRow = 3;              {row for the magnitude of the marker}
  AnglRow = 4;              {row for the angle of the marker}
  TypeRow = 5;              {row for the index of the marker type}
  d_p : char = ' ';         {decimal point: '.' or ','}
  RXnotZF : boolean = true; {true, when R and X saved, not Z and angle}

procedure TImpedanceForm.ImpedGridSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  ActRow := row;
  {if row for marker types selected, show the "menu"}
  if (row = TypeRow) then begin
    {cannot edit the row for the marker type}
    ImpedGrid.options := ImpedGrid.options-[goEditing,goAlwaysShowEditor];
    {show the menu}
    TypePanel.bringtofront;
    TypePanel.visible := true;
  end else begin
    ImpedGrid.options := ImpedGrid.options+[goEditing,goAlwaysShowEditor];
    TypePanel.visible := false;
  end;
end;

procedure TImpedanceForm.InitializeGrid;
begin
  with ImpedGrid do begin
    cells[1,NameRow] := 'marker 1';
    cells[1,ResiRow] := '0';
    cells[1,ReacRow] := '0';
    cells[1,MagnRow] := '0';
    cells[1,AnglRow] := '0';
    cells[1,TypeRow] := '1';
  end;
end;

procedure TImpedanceForm.FormCreate(Sender: TObject);
{initialize the form}
begin
  {determine the char.for the decimal point}
  DecimalPoint(d_p);  {in routines.pas}
  {initialize the grid for the parameters of the markers}
  with ImpedGrid do begin
    {first, the captions}
    cells[0,NameRow] := 'name';
    cells[0,ResiRow] := 'resistance (Ohm)';
    cells[0,ReacRow] := 'reactance (Ohm)';
    cells[0,MagnRow] := 'magnitude (Ohm)';
    cells[0,AnglRow] := 'angle (degrees)';
    cells[0,TypeRow] := 'type';
  end;
  {then the values}
  InitializeGrid;
(*
    cells[1,NameRow] := 'marker 1';
    cells[1,ResiRow] := '0';
    cells[1,ReacRow] := '0';
    cells[1,MagnRow] := '0';
    cells[1,AnglRow] := '0';
    cells[1,TypeRow] := '1';
  end;
*)
  {prepare image and its background panel for selecting the type of the marker}
  MarkerMenu(@TypeImage, @TypePanel, @ImpedGrid, TypeRow);
  {add flag that form created}
  Imagoo.ReadyEdit.text := Imagoo.ReadyEdit.text+'I';
end;

procedure TImpedanceForm.TypeImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MarkerIndexToCell(x,y, @ImpedGrid, @TypePanel,TypeRow);
end;

procedure TImpedanceForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  {accept only numbers, '+', '-', '.' or ',', when entering data}
  if (ActRow > 0) then begin             {first row is for text}
    if ((key = ',') or (key = '.')) then key := d_p
    {only numbers and . or , accepted}
    else if not(((key >= '0') and (key <= '9')) or
                 (key = '+') or (key = '-') or
                 (key  = #8 ) or
                 (key =  #3 ) or (key = #22)) then   {"copy" and "paste"}
      key := #0;
  end else if (key = ';') or (key = gImpedSep) then begin   {row for the name of the marker}
    key := #0;                         {';' used as separator}
  end;
end;

procedure TImpedanceForm.CloseButClick(Sender: TObject);
begin
  ImpedanceForm.visible := false;
end;

procedure TImpedanceForm.SaveMarkerData;
{save the marker data as a description of the component}
var
  cnt   : integer; {no of lines = no of markers}
  dp    : Char;      {decimal point: (,) or (.)}
  ii    : integer;
  line  : string;
  pComp : TpComp;   {pointer to the active relay}
begin
  if (tag <> 0) then begin   {pointer to relay saved in tag}
    pComp := TpComp(tag);
    line := '';
    cnt := MarkerList.Items.count;      {no of lines = no of markers}
    if (cnt > 0) then begin
      for ii := 0 to cnt-1 do begin
        line := line+MarkerList.Items[ii];
        {add type of marker (point or line) and the default,relative coord. of the identifier}
        line := line+'point\0\-18\';
      end;
    end;
    pComp^.PutDescrG(4,'','',line);     {save as a description}
  end;
end;

procedure ReplaceMarkerData(DescrOld,DataLine: string; iMark: integer; var DescrNew: string);
{replace (modified) data in the line of marker data (saved as a description)
 DewscrOld   old line of data = a line of desrcriptions of all markers
 DataLine    the new data replacing the old
             NOTE: the relative position of the identifier is not replaced (two last items)
 iMark       the number of the marker (first = 0)
 DescrNew    new line of data}
var
  err     : integer;  {error flag}
  ii,im   : integer;  {index}
  iPnt    : integer;  {index of the text 'point'}
  istrt   : integer;  {index of start of search}
  iSep    : longint;  {index of separator}
  len     : integer;  {length of a string}
  lenDO   : integer;  {length of old description}
  ll      : longint;  {temporary}
  nSep    : integer;  {counts separators}
  pCh     : pChar;    {pointer to a char in a string}
  pPnt    : pChar;    {pointer to the string 'point'}
  pSep    : pChar;    {pointer to a separator}
  Stxt    : string;   {text to be searched}
  txa,txb : string;   {temporary}
begin
  err := 0;                                     {initialize}
  pCh := nil;                                   {initialize}
  if (iMark < 0) then begin                     {error case}
    err := 1;
  end else if (iMark = 0) then begin            {first data set}
    err := 0;
    pCh := @DescrOld[1];            {pointer to the first char. of data set iMark = 0}
  end else begin                    {not the first one}
    {must find the beginning of data set iMark}
    lenDO := length(DescrOld);
    Stxt := 'point';                            {the searched text}
    txa := gImpedSep;
    pCh := @DescrOld[1];
    pPnt := @DescrOld[1];
    {search for the beginning of data set iMark}
    for ii := 1 to iMark do begin
      {find the first separator, or first after the identifier}
      pSep := StrPos(pCh, @txa[1]);       {the separator after the identifier}
      if (pSep <> nil) then begin
        {search for next 'point'}
        pPnt := StrPos(pSep, @Stxt[1]);
        if (pPnt <> nil) then begin
          {search for the next identifier = after three separators}
          pSep := pPnt;
          for im := 1 to 3 do begin         {for three separators}
            pSep := StrPos(pSep,@txa[1]);
            if (pSep = nil) then begin
              err := 2;
              break;
            end else begin
              inc(pSep);                    {next char. after the separator}
            end;
          end;
        end else begin                      {not found 'point'}
          err := 3;
        end;
      end else begin                        {not found the first separator (after identif.)}
        err := 4;
      end;
      if (err = 0) then begin
        pCh := pSep; inc(pCh);              {pointer to the next identifier}
      end else begin
        break;
      end;
    end;
  end;
  { - - - - - - - - - - - - - - - - }
  {if err = 0, the pointer to the first character of the data set number iMark is pCh}
  {find the next occurrence of 'point' = end of parameter set to be replaced}
  if (err = 0) and (pCh <> nil) then begin
    {find the first separator, to be used in a check}
    txa := gImpedSep;
    pSep := StrPos({@DescrOld[1]}pCh, @txa[1]);
    if (pSep <> nil) then begin                      {separator found}
      {iSep := longint(pSep)-longint(@DescrOld[1]);}
      {look for the first occurrence of the text 'point'}
      txa := 'point';                                {indicates a point marker}
      pPnt := StrPos(pSep, @txa[1]);                 {identifier might be 'point..}
      if (pPnt <> nil) then begin
        iPnt := longint(pPnt)-longint(@DescrOld[1])+1;{index of the'point'}
        {copy the new data}
        {reasonable data line?}
        if (length(DataLine) > 5) and
           (DataLine[length(DataLine)] = gImpedSep){ and}{last char = separator}
           {(iPnt > iSep)} then begin                  {the identifier could be 'point..}
          if (iMark = 0) then begin                  {first data set}
            DescrNew := '';
          end else begin                             {not the first, must copy up to set Imark}
            DescrNew := '';
            ll := longint(pCh)-longint(@DescrOld[1])-1; {index of last char. to be copied}
            for ii := 1 to ll do DescrNew := DescrNew+DescrOld[ii];
          end;
          {copy the new data}
          for ii := 1 to length(DataLine) do DescrNew := DescrNew+DataLine[ii];
          {add the old descr.line from the 'point' onwards}
          for ii := iPnt to length(DescrOld) do DescrNew := DescrNew+DescrOld[ii];
        end else begin
          err := 12;                                  {not reasonable data}
        end;
      end else begin                                 {error, 'point' not found}
        err := 13;
      end;
    end else begin                                   {separator not found, something wrong}
      err := 14;
    end;
  end;
  {if error, copy old to new as such}
  if (err <> 0) then begin
    len := length(DescrOld);
    DescrNew := '';
    for ii := 1 to len do DescrNew := DescrNew+DescrOld[ii];
  end;
end;

procedure TImpedanceForm.ConstructMarkerLine(var line: string; var ertot: integer);
{collect the parameters of a marker in the grid to one string
 line    the string of the parameters
 ertot   error flag}
var
  err : integer;  {error flag}
  ii  : integer;
  txt : string;   {temporary}
begin
  {check first that the numbers are correct}
  ertot := 0;
  for ii := 1 to ImpedGrid.RowCount-1 do begin
    line := ImpedGrid.cells[1,ii];
    chkfloat(line, err,txt);                 {in routines.pas}
    ertot := ertot+err;
    if (err <> 0) then begin
      display_message(ImpedanceForm,MsgBut,'error in row '+inttostr(ii+1));
    end;
  end;
  {add only if no errors}
  if (ertot = 0) then begin
    {save the last modified: R and X, or Z and  to the line}
    if (RXnotZF) then
      txt := ' R = '+ImpedGrid.cells[1,1]+' Ohm'+gImpedSep+
             ' X = '+ImpedGrid.cells[1,2]+' Ohm'+gImpedSep
    else
      txt := ' Z = '+ImpedGrid.cells[1,3]+' Ohm'+gImpedSep+
             '  = '+ImpedGrid.cells[1,4]+' deg'+gImpedSep;
    {add index of marker symbol}
    line :=         ImpedGrid.cells[1,0]+gImpedSep+txt+
            '   ( '+ImpedGrid.cells[1,5]+' )'+gImpedSep;
    {add indicator where the marker is displayed: phase plane, or residual, or both}
    if (PhaseRadBut.checked) then line := line+'21'+gImpedSep
    else if (ResidualRadBut.checked) then line := line+'21N'+gImpedSep
    else line := line+'both'+gImpedSep;
  end;
end;

procedure TImpedanceForm.AddButClick(Sender: TObject);
{add a new marker to the list of markers}
var
  cnt     : integer;  {no of lines = no of markers}
  dp      : Char;     {decimal point: (,) or (.)}
  err     : integer;  {error flag}
  ertot   : integer;  {total error}
  ii,jj   : integer;
  line    : string;
  pComp   : TpComp;   {pointer to the active relay}
  txa,txb : string;   {temporary}
  txc,txt : string;   {temporary}
begin
  {construct the parameter line}
  ConstructMarkerLine(line, ertot);
  if (ertot = 0) then begin
    MarkerList.items.add(line);
    {SaveMarkerData;}
    if (tag <> 0) then begin   {pointer to relay saved in tag}
      pComp := TpComp(tag);
      {save the data of the new marker = add to data line}
      pComp^.GetDescrG(txa,txb,txc);            {data in third string}
      {add data for the new marker, default position for the identifier}
      txc := txc+line+'point'+gImpedSep+'0'+gImpedSep+'-18'+gImpedSep;
      pComp^.PutDescrG(4,'','',txc);            {save, as #description"}
      if (isMho(pComp)) then DistanceRelayShow  {Mho-distance relay}
      else PolygonRelayShow;                    {polygon-distance relay}
    end;
  end;

end;

procedure CopyMarkerData(tag : longint; point : boolean; pList : TpListBox);
{copy marker data from component to list of markers
 tag    pointer to the component as integer
 point  flag for marker type: point or line
 pList  pointer to the list of markers}
var
  ii      : integer;   {index}
  len     : integer;   {length of line}
  line    : string;    {text with data for all markers}
  nCopy   : integer;   {no of items copied for display}
  nType   : integer;   {number of the item telling the type of the marker: line or point}
  pComp   : TpComp;    {pointer to the relay}
  semicol : integer;   {counter of separators}
  txa,txb : string;    {temporary}
begin
  pList^.clear;                       {clear initially}
  nType := 5;                         {initialize, change later}
  if (point) then begin
    nCopy := 5;
    nType := 8;
  end else begin
    nCopy := 7;
    nType := 10;
  end;
  {pointer to relay saved in tag (in MarkerMenuClick, in distance and polygon)}
  if (tag <> 0) then begin
    pComp := TpComp(tag);
    if (point) then
      pComp^.GetDescrG(txa,txb,line)  {point-marker data saved in description}
    else
      pComp^.GetDescrQ(line,txa,txb); {line-marker}
    len := length(line);
    if (len > 0) then begin           {there is some data}
      txa := '';                      {copy here several items (parameters)}
      semicol := 0;
      for ii := 1 to len do begin
        if (semicol < nCopy) then txa := txa+line[ii]; {copy char. by char, up to..}
        {note: txa updated below}
        txb := txb+line[ii];                    {one item here}
        if (line[ii] = gImpedSep) then begin    {.. used as a separator}
          inc(semicol);                         {count separators}
          if (semicol >= nType) then begin      {nType items in one set of data}
            pList^.items.add(txa);              {add dataline to list}
            txa := '';
            semicol := 0;                       {reset counter}
          end;
        end;
      end;
    end;
  end;
end;

procedure TImpedanceForm.FormShow(Sender: TObject);
begin
  {copy marker data from component}
  CopyMarkerData(tag,true, @MarkerList);
  InitializeGrid;                        {initial parameter values}
  ModifyBut.enabled := false;            {must be false, before a line is selected}
end;

procedure RemoveMarkerData(txin : string; iMark,nMark: integer; var txut : string);
{copy txin totxut without the data of marker iMark
 txin   line with data with all markers
 iMark  index of marker to be removed  in original list 0..nMark
 nMark  number of markers WITHOUT the marker to be removed
 txut   line without data of marker iMark}
var
  ii,kk : integer;  {index, counter}
  len   : integer;  {length of sting}
begin
  if (nMark > 0) and (iMark <= nMark) and (iMark >= 0) then begin
    len := length(txin);
    txut := '';                               {initialize}
    if (len > 0) and (nMark > 0) then begin
      kk := 0;                                {counts separators}
      for ii := 1 to len do begin
        if (txin[ii] = gImpedSep) then begin
          inc(kk);                            {separator}
          if (kk < {7}8*iMark) or ((kk >= {7}8*(iMark+1)) and (kk <> {7}8) ) then begin
            txut := txut+txin[ii];
          end;
        end else begin
          if ((kk < {7}8*iMark) or (kk >= {7}8*(iMark+1))) then begin
            txut := txut+txin[ii];
          end;
        end;
      end;
    end else begin                            {no markers, or no data(?)}
      {do nothing}
    end;
  end else begin  {error case}
    txut := '';
  end;
end;

procedure TImpedanceForm.DeleteButClick(Sender: TObject);
{delete the selected marker}
var
  iMark       : integer;
  nMark       : integer;  {no of markers}
  pComp       : TpComp;   {pointer to the (last clicked) relay of the graph}
  txa,txb,txc : string;   {temporary}
begin
  iMark := MarkerList.ItemIndex;              {index of selected marker}
  if (iMark >= 0) then begin                  {some item selected}
    MarkerList.Items.delete(iMark);           {delete it from the marker list}
    {save data and redraw picture without the deleted marker}
    if (tag <> 0) then begin                    {pointer to relay saved in tag}
      pComp := TpComp(tag);
      {must also delete from the string of data}
      nMark := MarkerList.Items.count;          {no of markers left}
      pComp^.GetDescrG(txa,txb,txc);            {data in third string}
      {update the line with the marker data}
      RemoveMarkerData(txc,iMark,nMark,txa);
(*
      len := length(txc);
      txa := '';                                {initialize}
      if (len > 0) and (nMark > 0) then begin
        kk := 0;                                {counts separators}
        for ii := 1 to len do begin
          if (txc[ii] = gImpedSep) then begin
            inc(kk);                            {separator}
            if (kk < 7*iMark) or ((kk >= 7*(iMark+1)) and (kk <> 7) ) then begin
              txa := txa+txc[ii];
            end;
          end else begin
            if ((kk < 7*iMark) or (kk >= 7*(iMark+1))) then begin
              txa := txa+txc[ii];
            end;
          end;
        end;
      end else begin                            {no markers, or no data(?)}
        {do nothing}
      end;
*)
      pComp^.PutDescrG(4,'','',txa);            {put data back}
      if (isMho(pComp)) then DistanceRelayShow  {Mho-distance relay}
      else PolygonRelayShow;                    {polygon-distance relay}
    end;
  end;
end;

function ManyChars(rr : single) : string;
{write a positive or negative real using several places, fill with leading spaces (?)
 rr    the real
 text  the outputted text}
const
  small = 1e-8;
var
  aa       : single;     {abs value of rr}
  first    : boolean;
  ii,jj,kk : integer;
  len      : integer;    {length of line}
  line     : string;     {text under construction}
  ss       : string;     {space or minus sign}
  started  : boolean;    {nonzero number found}
  text     : string;
begin
  aa := abs(rr);
  if (aa < small) then begin
    text := ' 0.0      ';
  end else if (aa < {0.0001}0.01) then
    text := format({'%9.7g'}'%5.3g',[aa])
  else if (aa < 1.0) then begin
    line := format({'%10.8f'}'%5.3f',[aa]);
    {remove trailing zeroes}
    len := length(line);
    if (len > 1) then begin
      text := '';
      for ii := 1 to len do begin
        text := text+' ';          {lengt same as line, but only spaces}
      end;
      started := false;            {not yet found a nonzero number}
      kk := len;                   {last char. in text}
      for ii := 1 to len do begin
        jj := len-ii+1;
        if (line[jj] = '0') and (not started) then begin
          {zeroes not copied}
        end else begin
          text[kk] := line[jj];
          dec(kk);
          started := true;         {a nonzero number found}
        end;
      end;
    end else begin
      text := line;
    end;
  end else if (aa < 1000{1.e5}) then begin
    if (aa = round(aa)) then text := format('%7.6g',[aa])+'.0'  {integer}
    else text := format({'%9.8g'}'%6.5g',[aa]);
  end else if (aa < 1.e6) then
    text := format({'%9.8g'}'%7.6g',[aa])
  else
    text := '1.0e6    ';
  len := length(text);
  line := '';
  if (len > 0) then begin
    {add the minus sign for a negative number}
    if (aa >= small) then begin    {no minus sign to zero}
      if (rr < 0) then ss := '-'
      else             ss := ' ';
      first := true;
      for ii := 1 to len do begin
        if (text[ii] <> ' ') and first then begin
          line := line+ss;
          first := false;
        end;
        line := line+text[ii];
      end;
    end else begin
      line := text;      {zero...}
    end;
  end;
  ManyChars := line;
end;

function spaceless(text : string): string;
{remove spaces from text, check decimal point}
var
  ii    : integer;
  len   : integer;
  txout : string;
begin
  len := length(text);
  txout := '';
  if (len > 0)  then begin
    for ii := 1 to len do begin
      if (text[ii] = '.') or (text[ii] = ',') then txout := txout+d_p  {check dec.point}
      else if (text[ii] <> ' ') then txout := txout+text[ii];  {copy other nonspace char.}
    end;
  end;
  spaceless := txout;
end;

procedure RecToPolar(textR, textX : string; var textZ,textF : string);
{change complex number from rectangular to polar form
 textZ  abs.value of the number, as text
 textF  angle in degrees, as text
 textR  real part, as text
 textX  imag.part, as text}
var
  apu,apv,apw : single;  {temporary}
  err1,err2   : integer; {error flags}
  ipu         : integer; {temporary}
  deg,fii     : single;  {angle in degrees, in rad}
  Rr          : single;  {real part}
  st1,st2     : string;  {temporary}
  st3,st4     : string;  {temporary}
  text        : string;  {temporary}
  Xx          : single;  {imag.part}
  Zz          : single;  {abs.value of the complex}
begin
  ValuesFromLine(textR, 1, false,ipu, Rr , apu, apv, apw,st1,st2,st3,st4, err1);
  ValuesFromLine(textX, 1, false,ipu, Xx , apu, apv, apw,st1,st2,st3,st4, err2);
  if (err1 = 0) and (err2 = 0) then begin
    Zz := sqrt(Rr*Rr+Xx*Xx);
    fii := Cangle(cplx(Rr,Xx));   {angle in radians}
    deg := fii*180/pi;
    text := ManyChars(Zz);        {numerical value to string}
    TextZ := spaceless(text);     {remove spaces}
    text := ManyChars(deg);
    TextF := spaceless(text);     {remove spaces}
  end else begin
    TextZ := '?';
    TextF := '?';
  end;
end;

procedure PolarToRec(textZ,textF : string; var textR, textX : string);
{change complex number from polar form to rectangular
 textZ  abs.value of the number to be changed, as text
 textF  angle in degrees, as text
 textR  real part, as text
 textX  imag.part, as text}
var
  apu,apv,apw : single;  {temporary}
  err1,err2   : integer; {error flags}
  deg,fii     : single;  {angle in degrees, in rad}
  ii,jj       : integer; {indices}
  ipu         : integer; {temporary}
  len         : integer; {length of text}
  Rr          : single;  {real part}
  st1,st2     : string;  {temporary}
  st3,st4     : string;  {temporary}
  text        : string;  {temporary}
  Xx          : single;  {imag.part}
  Zz          : single;  {abs.value of the complex}
begin
  ValuesFromLine(textZ, 1, false,ipu, Zz , apu, apv, apw,st1,st2,st3,st4, err1);
  ValuesFromLine(textF, 1, false,ipu, deg, apu, apv, apw,st1,st2,st3,st4, err2);
  if (err1 = 0) and (err2 = 0) then begin
      fii := deg*pi/180;       {change to radians}
      Rr := Zz*cos(fii);
      Xx := Zz*sin(fii);
      text := ManyChars(Rr);      {numerical value to string}
      TextR := spaceless(text); {remove spaces}
      text := ManyChars(Xx);
      TextX := spaceless(text); {remove spaces}
  end else begin
    TextR := '?';
    TextX := '?';
  end;
end;

procedure TImpedanceForm.ImpedGridKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{if resistance or reactance row, calculate and show magnitude and angle, and vice versa}
var
  Ftxt : string;   {angle of impedance as text}
  Rtxt : string;   {resistance as text}
  Xtxt : string;   {reactance as text}
  Ztxt : string;   {impedance as text}
begin
  if (ActRow = ResiRow) or (ActRow = ReacRow) then begin
    {entering resistance or reactance}
    Rtxt := ImpedGrid.cells[1,ResiRow];   {copy resistance to local}
    Xtxt := ImpedGrid.cells[1,ReacRow];   {copy reactance}
    RecToPolar(Rtxt, Xtxt,Ztxt,Ftxt);
    ImpedGrid.cells[1,MagnRow] := Ztxt;   {impedance = magnitude to grid}
    ImpedGrid.cells[1,AnglRow] := Ftxt;
    RXnotZF := true;                      {resistance and reactance will be saved}
  end else if (ActRow = MagnRow) or (ActRow = AnglRow) then begin
    {entering impedance or angle}
    Ztxt := ImpedGrid.cells[1,MagnRow];   {copy impedance to local}
    Ftxt := ImpedGrid.cells[1,AnglRow];   {copy angle}
    PolarToRec(Ztxt, Ftxt,Rtxt,Xtxt);
    ImpedGrid.cells[1,ResiRow] := Rtxt;   {Resistance to grid}
    ImpedGrid.cells[1,ReacRow] := Xtxt;
    RXnotZF := false;                      {impedance and angle will be saved}
  end;
end;

procedure TImpedanceForm.MsgButClick(Sender: TObject);
begin
  MsgBut.visible := false;
end;

procedure GetNumericalString(line: string; var txb: string);
{extract the numerical part of line, put it to txt}
var
  ii  : integer;  {index}
  len : integer;  {length of string}
begin
  {must extract numerical value}
  len := length(line);
  if (len > 0) then begin
    txb := '';
    for ii := 1 to len do begin
      {+ or - accepted as the first character}
      if ((txb = '') and ((line[ii] = '+') or (line[ii] ='-'))) or
         ((line[ii] >= '0') and (line[ii] <= '9')) or    { numbers...}
          (line[ii] = '.')  or  (line[ii] = ',') then    {.. or decimal point}
        txb := txb+line[ii];
    end;
  end else begin   {error case, just some value}
    txb := '0';
  end;
end;

procedure TImpedanceForm.MarkerListClick(Sender: TObject);
{copy the parameters of the markers to the grid}
var
  line    : string;  {the line with the parameters}
  txa,txb : string;  {temporary}
  txc     : string;  {temporary}
  txz,txf : string;  {temporary}
begin
  ModifyBut.enabled := true;               {true, when a line selected}
  line := MarkerList.items[MarkerList.ItemIndex];
  {copy parameters one by one}
  ExtractParam(1,gImpedSep,line,txa);      {identifier}
  ImpedGrid.Cells[1,NameRow] := txa;       {identifier taken as such}
  ExtractParam(2,gImpedSep,line,txa);      {resistance or impedance}
  {must extract numerical value}
  GetNumericalString(txa, txb);
  ExtractParam(3,gImpedSep,line,txa);      {reactance or angle}
  GetNumericalString(txa, txc);
  {must check: R and X, or Z and }
  if (length(txa) > 1) and (txa[2] = 'X') then begin  {R and X as such}
    ImpedGrid.cells[1,ResiRow] := txb;     {resistance}
    ImpedGrid.cells[1,ReacRow] := txc;     {reactance}
    {must calculate impedance and phase}
    RecToPolar(txb, txc,txz,txf);
    ImpedGrid.cells[1,MagnRow] := txz;     {impedance = magnitude to grid}
    ImpedGrid.cells[1,AnglRow] := txf;
    RXnotZF := true;                       {resistance and reactance will be saved}
  end else begin                           {Z and  = impedance and phase}
    ImpedGrid.cells[1,MagnRow] := txb;     {impedance}
    ImpedGrid.cells[1,AnglRow] := txc;     {phase}
    {must calculate resistance and reactance}
    PolarToRec(txb, txc,txz,txf);
    ImpedGrid.cells[1,ResiRow] := txz;     {Resistance to grid}
    ImpedGrid.cells[1,ReacRow] := txf;
    RXnotZF := false;                      {impedance and angle will be saved}
  end;
  ExtractParam(4,gImpedSep,line,txa);      {type of marker}
  GetNumericalString(txa, txb);            {index of marker}
  ImpedGrid.cells[1,TypeRow] := txb;
  ExtractParam(5,gImpedSep,line,txa);      {plane on which displayed}
  if (txa = '21') then PhaseRadBut.checked := true
  else if (txa = '21N') then ResidualRadBut.checked := true
  else BothRadBut.checked := true;
end;

procedure TImpedanceForm.ModifyButClick(Sender: TObject);
{modify the selected marker = replace parameters in the list and in the description
 with the parameters in the grid}
var
  ertot    : integer; {error flag}
  DataLine : string;  {line with the modified parameters of the marker (not ID-position)}
  DescrOld : string;  {old string (description) with the parameters of all markers}
  DescrNew : string;  {new string (description) with the parameters of all markers}
  iMark    : integer; {index of the marker, 0 = first}
  pComp    : TpComp;  {pointer to the relay}
  txa,txb  : string;  {temporary}
begin
  ertot := 0;                                 {initialize}
  ConstructMarkerLine(DataLine, ertot);       {collect the parameters to one line}
  iMark := MarkerList.ItemIndex;              {index of selected marker}
  if (ImpedanceForm.tag <> 0) and (ertot = 0) and    {pointer to relay saved in tag}
    (iMark >= 0) and (iMark < MarkerList.items.count) then begin
    {replace the data in the list visible on the lower part of the marker form}
    MarkerList.Items.Delete(iMark);           {delete the old line}
    MarkerList.Items.Insert(iMark,DataLine);  {add the new line}
    pComp := TpComp(ImpedanceForm.tag);
    {save the data of the new marker = add to data line}
    pComp^.GetDescrG(txa,txb,DescrOld);                     {data in third string}
    ReplaceMarkerData(DescrOld, DataLine, iMark, DescrNew); {in "description" = data for all markers}
    {save the data as a description}
    pComp^.PutDescrG(4,'','',DescrNew);       {save, as a "description"}
    {update the graph}
    if (isMho(pComp)) then DistanceRelayShow  {Mho-distance relay}
    else PolygonRelayShow;                    {polygon-distance relay}
  end;
  ModifyBut.enabled := false;                 {forces to select a row before modifying}
end;

end.
