unit Lines;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, ExtCtrls, Routines, RelCurr, Skemadef, Skemproc, marker,
  Buttons,utildef, Impmark,distance;

type
  TLineForm = class(TForm)
    LineGrid: TStringGrid;
    LineLabel: TLabel;
    AddBut: TButton;
    DelBut: TButton;
    CloseBut: TButton;
    LineList: TListBox;
    ColorPanel: TPanel;
    ColorImage: TImage;
    MsgBut: TBitBtn;
    CurrBut: TRadioButton;
    TimeBut: TRadioButton;
    ModifyBut: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CloseButClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure AddButClick(Sender: TObject);
    procedure MsgButClick(Sender: TObject);
    procedure DelButClick(Sender: TObject);
    procedure LineGridSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure ColorImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LineGridDrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure CurrButClick(Sender: TObject);
    procedure TimeButClick(Sender: TObject);
    procedure InitializeGrid;
    procedure LineListClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ConstructLine(var line: string; var err: integer);
    procedure ModifyButClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure DeleteAllLines;

var
  LineForm: TLineForm;

implementation

uses PQRelay, Imagopas;

{$R *.DFM}

const
  NameRow  = 0;
  ValueRow = 1;
  ColorRow = 2;
  StartRow = 3;
  EndRow   = 4;
  ThickRow = 5;
var
  d_p      : char;     {decimal point: '.' or ','}
  CRwidth  : integer;  {width (and height) of a small color rectangle}
  CRspace  : integer;  {width of space between color rectangles}
  LLclick  : boolean;  {true, when line list clicked, needed for radio buttons}

procedure TLineForm.FormCreate(Sender: TObject);
var
  ii  : integer;
  ipu : integer; {temporary}
begin
  {determine the char.for the decimal point}
  DecimalPoint(d_p);  {in routines.pas}
  {captions}
  LineGrid.Cells[0,NameRow] := 'Name';
  LineGrid.Cells[0,ValueRow] := 'Current (A)';
  LineGrid.Cells[0,ColorRow] := 'Color';
  LineGrid.Cells[0,StartRow] := 'Start (s)';
  LineGrid.Cells[0,EndRow] := 'End (s)';
  LineGrid.Cells[0,ThickRow] := 'thickness';
  {initialize the parameters for the lines}
  LineGrid.Cells[1,NameRow] := 'aa';
  LineGrid.Cells[1,ValueRow] := '123';
  LineGrid.Cells[1,ColorRow] := '1';
  LineGrid.Cells[1,StartRow] := '0.01';
  LineGrid.Cells[1,EndRow] := '10.0';
  LineGrid.Cells[1,ThickRow] := '2';
  {initialize the color selection image}
  {PQForm.}ColorInitialize(@ColorImage);
  {PQForm.}GetCRsize(CRwidth,CRspace);
  ColorPanel.width := ColorImage.width+2*CRspace;
  ColorPanel.height := ColorPanel.width;
  ipu := 0;
  for ii := 0 to ColorRow do  ipu := ipu+LineGrid.RowHeights[ii];
  ColorPanel.top := LineGrid.top+ipu+2*LineGrid.GridLineWidth;
  ColorPanel.left := LineGrid.left+LineGrid.ColWidths[0]+2*LineGrid.GridLineWidth;
  LLclick := false;   {line list not clicked, initially}
  {add flag that form created}
  Imagoo.ReadyEdit.text := Imagoo.ReadyEdit.text+'L';
end;

procedure TLineForm.CloseButClick(Sender: TObject);
begin
  LineForm.visible := false;
end;

procedure TLineForm.InitializeGrid;
var
  apu : single;
  ll  : longint;
begin
  apu := RelayCharact.EndCurr;
  ll := round(apu/2);
  LineGrid.Cells[0,ValueRow] := 'Current (A)';
  LineGrid.Cells[1,ValueRow] := inttostr(ll);
  LineGrid.Cells[0,StartRow] := 'Start (s)';
  LineGrid.Cells[1,StartRow] := '0.01';
  LineGrid.Cells[0,EndRow] := 'End (s)';
  LineGrid.Cells[1,EndRow] := '10.0';
end;

procedure TLineForm.FormActivate(Sender: TObject);
begin
(*{first guess for the position of the line }
  if (LineList.Items.count = 0) then begin
    InitializeGrid;
    CurrBut.checked := true;
    TimeBut.checked := false;
  end;
*)
end;

procedure TLineForm.ConstructLine(var line: string; var err: integer);
{collect the data of the marker from the grid to one line
 line   the line containing the data
 err    error flag}
var
  dp      : Char;      {decimal point: (,) or (.)}
  ii,jj   : integer;
  ipu     : integer;
  len     : integer;   {length of text}
  pComp   : TpComp;    {pointer to the active relay}
  txc,txt : string;    {temporary}
  tx      : string;
begin
  {check first that the numbers are correct and that either current or time is given}
  txc := LineGrid.cells[1,ValueRow];
  {check decimal point }
  len := length(txc);
  tx := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (txc[ii] = '.') or (txc[ii] = ',') then tx := tx+d_p
      else tx := tx+txc[ii];
    end;
    txc := tx;
  end;
  {check value and type, current or time}
  chkfloat(txc, err, tx);                                  {correct value}
  if (err = 0) then begin
    if (CurrBut.checked) then                          {current}
      line := ' I = '+txc+'A;'
    else                                               {time}
      line := ' t = '+txc+'s;';
  end else begin
    Display_Message(LineForm,MsgBut,'error in data in row '+inttostr(ValueRow));
  end;
  {add only if no errors}
  if (err = 0) then begin
    {name (label, identifier) of the marker}
    line :=         LineGrid.cells[1,NameRow]+';'+line;
    {color of the marker. Code the line thickness as number of spaces......}
    txt := LineGrid.cells[1,ThickRow];
    if      (txt = '1') then txc := ' '
    else if (txt = '2') then txc := '  '
    else if (txt = '3') then txc := '   '
    else if (txt = '4') then txc := '    '
    else if (txt = '5') then txc := '     '
    else  txc := '  ';
    {add the cooded line thickness and the color}
    line := line+' '+txc+'( '+LineGrid.cells[1,ColorRow]+' )';
    {add the start and end points of the line}
    for jj := 1 to 2 do begin   {1 = start, 2 = end}
      if (jj = 1) then ipu := StartRow else ipu := EndRow;
      txc := LineGrid.cells[1,ipu];
      {check decimal point }
      len := length(txc);
      tx := '';
      if (len > 0) then begin
        for ii := 1 to len do begin
          if (txc[ii] = '.') or (txc[ii] = ',') then tx := tx+d_p
          else tx := tx+txc[ii];
        end;
        txc := tx;
      end;
      line := line+';'+txc;
    end;
  end;
end;

procedure TLineForm.AddButClick(Sender: TObject);
{add a new line to the list of lines}
var
  err     : integer;   {error flag}
  line    : string;    {line to be added}
  pComp   : TpComp;    {pointer to the active relay}
begin
  ConstructLine(line, err);  {collect the marker data in one line}
  if (err = 0) then begin                            {add only if no errors}
    LineList.items.add(line);
    pComp := TpComp(RelayCharact.tag);               {recall pointer to the relay}
    if (pComp <> nil) then Relay_show(pComp,'real'); {redraw the curves, in skemproc.pas}
  end;

end;

procedure TLineForm.MsgButClick(Sender: TObject);
begin
  MsgBut.visible := false;
end;

procedure TLineForm.DelButClick(Sender: TObject);
var
  iLine : integer;
  pComp : TpComp;   {pointer to the (last clicked) relay of the graph}
begin
  iLine := LineList.ItemIndex;                {index of selected line}
  if (iLine >= 0) then begin                  {some item selected}
    LineList.Items.delete(iLine);             {delete it}
    RelayCharact.DeleteLineIDCoord(iLine);    {update list of ID-coordinates} 
    pComp := TpComp(RelayCharact.tag);        {recall pointer to the relay}
    if (pComp <> nil) then Relay_show(pComp,'real'); {redraw the curves}
  end;
end;

procedure TLineForm.LineGridSelectCell(Sender: TObject; Col, Row: Longint;
  var CanSelect: Boolean);
begin
  if (row = ColorRow) and (col = 1) then begin  {row for selecting the color}
    {must not edit the color row}
    LineGrid.options := LineGrid.options-[goEditing,goAlwaysShowEditor];
    ColorPanel.visible := true;             {show the color palette}
    ColorPanel.bringtofront;
  end else begin
    ColorPanel.visible := false;
    {can edit}
    LineGrid.options := LineGrid.options+[goEditing,goAlwaysShowEditor];
  end;
end;

procedure TLineForm.ColorImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{get color index from X and Y}
var
  ii,jj,kk : integer;
begin
  ii := (X div (CRwidth+CRspace))+1;
  jj := (Y div (CRwidth+CRspace))+1;
  kk := (ii-1)*4+jj;                 {color index}
  LineGrid.cells[1,ColorRow] := inttostr(kk);
  ColorPanel.visible := false;
  Repaint;  { ?? }

end;

procedure TLineForm.LineGridDrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var
  iclr : integer;   {color index}
  ipu  : integer;   {temporary}
  txt  : string;    {temporary}
begin
  {row for the selection of the color}
  if (row = ColorRow) and (col = 1) then begin
    txt := LineGrid.cells[col,row];
    if (txt <> '') then begin
      iclr := strtoint(txt);
      if (iclr < 1) then iclr := 1;
      if (iclr > 16) then iclr := 16;
      LineGrid.canvas.pen.color := gColor[iclr];
      LineGrid.canvas.brush.color := gColor[iclr];
      {a rectangle indicates the color}
      LineGrid.canvas.rectangle(rect.left+LineGrid.GridLineWidth,
                                rect.top +LineGrid.GridLineWidth,
                                rect.left+LineGrid.RowHeights[row]-LineGrid.GridLineWidth,
                                rect.top +LineGrid.RowHeights[row]-LineGrid.GridLineWidth);
    end;
  end;
end;

procedure DeleteAllLines;
{delete the selected marker}
begin
  LineForm.LineList.Items.clear;
  LineForm.visible := false;
end;

procedure TLineForm.CurrButClick(Sender: TObject);
begin
  if (not LLclick) then InitializeGrid;  {not, if copying data from list}
  LLclick := false;                      {reset}
end;

procedure TLineForm.TimeButClick(Sender: TObject);
begin
  if (not LLclick) then begin            {not copying data from list}
    LineGrid.Cells[0,ValueRow] := 'Time (s)';
    LineGrid.Cells[1,ValueRow] := '1';
    LineGrid.Cells[0,StartRow] := 'Start (A)';
    LineGrid.Cells[1,StartRow] := '1';
    LineGrid.Cells[0,EndRow] := 'End (A)';
    LineGrid.Cells[1,EndRow] := '1000';
  end;
  LLclick := false;                      {reset}  
end;

procedure TLineForm.LineListClick(Sender: TObject);
{copy the data from the selected line to the grid}
var
  ch      : char;    {one character}
  ii      : integer; {index}
  ipu     : integer; {temporary}
  len     : integer; {length of string}
  line    : string;  {line with the data of the marker}
  txa,txb : string;  {temporary}
begin
  LLclick := true;                      {set flag, for radio buttons}
  line := LineList.items[LineList.ItemIndex];
  {copy parameters one by one}
  ExtractParam(1,';',line,txa);         {identifier}
  LineGrid.Cells[1,NameRow] := txa;     {identifier taken as such}
  {current or time}
  ExtractParam(2,';',line,txa);         {current}
  GetNumericalString(txa, txb);         {extract numerical value as text}
  LineGrid.Cells[1,ValueRow] := txb;    {current or time to grid}
  {determine: current or time. Find first character (not a space}
  len := length(txa);
  if (len > 1) then begin
    for ii := 1 to len do begin
      ch := txa[ii];
      if (ch <> ' ') then begin
        if (ch = 't') then TimeBut.checked := true        {time}
        else if (ch = 'I') then CurrBut.checked := true {current}
        else {error, do nothing?};
        break;                          {first non-space character checked}
      end;
    end;
  end;
  {change the texts for current or time}
  if (TimeBut.checked) then LineGrid.cells[0,ValueRow] := 'time (s)'
  else LineGrid.cells[0,ValueRow] := 'Current (A)';
  {color}
  ExtractParam(3,';',line,txa);         {color index}
  GetNumericalString(txa, txb);         {extract numerical value as text}
  LineGrid.Cells[1,ColorRow] := txb;    {color index to grid}
  {line thickness, coded as number of spaces}
  len := length(txa);
  ipu := NumberOfSpaces(txa);
  if (ipu > 1) and (ipu <= 6) then LineGrid.Cells[1,ThickRow] := inttostr(ipu-1)
  else LineGrid.Cells[1,ThickRow] := '2';
  {start and stop of the line}
  ExtractParam(4,';',line,txa);         {start current or time}
  GetNumericalString(txa, txb);         {extract numerical value as text}
  if (txb = '0') then txb := '';        {0 = not defined}
  LineGrid.Cells[1,StartRow] := txb;    {start value to the grid}
  ExtractParam(5,';',line,txa);         {end current or time}
  GetNumericalString(txa, txb);         {extract numerical value as text}
  if (txb = '0') then txb := '';        {0 = not defined}
  LineGrid.Cells[1,EndRow] := txb;      {start value to the grid}
  ModifyBut.enabled := true;
end;

procedure TLineForm.FormShow(Sender: TObject);
{first guess for the position of the line }
begin
  if (LineList.Items.count = 0) then begin
    InitializeGrid;
    CurrBut.checked := true;
    TimeBut.checked := false;
  end;
  ModifyBut.enabled := false;
end;

procedure TLineForm.ModifyButClick(Sender: TObject);
{modify the selected marker = replace parameters in the list and in the description
 with the parameters in the grid}
var
  ertot    : integer;  {error flag}
  DataLine : string;   {data for the marker in one line}
  iMark    : integer;  {index of the marker}
  pComp    : TpComp;   {pointer to the relay}
begin
  ertot := 0;                                 {initialize}
  ConstructLine(DataLine, ertot);             {collect the parameters to one line}
  iMark := LineList.ItemIndex;                {index of selected marker}
  if (ertot  = 0) and                         {no errors}
     (iMark >= 0) and (iMark < LineList.items.count) then begin  {reasonable index}
    {replace the data in the list visible on the lower part of the marker form}
    LineList.Items.Delete(iMark);             {delete the old line}
    LineList.Items.Insert(iMark,DataLine);    {add the new line}
    pComp := TpComp(RelayCharact.tag);        {recall pointer to the relay}
    if (pComp <> nil) then Relay_show(pComp,'real'); {redraw the curves}
  end;
  ModifyBut.enabled := false;
end;



end.
