unit AllShort;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,Imagopas,Skemadef,elplex, ExtCtrls, routines,compdef, Cmatmul,
  matmul,utildef,Twoshort, imagodef, earthflt,skemproc, Buttons;

type
  TAllShortForm = class(TForm)
    CalcBut: TButton;
    ShortCircTimer: TTimer;
    ImpedanceBox: TGroupBox;
    ResEdit: TEdit;
    ZaLabel: TLabel;
    jEdit: TLabel;
    ReactEdit: TEdit;
    LocationBox: TGroupBox;
    BusRadioBut: TRadioButton;
    VoltRadioBut: TRadioButton;
    BothRadioBut: TRadioButton;
    FaultTypeBox: TGroupBox;
    ThreePhaseBut: TRadioButton;
    LineLineBut: TRadioButton;
    LineEarthBut: TRadioButton;
    LLEBut: TRadioButton;
    LLLImg: TImage;
    LLImg: TImage;
    LEImg: TImage;
    LLEImg: TImage;
    AllBut: TRadioButton;
    MethodBox: TGroupBox;
    IECBut: TRadioButton;
    SuppBut: TRadioButton;
    InitBut: TSpeedButton;
    Z012But: TRadioButton;
    procedure FormHide(Sender: TObject);
    procedure CalcButClick(Sender: TObject);
    procedure ShortCircTimerTimer(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    {"local"}
    procedure ReportButtons(enable: boolean);
    procedure IECButClick(Sender: TObject);
    procedure SuppButClick(Sender: TObject);
    procedure ResEditEnter(Sender: TObject);
    procedure ReactEditEnter(Sender: TObject);
    procedure Z012ButClick(Sender: TObject);
    procedure FaultsEnabled(enable: boolean);
  private
    { Private declarations }
    pRefCT     : TpComp;      {pointer to "reference" current transformer, set in RelCalc.pas}
    pMeasCt    : TpComp;      {pointer to "measurement" current transformer, set in RelCalc.pas}
    IfmeasA_a  : complex;     {phase A current in measurement.CT, when fault}
    IfmeasB_a  : complex;
    IfmeasC_a  : complex;
    IfrefA_a   : complex;     {phase A current in ref.CT, when fault}
    IfrefB_a   : complex;
    IfrefC_a   : complex;
  public
    { Public declarations }
    procedure AsymmFaults(ipSet : longint; Fault: string; probl: solve);
    procedure PutRefAndMeasCT(pRCT,pMCT : TpComp);
    procedure GetCTcurr(var ma,mb,mc,ra,rb,rc: complex);
  end;

//procedure UnsymmLoadOld(ipSet: longint);  {one unsymm load only}
procedure UnsymmLoad(ipSet: longint);
procedure UnsymmLoad2(ipSet: longint);

var
  AllShortForm: TAllShortForm;

implementation

uses Relcurr, voltphas, phasor, Tab, RelCalc, shortcmp, shortcir;

{$R *.DFM}
var
  dec_p  : char;  {decimal point character '.' or ','}
  pZmat   : pComplex;    {pointer to the zero sequence matrix}
  pIndexZ : pInteger;    {pointer to array keeping track on row exchanges}

procedure TAllShortForm.FormHide(Sender: TObject);
begin
  Imagoo.AllShortItem.checked := false;
  {enable the buttons in the report form}
  {ReportButtons(true);  moved to Skemadef.pas}
end;

procedure TAllShortForm.CalcButClick(Sender: TObject);
{calculate the short circuit current at all buses and/or voltmeters,
 according to the selections done in the dialog}
var
  ipSet : longint;   {pointer to the common as integer}
  pSet  : TpCompSet; {pointer to the set of comp. etc}
begin
  {hide unrelevant forms (that may be visible)}
  RelayCurr.visible := false;
  VoltPhasorForm.visible := false;
  CurrPhasorForm.visible := false;
  Imagopas.HideTimeEdit;
  ipSet := imagopas.ipSetG;         {copy to local}
  pSet := TpCompSet(ipSet);         {pointer to the set, from integer}
  pSet^.Deactivate;                 {deactivate evtl. active comp. and contr}
  pSet^.w_checked := false;         {flag for checking overlapping or shorted components}
  {IEC-calculation, three phase fault,but in all buses and/or voltmeters}
  { these are reset in ShortCircTimerTimer after calculations}
  pSet^.Bus_Short := BusRadioBut.checked or BothRadioBut.checked;   {fault in all busbars}
  pSet^.Volt_Short := VoltRadioBut.checked or BothRadioBut.checked; {fault in all voltmeters}
  {IEC or superposition method, or impedance calculation}
  if (Z012But.checked) then begin  {impedance calculation}
    pSet^.appl_type := AllZ012;
  end else begin                   {fault calculation}
    if (SuppBut.checked) then begin
      pSet^.appl_type := AllSup;   {moved to calcbutclick}
    end else begin
      pSet^.appl_type := AllStd;
    end;
  end;
  {disable some buttons}
  Imagoo.RIBut.enabled := SuppBut.checked or Z012But.checked;
  Imagoo.VoltPhasorBut.enabled := false;
  Imagoo.VoltBut.enabled := false;
  {hide display of current or power}
  if (imagoo.ShowCurrBut.caption = Icap_g) then begin  {displays current, two clicks to reset}
    imagoo.ShowCurrButClick(sender);
    imagoo.ShowCurrButClick(sender);
  end else if (imagoo.ShowCurrBut.caption = '') then begin
    {does not display, do nothing}
  end else begin                        {displays power, one click to reset}
    imagoo.ShowCurrButClick(sender);
  end;
  imagoo.ShowCurrBut.down := false;
  {hide display of bus voltages}
  if (imagoo.BusBut.caption = VVcap_g) then begin  {displays voltage, two clicks to reset}
    imagoo.BusButClick(sender);
    imagoo.BusButClick(sender);
  end else if (imagoo.BusBut.caption = '') then begin
    {does not display, do nothing}
  end else begin                        {displays pu voltage, one click to reset}
    imagoo.BusButClick(sender);
  end;
  imagoo.BusBut.down := false;
  {start...}
  pSet^.SetApplOp(14);              {flag for type of calculation}
  {sequence currents may not be shown}
  {IEC, or impedance: no sequence currents}
  if (pSet^.appl_type = AllStd) or (pSet^.appl_type = AllZ012) then begin
    if (Imagoo.PhaseBut.caption[1] = '+') or
       (Imagoo.PhaseBut.caption[1] = '-') or
       (Imagoo.PhaseBut.caption[1] = '0') then Imagoo.PhaseButMouseUp(Sender,mbLeft,[],0,0);
  end else begin                                   {superpos, some sequence currents}
    if (ThreePhaseBut.checked) and                 {3-phase fault}
       ((Imagoo.PhaseBut.caption[1] = '+') or
        (Imagoo.PhaseBut.caption[1] = '-') or
        (Imagoo.PhaseBut.caption[1] = '0')) then
      Imagoo.PhaseButMouseUp(Sender,mbLeft,[],0,0)
    else if (LineLineBut.checked) and (Imagoo.PhaseBut.caption[1] = '0') then
      Imagoo.PhaseButMouseUp(Sender,mbLeft,[],0,0)
    else ;
  end;
  {hide activation times of relays}
  ShowOrNotActivationTimes(pSet, true);  {in Skemproc.pas}
  {hide impedances of distance relays}
  ShowOrNotDistance(pSet, true);         {in Skemproc.pas}
  {hide ampacity indicators at transmission lines}
  HideAmpacity(pSet);                    {in skemproc.pas}
  {set the buttons on the report form to correspond to the settings on this form}
  ReportForm.VoltMetBut.down := VoltRadioBut.checked or BothRadioBut.checked;
  ReportForm.BusBut.down := BusRadioBut.checked or BothRadioBut.checked;
  ReportForm.BusBut.enabled := ReportForm.BusBut.down;
  ReportForm.VoltMetBut.enabled := ReportForm.VoltMetBut.down;
  ReportForm.Z012But.down := Z012But.checked;
  ReportForm.Z012But.enabled := Z012But.checked;
  {calculate, but only if there are components}
//if (pSet^.p1SchComp <> nil) then begin
  if (pSet^.pCompList.count > 0) then begin
    {set cursors}
    CalcBut.Cursor := crHourGlass;
    Cursor := crHourGlass;
    ShortCircTimer.enabled := true;
  end;
  {actual calculations are started in ShortCircTimerTimer of imagopas.pas}
end;

procedure TAllShortForm.ReportButtons(enable: boolean);
{most buttons in the ReportForm (Tab.pas) not enabled, not down,
 when fault at all buses
 enable   enabled/not
 Called in skemadef.pas}
begin
  {disable/enable buttons}
  ReportForm.AmpBut.enabled := enable;
  ReportForm.VoltBut.enabled := enable;
  ReportForm.OhmBut.enabled := enable;
  ReportForm.WattBut.enabled := enable;
  ReportForm.CTWattBut.enabled := enable;
  ReportForm.ActimeBut.enabled := enable;
  ReportForm.ImpedanceBut.enabled := enable;
  ReportForm.TapBut.enabled := enable;
//FilterBut := enable;
  ReportForm.AmpSeqBut.enabled := enable;
  ReportForm.VoltSeqBut.enabled := enable;
  ReportForm.OhmGroundBut.enabled := enable;
  ReportForm.TimeBut.enabled := enable;
  ReportForm.ZoneTimeBut.enabled := enable;
  ReportForm.FaultBut.enabled := enable;
  ReportForm.NotesBut.enabled := enable;
  ReportForm.ShortCurrBut.enabled := enable;
//ReportForm.TitleBut.enabled := enable;
//ReportForm.AbcBut.enabled := enable;
  ReportForm.ZeroBut.enabled := enable;
  {the buttons for the reports of fault at all buses}
  ReportForm.ShortBut.enabled := not enable;
  ReportForm.TwoShortBut.enabled := not enable;
  ReportForm.EarthFaultBut.enabled := not enable;
  ReportForm.LLEarthBut.enabled := not enable;
  ReportForm.BusBut.enabled := not enable;
  ReportForm.VoltMetBut.enabled := not enable;
  ReportForm.Z012But.enabled := not enable;
  {reset (= not down) if disabled}
  if (not enable) then begin
//  ReportForm.AmpBut.down := false;
    ReportForm.VoltBut.down := false;
    ReportForm.OhmBut.down := false;
    ReportForm.WattBut.down := false;
    ReportForm.CTWattBut.down := false;
    ReportForm.ActimeBut.down := false;
    ReportForm.ImpedanceBut.down := false;
    ReportForm.TapBut.down := false;
//  FilterBut := enable;
//  ReportForm.AmpSeqBut.down := false;
    ReportForm.VoltSeqBut.down := false;
    ReportForm.OhmGroundBut.down := false;
    ReportForm.TimeBut.down := false;
    ReportForm.ZoneTimeBut.down := false;
    ReportForm.FaultBut.down := false;
    ReportForm.NotesBut.down := false;
    ReportForm.ShortCurrBut.down := false;
//  ReportForm.TitleBut.down := false;
//  ReportForm.AbcBut.down := false;
    ReportForm.ZeroBut.down := false;
  end else begin
    {the buttons for the reports of fault at all buses}
    ReportForm.ShortBut.down := false;
    ReportForm.TwoShortBut.down := false;
    ReportForm.EarthFaultBut.down := false;
    ReportForm.LLEarthBut.down := false;
    ReportForm.BusBut.down := false;
    ReportForm.VoltMetBut.down := false;
  end;
end;

procedure TAllShortForm.ShortCircTimerTimer(Sender: TObject);
{start the short circuit calculations}
var
  err,erx : integer;        {error flag}
  ipu     : integer;        {temporary}
  jcomp   : integer;        {counts components}
  name    : string;         {name = type of a component}
  nComp   : integer;        {number of components}
  nNonc   : integer;        {number of components with no calculations}
  probl   : solve;          {type of problem to be solved}
  pComp   : TpComp;         {pointer to a component}
  pSet    : TpCompSet;      {pointer to the set of comp. etc}
  Rf,Xf   : single;         {fault resistance, reactance}
  str,stx : string;         {impedance as string}
  txt     : string;         {temporary}
begin
  ShortCircTimer.enabled := false;    {Do this first !!}
  pSet := TpCompSet(imagopas.ipSetG);
  {get memory for results, unless already reserved}
  nComp := 0;
  nNonc := 0;
  pComp := nil;                               {initialize pointer to comp.}
  err := 0;
  jcomp := 0;
  while (err = 0) do begin                         {scan all components}
    pSet^.NextComp(pComp, err,jcomp,false);        {pointer to next component}
    if (err = 0) then begin                        {no errors, else no more components}
      inc(nComp);                                  {count components}
      pComp^.GetName(name);                        {name (type) of the component}
      if (name = gBus ) or (name = gVolt) or       {bus or voltmeter}
         (name = gShort) then begin                {fault, for tests}
        pComp^.GetMemMultix1(4);                    {for 4 types of faults, unless already reserved}
        inc(nNonc);                                {count "noncalculable" components}
      end else if (name = gGrnd) or (name = gWire) or   {more "noncalculable" components}
                  (name = gBrkr) or (name = gSymbol) or
                  (name = gText) or (name = gStar) or
                  (name = gGrImped) or (name = gRelay) or
                  (name = gRelay_inv) or (name = gRelay_vinv) or
                  (name = gRelay_einv) or (name = grelay_Ltinv) or
                  (name = gSpecCT) then begin
        inc(nNonc);
      end;
    end;
  end;
  {impedance or short circuit}
  if (Z012But.checked) then begin
    probl := Z012Th;
  end else begin
    {IEC or superposition method}
    if (SuppBut.checked) then begin
      probl := superpos;
//  pSet^.appl_type := AllSup;   {moved to calcbutclick}
    end else begin
      probl := IECfault;
//    pSet^.appl_type := AllStd;
    end;
  end;
  {read fault impedance}
  txt := ResEdit.text;                             {resistance, as text}
  NextValueFromLine(txt, 1, ipu, Rf, str, err);    {resistance}
  txt := ReactEdit.text;                           {reactance, as text}
  NextValueFromLine(txt, 1, ipu, Xf, stx, erx);    {reactance}
  if (err = 0) and (erx = 0) and
     {calculate, but only if there are "calculable" components}
     (nComp > 0) and (nComp > nNonc) then begin    {no errors}
    pSet^.fault_Z := str+' + j '+stx;              {fault impedance as string, needed in report}
    pSet^.fault_imped := cplx(Rf,Xf);              {fault impedance}
    {set flag for fault type}
    if (Z012But.checked      ) then                {impedance calculation}
      pSet^.Appl_but := 23
    else begin                                     {short circuit}
      if      (ThreePhaseBut.checked) then pSet^.Appl_but := 18
      else if (LineLineBut.checked  ) then pSet^.Appl_but := 19
      else if (LineEarthBut.checked ) then pSet^.Appl_but := 20
      else if (LLEBut.checked       ) then pSet^.Appl_but := 21
      else if (AllBut.checked       ) then pSet^.Appl_but := 22
      else                                 pSet^.Appl_but := 18; {error case}
    end;
    {calculate initial state, if superpos.method, and not yet calculated}
    if (SuppBut.checked) and (not InitBut.down) then begin
      SolveNet(imagopas.ipSetG,false, normal);     {initial state}
      pSet^.SavePrefaultVoltages('parz');          {especially needed when all faults}
    end else begin
      Imagoo.NodeButClick(nil);                    {determine nodes (necessary?)}
    end;
    {impedance, or short circuit}
    if (Z012But.checked ) then begin      {Thevenin sequence impedances}
      AsymmFaults(imagopas.ipSetG,'Z012',superpos); {calculated as line-earth, with superposition}
      {set the buttons in the report form}
      ReportForm.ShortBut.enabled := false;
      ReportForm.TwoShortBut.enabled := false;
      ReportForm.EarthFaultBut.enabled := false;
      ReportForm.LLEarthBut.enabled := false;
    end else begin                                 {short circuit}
      if      (LineLineBut.checked  ) then begin
        AsymmFaults(imagopas.ipSetG,'LL',probl );    {line to line fault}
        {set the buttons in the report form}
        ReportForm.ShortBut.enabled := false;
        ReportForm.TwoShortBut.down := true;
        ReportForm.EarthFaultBut.enabled := false;
        ReportForm.LLEarthBut.enabled := false;
      end else if (LineEarthBut.checked ) then begin
        AsymmFaults(imagopas.ipSetG,'LE',probl  );   {line-earth}
        {set the buttons in the report form}
        ReportForm.ShortBut.enabled := false;
        ReportForm.TwoShortBut.enabled := false;
        ReportForm.EarthFaultBut.down := true;
        ReportForm.LLEarthBut.enabled := false;
      end else if (LLEBut.checked       ) then begin
        AsymmFaults(imagopas.ipSetG,'LLE',probl );   {line-line-earth}
        {set the buttons in the report form}
        ReportForm.ShortBut.enabled := false;
        ReportForm.TwoShortBut.enabled := false;
        ReportForm.EarthFaultBut.enabled := false;
        ReportForm.LLEarthBut.down := true;
      end else if (ThreePhaseBut.checked) then  begin
        if (SuppBut.checked) then                    {superposition}
          SolveNet(imagopas.ipSetG,InitBut.down, Superpos)  {3-phase fault}
        else                                         {IEC-calculation}
          SolveNet(imagopas.ipSetG,false, IECFault); {3-phase fault}
        {set the buttons in the report form}
        ReportForm.ShortBut.down := true;
        ReportForm.TwoShortBut.enabled := false;
        ReportForm.EarthFaultBut.enabled := false;
        ReportForm.LLEarthBut.enabled := false;
      end else if (AllBut.checked) then begin        {calculate all faults}
        if (SuppBut.checked) then                    {superposition}
          SolveNet(imagopas.ipSetG,InitBut.down, Superpos)  {3-phase fault}
        else                                         {IEC-calculation}
          SolveNet(imagopas.ipSetG,false, IECFault); {3-phase fault}
        ReportForm.ShortBut.enabled := true;
        ReportForm.TwoShortBut.enabled := true;
        ReportForm.EarthFaultBut.enabled := true;
        ReportForm.LLEarthBut.enabled := true;
//      ReportForm.ShortBut.down := true;
        AsymmFaults(imagopas.ipSetG,'LL' ,probl);    {line to line fault}
        AsymmFaults(imagopas.ipSetG,'LE' ,probl);    {line-earth}
        AsymmFaults(imagopas.ipSetG,'LLE',probl);    {line-line-earth}
      end;
    end;
    {update the eventual result boxes}
    pSet^.UpdateResultBoxes(1);
    pSet^.UpdateResultBoxes(2);
    {show the last operation with a picture on the main form}
    Imagoo.SolvedProb.picture := nil;
    if (Z012But.checked) then begin        {sequence impedances}
      Imagoo.SolvedProb.canvas.font.name := 'arial';
      Imagoo.SolvedProb.canvas.font.size := 10;
      Imagoo.SolvedProb.canvas.font.color := clBlue;
      Imagoo.SolvedProb.canvas.TextOut(1,8,'Z012');
    end else begin                         {short circuit currents}
      if      (LineLineBut.checked  ) then Imagoo.SolvedProb.picture.bitmap := Imagoo.TwoShortBut.glyph
      else if (LineEarthBut.checked ) then Imagoo.SolvedProb.picture.bitmap := Imagoo.EarthFaultBut.glyph
      else if (LLEBut.checked       ) then Imagoo.SolvedProb.picture.bitmap := Imagoo.LLEarthBut.glyph
      else if (ThreePhaseBut.checked) then Imagoo.SolvedProb.picture.bitmap := Imagoo.ShortBut.glyph;
    end;
    Imagoo.SolvedProb.hint := 'Solved three phase fault';
    Imagoo.SolvedProb.canvas.font.style := [];
    Imagoo.SolvedProb.canvas.font.color := clRed;
    Imagoo.SolvedProb.canvas.font.name := 'arial';
    Imagoo.SolvedProb.canvas.font.size := 8;
    Imagoo.SolvedProb.canvas.textout(0,-5,'x all');  {a small x for deleting}
    Imagoo.SolvedProb.BringToFront;
  end else begin                      {some error in data, indicate it}
    if (err <> 0) then ResEdit.color := clYellow;
    if (erx <> 0) then ReactEdit.color := clYellow;
  end;
  Cursor := crDefault;              {waiting is over, return the cursor}
  CalcBut.cursor := crDefault;
  {reset the flags for calculated fault positions: buses or "voltmeters"}
  pSet^.Bus_Short := false;
  pSet^.Volt_Short := false;
end;

procedure TAllShortForm.FormKeyPress(Sender: TObject; var Key: Char);
{check that only numerals and comma or full stop are accepted at the Edit}
begin
{showmessage(activeControl.name);}
  {only numbers and . or , accepted}
  if not(((key >= '0') and (key <= '9')) or
          (key  = ',') or  (key  = '.')  or
          (key  = 'e') or  (key  = 'E')  or
          (key  = '-') or  (key  = '+')  or
          (key  = #8 ) or
          (key =  #3 ) or (key = #22)) then       {"copy" and "paste"}
           key := #0;                             {neglect other keys }
  {check correct char for decimal point}
  if      (dec_p = ',') and (key = '.') then key := ','
  else if (dec_p = '.') and (key = ',') then key := '.';
end;

procedure TAllShortForm.FormCreate(Sender: TObject);
begin
  DecimalPoint(dec_p);      {determine the char. for the decimal point, in routines.pas}
  pRefCT := nil;
  pMeasCT := nil;
  {add flag that form created}
  Imagoo.ReadyEdit.text := Imagoo.ReadyEdit.text+'a';
end;

procedure TAllShortForm.FormShow(Sender: TObject);
{disable, reset buttons in the report form}
begin
  {initialize the cursor, may be wrong after an error}
  CalcBut.Cursor := crDefault;
  Cursor := crDefault;
  {ReportButtons(false);  moved to Skemadef.pas}
end;

procedure TAllShortForm.PutRefAndMeasCT(pRCT,pMCT : TpComp);
{put the pointers to the reference and measurement current transformers
 Needed in RelCalc.pas for relay coordination
 pRCT  pointer to the reference CT
 pMCT  pointer to the measurement CT}
begin
  pRefCT := pRCT;
  pMeasCT := pMCT;
end;

procedure TAllShortForm.GetCTcurr(var ma,mb,mc,ra,rb,rc: complex);
{get the (unsymmetric) currents at measurement CT and reference CT when fault
 ma,mb,mc  phase currents of measurement CT
 ra,rb,rc  phase currents of reference CT}
begin
  ma := IfmeasA_a;
  mb := IfmeasB_a;
  mc := IfmeasC_a;
  ra := IfrefA_a;
  rb := IfrefB_a;
  rc := IfrefC_a;
end;

procedure TAllShortForm.AsymmFaults(ipSet : longint; fault: string; probl: solve);
{calculate line to line, line to earth short, line-line-earth circuit
 according to IEC 60909 or the superposition method
 at all buses and/or voltmeters
 ipSet   pointer to the "common" as integer
 fault   fault type: 'LL' = line-to-line; 'LE' = line-to-earth
         'LLE' = line-line-earth
 probl   the calculation method, IEC or superposition (here)
}
{ The principle in the line-line fault is the following:
  get the nominal voltage Ua-Ub  (multiplied by the c-factor) = Uabnom
  inject a current Ii to phase a, and drain the same current from phase b
  solve the net, get the changes in voltages of phase a (dUa) and phase b (dUb)
  the change in the voltage (dUa-dUb) = kIi is a linear function of the injected
      current Ii, so that k = (dUa-dUb)/Ii
  When the fault impedance is zero (so far..) it is required that the voltages
      in phase a and b are the same, dUa-dUb+Uabnom = 0
  The fault current is thus Isc = -Ii*Uabnom/(dUa-dUb)
  The other fault types solved similarly}
const
  Iinj = 1000.0;             {injected current}
var
  apu,apv,apw : single;      {temporary}
  cpu,cpv     : complex;     {temporary}
  cpw,cpx     : complex;     {temporary}
  det         : complex;     {determinant}
  dodd,doddN  : single;      {keep track of permutations}
  doddZ       : single;
  dUa,dUb,dUc : complex;     {voltage changes}
  dUadIa      : complex;     {derivative dUa/dIa due to injected current}
  dUbdIa      : complex;     {derivative dUb/dIa due to injected current}
  dUadIb      : complex;     {derivative dUa/dIb due to injected current}
  dUbdIb      : complex;     {derivative dUb/dIb due to injected current}
  dUadIaZ     : complex;     {derivative dUa/dIa due to injected current+Zf}
  dUbdIaZ     : complex;     {derivative dUb/dIa due to injected current+Zf}
  dUadIbZ     : complex;     {derivative dUa/dIb due to injected current+Zf}
  dUbdIbZ     : complex;     {derivative dUb/dIb due to injected current+Zf}
  err         : integer;     {error flag}
  errcom      : boolean;     {error flag}
  er1         : integer;     {error flag}
  freq        : single;      {frequency}
  IECvolt     : complex;     {line to neutral voltage, with the IEC correction}
//  IfmeasA     : complex;     {phase A current in measurement.CT, when fault}
//  IfmeasB     : complex;
//  IfmeasC     : complex;
//  IfrefA      : complex;     {phase A current in ref.CT, when fault}
//  IfrefB      : complex;
//  IfrefC      : complex;
  Ip,Im,Iz    : complex;     {pos,neg,zero seq. currents}
  Ia,Ib,Ic    : complex;     {fault currents}
  IpmeasA     : complex;     {prefault current in phase a of measurement CT (for relay coord.)}
  IpmeasB     : complex;
  IpmeasC     : complex;
  IprefA      : complex;     {prefault current in phase a of reference CT (for relay coord.)}
  IprefB      : complex;
  IprefC      : complex;
  Isc,Iscb    : complex;     {short circuit current (to phase a and b)}
  Iscc        : complex;     {short circuit current of phase c}
  ii          : integer;     {index}
  ipu         : integer;     {temporary}
  ix,iy       : integer;
  jcomp       : integer;     {counts components}
  ju          : integer;     {voltage node}
  kf          : complex;
  lbl         : string;
  MTa,MTb,MTc : complex;     {phase currents in measurement CT}
  MTp,MTn,MTz : complex;     {pos, neg, zero seq. currents of measurement curr.trafo}
  M2a,M2b,M2c : complex;     {phase currents in measurement CT}
  M2p,M2n,M2z : complex;     {+,-,0 seq- currents of measurement CT at second injection of LLE}
  name        : string;      {name (type) of a component}
  nnod        : integer;     {number of nodes}
  pComp       : TpComp;      {pointer to a component}
//pIndexZ     : pInteger;    {pointer to array keeping track on row exchanges}
  pSet        : TpCompSet;   {pointer to the common}
  pVrhs       : pComplex;    {temporary right hand side}
//pZmat       : pComplex;    {pointer to the zero sequence matrix}
  RTa,RTb,RTc : complex;     {phase currents in reference CT}
  RTp,RTn,RTz : complex;     {pos, neg, zero seq. currents of reference curr.trafo}
  R2a,R2b,R2c : complex;     {phase currents in reference CT}
  R2p,R2n,R2z : complex;     {+,-,0 seq- currents of reference CT at second injection of LLE}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  SnglZ       : integer;
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  ToutZ       : complex;
  Uab         : complex;     {Ua-Ub (*C-factor)}
  UseInitSt   : boolean;     {use initial state of a previous calculation (load flow, etc.)}
  Va,Vb       : complex;     {voltages at phases a and b, for LLE-fault}
  Vp,Vn,Vz    : complex;     {sequence voltages}
  Vnom        : single;      {nominal voltage at fault, * C-factor}
  Zf          : complex;     {fault impedance}
  Z0,Z1,Z2    : complex;     {thevenin sequence impedances}
begin
  pSet := TpCompSet(ipSet);          {integer to pointer}
  freq := pSet^.appl_Hz;
  nnod := pSet^.appl_nn;             {copy number of nodes}
  {reserve and initialize memory for matrices}
  ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
  {reserve memory for the neg.seq matrix}
  getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
  clearCmat(pNmat,nnod);
  getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
  clearIvec(pIndexN, nnod);
  {zero sequence needed for line-earth faults}
  if (fault = 'LE') or (fault = 'LLE') or (fault = 'Z012') then begin
    getmem(pZmat,nnod*nnod*SizeOf(complex));  {reserve matrix for neg seq.}
    clearCmat(pZmat,nnod);
    getmem(pIndexZ,nnod*Sizeof(integer));     {for index array, neg seq.}
    clearIvec(pIndexZ, nnod);
  end else begin
    pZmat := nil;
    pIndexZ := nil;
  end;
  {deduce, is old initial state used}
  UseInitSt :=
    ((pSet^.Appl_Op = 18) and (RelayCalcForm.InitBut.down)) or  {relay coordination, in RelCalc.pas}
    ((pSet^.Appl_Op = 14) and (AllShortForm.InitBut.down));      {fault in all buses}
  getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
  clrCrow(pVrhs,1,nnod);
  {clear rhs, it is not (yet) needed}
  clrCrow(pWs,1,nnod);
  ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, probl{IECFault},
             {false}UseInitSt, errcom);
  {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
  PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
  {zero sequence needed for line-earth faults}
  if (fault = 'LE') or (fault = 'LLE') or (fault = 'Z012') then begin
    {copy the matrix to zero seq. matrix, convert to correspond to zero seq}
    PosToZeroMat(ipSet, pAdmat,pZmat, freq, 0.0, nnod);
  end;
  if (not(errcom)) then begin
    try
      CLUdecomp(pAdmat,pWs,nnod,pIndex,Dodd,Sngl,Tout);     {decompose, pos.seq}
      CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg.seq}
      {zero sequence needed for line-earth faults}
      if (fault = 'LE') or (fault = 'LLE') or (fault = 'Z012') then begin
        CLUdecomp(pZmat ,pWs,nnod,pIndexZ,DoddZ,SnglZ,ToutZ); {decompose, zero seq}
      end;
      {solve the voltage at the fault for each bus/voltmeter in turn. . . . . .}
      Zf := pSet^.fault_imped;                    {fault impedance}
      pComp := nil;                               {initialize pointer to comp.}
      err := 0;
      jcomp := 0;
      while (err = 0) do begin                    {scan all components}
        pSet^.NextComp(pComp, err,jcomp,true);    {pointer to next component}
        if (err = 0) then begin                   {no errors, else no more components}
          pComp^.GetName(name);                   {name (type) of the component}
          if ((name = gBus ) and (pSet^.Bus_Short )) or {fault in buses}
             ((name = gVolt) and (pSet^.Volt_Short)) or {fault in voltmeters}
             (name = gShort) then begin                 {fault, for tests}
            pComp.GetLabel(lbl,ix,iy);
            if (probl = IECfault) then begin
              pComp^.GetSystlev(apu);               {get system voltage level}
              apv := VoltageFactor(apu);            {c-factor}
              IECVolt := cplx(-apv*apu/sqrt(3),0);  {line to neutral voltage}
              Vnom := Cabs(IECVolt);                {nominal voltage *C-factor}
              if (Vnom < 1) then Vnom := 1;         {just to be sure}
              Uab := cplx(1.5*Vnom,0.5*sqrt3*Vnom); {nominal Ua-Ub *C-factor}
            end else begin                          {superposition}
            {coordination with relay calculations? see RelCalc.pas}
              if (pSet^.appl_op = 18) then begin
                cpu := RelayCalcForm.GetPrefVoltage; {prefault voltage in the tr.line}
                IECVolt.x := -cpu.x;                {opposite voltage for superposition}
                IECVolt.y := -cpu.y;
              end else begin                        {"normal" calculation}
                if (InitBut.down) then
                  pComp^.GetPref(apu,apv)             {prefault volt. after Loadflow, genvolt}
                else
                  pComp^.GetZpar(apu,apv,apw);        {prefault volt, after "normal calc., apw not needed}
                IECVolt := cplx(-apu,-apv);           {neg. of the line to neutral voltage}
              end;
//            cpu := cplx(apu,apv);                 {prefault voltage of phase a}
              cpu := cplx(-IECVolt.x,-IECVolt.y);
              cpv := Cmult(emifii,cpu);             {voltage of phase b}
              Uab := Csub(cpu,cpv);                 {prefault Ua-Ub }
              Vnom := Cabs(Uab);                    {not needed, just to keep the compiler happy}
            end;
            pComp^.GetNode(1,ju,er1);               {get upstream node}
            if (ju > 0) and (ju <= nnod) then begin {reasonable node}
              if (fault = 'LL')  then begin
                Ia := cplx( Iinj,0);                  {current injected to phase a}
                Ib := cplx(-Iinj,0);                  {to phase b (drained)}
                Ic := cplx(0,0);                      {nothing to phase c}
              end else if (fault = 'LE') or (fault = 'LLE') or (fault = 'Z012') then begin
                Ia := cplx(-Iinj,0);                  {current injected to phase a}
                Ib := cplx(0,0);
                Ic := cplx(0,0);
              end else begin                          {error case}
                Ia := cplx(0,0);
                Ib := cplx(0,0);
                Ic := cplx(0,0);
              end;
              PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);        {to sequence currents}
              clearCvec(pRhs,nnod);                 {clear right hand side, pos.seq}
              clearCvec(pVrhs,nnod);                {clear right hand side, neg.seq}
              putCvec(pRhs,ju,nnod,Ip);             {current to rhs, pos.seq.net}
              putCvec(pVrhs,ju,nnod,Im);            {current to rhs, neq.seq. net}
              CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
              CLUbacksub(pNmat,nnod,pIndexN,pVrhs); {backsubstitute = solve neg.seq.net}
{RELCALC}     {try to get currents in reference and measurement current trafos, needed in RelCalc.pas...}
              {.. but only when coordinating in RelCalc. = when ref. and measurement CT's are defined}
              if (pRefCT <> nil) and (pMeasCT <> nil) then begin
                {reference CT}
                ResultsToComp(pSet, nnod, freq, pRhs, 1, ForNode);   {positive seq. to all components}
                RelayCalcForm.PutShortCurrent(Ip);  {needed in sum current}
                pRefCT^.SumCurr(RTp.x,RTp.y);       {pos.seq. current of reference CT}
                pMeasCT^.SumCurr(MTp.x,MTp.y);      {pos.seq. current of measurement CT}
                ResultsToComp(pSet, nnod, freq, pVrhs, 2, ForNode);  {negative seq. to all components}
                RelayCalcForm.PutShortCurrent(Im);
                pRefCT^.SumCurr(RTn.x,RTn.y);       {neg.seq. current of reference CT}
                pMeasCT^.SumCurr(MTn.x,MTn.y);      {neg.seq. current of measurement CT}
              end else begin                        {not needed in other calculations}
                MTp := cplx(0,0); MTn := cplx(0,0);
                RTp := cplx(0,0); RTn := cplx(0,0);
              end;
              getCvec(pRhs,ju,nnod,Vp);             {get pos.seq.voltage at fault}
              getCvec(pVrhs,ju,nnod,Vn);            {get neg.seq.voltage at fault}
              if (fault = 'LL') then begin
                Vz := cplx(0,0);                    {zero seq. voltage = 0}
{RELCALC}       RTz.x := 0;                         {zero seq. current = 0 (?)}
                RTz.y := 0;
                MTz.x := 0;
                MTz.y := 0;
              {zero sequence needed for line-earth faults}
              end else if (fault = 'LE') or (fault = 'LLE') or (fault = 'Z012') then begin
                clearCvec(pVrhs,nnod);              {clear right hand side, zero.seq}
                putCvec(pVrhs,ju,nnod,Iz);          {current to rhs, zero zero. net}
                CLUbacksub(pZmat,nnod,pIndexZ,pVrhs); {backsubstitute = solve zero seq.net}
                getCvec(pVrhs,ju,nnod,Vz);            {get zero seq.voltage at fault}
{RELCALC}       {currents at ref. and meas. CT, if relay coordination}
                if (pRefCT <> nil) and (pMeasCT <> nil) then begin
                  ResultsToComp(pSet, nnod, freq, pVrhs, 3, ForNode);  {negative seq. to all components}

                  RelayCalcForm.PutShortCurrent(Iz);  {needed in sum current}
                  pRefCT^.SumCurr(RTz.x,RTz.y);       {zero.seq. current of reference CT}
                  pMeasCT^.SumCurr(MTz.x,MTz.y);      {zero.seq. current of measurem. CT}
                end else begin                        {not needed in other calculations}
                  MTz := cplx(0,0); MTz := cplx(0,0);
                  RTz := cplx(0,0); RTz := cplx(0,0);
                end;
              end;
              SeqToPhase(Vp,Vn,Vz,dUa,dUb,dUc);     {changes in phase voltages}
{RELCALC}     SeqToPhase(RTp,RTn,RTz,RTa,RTb,RTc);  {reference phase currents (with injected fault current)}
              SeqToPhase(MTp,MTn,MTz,MTa,MTb,MTc);  {measurement phase currents (with injected fault current)}
              if (fault = 'LL') then begin
                kf := Cdiv(Csub(dUa,dUb),Ib);         {coeff. for dVoltage from current}
                                 { Note:  Ib, because pos direction of current = from a to b}
                Isc := Cdiv(Uab,Csub(Zf,kf));         {short circuit current, when impedance}
                Iscb := cplx(-Isc.x,-Isc.y);
                Iscc := cplx(0,0);
{RELCALC}       {relay coordination: current in reference CT, linear function of fault current}
                if (pRefCT <> nil) and (pMeasCT <> nil) then begin
                  {find current in reference CT when fault}
                  RelayCalcForm.GetPrefaultRefCTCurrent(IprefA.x,IprefA.y);  {prefault current in ref CT}
                  IprefB := Cmult(emifii,IprefA);
                  IprefC := Cmult(eifii,IprefA);
                  {currents when fault}
                  cpu := Cdiv(Isc,cplx(Iinj,0));             {linear coeff.}
                  IfrefA_a := Csub(IprefA,Cmult(cpu,RTa));   {current when fault}
                  IfrefB_a := Csub(IprefB,Cmult(cpu,RTb));
                  IfrefC_a := Csub(IprefC,Cmult(cpu,RTc));
                  {find current in measurement CT when fault}
                  RelayCalcForm.GetPrefaultMeasCTCurrent(IpmeasA.x,IpmeasA.y);  {prefault current in ref CT}
                  IpmeasB := Cmult(emifii,IpmeasA);
                  IpmeasC := Cmult(eifii,IpmeasA);
                  {currents when fault}
                  cpu := Cdiv(Isc,cplx(Iinj,0));             {linear coeff.}
                  IfmeasA_a := Csub(IpmeasA,Cmult(cpu,MTa)); {current when fault}
                  IfmeasB_a := Csub(IpmeasB,Cmult(cpu,MTb));
                  IfmeasC_a := Csub(IpmeasC,Cmult(cpu,MTc));
                end;
              end else if (fault = 'LE') then begin
                kf := Cdiv(dUa,Ia);
                Isc := Cdiv(IECVolt,Cadd(kf,Zf));
                Isc := cplx(-Isc.x,-Isc.y);           {must change sign, for some reason?}
                Iscb := cplx(0,0);
                Iscc := cplx(0,0);
{RELCALC}       {relay coordination: current in reference CT, linear function of fault current}
                if (pRefCT <> nil) and (pMeasCT <> nil) then begin
                  {find current in reference CT when fault}
                  RelayCalcForm.GetPrefaultRefCTCurrent(IprefA.x,IprefA.y);  {prefault current in ref CT}
                  IprefB := Cmult(emifii,IprefA);
                  IprefC := Cmult(eifii,IprefA);
                  {currents when fault}
                  cpu := Cdiv(Isc,cplx(Iinj,0));             {linear coeff.}
{Cadd??}          IfrefA_a := {Csub}Cadd(IprefA,Cmult(cpu,RTa));   {current when fault}
                  IfrefB_a := {Csub}Cadd(IprefB,Cmult(cpu,RTb));
                  IfrefC_a := {Csub}Cadd(IprefC,Cmult(cpu,RTc));
                  {find current in measurement CT when fault}
                  RelayCalcForm.GetPrefaultMeasCTCurrent(IpmeasA.x,IpmeasA.y);  {prefault current in ref CT}
                  IpmeasB := Cmult(emifii,IpmeasA);
                  IpmeasC := Cmult(eifii,IpmeasA);
                  {currents when fault}
                  cpu := Cdiv(Isc,cplx(Iinj,0));             {linear coeff.}
                  IfmeasA_a := {Csub}Cadd(IpmeasA,Cmult(cpu,MTa)); {current when fault}
                  IfmeasB_a := {Csub}Cadd(IpmeasB,Cmult(cpu,MTb));
                  IfmeasC_a := {Csub}Cadd(IpmeasC,Cmult(cpu,MTc));
                end;
{start of LLE}
              end else if (fault = 'LLE') then begin  {line-line-earth fault}
                dUadIa := Cdiv(dUa,Ia);
                dUbdIa := Cdiv(dUb,Ia);
                {inject a current to phase b also}
                Ia := cplx({0}Iinj,0);                {zero seq. current = zero, else problems}
                Ib := cplx(-Iinj,0);                  {current injected to phase b}
                Ic := cplx(0,0);
                PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);        {to sequence currents}
                clearCvec(pRhs,nnod);                 {clear right hand side, pos.seq}
                clearCvec(pVrhs,nnod);                {clear right hand side, neg.seq}
                putCvec(pRhs,ju,nnod,Ip);             {current to rhs, pos.seq.net}
                putCvec(pVrhs,ju,nnod,Im);            {current to rhs, neq.seq. net}
                CLUbacksub(pAdmat,nnod,pIndex,pRhs);  {backsubstitute = solve pos.seq.net}
                CLUbacksub(pNmat,nnod,pIndexN,pVrhs); {backsubstitute = solve neg.seq.net}
                getCvec(pRhs,ju,nnod,Vp);             {get pos.seq.voltage at fault}
                getCvec(pVrhs,ju,nnod,Vn);            {get neg.seq.voltage at fault}
                {currents at measurement and ref.curr.transformers, when relay coordination}
                if (pRefCT <> nil) and (pMeasCT <> nil) then begin
                  {prefault measurement and referene (again... do this only once)}
                  RelayCalcForm.GetPrefaultMeasCTCurrent(IpmeasA.x,IpmeasA.y);  {prefault current in meas. CT}
                  IpmeasB := Cmult(emifii,IpmeasA);
                  IpmeasC := Cmult(eifii,IpmeasA);
                  RelayCalcForm.GetPrefaultRefCTCurrent(IprefA.x,IprefA.y);     {prefault current in ref CT}
                  IprefB := Cmult(emifii,IprefA);
                  IprefC := Cmult(eifii,IprefA);
                  {reference and measurement CT when injected current }
                  ResultsToComp(pSet, nnod, freq, pRhs, 1, ForNode);   {positive seq. to all components}
                  RelayCalcForm.PutShortCurrent(Ip);  {needed in sum current}
                  pRefCT^.SumCurr(R2p.x,R2p.y);       {pos.seq. current of reference CT}
                  pMeasCT^.SumCurr(M2p.x,M2p.y);      {pos.seq. current of measurement CT}
                  ResultsToComp(pSet, nnod, freq, pVrhs, 2, ForNode);  {negative seq. to all components}
                  RelayCalcForm.PutShortCurrent(Im);  {needed in sum current}
                  pRefCT^.SumCurr(R2n.x,R2n.y);       {neg.seq. current of reference CT}
                  pMeasCT^.SumCurr(M2n.x,M2n.y);      {neg.seq. current of measurement CT}
                end else begin                        {not needed in other calculations}
                  M2p := cplx(0,0); M2n := cplx(0,0);
                  R2p := cplx(0,0); R2n := cplx(0,0);
                end;
                clearCvec(pVrhs,nnod);                {clear right hand side, zero.seq}
                putCvec(pVrhs,ju,nnod,Iz);            {current to rhs, zero zero. net}
                CLUbacksub(pZmat,nnod,pIndexZ,pVrhs); {backsubstitute = solve zero seq.net}
                getCvec(pVrhs,ju,nnod,Vz);            {get zero seq.voltage at fault}
{RELCALC}       {currents at ref. and meas. CT, for relay coordination (zero, as stated above)}
                R2z := cplx(0,0);
                M2z := cplx(0,0);
//ResultsToComp(pSet, nnod, freq, pVrhs, 3, ForNode);
//pRefCT^.SumCurr(R2z.x,R2z.y);       {neg.seq. current of reference CT}
//pMeasCT^.SumCurr(M2z.x,M2z.y);      {neg.seq. current of measurement CT}
//showmessage('z  '+comptostr(M2z));
                {continue ...}
                SeqToPhase(Vp,Vn,Vz,dUa,dUb,dUc);     {changes in phase voltages}
                dUadIb := Cdiv(dUa,Ib);
                dUbdIb := Cdiv(dUb,Ib);
                {solve the currents giving the (negative) of the voltages at phases a and b}
                {add the contribution of the fault impedance (Csub or Cadd ??)}
                dUadIaZ := Cadd(dUadIa,Zf);
                dUbdIaZ := Cadd(dUbdIa,Zf);
                dUadIbZ := dUadIb;
                dUbdIbZ := dUbdIb;
                if (probl = IECFault) then begin      {IEC-calculation}
                  Va := cplx(-Vnom,0);
                  Vb := cplx(0.5*Vnom,sqrt(3)*0.5*Vnom);
                end else begin                        {superposition}
                  Va := cplx(-IECVolt.x,-IECVolt.y);
                  Vb := Cmult(Va,emifii);
                end;
                det := Csub(Cmult(dUadIaZ,dUbdIbZ),Cmult(dUbdIaZ,dUadIbZ)); {determinant}
                if (Cabs(det) > 1.e-5) then begin
                  cpu := Csub(Cmult(dUbdIbZ,Va),Cmult(dUadIbZ,Vb));
                  cpv := Csub(Cmult(dUadIaZ,Vb),Cmult(dUbdIaZ,Va));
                  cpw := Cdiv(cpu,det);             {part of current in phase a ...}
                  Iscb := Cdiv(cpv,det);            {current in phase b}
                  Iscc := cplx(0,0);
                  Isc := Csub(cpw,Iscb);            {current in phase a}
                end else begin
                  Isc := cplx(0,0);                 {error case}
                  Iscb := cplx(0,0);
                  Iscc := cplx(0,0);
                end;
{RELCALC}       {if relay coordination (RelCalc.pas), calculate currents at reference and measurement CT's}
                if (pRefCT <> nil) and (pMeasCT <> nil) then begin
                  SeqToPhase(R2p,R2n,R2z,R2a,R2b,R2c);  {reference phase currents (with injected fault current)}
                  SeqToPhase(M2p,M2n,M2z,M2a,M2b,M2c);  {measurement phase currents (with injected fault current)}
                  cpu := Cdiv(Iscb,cplx(-Iinj,0));      {linear coeff.}
                  cpv := Cdiv(Cadd(Isc,Iscb),cplx(-Iinj,0));       {does this work ...?}
                  {measurement, phase a}
                  cpw := Cmult(cpu,M2a);
                  cpx := Csub(IpmeasA,Cmult(cpv,MTa));
                  IfmeasA_a := Csub(cpx,cpw);
                  {measurement, phase b}
                  cpw := Cmult(cpu,M2b);
                  cpx := Csub(IpmeasB,Cmult(cpv,MTb));
                  IfmeasB_a := Csub(cpx,cpw);
                  {measurement, phase c}
                  cpw := Cmult(cpu,M2c);
                  cpx := Csub(IpmeasC,Cmult(cpv,MTc));
                  IfmeasC_a := Csub(cpx,cpw);
                  {reference, phase a}
                  cpw := Cmult(cpu,R2a);
                  cpx := Csub(IprefA,Cmult(cpv,RTa));
                  IfrefA_a := Csub(cpx,cpw);
                  {reference, phase b}
                  cpw := Cmult(cpu,R2b);
                  cpx := Csub(IprefB,Cmult(cpv,RTb));
                  IfrefB_a := Csub(cpx,cpw);
                  {reference, phase c}
                  cpw := Cmult(cpu,R2c);
                  cpx := Csub(IprefC,Cmult(cpv,RTc));
                  IfrefC_a := Csub(cpx,cpw);
//showmessage(comptostr(IfrefA_a));
{end of LLE}    end;
              end else if (fault = 'Z012') then begin  {sequence impedances}
                Z1 := Cdiv(Vp,Ip);
                if (abs(Z1.x) < 0.0005) then Z1.x := 0;
                if (abs(Z1.y) < 0.0005) then Z1.y := 0;
                Z2 := Cdiv(Vn,Im);
                if (abs(Z2.x) < 0.0005) then Z2.x := 0;
                if (abs(Z2.y) < 0.0005) then Z2.y := 0;
                Z0 := Cdiv(Vz,Iz);
                if (abs(Z0.x) < 0.0005) then Z0.x := 0;
                if (abs(Z0.y) < 0.0005) then Z0.y := 0;
                Isc := cplx(0,0);                   {some value, not needed}
                Iscb := cplx(0,0);
                Iscc := cplx(0,0);
              end else begin
                Isc := cplx(0,0);                   {error case}
                Iscb := cplx(0,0);
                Iscc := cplx(0,0);
              end;
              pComp^.PutVar(1,Cabs(Isc),'AsymmFaults');           {current to component}
              pComp^.PutVar(2,Cabs(Iscb),'AsymmFaults');          {current to component}
              pComp^.PutVar(3,Cabs(Iscc),'AsymmFaults');          {current to component}
              {save according to fault type}
              if (fault = 'LL') then ipu := 2
              else if (fault = 'LE') then ipu := 3
              else if (fault = 'LLE') then ipu := 4
              else if (fault = 'Z012') then ipu := 3    {save as for line-earth fault}
              else ipu := 3;
              {save as multiples}
              if (fault = 'Z012') then begin            {impedances differently}
                pComp^.PutMultix(ipu,63,Z1.x,Z1.y,Z2.x,Z2.y,Z0.x,Z0.y);
              end else begin
                if (pSet^.appl_type = AllStd) then begin  {IEC method}
                  pComp^.PutMultix(ipu,63,Cabs(Isc ),0,Cabs(Iscb),0,Cabs(Iscc),0);
                end else begin                            {superposition}
                  pComp^.PutMultix(ipu,63,Isc.x,Isc.y,Iscb.x,Iscb.y,Iscc.x,Iscc.y);
                end;
              end;
            end;
          end else begin                            {not bus or voltmeter with fault}
            if ((name = gBus ) and (not pSet^.Bus_Short )) or         {no fault in buses}
               ((name = gVolt) and (not pSet^.Volt_Short)) then begin {no fault in voltmeters}
              {save according to fault type}
              if (fault = 'LL') then ipu := 2
              else if (fault = 'LE') then ipu := 3
              else if (fault = 'LLE') then ipu := 4
              else ipu := 3;
              pComp^.PutVar(1,0,'AsymmFaults');  {no short circuit current}
              pComp^.PutMultix(ipu,63,0,0,0,0,0,0);
            end;
          end;
        end;
      end;        {end of loop over components}
    finally
    end;
  end;
  FreeMat(nnod);                             {free pAdmat, pRhs, pWs, pIndex}
  Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
  Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
  Freemem(pZmat,nnod*nnod*SizeOf(complex));  {free matrix for zero seq.}
  Freemem(pIndexZ,nnod*Sizeof(integer));     {free index array, zero seq}
  Freemem(pVrhs,nnod*SizeOf(complex));
  pRefCT := nil;                             {reset, set again in RelCalc, if needed}
  pMeasCT := nil;
end;

procedure TAllShortForm.FaultsEnabled(enable: boolean);
{enable/disable the radio buttons for selecting the fault type}
begin
  ThreePhaseBut.enabled := enable;
  LineLineBut.enabled := enable;
  LineEarthBut.enabled := enable;
  LLEBut.enabled := enable;
  AllBut.enabled := enable;
  ResEdit.enabled := enable;
  ReactEdit.enabled := enable;
  if (enable) then begin
    ResEdit.color := clWhite;
    ReactEdit.color := clWhite;
  end else begin
    ResEdit.color := clSilver;
    ReactEdit.color := clSilver;
  end;
end;

procedure TAllShortForm.IECButClick(Sender: TObject);
begin
  InitBut.enabled := not IECBut.checked;
  FaultsEnabled(true);     {is possible to select fault type}
end;

procedure TAllShortForm.SuppButClick(Sender: TObject);
begin
  InitBut.enabled := SuppBut.checked;
  FaultsEnabled(true);     {is possible to select fault type}
end;

procedure TAllShortForm.ResEditEnter(Sender: TObject);
begin
  ResEdit.color := clWhite;
end;

procedure TAllShortForm.ReactEditEnter(Sender: TObject);
begin
  ReactEdit.color := clWhite;
end;

procedure TAllShortForm.Z012ButClick(Sender: TObject);
begin
  InitBut.enabled := false;
  FaultsEnabled(false);     {not possible to select fault type}
end;
(*
procedure UnsymmLoadOld(ipSet: longint);
{solve the net, when there is one unsymmetrical load in the net
 ipSet  pointer to the "common", as integer}
const
  Ix0 : array[1..3] of complex = ((x:1;y:0),(x: 1;y:0),(x:-2;y:0));  {first test current}
  Iy0 : array[1..3] of complex = ((x:1;y:0),(x:-1;y:0),(x: 0;y:0));  {second test current}
var
  b1,b2       : complex;     {right hand side}
  c11,c12     : complex;     {coefficients}
  c21,c22     : complex;     {coefficients}
  cpu,cpv     : complex;     {temporary}
  det         : complex;     {determinant}
  Dodd,DoddN  : single;      {keeps track on permutations}
  er1         : integer;     {error flag}
  errcom      : boolean;     {error flag}
  freq        : single;      {frequency}
  Ia,Ib,Ic    : complex;     {phase currents}
  Ip,Im,Iz    : complex;     {sequence currents}
  it          : integer;     {index for test currents}
  Ix,Iy       : complex;     {solved injected currents}
  ju,jc       : integer;     {upstream, current node of the load}
  k11,k12     : complex;     {coefficients, dU/dI, phase a}
  k21,k22     : complex;     {coefficients, dU/dI, phase b}
  k31,k32     : complex;     {coefficients, dU/dI, phase c}
  nnod        : integer;     {number of nodes}
  nvar        : integer;     {number of (result) variables}
  pSet        : TpCompSet;
  pUnsym      : TpComp;      {pointer to the unsymm.load}
  pVrhs       : pComplex;    {temporary right hand side}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  Tout,ToutN  : Complex;     {test output from matrix decomposition}
  Ua0,Ub0,Uc0 : complex;     {voltage at the load, when no load}
  Ua,Ub,Uc    : complex;     {voltage at the load}
  Up,Un,Uz    : complex;     {voltages}
  Vn          : complex;     {neutral voltage}
  Za,Zb,Zc    : complex;     {load impedances in phases a,b,c}
begin
  pSet := TpCompSet(ipSet);
{showmessage(inttostr(pSet^.nUns_load));}
  Imagoo.NodeButClick(nil);             {determine nodes (necessary?)}
  if (pSet^.appl_temp <> 0) then begin  {there is an unsymm.load}
    pSet^.SetApplOp(15);                {flag for type of calculation}
    pUnsym := TpComp(pSet^.appl_temp);  {pointer to the load, saved as integer (temporary)}
    pUnsym^.GetPar(1,Za.x);             {resistance in phase a}
    pUnsym^.GetPar(2,Za.y);             {reactance in phase a}
    pUnsym^.GetPar(3,Zb.x);             {resistance in phase b}
    pUnsym^.GetPar(4,Zb.y);             {reactance in phase b}
    pUnsym^.GetPar(5,Zc.x);             {resistance in phase c}
    pUnsym^.GetPar(6,Zc.y);             {reactance in phase c}
    nnod := pSet^.appl_nn;              {number of nodes}
    freq := pSet^.appl_hz;              {frequency}
    pUnsym^.getNode(1,ju,er1);          {upstream node}
    pUnsym^.getNode(2,jc,er1);          {current "node"}
    if (nnod > 0) then begin            {there are nodes}
      {reserve mem for curr.trafos and relays (if there is any)}
      pSet^.GetMemForCurrTrAndRelays;
      {reserve and initialize memory for matrices}
      ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
      {reserve memory for the neg.seq matrix}
      getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
      clearCmat(pNmat,nnod);
      getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
      clearIvec(pIndexN, nnod);
      getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
      {construct the extended admittance matrix}
      ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, normal,false, errcom);
      {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
      PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
      {solve voltages, but only if no unknown components}
      if (not(errcom)) then begin
        try
          CLUdecomp(pAdmat,pWs,nnod,pIndex,Dodd,Sngl,Tout); {decompose first}
          CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg.seq}
          {first, solve the net without the unsymm. load}
          copyrhs(pRhs,pVrhs,nnod);   {copy the rhs to temporary, it must not be destroyed}
          CLUbacksub(pAdmat,nnod,pIndex,pVrhs);           {backsubstitute = solve net}
          {get voltage at the location of the unsymm.load, without the load}
          getCvec(pVrhs,ju,nnod,Ua0);                     {phase a voltage}
          Ub0 := Cmult(emifii,Ua0);                       {phase b voltage}
          Uc0 := Cmult(eifii ,Ua0);                       {phase c voltage}
          {inject (or drain) test currents, to determine dU/dI, two different test currents}
          for it := 1 to 2 do begin
            if (it = 1) then begin                        {the first test current}
              Ia := Ix0[1];                               {current of phase a}
              Ib := Ix0[2];                               {current of phase b}
              Ic := Ix0[3];                               {current of phase c}
            end else if (it = 2) then begin               {the second test current}
              Ia := Iy0[1];                               {current of phase a}
              Ib := Iy0[2];                               {current of phase b}
              Ic := Iy0[3];                               {current of phase c}
            end;
            PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);                  {sequence currents}
            clearCvec(pVrhs,nnod);                        {clear rhs}
            putCvec(pVrhs,ju,nnod,Ip);                    {current to rhs, pos.seq. net}
            CLUbacksub(pAdmat,nnod,pIndex,pVrhs);         {backsubstitute = solve pos.seq.net}
            getCvec(pVrhs,ju,nnod,Up);                    {pos.seq.voltage at load}
            clearCvec(pVrhs,nnod);                          {clear rhs}
            putCvec(pVrhs,ju,nnod,Im);                      {inject neg.seq.current}
            CLUbacksub(pNmat,nnod,pIndexN,pVrhs);           {backsubst. = solve neg.seq.net}
            getCvec(pVrhs,ju,nnod,Un);                    {neg.seq.voltage at load}
            Uz := cplx(0,0);                              {no zero seq. current}
            {voltages = coefficients}
            if (it = 1) then begin                        {the first test currents}
              SeqToPhase(Up,Un,Uz,k11,k21,k31);           {phase voltages at the load = coefficients}
              {positive current is inwards (is it?)}
              k11 := cplx(-k11.x,-k11.y);
              k21 := cplx(-k21.x,-k21.y);
              k31 := cplx(-k31.x,-k31.y);
            end else if (it = 2) then begin               {the second test current}
              SeqToPhase(Up,Un,Uz,k12,k22,k32);
              {positive current is inwards (is it?)}
              k12 := cplx(-k12.x,-k12.y);
              k22 := cplx(-k22.x,-k22.y);
              k32 := cplx(-k32.x,-k32.y);
            end;
          end;
          {solve voltages. Use condition that voltage at neutral is the same for all phases}
          c11 := Csub(Cadd(k11,Zb),Cadd(k21,Za));
          c12 := Csub(Csub(k12,Zb),Cadd(k22,Za));
          c21 := Csub(Csub(k11,Crmult(2,Zc)),Cadd(Za,k31));
          c22 := Csub(k12,Cadd(Za,k32));
          b1 := Csub(Ub0,Ua0);
          b2 := Csub(Uc0,Ua0);
          det := Csub(Cmult(c11,c22),Cmult(c12,c21));       {determinant}
          if (det.x <> 0) or (det.y <> 0) then begin
            cpu := cplx(-c12.x,-c12.y);
            Ix := Cdiv(Cadd(Cmult(c22,b1),Cmult(cpu,b2)),det);
            cpv := cplx(-c21.x,-c21.y);
            Iy := Cdiv(Cadd(Cmult(cpv,b1),Cmult(c11,b2)),det);
          end else begin                                    {cannot solve}
            Ix := cplx(0,0);
            Iy := cplx(0,0);
          end;
//showmessage(comptostr(ix)+'   '+comptostr(iy));
          {currents solved, solve the whole net with these currents}
          {phase currents}
          {note positive direction....!?}
          cpu := cplx(-Ix.x,-Ix.y);
          cpv := cplx(-Iy.x,-Iy.y);
          Ia := Cadd(Cmult(cpu,Ix0[1]),Cmult(cpv,Iy0[1]));
          Ib := Cadd(Cmult(cpu,Ix0[2]),Cmult(cpv,Iy0[2]));
          Ic := Cadd(Cmult(cpu,Ix0[3]),Cmult(cpv,Iy0[3]));
          PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);                  {sequence currents}
          putCvec(pRhs,ju,nnod,Ip);                       {current to original rhs, pos.seq. net}
          CLUbacksub(pAdmat,nnod,pIndex,pRhs);            {backsubstitute = solve pos.seq.net}
          putCvec(pRhs,jc,nnod,Ip);                       {complete the solution with the current}
          clearCvec(pVrhs,nnod);                          {clear rhs}
          putCvec(pVrhs,ju,nnod,Im);                      {inject neg.seq.current}
          CLUbacksub(pNmat,nnod,pIndexN,pVrhs);           {backsubst. = solve neg.seq.net}
          putCvec(pVrhs,jc,nnod,Im);                      {complete the solution with the current}
          {Results to components. pos.seq. results in pRhs, neg.seq results in pVrhs}
          ClearAllVar(pSet);                              {clear initially}
          ResultsToComp(pSet, nnod, freq, pVrhs,2,ForBoth); {neg.seq to comp., as first set of var.}
          CopyAllVar(pSet,1,2);                             {copy to second set of var. in Elplex.pas}
          ResultsToComp(pSet, nnod, freq, pRhs ,1,ForBoth); {pos.seq to comp, as first set of var}
          AllVarSeqToPhase(pSet);                           {from seq to phase, in Twoshort.pas}
          {voltage of the neutral point}
          pUnsym^.GetVar(gUnsymm_Vr,cpu.x);                 {voltage in phase a}
          pUnsym^.GetVar(gUnsymm_Vi,cpu.y);
          Vn := Cadd(cpu,Cmult(Ia,Za));                    {neutral voltage}
          {save same voltage to all phases}
          pUnsym^.GetNvar(nvar);
          pUnsym^.PutVar(gUnsymm_Vnr,Vn.x);
          pUnsym^.PutVar(gUnsymm_Vni,Vn.y);
          pUnsym^.PutVar(gUnsymm_Vnr+nvar,Vn.x);
          pUnsym^.PutVar(gUnsymm_Vni+nvar,Vn.y);
          pUnsym^.PutVar(gUnsymm_Vnr+nvar*2,Vn.x);
          pUnsym^.PutVar(gUnsymm_Vni+nvar*2,Vn.y);
          {sum current trafos must display the sum current}
//        if (pCurrT <> nil) then pCurrT^.SumToOutVar;     {in shortcmp.pas}
          {display relay current using polar coordinates}
          PolarToRelaysAndBuses(pSet);
          {show also the current through the current trafos, if allowed}
          ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);

          {update the eventual result boxes}
          pSet^.UpdateResultBoxes(1);

//showmessage('Ia  '+comptostr(Ia)+'  Ib  '+comptostr(Ib)+'  Ic  '+comptostr(Ic));
        finally

        end;
      end;
      {free the reserved memory from matrices}
      FreeMat(nnod);
      Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
      Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
      Freemem(pVrhs,nnod*SizeOf(complex));
      pSet^.FreeMemForCurrTrAndRelays;
    end;
  end;
end;
*)
procedure ListUnsymm(pSet: TpCompSet; pList: TpList; var nUns : integer);
{fill the list of pointers to the unsymmetrical loads
pSet    pointer to the common
pList   pointer to the list of pointers to the unsymm.loads, to be filled
nUns    counts the unsymm.loads}
var
  err        : integer;     {error flag}
  jcomp      : integer;     {counts components}
  name       : string;      {name (type) of a component}
  pComp      : TpComp;      {pointer to a component}
begin
  pComp := nil;
  err := 0;
  jcomp := 0;
  nUns := 0;
  while (err = 0) do begin                 {scan all components}
    pSet^.NextComp(pComp, err,jcomp,true); {pointer to next component}
    if (err = 0) then begin                {no errors, no more components}
      pComp^.GetName(name);                {get name (type) of the comp.}
      if (name = gUnsymm) then begin       {unsymmetrical load}
        pList^.add(pComp);
        inc(nUns);                         {count unsym.loads}
        if (nUns >= pSet^.nUns_load) then break;  {all found}
      end;
    end;
  end;
end;

procedure GetMemoryForEquations2(var paa,pbb,pWr :pComplex; var pind: pInteger; nUns: integer);
{get and clear memory for the equations for the unsymmetrical loads
 paa   pointer to the matrix
 pbb   pointer to the right hand side
 pWr   pointer to working space
 pind  pointer to the array for row exchanges
 nUns  number of unsymm. loads}
var
  cpu      : complex;   {temporary}
  ii       : integer;   {index}
  nn,nn2   : integer;   {dimensions}
  pa,pb,pw : pComplex;  {pointer to an element}
  pin      : pInteger;  {pointer to an element}
begin
  nn := nUns*3;       {three unknowns for each load}
  nn2 := nn*nn;
  getmem(paa,nn2*sizeof(complex));
  getmem(pbb,nn*sizeof(complex));
  getmem(pWr,nn*sizeof(complex));
  getmem(pind,nn*sizeof(integer));
  {clear the matrix and rhs}
  pa := paa;
  cpu := cplx(0,0);
  for ii := 1 to nn2 do begin
    pa^:= cpu;
    inc(pa);
  end;
  pb := pbb;
  pw := pWr;
  pin := pind;
  for ii := 1 to nn do begin
    pb^:= cpu;
    pw^:= cpu;
    pin^:= 0;
    inc(pb);
    inc(pw);
    inc(pin);
  end;
end;

procedure GetMemoryForEquations(var paa,pbb :pComplex; var pind: pInteger; nUns: integer);
{get and clear memory for the equations for the unsymmetrical loads
 paa   pointer to the matrix
 pbb   pointer to the right hand side
 pind  pointer to the array for row exchanges
 nUns  number of unsymm. loads}
var
  cpu    : complex;   {temporary}
  ii     : integer;   {index}
  nn,nn2 : integer;   {dimensions}
  pa,pb  : pComplex;  {pointer to an element}
  pin    : pInteger;  {pointer to an element}
begin
  nn := nUns*2;       {two unknowns for each load}
  nn2 := nn*nn;
  getmem(paa,nn2*sizeof(complex));
  getmem(pbb,nn*sizeof(complex));
  getmem(pind,nn*sizeof(integer));
  {clear the matrix and rhs}
  pa := paa;
  cpu := cplx(0,0);
  for ii := 1 to nn2 do begin
    pa^:= cpu;
    inc(pa);
  end;
  pb := pbb;
  pin := pind;
  for ii := 1 to nn do begin
    pb^:= cpu;
    pin^:= 0;
    inc(pb);
    inc(pin);
  end;
end;

procedure GetMemoryFordUdI2(var p11,p12,p13,p21,p22,p23,p31,p32,p33: pComplex;nUns: integer);
begin
  getmem(p11,nUns*nUns*sizeof(complex));
  getmem(p12,nUns*nUns*sizeof(complex));
  getmem(p13,nUns*nUns*sizeof(complex));
  getmem(p21,nUns*nUns*sizeof(complex));
  getmem(p22,nUns*nUns*sizeof(complex));
  getmem(p23,nUns*nUns*sizeof(complex));
  getmem(p31,nUns*nUns*sizeof(complex));
  getmem(p32,nUns*nUns*sizeof(complex));
  getmem(p33,nUns*nUns*sizeof(complex));
end;

procedure GetMemoryFordUdI(var p11,p12,p21,p22,p31,p32: pComplex;nUns: integer);
begin
  getmem(p11,nUns*nUns*sizeof(complex));
  getmem(p12,nUns*nUns*sizeof(complex));
  getmem(p21,nUns*nUns*sizeof(complex));
  getmem(p22,nUns*nUns*sizeof(complex));
  getmem(p31,nUns*nUns*sizeof(complex));
  getmem(p32,nUns*nUns*sizeof(complex));
end;

procedure NetEquations2(pSet: TpCompSet; var pVrhs: pComplex; nnod: integer; var errcom: boolean);
{get memory and construct equations for solving the network
 pSet    pointer to the common
 pVrhs   temporary right hand side
 nnod    number of nodes}
var
  freq : single;
begin
  freq := pSet^.appl_hz;                      {frequency}
  ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
  {reserve memory for the neg.seq matrix}
  getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
  clearCmat(pNmat,nnod);
  getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
  clearIvec(pIndexN, nnod);
  {reserve memory for the zero seq matrix}
  getmem(pZmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
  clearCmat(pZmat,nnod);
  getmem(pIndexZ,nnod*Sizeof(integer));       {for index array, neg seq.}
  clearIvec(pIndexZ, nnod);
  getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
  {construct the extended admittance matrix}
  ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, normal,false, errcom);
  {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
  PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
  {copy the matrix to zero seq. matrix, convert to correspond to zero seq}
  PosToZeroMat(longint(pSet), pAdmat,pZmat, freq, 0.0, nnod);
end;


procedure NetEquations(pSet: TpCompSet; var pVrhs: pComplex; nnod: integer; var errcom: boolean);
{get memory and construct equations for solving the network
 pSet    pointer to the common
 pVrhs   temporary right hand side
 nnod    number of nodes}
var
  freq : single;
begin
  freq := pSet^.appl_hz;                      {frequency}
  ReserveMat(nnod,pAdmat,pWs,pRhs,pIndex);
  {reserve memory for the neg.seq matrix}
  getmem(pNmat,nnod*nnod*SizeOf(complex));    {reserve matrix for neg seq.}
  clearCmat(pNmat,nnod);
  getmem(pIndexN,nnod*Sizeof(integer));       {for index array, neg seq.}
  clearIvec(pIndexN, nnod);
  getmem(pVrhs,nnod*SizeOf(complex));         {for temporary right hand side}
  {construct the extended admittance matrix}
  ConstrMatr(pSet, pAdmat, pRhs, nnod, freq, Steady, normal,false, errcom);
  {copy the matrix to neg seq. matrix, convert to correspond to neg. seq}
  PosToNegMat(pSet, pAdmat,pNmat, nnod, 1.0);
end;

procedure PutNoLoadVoltages(pVrhs,pU0: pComplex; pList: TpList; nnod: integer);
{get the solved voltages from the vector pVrhs and save them to pU0
pVrhs   solved voltages at nodes
pU0     pointer to voltages
pList   pointer to pointers to unsymm.loads
nnod    number of nodes}
var
  er       : integer;   {error flag}
  ii       : integer;
  ju       : integer;   {index of voltage node}
  pComp    : TpComp;    {pointer to an unsymm.load}
  pU       : pComplex;  {pointer to a voltage}
  Ua,Ub,Uc : complex;   {voltages at the loads for phases a,b,c}
begin
  pU := pU0;
  for ii := 0 to pList^.count-1 do begin
    pComp := pList^[ii];        {pointer to an unsymm.load}
    pComp^.GetNode(1,ju,er);
    getCvec(pVrhs,ju,nnod,Ua);  {phase a voltage}
    Ub := Cmult(emifii,Ua);     {phase b voltage}
    Uc := Cmult(eifii ,Ua);     {phase c voltage}
    pU^:= Ua;
    inc(pU);
    pU^:= Ub;
    inc(pU);
    pU^:= Uc;
    inc(pU);
  end;
end;

procedure GetNoLoadVoltages(pU0: pComplex; ii: integer; var Ua0,Ub0,Uc0: complex);
{get the no load phase voltages for the unsymm.load number ii
 pU0  pointer to the voltages
 ii   index of the unsymm.load
 Ua0,Ub0,Uc0  the no load voltages}
var
  pU :pComplex;  {pointer to a voltage}
begin
  pU := pU0;
  inc(pu,3*(ii-1));
  Ua0 := pU^;
  inc(pU);
  Ub0 := pU^;
  inc(pU);
  Uc0 := pU^;
end;

procedure PutdUdI(ii,jj: integer; k1,k2,k3: complex; pa,pb,pc : pComplex; nUns: integer);
{save the derivatives dU/dI to locations reserved for them
 Coefficients have been calculated using the test current 1 (Ix)
 ii           index of unsymm.load at which test currents injected, ii = 1..nUns
 jj           index of unsymm.load at which dU/dI calculated, jj = 1..nUns
 k1,k2,k3     the derivatives (dUjj/dIii) for phases a,b,c
 pa,pb,pc  pointers to the reserved memory
 nUns         number of unsymm.loads}
var
  p1,p2,p3  : pComplex;  {pointers to coefficients}
begin
  p1 := pa;
  inc (p1,(ii-1)*nUns+jj-1);
  p1^:= k1;
  p2 := pb;
  inc (p2,(ii-1)*nUns+jj-1);
  p2^:= k2;
  p3 := pc;
  inc (p3,(ii-1)*nUns+jj-1);
  p3^:= k3;
end;

procedure GetdUdI(ii,jj: integer; var k1,k2,k3: complex; pa,pb,pc : pComplex; nUns: integer);
{get the derivatives dU/dI from locations reserved for them
 Coefficients have been calculated using the test current 1 (Ix)
 ii           index of unsymm.load at which test currents injected, ii = 1..nUns
 jj           index of unsymm.load at which dU/dI calculated, jj = 1..nUns
 k1,k2,k3     the derivatives (dUjj/dIii) for phases a,b,c
 pa,pb,pc  pointers to the reserved memory
 nUns         number of unsymm.loads}
var
  p1,p2,p3  : pComplex;  {pointers to coefficinets}
begin
  p1 := pa;
  inc (p1,(ii-1)*nUns+jj-1);
  k1 := p1^;
  p2 := pb;
  inc (p2,(ii-1)*nUns+jj-1);
  k2 := p2^;
  p3 := pc;
  inc (p3,(ii-1)*nUns+jj-1);
  k3 := p3^;
end;

procedure CalculatedUdI2(pList: TpList; Ix0,Iy0,Iz0: array of complex;
                        p11,p21,p31,p12,p22,p32,p13,p23,p33: pComplex; nnod : integer);
{inject (or drain) test currents, to determine dU/dI, two different test currents
 for each unsymm.load
 pList    list of pointers to the unsymm.loads
 Ix0,Iy0  arrays [0..2] of the test currents
 p11..p33 pointers to derivatives dU/dI
          p11,p21,p31  phases a,b,c for test current 1
          p12,p22,p32  phases a,b,c for test current 2
          p13,p23,p33  phases a,b,c for test current 3
 nnod     number of nodes}
var
  er       : integer;    {error flag}
  Ia,Ib,Ic : complex;    {phase currents}
  Ip,Im,Iz : complex;    {sequence currents}
  ii,jj    : integer;    {index}
  it       : integer;    {index for the test currents}
  iu,ju    : integer;    {upstream (voltage) node}
  k1,k2,k3 : complex;    {voltages = derivatives}
  nUns     : integer;    {number of unsymm.loads}
  pComp    : TpComp;     {pointer to an unsymm.load}
  pNrhs    : pComplex;   {right hand side for neg.seq}
  pPrhs    : pComplex;   {right hand side for pos.seq}
  pUns     : TpComp;     {pointer to another unsymm.load}
  pZrhs    : pComplex;   {right hand side for zero seq}
  Up,Un,Uz : complex;    {sequence voltages}
begin
  getmem(pPrhs,nnod*sizeof(complex));         {right hand side for pos.seq}
  getmem(pNrhs,nnod*sizeof(complex));         {right hand side for neg.seq}
  getmem(pZrhs,nnod*sizeof(complex));         {right hand side for zero seq}
  nUns := pList^.count;                       {number of unsymm.loads}
  for ii := 0 to pList^.count-1 do begin
    pComp := pList^[ii];                      {pointer to (next) unsymm.load}
    pComp^.GetNode(1,iu,er);                  {voltage node}
    for it := 1 to 3 do begin
      if (it = 1) then begin                        {the first test current}
        Ia := Ix0[0];                               {current of phase a}
        Ib := Ix0[1];                               {current of phase b}
        Ic := Ix0[2];                               {current of phase c}
      end else if (it = 2) then begin               {the second test current}
        Ia := Iy0[0];                               {current of phase a}
        Ib := Iy0[1];                               {current of phase b}
        Ic := Iy0[2];                               {current of phase c}
      end else if (it = 3) then begin               {the third test current}
        Ia := Iz0[0];                               {current of phase a}
        Ib := Iz0[1];                               {current of phase b}
        Ic := Iz0[2];                               {current of phase c}
      end;
      PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);                {sequence currents}
      clearCvec(pPrhs,nnod);                        {clear rhs}
      putCvec(pPrhs,iu,nnod,Ip);                    {current to rhs, pos.seq. net}
      CLUbacksub(pAdmat,nnod,pIndex,pPrhs);         {backsubstitute = solve pos.seq.net}
      clearCvec(pNrhs,nnod);                        {clear rhs}
      putCvec(pNrhs,iu,nnod,Im);                    {inject neg.seq.current}
      CLUbacksub(pNmat,nnod,pIndexN,pNrhs);         {backsubst. = solve neg.seq.net}
      clearCvec(pZrhs,nnod);                        {clear rhs}
      putCvec(pZrhs,iu,nnod,Iz);                    {inject neg.seq.current}
      CLUbacksub(pZmat,nnod,pIndexZ,pZrhs);         {backsubst. = solve neg.seq.net}
{Iz on nolla kun it = 1 tai 2, muut nollia kun it = 3}
      {get the voltages at each unsymm.load = coefficients}
      for jj := 0 to pList^.count-1 do begin
        pUns := pList^[jj];
        pUns^.GetNode(1,ju,er);                     {voltage node}
        getCvec(pPrhs,ju,nnod,Up);                  {pos.seq.voltage at load}
        getCvec(pNrhs,ju,nnod,Un);                  {neg.seq.voltage at load}
        getCvec(pZrhs,ju,nnod,Uz);                  {neg.seq.voltage at load}
        {voltages = coefficients}
        SeqToPhase(Up,Un,Uz,k1,k2,k3);              {phase voltages at the load = coefficients}
        {positive current is inwards (is it?)}
        k1 := cplx(-k1.x,-k1.y);
        k2 := cplx(-k2.x,-k2.y);
        k3 := cplx(-k3.x,-k3.y);
        {save coefficinets}
        if (it = 1) then begin                         {the first test currents}
          PutdUdI(ii+1,jj+1,k1,k2,k3,p11,p21,p31,nUns);
        end else if (it = 2) then begin                {the second test currents}
          PutdUdI(ii+1,jj+1,k1,k2,k3,p12,p22,p32,nUns);
        end else begin                                 {the third test currents}
          PutdUdI(ii+1,jj+1,k1,k2,k3,p13,p23,p33,nUns);
        end;
      end;
    end;
  end;
  freemem(pPrhs,nnod*sizeof(complex));
  freemem(pNrhs,nnod*sizeof(complex));
  freemem(pZrhs,nnod*sizeof(complex));
end;

procedure CalculatedUdI(pList: TpList; Ix0,Iy0: array of complex;
                        p11,p21,p31,p12,p22,p32: pComplex; nnod : integer);
{inject (or drain) test currents, to determine dU/dI, two different test currents
 for each unsymm.load
 pList    list of pointers to the unsymm.loads
 Ix0,Iy0  arrays [0..2] of the test currents
 p11..p32 pointers to derivatives dU/dI
          p11,p21,p31  phases a,b,c for test current 1
          p12,p22,p32  phases a,b,c for test current 2
 nnod     number of nodes}
var
  er       : integer;    {error flag}
  Ia,Ib,Ic : complex;    {phase currents}
  Ip,Im,Iz : complex;    {sequence currents}
  ii,jj    : integer;    {index}
  it       : integer;    {index for the test currents}
  iu,ju    : integer;    {upstream (voltage) node}
  k1,k2,k3 : complex;    {voltages = derivatives}
  nUns     : integer;    {number of unsymm.loads}
  pComp    : TpComp;     {pointer to an unsymm.load}
  pNrhs    : pComplex;   {right hand side for neg.seq}
  pPrhs    : pComplex;   {right hand side for pos.seq}
  pUns     : TpComp;     {pointer to another unsymm.load}
  Up,Un,Uz : complex;    {sequence voltages}
begin
  getmem(pPrhs,nnod*sizeof(complex));         {right hand side for pos.seq}
  getmem(pNrhs,nnod*sizeof(complex));         {right hand side for neg.seq}
  nUns := pList^.count;                       {number of unsymm.loads}
  for ii := 0 to pList^.count-1 do begin
    pComp := pList^[ii];                      {pointer to (next) unsymm.load}
    pComp^.GetNode(1,iu,er);                  {voltage node}
    for it := 1 to 2 do begin
      if (it = 1) then begin                        {the first test current}
        Ia := Ix0[0];                               {current of phase a}
        Ib := Ix0[1];                               {current of phase b}
        Ic := Ix0[2];                               {current of phase c}
      end else if (it = 2) then begin               {the second test current}
        Ia := Iy0[0];                               {current of phase a}
        Ib := Iy0[1];                               {current of phase b}
        Ic := Iy0[2];                               {current of phase c}
      end;
      PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);                {sequence currents}
      clearCvec(pPrhs,nnod);                        {clear rhs}
      putCvec(pPrhs,iu,nnod,Ip);                    {current to rhs, pos.seq. net}
      CLUbacksub(pAdmat,nnod,pIndex,pPrhs);         {backsubstitute = solve pos.seq.net}
      clearCvec(pNrhs,nnod);                        {clear rhs}
      putCvec(pNrhs,iu,nnod,Im);                    {inject neg.seq.current}
      CLUbacksub(pNmat,nnod,pIndexN,pNrhs);         {backsubst. = solve neg.seq.net}
      {get the voltages at each unsymm.load = coefficients}
      for jj := 0 to pList^.count-1 do begin
        pUns := pList^[jj];
        pUns^.GetNode(1,ju,er);                     {voltage node}
        getCvec(pPrhs,ju,nnod,Up);                  {pos.seq.voltage at load}
        getCvec(pNrhs,ju,nnod,Un);                  {neg.seq.voltage at load}
        Uz := cplx(0,0);                            {no zero seq. current}
        {voltages = coefficients}
        SeqToPhase(Up,Un,Uz,k1,k2,k3);              {phase voltages at the load = coefficients}
        {positive current is inwards (is it?)}
        k1 := cplx(-k1.x,-k1.y);
        k2 := cplx(-k2.x,-k2.y);
        k3 := cplx(-k3.x,-k3.y);
//showmessage(comptostr(k1));
        {save coefficinets}
        if (it = 1) then begin                      {the first test currents}
          PutdUdI(ii+1,jj+1,k1,k2,k3,p11,p21,p31,nUns);
        end else begin                              {the second test currents}
          PutdUdI(ii+1,jj+1,k1,k2,k3,p12,p22,p32,nUns);
        end;
      end;
    end;
  end;
  freemem(pPrhs,nnod*sizeof(complex));
  freemem(pNrhs,nnod*sizeof(complex));
end;

procedure PutCoef(cc: complex; paa: pComplex; i,j: integer; nDim: integer);
{put coefficinet (i,j) to matrix paa}
var
  pa: pComplex;
begin
  if (i > 0) and (i <= nDim) and (j > 0) and (j <= nDim) then begin
    pa := paa;
    inc(pa,nDim*(i-1)+j-1);
    pa^:= cc;
  end;
end;

procedure GetCoef(var cc: complex; paa: pComplex; i,j: integer; nDim: integer);
{put coefficinet (i,j) to matrix paa}
var
  pa: pComplex;
begin
  if (i > 0) and (i <= nDim) and (j > 0) and (j <= nDim) then begin
    pa := paa;
    inc(pa,nDim*(i-1)+j-1);
    cc := pa^;
  end;
end;

procedure EquationsForCurrents2(pList: TpList; Ix0,Iy0,Iz0: array of complex;
                               paa,p11,p21,p31,p12,p22,p32,p13,p23,p33: pComplex);
{construct the equations for the currents at the unsymm.loads
 pList       pointer to the list of unsymm.loads
 Ix0,Iy0,Iz0 arrays [0..2] of the test currents
 paa         pointer to the coeff.matrix
 p11..p33    pointers to derivatives dU/dI
           p11,p21,p31  phases a,b,c for test current 1
           p12,p22,p32  phases a,b,c for test current 2
           p13,p23,p33  phases a,b,c for test current 2}
var
  a11,a12,a13 : complex;  {"diagonal" elements}
  a21,a22,a23 : complex;  {more diagonal elements}
  a31,a32,a33 : complex;  {more diagonal elements}
  cpu,cpv     : complex;  {temporary}
  cpw,cpx     : complex;  {temporary}
  cpy,cpz     : complex;
  ii,jj       : integer;  {indices}
  k1,k2,k3    : complex;  {derivatives dU/dI for phases a,b,c wrt test current 1}
  l1,l2,l3    : complex;  {derivatives dU/dI for phases a,b,c wrt test current 2}
  m1,m2,m3    : complex;  {derivatives dU/dI for phases a,b,c wrt test current 3}
  mult        : single;   {multiplier (of load!)}
  n3          : integer;  {no of equations = dim. of matrix}
  nUns        : integer;  {number of unsymm.loads}
  pa          : pComplex; {pointer to an element in paa}
  pUnsym      : TpComp;   {pointer to an umsymm.load}
  Za,Zb,Zc    : complex;  {impedance of load, phases a,b,c}
  Zg          : complex;  {grounding impedance}
begin
  nUns := pList^.count;                 {number of unsymm.loads}
  n3 := 3*nUns;                         {no of equations}
  for ii := 1 to nUns do begin          {eq. for all loads}
    pUnsym := pList^[ii-1];             {pointer to an unsymm.load}
    {get impedaces of this load}
    pUnsym^.GetPar(gUnsym_Ra{1},Za.x);  {resistance in phase a}
    pUnsym^.GetPar(gUnsym_Xa{2},Za.y);  {reactance in phase a}
    pUnsym^.GetPar(gUnsym_Rb{3},Zb.x);  {resistance in phase b}
    pUnsym^.GetPar(gUnsym_Xb{4},Zb.y);  {reactance in phase b}
    pUnsym^.GetPar(gUnsym_Rc{5},Zc.x);  {resistance in phase c}
    pUnsym^.GetPar(gUnsym_Xc{6},Zc.y);  {reactance in phase c}
    pUnsym^.GetPar(gUnsym_Rg{7},Zg.x);  {grounding resistance}
    pUnsym^.GetPar(gUnsym_Xg{8},Zg.y);  {grounding reactance}
    pUnsym^.GetPar(gUnsym_m    ,mult);  {multiplier}
    {multiply load = divide impedance}
    if (mult <= 0) then mult := 1;
    if (mult <> 1) then begin
      Za := CrMult(1/mult,Za);
      Zb := CrMult(1/mult,Zb);
      Zc := CrMult(1/mult,Zc);
    end;
    {calculate...}
    for jj := 1 to nUns do begin
      GetdUdI(ii,jj,k1,k2,k3,p11,p21,p31,nUns);  {derivatives dU/dI wrt test current 1}
      GetdUdI(ii,jj,l1,l2,l3,p12,p22,p32,nUns);  {derivatives dU/dI wrt test current 2}
      GetdUdI(ii,jj,m1,m2,m3,p13,p23,p33,nUns);  {derivatives dU/dI wrt test current 3}
      cpu := Csub(k1,k2);
      cpv := Csub(l1,l2);
      cpw := Csub(m1,m2);
      PutCoef(cpu,paa,3*ii-2,3*jj-2,n3);
      PutCoef(cpv,paa,3*ii-2,3*jj-1,n3);
      PutCoef(cpw,paa,3*ii-2,3*jj  ,n3);
      cpu := Csub(k1,k3);
      cpv := Csub(l1,l3);
      cpw := Csub(m1,m3);
      PutCoef(cpu,paa,3*ii-1,3*jj-2,n3);
      PutCoef(cpv,paa,3*ii-1,3*jj-1,n3);
      PutCoef(cpw,paa,3*ii-1,3*jj  ,n3);
      PutCoef(k1 ,paa,3*ii  ,3*jj-2,n3);
      PutCoef(l1 ,paa,3*ii  ,3*jj-1,n3);
      PutCoef(m1 ,paa,3*ii  ,3*jj  ,n3);
    end;
    {complete "diagonal", because of the impedances}
    GetCoef(a11,paa,3*ii-2,3*ii-2,n3);
    GetCoef(a12,paa,3*ii-2,3*ii-1,n3);
    GetCoef(a13,paa,3*ii-2,3*ii  ,n3);
    GetCoef(a21,paa,3*ii-1,3*ii-2,n3);
    GetCoef(a22,paa,3*ii-1,3*ii-1,n3);
    GetCoef(a23,paa,3*ii-1,3*ii  ,n3);
    GetCoef(a31,paa,3*ii  ,3*ii-2,n3);
    GetCoef(a32,paa,3*ii  ,3*ii-1,n3);
    GetCoef(a33,paa,3*ii  ,3*ii  ,n3);
    cpu := Csub(Cmult(Zb,Ix0[1]),Cmult(Za,Ix0[0]));
    cpv := Csub(Cmult(Zb,Iy0[1]),Cmult(Za,Iy0[0]));
    cpw := Csub(Cmult(Zb,Iz0[1]),Cmult(Za,Iz0[0]));
    a11 := Cadd(a11,cpu);
    a12 := Cadd(a12,cpv);
    a13 := Cadd(a13,cpw);
    PutCoef(a11,paa,3*ii-2,3*ii-2,n3);
    PutCoef(a12,paa,3*ii-2,3*ii-1,n3);
    PutCoef(a13,paa,3*ii-2,3*ii  ,n3);
    cpu := Csub(Cmult(Zc,Ix0[2]),Cmult(Za,Ix0[0]));
    cpv := Csub(Cmult(Zc,Iy0[2]),Cmult(Za,Iy0[0]));
    cpw := Csub(Cmult(Zc,Iz0[2]),Cmult(Za,Iz0[0]));
    a21 := Cadd(a21,cpu);
    a22 := Cadd(a22,cpv);
    a23 := Cadd(a23,cpw);
    PutCoef(a21,paa,3*ii-1,3*ii-2,n3);
    PutCoef(a22,paa,3*ii-1,3*ii-1,n3);
    PutCoef(a23,paa,3*ii-1,3*ii  ,n3);
    cpx := Cmult(Za,Ix0[0]);
    cpy := Cmult(Za,Iy0[0]);
    cpz := Cmult(Za,Iz0[0]);
    cpu := Cmult(Zg,cplx(-Ix0[0].x-Ix0[1].x-Ix0[2].x,-Ix0[0].y-Ix0[1].y-Ix0[2].y));
    cpu := Csub(cpu,cpx);
    cpv := Cmult(Zg,cplx(-Iy0[0].x-Iy0[1].x-Iy0[2].x,-Iy0[0].y-Iy0[1].y-Iy0[2].y));
    cpv := Csub(cpv,cpy);
    cpw := Cmult(Zg,cplx(-Iz0[0].x-Iz0[1].x-Iz0[2].x,-Iz0[0].y-Iz0[1].y-Iz0[2].y));
    cpw := Csub(cpw,cpz);
    a31 := Cadd(a31,cpu);
    a32 := Cadd(a32,cpv);
    a33 := Cadd(a33,cpw);
    PutCoef(a31,paa,3*ii  ,3*ii-2,n3);
    PutCoef(a32,paa,3*ii  ,3*ii-1,n3);
    PutCoef(a33,paa,3*ii  ,3*ii  ,n3);
  end;
end;

procedure EquationsForCurrents(pList: TpList; Ix0,Iy0: array of complex;
                               paa,p11,p21,p31,p12,p22,p32: pComplex);
{construct the equations for the currents at the unsymm.loads
 pList     pointer to the list of unsymm.loads
 Ix0,Iy0   arrays [0..2] of the test currents
 paa       pointer to the coeff.matrix
 p11..p32  pointers to derivatives dU/dI
           p11,p21,p31  phases a,b,c for test current 1
           p12,p22,p32  phases a,b,c for test current 2}
var
  a11,a12  : complex;  {"diagonal" elements}
  a21,a22  : complex;  {more diagonal elements}
  cpu,cpv  : complex;  {temporary}
  ii,jj    : integer;  {indices}
  k1,k2,k3 : complex;  {derivatives dU/dI for phases a,b,c wrt test current 1}
  l1,l2,l3 : complex;  {derivatives dU/dI for phases a,b,c wrt test current 2}
  n2       : integer;  {no of equations = dim. of matrix}
  nUns     : integer;  {number of unsymm.loads}
  pa       : pComplex; {pointer to an element in paa}
  pUnsym   : TpComp;   {pointer to an umsymm.load}
  Za,Zb,Zc : complex;  {impedance of load, phases a,b,c}
begin
  nUns := pList^.count;                 {number of unsymm.loads}
  n2 := 2*nUns;                         {no of equations}
  for ii := 1 to nUns do begin          {eq. for all loads}
    pUnsym := pList^[ii-1];             {pointer to an unsymm.load}
    {get impedaces of this load}
    pUnsym^.GetPar(gUnsym_Ra{1},Za.x);  {resistance in phase a}
    pUnsym^.GetPar(gUnsym_Xa{2},Za.y);  {reactance in phase a}
    pUnsym^.GetPar(gUnsym_Rb{3},Zb.x);  {resistance in phase b}
    pUnsym^.GetPar(gUnsym_Xb{4},Zb.y);  {reactance in phase b}
    pUnsym^.GetPar(gUnsym_Rc{5},Zc.x);  {resistance in phase c}
    pUnsym^.GetPar(gUnsym_Xc{6},Zc.y);  {reactance in phase c}
    for jj := 1 to nUns do begin
      GetdUdI(ii,jj,k1,k2,k3,p11,p21,p31,nUns);  {derivatives dU/dI wrt test current 1}
      GetdUdI(ii,jj,l1,l2,l3,p12,p22,p32,nUns);  {derivatives dU/dI wrt test current 2}
      cpu := Csub(k1,k2);
      cpv := csub(l1,l2);
      PutCoef(cpu,paa,2*ii-1,2*jj-1,n2);
      PutCoef(cpv,paa,2*ii-1,2*jj  ,n2);
      cpu := Csub(k1,k3);
      cpv := Csub(l1,l3);
      PutCoef(cpu,paa,2*ii,2*jj-1  ,n2);
      PutCoef(cpv,paa,2*ii,2*jj    ,n2);
    end;
    {complete "diagonal", because of the impedances}
    GetCoef(a11,paa,2*ii-1,2*ii-1,n2);
    GetCoef(a12,paa,2*ii-1,2*ii  ,n2);
    GetCoef(a21,paa,2*ii  ,2*ii-1,n2);
    GetCoef(a22,paa,2*ii  ,2*ii  ,n2);
    cpu := Csub(Cmult(Zb,Ix0[1]),Cmult(Za,Ix0[0]));
    cpv := Csub(Cmult(Zb,Iy0[1]),Cmult(Za,Iy0[0]));
    a11 := Cadd(a11,cpu);
    a12 := Cadd(a12,cpv);
    PutCoef(a11,paa,2*ii-1,2*ii-1,n2);
    PutCoef(a12,paa,2*ii-1,2*ii  ,n2);
    cpu := Csub(Cmult(Zc,Ix0[2]),Cmult(Za,Ix0[0]));
    cpv := Csub(Cmult(Zc,Iy0[2]),Cmult(Za,Iy0[0]));
    a21 := Cadd(a21,cpu);
    a22 := Cadd(a22,cpv);
    PutCoef(a21,paa,2*ii  ,2*ii-1,n2);
    PutCoef(a22,paa,2*ii  ,2*ii  ,n2);
  end;
end;

procedure RhsForCurrents2(pbb,pU0: pComplex; nUns: integer);
{constructs the right hand side for the current equations
 pbb   pointer to the right hand side
 pU0   pointer to the voltages without the load
 nUns  number of unsymm.loads}
var
  cpu,cpv  : complex;   {temporary}
  ii       : integer;   {index}
  pb       : pComplex;  {pointer to an element on the rhs}
  Ua,Ub,Uc : complex;   {the voltages without the load}
begin
  pb := pbb;
  for ii := 1 to nUns do begin
    GetNoLoadVoltages(pU0,ii,Ua,Ub,Uc);
    cpu := Csub(Ub,Ua);
    cpv := Csub(Uc,Ua);
    pb^:= cpu;
    inc(pb);
    pb^:= cpv;
    inc(pb);
    pb^:= cplx(-Ua.x,-Ua.y);
    inc(pb);
  end;
end;

procedure RhsForCurrents(pbb,pU0: pComplex; nUns: integer);
{constructs the right hand side for the current equations
 pbb   pointer to the right hand side
 pU0   pointer to the voltages without the load
 nUns  number of unsymm.loads}
var
  cpu,cpv  : complex;   {temporary}
  ii       : integer;   {index}
  pb       : pComplex;  {pointer to an element on the rhs}
  Ua,Ub,Uc : complex;   {the voltages without the load}
begin
  pb := pbb;
  for ii := 1 to nUns do begin
    GetNoLoadVoltages(pU0,ii,Ua,Ub,Uc);
    cpu := Csub(Ub,Ua);
    cpv := Csub(Uc,Ua);
    pb^:= cpu;
    inc(pb);
    pb^:= cpv;
    inc(pb);
  end;
end;

procedure UnsymmLoad2(ipSet: longint);
{solve the net, when there are unsymmetrical loads in the net
 ipSet  pointer to the "common", as integer}
const
  Ix0 : array[1..3] of complex = ((x:1;y:0),(x: 1;y:0),(x:-2;y:0));  {first test current}
  Iy0 : array[1..3] of complex = ((x:1;y:0),(x:-1;y:0),(x: 0;y:0));  {second test current}
  Iz0 : array[1..3] of complex = ((x:1;y:0),(x: 1;y:0),(x: 1;y:0));  {third test current}
//  Ix0 : array[1..3] of complex = ((x:1;y:0),(x: 0;y:0),(x: 0;y:0));  {first test current}
//  Iy0 : array[1..3] of complex = ((x:0;y:0),(x: 1;y:0),(x: 0;y:0));  {second test current}
//  Iz0 : array[1..3] of complex = ((x:0;y:0),(x: 0;y:0),(x: 1;y:0));  {third test current}
var
  er          : integer;     {error flag}
  cpu,cpv     : complex;     {temporary}
  cpw         : complex;     {temporary}
  Dodd,DoddN  : single;      {keeps track on permutations}
  DoddZ       : single;      {keeps track on permutations}
  errcom      : boolean;     {error flag}
  freq        : single;      {frequency}
  Ia,Ib,Ic    : complex;     {phase currents}
  ii          : integer;     {index}
  Ip,Im,Iz    : complex;     {sequence currents}
  jc,ju       : integer;     {current node, upstream node}
  mult        : single;      {multiplier (of Load!)}
  nCurrT      : integer;     {number of current transformers}
  nGenMot,nRelay,nFault,nAsyMot,nSource,nPQRel : integer;
  nn,nn2      : integer;     {dimensions}
  nnod        : integer;     {number of nodes}
  nUns        : integer;     {counts unsymmetrical loads}
  nvar        : integer;     {number of variables}
  paa         : pComplex;    {pointer to coeff. of equation}
  pbb         : pComplex;    {pointer to rhs, used also to save Ip,Im after solving}
  pin         : pInteger;    {for row exchanges}
  p11,p12,p13 : pcomplex;    {pointers to coefficients, dU/dI, phase a}
  p21,p22,p23 : pcomplex;    {pointers to coefficients, dU/dI, phase b}
  p31,p32,p33 : pcomplex;    {pointers to coefficients, dU/dI, phase c}
  pCurrT      : TpCurrT;     {pointer to object for current trafos}
  pList       : TpList;      {pointers to the unsymmetrical loads}
  pSet        : TpCompSet;   {pointer to the common}
  pU0         : pComplex;    {pointer to the voltages without the loads}
  pUnsym      : TpComp;      {pointer to the unsymm.load}
  pVrhs       : pComplex;    {temporary right hand side}
  pWr         : pComplex;    {working space}
  Sngl,SnglN  : Integer;     {flag for singular matrix}
  SnglZ       : integer;
  Tout,ToutN  : complex;     {test output from matrix decomposition}
  ToutZ       : complex;
  Va          : complex;     {voltage in phase a}
  Vn          : complex;     {voltage at the neutral point}
  Za          : complex;     {impedance in phase a}
  Zg          : complex;     {grounding impedance}
begin
  pSet := TpCompSet(ipSet);
  freq := pSet^.appl_Hz;
  pCurrT := nil;
  if (pSet^.nUns_load > 0) then begin     {there are unsymmetrical load(s)}
    Imagoo.NodeButClick(nil);             {determine nodes }
    nnod := pSet^.appl_nn;                {copy number of nodes}
    new(pList);
    pList^ := TList.create;               {list for pointers to the unsymm.loads}
    {search the unsymmetrical loads, and fill the list}
    ListUnsymm(pSet, pList, nUns);
    nn := 3*nUns;                          {number of unknowns}
    nn2 := nn*nn;
    {get and clear memory for equations}
    paa := nil;                                  {initialize..}
    pbb := nil;                                  {..to keep the compiler happy}
    GetMemoryForEquations2(paa,pbb,pWr,pin,nUns);
    {get memory for dU/dI, all phases, all loads}
    p11 := nil; p12 := nil; p13 := nil;
    p21 := nil; p22 := nil; p23 := nil;
    p31 := nil; p32 := nil; p33 := nil;
    GetMemoryFordUdI2(p11,p12,p13,p21,p22,p23,p31,p32,p33,nUns);
    getmem(pU0,3*nUns*sizeof(complex));          {get memory for voltages without loads}
    {reserve mem for curr.trafos and relays (if there is any)}
    pSet^.GetMemForCurrTrAndRelays;
    {reserve and initialize memory for matrices, construct the equations}
    pVrhs := nil;                                {initialize, to keep the compiler happy}
    NetEquations2(pSet, pVrhs, nnod, errcom);
    {solve voltages, but only if no unknown components}
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex,Dodd,Sngl,Tout);     {decompose first}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg.seq}
        CLUdecomp(pZmat ,pWs,nnod,pIndexZ,DoddZ,SnglZ,ToutZ); {decompose, zero seq}
        {first, solve the net without the unsymm. loads}
        copyrhs(pRhs,pVrhs,nnod);             {copy the rhs to temporary, it must not be destroyed}
        CLUbacksub(pAdmat,nnod,pIndex,pVrhs);                 {backsubstitute = solve net}
        {get and save voltage at the location of the unsymm.loads, but without the loads}
        PutNoLoadVoltages(pVrhs,pU0, pList, nnod);
        {calculate the derivatives dU/dI = d(Voltage at load)/d(injected current at load)}
        CalculatedUdI2(pList,Ix0,Iy0,Iz0,p11,p21,p31,p12,p22,p32,p13,p23,p33,nnod);
        {construct equations for the currents at the loads}
        EquationsForCurrents2(pList,Ix0,Iy0,Iz0,paa,p11,p21,p31,p12,p22,p32,p13,p23,p33);
        RhsForCurrents2(pbb,pU0,nUns);                                     {right hand side}
        {solve the equations}
        CLUdecomp(paa,pWr,3*nUns,pIn,Dodd,Sngl,Tout);        {decompose}
        CLUbacksub(paa,3*nUns,pIn,pbb);                      {backsubstitute = solve currents}
        {currents solved, solve the whole net with these currents}
        clearCvec(pVrhs,nnod);                            {clear rhs for neg.seq}
        clearCvec(pWs,nnod);                              {clar rhs for zero seq}
        for ii := 1 to nUns do begin
          pUnsym := pList^[ii-1];
          pUnsym^.GetNode(1,ju,er);                       {upstream (voltage) node}
          {phase currents. note the positive direction....!?}
          getCvec(pbb,ii*3-2,nUns*3,cpu);
          getCvec(pbb,ii*3-1,nUns*3,cpv);
          getCvec(pbb,ii*3  ,nUns*3,cpw);
          cpu := cplx(-cpu.x,-cpu.y);
          cpv := cplx(-cpv.x,-cpv.y);
          cpw := cplx(-cpw.x,-cpw.y);
          Ia := Cadd(Cmult(cpu,Ix0[1]),Cmult(cpv,Iy0[1]));
          Ia := Cadd(Ia,Cmult(cpw,Iz0[1]));
          Ib := Cadd(Cmult(cpu,Ix0[2]),Cmult(cpv,Iy0[2]));
          Ib := Cadd(Ib,Cmult(cpw,Iz0[2]));
          Ic := Cadd(Cmult(cpu,Ix0[3]),Cmult(cpv,Iy0[3]));
          Ic := Cadd(Ic,Cmult(cpw,Iz0[3]));
          PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);                  {sequence currents}
          getCvec(pRhs,ju,nnod,cpu);
          cpu := Cadd(cpu,Ip);
          putCvec(pRhs,ju,nnod,cpu);                      {current to original rhs, pos.seq. net}
          getCvec(pVrhs,ju,nnod,cpv);
          cpv := Cadd(cpv,Im);
          putCvec(pVrhs,ju,nnod,cpv);                     {inject neg.seq.current}
          getCvec(pWs  ,ju,nnod,cpw);
          cpw := Cadd(cpw,Iz);
          putCvec(pWs,ju,nnod,cpw);                     {inject zero seq.current}
          {save the currents (back) to pbb, needed below}
          putCvec(pbb,ii*3-2,nUns*3,Ip);
          putCvec(pbb,ii*3-1,nUns*3,Im);
          putCvec(pbb,ii*3  ,nUns*3,Iz);
        end;
        CLUbacksub(pAdmat,nnod,pIndex,pRhs);              {backsubstitute = solve pos.seq.net}
        CLUbacksub(pNmat,nnod,pIndexN,pVrhs);             {backsubst. = solve neg.seq.net}
        CLUbacksub(pZmat,nnod,pIndexZ,pWs);               {backsubst. = solve neg.seq.net}
        {complete the solution, put currents back}
        for ii := 1 to nUns do begin
          pUnsym := pList^[ii-1];
          pUnsym^.GetNode(2,jc,er);                       {current "node"}
          getCvec(pbb,ii*3-2,nUns*3,Ip);                  {saved above}
          getCvec(pbb,ii*3-1,nUns*3,Im);
          getCvec(pbb,ii*3  ,nUns*3,Iz);
          putCvec(pRhs ,jc,nnod,Ip);                      {complete the solution with the current}
          putCvec(pVrhs,jc,nnod,Im);                      {complete the solution with the current}
          putCvec(pWs  ,jc,nnod,Iz);                      {complete the solution with the current}
        end;
        {Results to components. pos.seq. results in pRhs, neg.seq results in pVrhs}
        ClearAllVar(pSet);                                {clear initially}
        ResultsToComp(pSet, nnod, freq, pWs,3,ForBoth);   {neg.seq to comp., as first set of var.}
        CopyAllVar(pSet,1,3);                             {copy to third set of var.}
        ResultsToComp(pSet, nnod, freq, pVrhs,2,ForBoth); {neg.seq to comp., as first set of var.}
        CopyAllVar(pSet,1,2);                             {copy to second set of var. in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pRhs ,1,ForBoth); {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);                           {from seq to phase, in Twoshort.pas}
        {voltage of the neutral point}
        for ii := 0 to pList^.count-1 do begin
          pUnsym := pList^[ii];                           {pointer to an unsymm.load}
          pUnsym^.GetNvar(nvar);
          pUnsym^.GetVar(gUnsymm_Vr,Va.x);                {voltage in phase a}
          pUnsym^.GetVar(gUnsymm_Vi,Va.y);
          pUnsym^.GetVar(gUnsymm_Ir,Ia.x);                {current in phase a}
          pUnsym^.GetVar(gUnsymm_Ii,Ia.y);
          pUnsym^.GetPar(gUnsym_Ra{1},Za.x);              {resistance in phase a}
          pUnsym^.GetPar(gUnsym_Xa{2},Za.y);              {reactance in phase a}
          pUnsym^.GetPar(gUnsym_Rg{7},Zg.x);              {grounding resistance}
          pUnsym^.GetPar(gUnsym_Xg{8},Zg.y);              {grounding reactance}
          pUnsym^.GetPar(gUnsym_m    ,mult);  {multiplier}
          {multiply load = divide impedance}
          if (mult <= 0) then mult := 1;
          if (mult <> 1) then Za := CrMult(1/mult,Za);
          {calculate...}
          if (Zg.x = 0) and (Zg.y = 0) then begin         {solidly grounded}
            Vn := cplx(0,0);                              {must be zero}
          end else if (Cabs(Zg) < 1000) then begin
            pUnsym^.GetVar(gUnsymm_Ir+nvar,Ib.x);         {current in phase b}
            pUnsym^.GetVar(gUnsymm_Ii+nvar,Ib.y);
            pUnsym^.GetVar(gUnsymm_Ir+nvar*2,Ic.x);       {current in phase c}
            pUnsym^.GetVar(gUnsymm_Ii+nvar*2,Ic.y);
            cpu := Cadd(Ia,Cadd(Ib,Ic));                  {sum current = neutral current}
            Vn := Cmult(Zg,cpu);
          end else begin                                  {almost not grounded}
            Vn := Csub(Va,Cmult(Ia,Za));                  {neutral voltage}
          end;
          {save same voltage to all phases}
          pUnsym^.GetNvar(nvar);
          pUnsym^.PutVar(gUnsymm_Vnr,Vn.x,'UnsymmLoad2');
          pUnsym^.PutVar(gUnsymm_Vni,Vn.y,'UnsymmLoad2');
          pUnsym^.PutVar(gUnsymm_Vnr+nvar,Vn.x,'UnsymmLoad2');
          pUnsym^.PutVar(gUnsymm_Vni+nvar,Vn.y,'UnsymmLoad2');
          pUnsym^.PutVar(gUnsymm_Vnr+nvar*2,Vn.x,'UnsymmLoad2');
          pUnsym^.PutVar(gUnsymm_Vni+nvar*2,Vn.y,'UnsymmLoad2');
        end;
        {sum current trafos must display the sum current}
{if there are current transformers, create object for operations with current trafos}
CountComp(ipSet,nGenMot,nRelay,nFault,nCurrT,nAsyMot,nSource,nPQRel);
CreateCurrT(ipSet, nCurrT, pSet^.nCalc, pCurrT);
if (pCurrT <> nil) then pCurrT^.SumToOutVar;     {in shortcmp.pas}
        {display relay current using polar coordinates}
        PolarToRelaysAndBuses(pSet);
        {show also the current through the current trafos, if allowed} {in skemproc.pas}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
        {update the eventual result boxes}
        pSet^.UpdateResultBoxes(1);
      finally
        {}
      end;
    end;
    {free the reserved memory from matrices}
    pSet^.FreeMemForCurrTrAndRelays;
    FreeMat(nnod);
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    Freemem(pZmat,nnod*nnod*SizeOf(complex));  {free matrix for zero seq.}
    Freemem(pIndexZ,nnod*Sizeof(integer));     {free index array, zero seq}
    Freemem(pVrhs,nnod*SizeOf(complex));
    freemem(p11,nUns*nUns*sizeof(complex));
    freemem(p12,nUns*nUns*sizeof(complex));
    freemem(p13,nUns*nUns*sizeof(complex));
    freemem(p21,nUns*nUns*sizeof(complex));
    freemem(p22,nUns*nUns*sizeof(complex));
    freemem(p23,nUns*nUns*sizeof(complex));
    freemem(p31,nUns*nUns*sizeof(complex));
    freemem(p32,nUns*nUns*sizeof(complex));
    freemem(p33,nUns*nUns*sizeof(complex));
    freemem(pU0,3*nUns*sizeof(complex));
    freemem(paa,nUns*nUns*9*sizeof(complex));
    freemem(pbb,nUns*3*sizeof(complex));
    freemem(pWr,nUns*3*sizeof(complex));
    freemem(pin,nUns*3*sizeof(integer));
    pList^.free;
    dispose(pList);
    {destroy object for current trafos, if created}
    if (nCurrT > 0) and (pCurrT <> nil) then begin
      pCurrT^.Destr;                 {destroy object for current trafors}
      pCurrT^.Free;
      dispose(pCurrT);
      pCurrT := nil;
    end;
  end;
end;

procedure UnsymmLoad(ipSet: longint);
{solve the net, when there are unsymmetrical loads in the net
 ipSet  pointer to the "common", as integer}
const
  Ix0 : array[1..3] of complex = ((x:1;y:0),(x: 1;y:0),(x:-2;y:0));  {first test current}
  Iy0 : array[1..3] of complex = ((x:1;y:0),(x:-1;y:0),(x: 0;y:0));  {second test current}
var
  er         : integer;     {error flag}
  cpu,cpv    : complex;     {temporary}
  Dodd,DoddN : single;      {keeps track on permutations}
  errcom     : boolean;     {error flag}
  freq       : single;      {frequency}
  Ia,Ib,Ic   : complex;     {phase currents}
  ii         : integer;     {index}
  Ip,Im,Iz   : complex;     {sequence currents}
  jc,ju      : integer;     {current node, upstream node}
  nn,nn2     : integer;     {dimensions}
  nnod       : integer;     {number of nodes}
  nUns       : integer;     {counts unsymmetrical loads}
  nvar       : integer;     {number of variables}
  paa        : pComplex;    {pointer to coeff. of equation}
  pbb        : pComplex;    {pointer to rhs, used also to save Ip,Im after solving}
  pin        : pInteger;    {for row exchanges}
  p11,p12    : pcomplex;    {pointers to coefficients, dU/dI, phase a}
  p21,p22    : pcomplex;    {pointers to coefficients, dU/dI, phase b}
  p31,p32    : pcomplex;    {pointers to coefficients, dU/dI, phase c}
  pList      : TpList;      {pointers to the unsymmetrical loads}
  pSet       : TpCompSet;   {pointer to the common}
  pU0        : pComplex;    {pointer to the voltages without the loads}
  pUnsym     : TpComp;      {pointer to the unsymm.load}
  pVrhs      : pComplex;    {temporary right hand side}
  Sngl,SnglN : Integer;     {flag for singular matrix}
  Tout,ToutN : complex;     {test output from matrix decomposition}
  Vn         : complex;     {voltage at the neutral point}
  Za         : complex;     {impedance in phase a}
begin
  pSet := TpCompSet(ipSet);
  freq := pSet^.appl_Hz;
  if (pSet^.nUns_load > 0) then begin     {there are unsymmetrical load(s)}
//  pSet^.SetApplOp(15);                  {flag for type of calculation}{moved to imagopas.pas}
    Imagoo.NodeButClick(nil);             {determine nodes }
    nnod := pSet^.appl_nn;                {copy number of nodes}
    new(pList);
    pList^ := TList.create;               {list for pointers to the unsymm.loads}
    {search the unsymmetrical loads, and fill the list}
    ListUnsymm(pSet, pList, nUns);
    nn := 2*nUns;                          {number of unknowns}
    nn2 := nn*nn;
    {get and clear memory for equations}
    paa := nil;                                  {initialize..}
    pbb := nil;                                  {..to keep the compiler happy}
    GetMemoryForEquations(paa,pbb,pin,nUns);
    {get memory for dU/dI, all phases, all loads}
    p11 := nil; p12 := nil; p21 := nil; p22 := nil; p31 := nil; p32 := nil;
    GetMemoryFordUdI(p11,p12,p21,p22,p31,p32,nUns);
    getmem(pU0,3*nUns*sizeof(complex));          {get memory for voltages without loads}
    {reserve mem for curr.trafos and relays (if there is any)}
    pSet^.GetMemForCurrTrAndRelays;
    {reserve and initialize memory for matrices, construct the equations}
    pVrhs := nil;                                {initialize, to keep the compiler happy}
    NetEquations(pSet, pVrhs, nnod, errcom);
    {solve voltages, but only if no unknown components}
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod,pIndex,Dodd,Sngl,Tout);     {decompose first}
        CLUdecomp(pNmat ,pWs,nnod,pIndexN,DoddN,SnglN,ToutN); {decompose, neg.seq}
        {first, solve the net without the unsymm. loads}
        copyrhs(pRhs,pVrhs,nnod);             {copy the rhs to temporary, it must not be destroyed}
        CLUbacksub(pAdmat,nnod,pIndex,pVrhs);                 {backsubstitute = solve net}
        {get and save voltage at the location of the unsymm.loads, but without the loads}
        PutNoLoadVoltages(pVrhs,pU0, pList, nnod);
        {calculate the derivatives dU/dI = d(Voltage at load)/d(injected current at load)}
        CalculatedUdI(pList,Ix0,Iy0,p11,p21,p31,p12,p22,p32,nnod);
        {construct equations for the currents at the loads}
        EquationsForCurrents(pList,Ix0,Iy0,paa,p11,p21,p31,p12,p22,p32);  {coeff.matrix}
        RhsForCurrents(pbb,pU0,nUns);                                     {right hand side}
        {solve the equations}
        CLUdecomp(paa,pWs,2*nUns,pIn,Dodd,Sngl,Tout);        {decompose}
        CLUbacksub(paa,2*nUns,pIn,pbb);                      {backsubstitute = solve currents}
        {currents solved, solve the whole net with these currents}
        clearCvec(pVrhs,nnod);                            {clear rhs for neg.seq}
        for ii := 1 to nUns do begin
          pUnsym := pList^[ii-1];
          pUnsym^.GetNode(1,ju,er);                       {upstream (voltage) node}
          {phase currents. note the positive direction....!?}
          getCvec(pbb,ii*2-1,nUns*2,cpu);
          getCvec(pbb,ii*2  ,nUns*2,cpv);
          cpu := cplx(-cpu.x,-cpu.y);
          cpv := cplx(-cpv.x,-cpv.y);
          Ia := Cadd(Cmult(cpu,Ix0[1]),Cmult(cpv,Iy0[1]));
          Ib := Cadd(Cmult(cpu,Ix0[2]),Cmult(cpv,Iy0[2]));
          Ic := Cadd(Cmult(cpu,Ix0[3]),Cmult(cpv,Iy0[3]));
          PhaseToSeq(Ia,Ib,Ic,Ip,Im,Iz);                  {sequence currents}
          getCvec(pRhs,ju,nnod,cpu);
          cpu := Cadd(cpu,Ip);
          putCvec(pRhs,ju,nnod,cpu);                      {current to original rhs, pos.seq. net}
          getCvec(pVrhs,ju,nnod,cpv);
          cpv := Cadd(cpv,Im);
          putCvec(pVrhs,ju,nnod,cpv);                     {inject neg.seq.current}
          {save the currents (back) to pbb, needed below}
          putCvec(pbb,ii*2-1,nUns*2,Ip);
          putCvec(pbb,ii*2  ,nUns*2,Im);
        end;
        CLUbacksub(pAdmat,nnod,pIndex,pRhs);              {backsubstitute = solve pos.seq.net}
        CLUbacksub(pNmat,nnod,pIndexN,pVrhs);             {backsubst. = solve neg.seq.net}
        {complete the solution, put currents back}
        for ii := 1 to nUns do begin
          pUnsym := pList^[ii-1];
          pUnsym^.GetNode(2,jc,er);                       {current "node"}
          getCvec(pbb,ii*2-1,nUns*2,Ip);                  {saved above}
          getCvec(pbb,ii*2  ,nUns*2,Im);
          putCvec(pRhs,jc,nnod,Ip);                       {complete the solution with the current}
          putCvec(pVrhs,jc,nnod,Im);                      {complete the solution with the current}
        end;
        {Results to components. pos.seq. results in pRhs, neg.seq results in pVrhs}
        ClearAllVar(pSet);                                {clear initially}
        ResultsToComp(pSet, nnod, freq, pVrhs,2,ForBoth); {neg.seq to comp., as first set of var.}
        CopyAllVar(pSet,1,2);                             {copy to second set of var. in Elplex.pas}
        ResultsToComp(pSet, nnod, freq, pRhs ,1,ForBoth); {pos.seq to comp, as first set of var}
        AllVarSeqToPhase(pSet);                           {from seq to phase, in Twoshort.pas}
        {voltage of the neutral point}
        for ii := 0 to pList^.count-1 do begin
          pUnsym := pList^[ii];                           {pointer to an unsymm.load}
          pUnsym^.GetVar(gUnsymm_Vr,cpu.x);               {voltage in phase a}
          pUnsym^.GetVar(gUnsymm_Vi,cpu.y);
          pUnsym^.GetVar(gUnsymm_Ir,Ia.x);                {current in phase a}
          pUnsym^.GetVar(gUnsymm_Ii,Ia.y);
          pUnsym^.GetPar(gUnsym_Ra{1},Za.x);              {resistance in phase a}
          pUnsym^.GetPar(gUnsym_Xa{2},Za.y);              {reactance in phase a}
          Vn := Csub(cpu,Cmult(Ia,Za));                   {neutral voltage}
          {save same voltage to all phases}
          pUnsym^.GetNvar(nvar);
          pUnsym^.PutVar(gUnsymm_Vnr,Vn.x,'UnsymmLoad');
          pUnsym^.PutVar(gUnsymm_Vni,Vn.y,'UnsymmLoad');
          pUnsym^.PutVar(gUnsymm_Vnr+nvar,Vn.x,'UnsymmLoad');
          pUnsym^.PutVar(gUnsymm_Vni+nvar,Vn.y,'UnsymmLoad');
          pUnsym^.PutVar(gUnsymm_Vnr+nvar*2,Vn.x,'UnsymmLoad');
          pUnsym^.PutVar(gUnsymm_Vni+nvar*2,Vn.y,'UnsymmLoad');
        end;
        {sum current trafos must display the sum current}
//      if (pCurrT <> nil) then pCurrT^.SumToOutVar;     {in shortcmp.pas}
        {sum calculated in PolarToRelay of elplex.pas }
        {display relay current using polar coordinates}
        PolarToRelaysAndBuses(pSet);
        {show also the current through the current trafos, if allowed}
        ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down,Imagoo.ShowCurrBut.caption);
        {update the eventual result boxes}
        pSet^.UpdateResultBoxes(1);
      finally
        {}
      end;
    end;
    {free the reserved memory from matrices}
    pSet^.FreeMemForCurrTrAndRelays;
    FreeMat(nnod);
    Freemem(pNmat,nnod*nnod*SizeOf(complex));  {free matrix for neg. seq.}
    Freemem(pIndexN,nnod*Sizeof(integer));     {free index array, neg. seq}
    Freemem(pVrhs,nnod*SizeOf(complex));
    freemem(p11,nUns*nUns*sizeof(complex));
    freemem(p12,nUns*nUns*sizeof(complex));
    freemem(p21,nUns*nUns*sizeof(complex));
    freemem(p22,nUns*nUns*sizeof(complex));
    freemem(p31,nUns*nUns*sizeof(complex));
    freemem(p32,nUns*nUns*sizeof(complex));
    freemem(pU0,3*nUns*sizeof(complex));
    freemem(paa,nUns*nUns*4*sizeof(complex));
    freemem(pbb,nUns*2*sizeof(complex));
    freemem(pin,nUns*2*sizeof(integer));
    pList^.free;
    dispose(pList);
  end;
end;

end.

