{unit for writing an EMTP *.dat file of the network}
{7.1.2003 subtransient impedances of as.motors written to *.dat file}
unit emtp;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls,skemadef,imagodef,shortcir,Earthflt,
  Compdef,Cmatmul,routines;

{define type for transmitting data to emtp-procedures}
type
  TpEdat = ^TEdat;
  TEdat = record
    branch   : integer;       {branch number, for transmission lines}
    ipSet    : longint;       {pointer to the "common" as integer}
    pComp    : TpComp;        {pointer to a component}
    pAlist   : ^TStringList;  {pointer to text for asynch.motors}
    pGlist   : ^TStringList;  {pointer to text for generators}
    pIlist   : ^TStringList;  {pointer to text for impedances}
    pSlist   : ^TstringList;  {pointer to text for short circuits = switches}
    pTlist   : ^TstringList;  {pointer to text for Tacs}
    pVlist   : ^TstringList;  {pointer to text for voltage sources}
    StarList : TList;         {list of wye-terminations}
    EarthList: Tlist;         {list of earthing impedances}
    text     : string;        {use depends on component type}
    dt       : single;        {time step length}
    nAsMot   : integer;       {no of asynch.motors}
    nGenMot  : integer;       {no of synchronous machines}
end;

procedure EmtpFile(ipSet : longint; FileName : string);
procedure GeneratorEMTP(ipPar : longint);
procedure AsynchMotEMTP(ipPar : longint);
procedure ImpedanceEMTP(ipPar : longint);
procedure CapacitorEMTP(ipPar : longint);
procedure FaultEMTP(ipPar : longint);
procedure OpenLineEMTP(ipPar : longint);
procedure LineEMTP(ipPar : longint);
procedure LoadEMTP(ipPar : longint);
procedure UnsymmLoadEMTP(ipPar : longint);
procedure SourceEMTP(ipPar : longint);
procedure TrafoYYEMTP(ipPar : longint);
procedure TrafoYDEMTP(ipPar : longint);
procedure TrafoDYEMTP(ipPar : longint);
procedure TrafoDDEMTP(ipPar : longint);
procedure FindStar(pComp : TpComp; StarList : TList);
{}
procedure NodeNames(nn : integer; var texta,textb,textc : string);
procedure NodeNamez(nn : integer; var texta,textb,textc : string);
procedure PhaseNames(pComp : TpComp; ni : integer;  pPar  : TpEdat;
          var nn : integer; var texta,textb,textc : string);
procedure ImpedanceLines(namea,nameb,namec,namer,names,namet : string;
          first : boolean; rr,xx,cc : single; var linea, lineb,linec : string);
procedure ThreeImpedLines(namea,nameb,namec,namer,names,namet : string;
          first : boolean; r1,x1,c1,r2,x2,c2,r3,x3,c3 : single;
          var linea, lineb,linec : string);
procedure CheckedImedance(rr,xx: single; var r1,x1,c1: single);                    
function TwoChar(ii : integer) : string;
function SixChar(rr : single) : string;
function SixSignedChar(aa : single) : string;
function SixPlaces(rr : single) : string;
function TenChar(rr : single) : string;
function SixCharOneDecimal(rr : single): string;
function TwoDecimalsAtMost(rr : single): string;
function ExactlyTwoDecimals(rr : single): string;
function ThreeDecimals(rr : single): string;

implementation

uses
  imagopas, elplex;

procedure NodeNames(nn : integer; var texta,textb,textc : string);
{generate the six character long node names from the node number
 nn                  the node number (ID)
 texta,textb,textc   the node names in the three phases}
var
  ii   : integer;     {index}
  len  : integer;     {length of string}
begin
  if (nn = 0) then begin        {connected to ground}
    texta := '      ';          {ground = six blanks}
    textb := '      ';
    textc := '      ';
  end else begin                {proper node}
    texta := 'A'+inttostr(nn);  {construct the node name}
    textb := 'B'+inttostr(nn);
    textc := 'C'+inttostr(nn);
    len := length(texta);
    if (len < 6) then begin     {add spaces so that total length = 6}
      for ii := 1 to 6-len do begin
        texta := texta+' ';
        textb := textb+' ';
        textc := textc+' ';
      end;
    end;
  end;
end;

procedure NodeNamez(nn : integer; var texta,textb,textc : string);
{generate the six character long node names from the node number
 nn                  the node number (ID)
 texta,textb,textc   the node names in the three phases}
var
  ii   : integer;     {index}
  len  : integer;     {length of string}
begin
  if (nn = 0) then begin        {connected to ground}
    texta := '      ';          {ground = six blanks}
    textb := '      ';
    textc := '      ';
  end else begin                {proper node}
    texta := 'R'+inttostr(nn);  {construct the node name}
    textb := 'S'+inttostr(nn);
    textc := 'T'+inttostr(nn);
    len := length(texta);
    if (len < 6) then begin     {add spaces so that total length = 6}
      for ii := 1 to 6-len do begin
        texta := texta+' ';
        textb := textb+' ';
        textc := textc+' ';
      end;
    end;
  end;
end;

procedure PhaseNames(pComp : TpComp; ni : integer; pPar  : TpEdat;
                     var nn : integer; var texta,textb,textc : string);
{generates six char. long names for the three phases, taking into account
 an eventual wye termination (star).
 If the star point of the wye is earthed through an impedance, add
 the impedance to the list of impedances
 pComp      pointer to the component
 ni         node index within the component (1,2, etc)
 pPar       pointer to lists, etc
 nn         the node number (identifier) in the whole network
 texa,textb,textc  the node names for the three phases}
var
  EiFound  : boolean;  {earthing impedance found}
  err,errn : integer;  {error flag}
  ii       : integer;  {index}
  ix,iy    : integer;  {temporary}
  lbl      : string;   {label of the component}
  na,nb,nc : string;   {temporary node names}
  nei,nst  : integer;  {nodes of earthing impedance, star}
  nstar    : integer;  {index of the wye}
  pEimp    : TpComp;   {pointer to an earthing impedance}
  pStar    : TpComp;   {pointer to the wye-termination}
  Rg,Xg    : single;   {earthing impedance}
begin
  pComp^.getNode(ni,nn,err);         {get the node number}
  pComp^.GetLabel(lbl,ix,iy);        {label of the component}
  if (err = 0) then begin            {no errors}
    {check eventual wye-termination (star), but only for "proper" nodes}
    if (ni > 0) and (ni < 4) then begin
      nstar := -1;                      {initialize index}
      {int1_s, 2_s, 3_s correspond to three first nodes, value = index of star}
      if      ((ni = 1) and (pComp^.int1_s >= 0)) then nstar := pComp^.int1_s
      else if ((ni = 2) and (pComp^.int2_s >= 0)) then nstar := pComp^.int2_s
      else if ((ni = 3) and (pComp^.int3_s >= 0)) then nstar := pComp^.int3_s;
      if (nstar >= 0) then begin                 {a wye found}
        {is the star point earthed through an impedance, or directly?}
        if (pPar^.EarthList.count > 0) then begin{there are earthing impedances}
          pStar := pPar^.StarList[nstar];        {pointer to the star}
          pStar^.GetNode(3,nst,errn);            {neutral node (star point)}
          EiFound := false;                      {initialize "imped.found"}
          pEimp := nil;
          for ii := 0 to pPar^.EarthList.count-1 do begin
            pEimp := pPar^.EarthList[ii];
            pEimp^.GetNode(1,nei,errn);
            if (nei = nst) then begin            {found earthing impedance}
              EiFound := true;
              break;
            end;
          end;
          if (EiFound and (pEimp <> nil)) then begin {there is an earthing impedance}
            pEimp^.GetPar(gGrImped_R,Rg);         {earthing resistance}
            pEimp^.GetPar(gGrImped_X,Xg);         {earthing reactance}
            if (Rg <= 0) and (Xg <= 0) then begin {solidly grounded}
              texta := '      ';                  {no name = earth}
              textb := '      ';
              textc := '      ';
            end else begin                        {finite impedance}
              NodeNames(nn,na,nb,nc);
              texta := na;                        {all phases get the same name}
              textb := na;
              textc := na;
              {add the earthing impedance to the list of impedances, if not written yet}
              if (pEimp^.int1_s = 0) then begin
{                pPar^.pIlist^.add('*   earthing impedance of '+lbl +' - - - - - - - ');}
                pPar^.pIlist^.add('  '+na+'      '+'      '+'      '+SixChar(Rg)+SixChar(Xg));
                pEimp^.int1_s := 1;
              end;
            end;
          end else begin                          {no earthing impedances}
            NodeNames(nn,na,nb,nc);
            texta := na;                        {all phases get the same name}
            textb := na;
            textc := na;
          end;
        end else begin                          {star point not earthed}
          NodeNames(nn,na,nb,nc);
          texta := na;                          {all phases get the same name}
          textb := na;
          textc := na;
        end;
      end else begin                 {no wye}
        NodeNames(nn,texta,textb,textc);
      end;
    end else begin                   {no need to check wye}
      NodeNames(nn,texta,textb,textc);
    end;
  end else begin                     {could not get node number}
    texta := '      ';
    textb := '      ';
    textc := '      ';
  end;
end;

function ClearSpaces(text : string): string;
{copy text without any spaces}
var
  ii  : integer;
  len : integer;
  line: string;
begin
  len := length(text);
  line := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (text[ii] <> ' ') then line := line+text[ii];
    end;
  end;
  ClearSpaces := line;
end;

function TwoChar(ii : integer) : string;
{write a nonnegative integer using two characters}
var
  text : string;
begin
  if (ii <= 0) then begin
    text := ' 0';
  end else if (ii < 10) then begin
    text := ' '+inttostr(ii);
  end else if (ii < 100) then begin
    text := inttostr(ii);
  end else begin
    text := '99';
  end;
  TwoChar := text;
end;

function SixPlaces(rr : single) : string;
{write a positive real using at most six places, no spaces
 and the default decimal point
 rr    the real }
var
  line  : string;
begin
  if (rr < 0.000099) then
    line := '   0.0'
  else if (rr < 1.0) then
    line := format('%6.4f',[rr])
  else if (rr < 1.e5) then
    line := format('%6.5g',[rr])
  else if (rr < 1.e8) then
    line := format('%6.3g',[rr])
  else
    line := '1.0e8 ';
  {remove eventual spaces}
  SixPlaces := ClearSpaces(line);
end;

procedure AddDecimalPoint(text : string; var line : string);
{adds a decimal point to a number as text if missing
 text  the text to be checked
 line  text with decimal point}
var
  first    : boolean;   {first space (after numbers)}
  founde   : integer;   {position of e or E in text, -1 if there is none}
  foundot  : boolean;   {true, when found a decimal point}
  ii       : integer;   {index}
  len      : integer;   {length of text}
begin
  len := length(text);
  if (len >= 4) then begin
    {there must be a decimal point (atp needs it)}
    foundot := false;
    for ii := 1 to len do begin
      if (text[ii] = '.') or (text[ii] = ',') then begin
        foundot := true;
        break;
      end;
    end;
    {there might be an 'E'}
    founde := 0;
    for ii := 1 to len do begin
      if (text[ii] = 'e') or (text[ii] = 'E') then begin
        founde := ii;
        break;
      end;
    end;
    {there must be a decimal point (atp needs it)}
    first := false;
    if (foundot) then begin  {there was a decimal point}
      line := text;
    end else begin           {add the decimal point}
      if (founde = 0) then begin  {no 'E'}
        line := '';
        for ii := 2 to len do begin
          if (text[ii] <> ' ') then begin
            line := line+text[ii];          {copy all numbers as such}
            first := true;                  {next space = first space after numbers}
          end else begin
            if (first) then begin           {first space}
              line := line+'. ';            {add decimal point and one space}
              first := false;               {next is not first}
            end else begin
              line := line+' ';
            end;
          end;
        end;
        if (first) then begin               {no space found, must add the decimal point}
          line := line+'.';
        end;
      end else begin         {there is an 'E', but no decimal point}
        if (text[1] = ' ') then begin  {there should be a space}
          line := '';
          for ii := 2 to founde-1 do line := line+text[ii];
          line := line+'.';
          for ii := founde to len do line := line+text[ii];
        end else begin
          line := text;
        end;
      end;
    end;
  end else begin
    line := text;          {copy a short text as such}
  end;
end;

function NoZeroesExp(rr: single) : string;
{expresses rr in a format with six characters
 removes the unnecessary zeroes in the exponential
 length of string = 6
 rr  the number}
var
  efound : boolean;
  ii     : integer;
  len    : integer;
  line   : string;
  text   : string;
begin
  line := format('%6.1e',[rr]);
  {remove unnecessary zeroes in the exponential}
  len := length(line);
  if (len > 6) then begin
    text := '';
    efound := false;
    for ii := 1 to len do begin
      if (not efound) then begin   {e or E of the exponential not yet found}
        if (line[ii] = 'e') or (line[ii] = 'E') then efound := true;
        text := text+line[ii];
      end else begin
        if (line[ii] <> '0') then text := text+line[ii];
      end;
    end;
  end else begin
    text := line;
  end;
  NoZeroesExp := text;
end;

function SixChar(rr : single) : string;
{write a positive real using six places
 rr    the real }
var
  efound  : boolean;
  ii      : integer;
  len     : integer;
  line    : string;
  text    : string;
begin
  if (rr <= 0) then
    text := '   0.0'
  else if (rr < 1.e-9) then
    text := '1.0e-9'
  else if (rr < 0.001) then
    text := NoZeroesExp(rr)
  else if (rr < 1.0) then
    text := format('%6.4f',[rr])
  else if (rr < 1e5) then begin
    text := format('%6.5g',[rr]);
  end else if (rr < 1.e8) then
    text := format('%6.3g',[rr])
  else
    text := '1.0e8 ';
  {there must be a decimal point (atp needs it)}
  AddDecimalPoint(text,line);    {add, if missing}
  {replace eventual commas (,) with full stops (.)}
  len := length(line);
  text := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> ',') then text := text+line[ii]
      else                      text := text+'.';
    end;
  end else begin
    text := line;
  end;
  SixChar := text;
end;

(*
function SixChar(rr : single) : string;
{write a positive real using six places
 rr    the real }
var
{ foundot : boolean;
  founde  : integer; }
  ii      : integer;
  len     : integer;
  line    : string;
  text    : string;
begin
  if (rr <= 0) then
    text := '    0 '
  else if (rr < 1.e-9) then
    text := '1.0e-9'
  else if (rr < 0.001) then
    text := '0.001 '{format('%6.1e',[rr])}
  else if (rr < 1.0) then
    text := format('%6.4f',[rr])
  else if (rr < 1e5) then begin
    text := format('%6.5g',[rr]);
  end else if (rr < 1.e8) then
    text := format('%6.3g',[rr])
  else
    text := '1.0e8 ';
  {there must be a decimal point (atp needs it)}
  AddDecimalPoint(text,line);    {add, if missing}
  {replace eventual commas (,) with full stops (.)}
  len := length(line);
  text := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> ',') then text := text+line[ii]
      else                      text := text+'.';
    end;
  end else begin
    text := line;
  end;
  SixChar := text;
end;
*)
(*
function SixNegChar(aa : single) : string;
{write a negative real using six places
 aa    the real}
var
  first : boolean;
  ii    : integer;
  len   : integer;
  line  : string;
  rr    : single;
  text  : string;
begin
  rr := abs(aa);
  if (rr < 0.001) then
    text := '0.001'
  else if (rr < 1.0) then
    text := format('%5.3f',[rr])
  else if (rr < 1.e4) then
    text := format('%5.4g',[rr])
  else if (rr < 1.e8) then
    text := format('%5.2g',[rr])
  else
    text := '1.0e8';
  {add eventually missing decimal point}
  AddDecimalPoint(text,line);
  {replace eventual commas (,) with full stops (.)}
  len := length(line);
  text := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> ',') then text := text+line[ii]
      else                      text := text+'.';
    end;
  end else begin
    text := line;
  end;
  {add the minus sign}
  first := true;
  line := '';
  for ii := 1 to len do begin
    if (text[ii] <> ' ') and first then begin
      line := line+'-';
      first := false;
    end;
    line := line+text[ii];
  end;
  SixNegChar := line;
{  SixNegChar := '-'+text;}
end;
*)
function NoDecPoint(rr: single) : string;
{expresses rr in a format with five characters
 removes the decimal point and
 the unnecessary zeroes in the exponential
 length of string = 5
 rr  the number}
var
  efound : boolean;
  ii,jj  : integer;
  len    : integer;
  line   : string;
  text   : string;
begin
  line := format('%6.1e',[rr]);
  {remove unnecessary zeroes in the exponential}
  len := length(line);
  if (len > 5) then begin
    text := '';
    efound := false;
    for ii := 1 to len do begin
      if (not efound) then begin   {e or E of the exponential not yet found}
        if (line[ii] = 'e') or (line[ii] = 'E') then efound := true;
        if (line[ii] <> '.') and (line[ii] <> ',') then text := text+line[ii];
      end else begin
        if (line[ii] > '0') and (line[ii] <= '9') then begin
          jj := strtoint(line[ii]);
          jj := jj+1;
          text := text+inttostr(jj);
        end else if (line[ii] <> '0') then begin
          text := text+line[ii];
        end;
      end;
    end;
  end else begin
    text := line;
  end;
  NoDecPoint := text;
end;

function SixNegChar(aa : single) : string;
{write a negative real using six places
 aa    the real}
var
  first : boolean;
  ii    : integer;
  len   : integer;
  line  : string;
  rr    : single;
  text  : string;
begin
  rr := abs(aa);
  if (rr < 1.e-9) then
    text := '1.0e-9'
  else if (rr < 0.01) then
    text := NoDecPoint(rr)
  else if (rr < 1.0) then
    text := format('%5.3f',[rr])
  else if (rr < 1.e4) then
    text := format('%5.4g',[rr])
  else if (rr < 1.e8) then
    text := format('%5.2g',[rr])
  else
    text := '1.0e8';
  {add eventually missing decimal point}
  AddDecimalPoint(text,line);
  {replace eventual commas (,) with full stops (.)}
  len := length(line);
  text := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> ',') then text := text+line[ii]
      else                      text := text+'.';
    end;
  end else begin
    text := line;
  end;
  {add the minus sign}
  first := true;
  line := '';
  for ii := 1 to len do begin
    if (text[ii] <> ' ') and first then begin
      line := line+'-';
      first := false;
    end;
    line := line+text[ii];
  end;
  SixNegChar := line;
{  SixNegChar := '-'+text;}
end;

function SixSignedChar(aa : single) : string;
{write a positive or negative real using six places
 aa    the real}
begin
  if (aa >= 0) then SixSignedChar := SixChar(aa)
  else              SixSignedChar := SixNegChar(aa);
end;

function SixCharOneDecimal(rr : single): string;
{write a positive or negative real using six char. and one decimal}
var
  eFound   : boolean;   {exponential found}
  ii       : integer;   {index}
  len      : integer;   {length of string}
  ndec     : integer;   {position of decimal point}
  nf       : integer;   {position of first character}
  txa,txb  : string;
begin
  txa := sixSignedChar(rr);    {change to string with six char.}
  {look for the position of the decimal point and first character}
  len := length(txa);
  ndec := 0;
  nf := 0;
  {check, is exponential notation used (2.3e6)}
  eFound := false;
  for ii := 1 to len do begin
    if (txa[ii] = 'e') or (txa[ii] = 'E') then begin
      eFound := true;
      break;
    end;
  end;
  {not exponential format, process}
  if (not eFound) then begin          {no exponential form, process nromally}
    for ii := 1 to len do begin
      {check first non-space char.}
      if (nf = 0) then begin                 {first nonspace char.not yet found}
        if (txa[ii] <> ' ') then nf := ii;   {position of first nonspace char.}
      end;
      if (txa[ii] = '.') or (txa[ii] = ',') then begin  {decimal point found}
        ndec := ii;
        break;
      end;
    end;
    {handle different cases}
    if ( ndec = 0) then begin      {no decimal point}
      if (nf > 2) then begin       {room for '.0'}
        txb := '';
        for ii := 3 to len do txb := txb+txa[ii];
        txb := txb+'.0';
      end else begin                            {no room for decimals}
        txb := txa;
      end;
    end else if (ndec >= 1) and (ndec <= 4) then begin  {must limit no of decimals}
      txb := '';
      for ii := 1 to 5-ndec do txb := txb+' ';  {fill with leading spaces}
      for ii := 1 to ndec+1 do txb := txb+txa[ii];    {copy from the beginning to the first dec.}
    end else begin
      txb := txa;                   {cannot adjust no of decimals}
    end;
    SixCharOneDecimal := txb;
  end else begin                    {exponential format}
    {for compatibility, use full stop as decimal point}
    for ii := 1 to len do begin
      if (txa[ii] = ',') then txa[ii] := '.';
    end;
    SixCharOneDecimal := txa;
  end;
end;

function TwoDecimalsAtMost(rr : single): string;
{express a small single as a string with at most two decimals}
var
  found : boolean;  {true, when the decimal point found}
  id    : integer;   {counts numbers after dec.point}
  ii    : integer;   {index}
  len   : integer;   {length of string}
  txa   : string;    {temporary}
  txt   : string;    {temporary string}
begin
  txt := floattostr(rr);
  len := length(txt);
  id := 0;
  found := false;
  {scan and copy, copy at most two decimals}
  txa := '';
  for ii := 1 to len do begin
    txa := txa+txt[ii];
    if (found) then begin        {decimal point has been found, count characters}
      inc(id);
      if (id >= 2) then break;   {at most two char. after the decimal point}
    end;
    if (txt[ii] = '.') or (txt[ii] = ',') then found := true; {decimal point}
  end;
  TwoDecimalsAtMost := txa;
end;

function ExactlyTwoDecimals(rr : single): string;
{expresses a (small) positive single with exactly two decimals}
var
  apu : single;    {temporary}
  dp  : char;      {decimal point}
  dec : string;    {rr with two decimals}
  ii  : integer;   {index}
  len : integer;   {length of a string}
  ll  : longint;   {temporary}
  txt : string;    {temporary}
begin
  DecimalPoint(dp);      {determine decimal point: , or . }
  apu := 100*rr;
  ll := round(apu);      {100*rr as rounded integer}
  if (ll < 1) then begin
    dec := '0'+dp+'00';
  end else if (ll < 10) then begin
    dec := '0'+dp+'0'+inttostr(ll);
  end else if (ll < 100) then begin
    dec := '0'+dp+inttostr(ll);
  end else begin
    txt := inttostr(ll);   {to string}
    len := length(txt);
    dec := '';
    for ii := 1 to len-2 do begin
      dec := dec+txt[ii];
    end;
    dec := dec+dp;
    for ii := len-1 to len do begin
      dec := dec+txt[ii];
    end;
  end;
  ExactlyTwoDecimals := dec;
end;


function ThreeDecimals(rr : single): string;
{express a small single as a string with at most three decimals}
var
  arr   : single;    {abs.value of rr}
  found : boolean;   {true, when the decimal point found}
  id    : integer;   {counts numbers after dec.point}
  ii,jj : integer;   {index}
  len   : integer;   {length of string}
  lem   : integer;   {length of modified string}
  ll    : longint;   {temporary}
  txa   : string;    {temporary}
  txt   : string;    {temporary string}
begin
  arr := abs(rr);
  if (arr < 2e6) then ll := round(1000*arr) else ll := round(1000*2e6);
  txt := inttostr(ll);
  len := length(txt);
  if (len <= 3) then begin  {too short, must add zeroes}
    if (len = 0) then txt := '0000'
    else if (len = 1) then txt := '000'+txt
    else if (len = 2) then txt := '00'+txt
    else if (len = 3) then txt := '0'+txt;
  end;
  {add the decimal point}
  lem := length(txt);
  txa := '';
  for ii:= 1 to lem-3 do txa := txa+txt[ii];  {copy up to the decimal point}
  txa := txa+',';
  for ii:= lem-2 to lem do txa := txa+txt[ii];  {copy the rest}
  if (rr >= 0) then ThreeDecimals := txa
  else ThreeDecimals := '-'+txa;
end;

function TenChar(rr : single) : string;
{write a positive or negative real using ten places, of which some are spaces (!)
 rr    the real
 text  the outputted text}
const
//  small = 0.00001;
  small = 1.e-7;
var
  aa    : single;     {abs value of rr}
  first : boolean;
  ii    : integer;
  len   : integer;    {length of line}
  line  : string;     {text under construction}
  ss    : string;     {space or minus sign}
  txt   : string;
  text  : string;
begin
  aa := abs(rr);
{ if (rr < 0) then ss := '-'
  else             ss := ' ';}
  if (aa < small) then
    text := ' 0.0      '
  else if (aa < 0.0001) then begin
    txt := format('%11.5e',[aa]);
    {remove unnecessary zeroes from exponential}
    text := '';
    for ii := 1 to length(txt) do begin
      if (ii < 9 ) or (ii > 10) then text := text+txt[ii];
    end;
  end else if (aa < 1.0) then
    text := {' '+format('%8.6f',[aa])}format('%9.7f',[aa])
  else if (aa < 1.e5) then begin
    if (aa = round(aa)) then text := format('%7.6g',[aa])+'.0'  {integer}
    else text := '  '+format('%7.6g',[aa]);
  end else if (aa < 1.e8) then
    text := '  '+format('%7.4g',[aa])
  else
    text := '1.0e8    ';
  AddDecimalPoint(text,line);  {add an eventually missing decimal point)
  {replace eventual commas (,) with full stops (.)}
  len := length(line);
  text := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> ',') then text := text+line[ii]
      else                      text := text+'.';
    end;
  end else begin
    text := line;
  end;
  {add the minus sign for a negative number}
  if (aa >= small) then begin    {no minus sign to zero}
    if (rr < 0) then ss := '-'
    else             ss := ' ';
    first := true;
    line := '';
    for ii := 1 to len do begin
      if (text[ii] <> ' ') and first then begin
        line := line+ss;
        first := false;
      end;
      line := line+text[ii];
    end;
  end;
  TenChar := line;
end;

function FifteenChar(rr : single) : string;
{write a positive or negative real using fifteen places, of which some are spaces (!)
 rr    the real
 text  the outputted text}
const
  small = 0.0000001;
var
  aa    : single;     {abs value of rr}
  first : boolean;
  ii    : integer;
  len   : integer;    {length of line}
  line  : string;     {text under construction}
  ss    : string;     {space or minus sign}
  text  : string;
begin
  aa := abs(rr);
  if (aa < small) then
    text := ' 0.0           '
  else if (aa < 0.1) then
    text := format('%10.8e',[aa])
  else if (aa < 1.0) then
    text := '    '+format('%10.8f',[aa])
  else if (aa < 1.e5) then begin
    if (aa = round(aa)) then text := '     '+format('%7.6g',[aa])+'.0'  {integer}
    else text := '   '+format('%11.9g',[aa]);
  end else if (aa < 1.e8) then
    text := '  '+format('%12.9g',[aa])
  else
    text := '  1.0e8       ';
  AddDecimalPoint(text,line);  {add an eventually missing decimal point)
  {replace eventual commas (,) with full stops (.)}
  len := length(line);
  text := '';
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> ',') then text := text+line[ii]
      else                      text := text+'.';
    end;
  end else begin
    text := line;
  end;
  {add the (eventual) minus sign}
  if (aa > small) then begin        {no minus sign to zero}
    if (rr < 0) then ss := '-'
    else             ss := ' ';
    first := true;
    line := '';
    for ii := 1 to len do begin
      if (text[ii] <> ' ') and first then begin
        line := line+ss;
        first := false;
      end;
      line := line+text[ii];
    end;
  end;
  FifteenChar := line;
end;
(**)

procedure ImpedanceLines(namea,nameb,namec,namer,names,namet : string;
          first : boolean; rr,xx,cc : single; var linea, lineb,linec : string);
{generate the EMTP-lines for an impedance
 namea,nameb,namec  names of upstream nodes, length must be 6 !, earth = '      '
 namer,names,namet  names of downstream nodes, --"--
 first              falg for first impedance
 rr,xx,cc           resistance (ohms), inductance(ohms),capacitance (uS) in series
 linea,lineb,linec  the lines for the three phases}
var
  plot              : string;
  texta,textb,textc : string;
begin
  {two first characters are blanks}
  linea := '  ';
  lineb := '  ';
  linec := '  ';
  linea := linea+namea;    {upstream node}
  lineb := lineb+nameb;
  linec := linec+namec;
  linea := linea+namer;
  lineb := lineb+names;
  linec := linec+namet;
  {no reference names}
  linea := linea+'            ';
  lineb := lineb+'            ';
  linec := linec+'            ';
  {resistance}
  texta := SixSignedChar(rr);
(*if (rr > 0) then begin
    texta := SixChar(rr)     {resistance with six characters}
  end else begin
    texta := '      ';        {six spaces corresponds to zero}
  end; *)
  {reactance}
  textb := SixSignedChar(xx);
(*if (xx > 0) then begin
    textb := SixChar(xx);      {inductive reactance with six characters}
  end else begin
    textb := '      '          {capacitive Not (yet?) included}
  end; *)
  {capacitance}
  if (cc > 0 ) then  begin
    textc := SixChar(cc);
  end else begin
    textc := '      ';
  end;
  linea := linea+texta+textb+textc; {all lumped linear elements}
  lineb := lineb+texta+textb+textc;
  linec := linec+texta+textb+textc;
  {add an one or a zero to position 80, to show the place for the eventual output flag}
  if (first) then plot := '1' else plot := '0';
  linea := linea+'                                   '+plot;
  lineb := lineb+'                                   '+plot;
  linec := linec+'                                   '+plot;
end;

procedure OneImpedLine(rr,xx,cc: single; var line: string);
{add the impedance values to a line under construction
 rr,xx,cc   resistance(ohm), inductive reactance(ohm), capacitive susceptance (uS)
 NOTE       at least one must be nonzero, no chekcs here
 line       the line under construction}
var
  texta,textb,textc: string;  {temporary}
begin
  {resistance 1}
  texta := SixSignedChar(rr);
  {reactance 1}
  textb := SixSignedChar(xx);
  {capacitance 1}
  if (cc > 0 ) then  begin
    textc := SixChar(cc);
  end else begin
    textc := '      ';
  end;
  line := line+texta+textb+textc; {all lumped linear elements}
end;

procedure ThreeImpedLines(namea,nameb,namec,namer,names,namet : string;
          first : boolean; r1,x1,c1,r2,x2,c2,r3,x3,c3 : single;
          var linea, lineb,linec : string);
{generate the EMTP-lines for three impedance, one in each phase
 namea,nameb,namec  names of upstream nodes, length must be 6 !, earth = '      '
 namer,names,namet  names of downstream nodes, --"--
 first              falg for first impedance
 r1,x1,c1           resistance (ohms), inductance(ohms),capacitance (uS) in series
 ...                for first impedance
 NOTE               at least one must be nonzero, NO checks here
 linea,lineb,linec  the lines for the three phases}
var
  plot              : string;
  texta,textb,textc : string;
begin
  {two first characters are blanks}
  linea := '  ';
  lineb := '  ';
  linec := '  ';
  linea := linea+namea;    {upstream node}
  lineb := lineb+nameb;
  linec := linec+namec;
  linea := linea+namer;
  lineb := lineb+names;
  linec := linec+namet;
  {no reference names}
  linea := linea+'            ';
  lineb := lineb+'            ';
  linec := linec+'            ';
  {add the impedances}
  OneImpedLine(r1,x1,c1,linea);
  OneImpedLine(r2,x2,c2,lineb);
  OneImpedLine(r3,x3,c3,linec);
  {add an one or a zero to position 80, to show the place for the eventual output flag}
  if (first) then plot := '1' else plot := '0';
  linea := linea+'                                   '+plot;
  lineb := lineb+'                                   '+plot;
  linec := linec+'                                   '+plot;
end;

procedure ImpedanceEMTP(ipPar : longint);
{add the lines for an impedance in the EMTP *.dat format}
var
  cc    : single;        {capacitance}
 { err   : integer;       {error flag}
  first : boolean;       {flag for first impedance}
  ix,iy : integer;       {temporary}
  lbl   : string;        {label (name) of the component}
  linea : string;        {EMTP data for impedance in phase a}
  lineb : string;
  linec : string;
  namea : string;        {upstream node in phase a}
  nameb : string;        {upstream node in phase b}
  namec : string;        {upstream node in phase c}
  namer : string;        {downstream node in phase a}
  names : string;        {downstream node in phase b}
  namet : string;        {downstream node in phase c}
  n1,n2 : integer;       {nodes}
  pComp : TpComp;        {pointer to the impedance}
  pPar  : TpEdat;        {pointer to the data}
  rr    : single;        {resistance}
  x1    : single;        {reactance}
  xx    : single;        {reactance}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);       {pointer to data}
    first := (pPar^.pIlist.count <= 1); {empty, when first impedance}
    pComp := pPar^.pComp;        {pointer to the impedance}
    pComp^.GetLabel(lbl,ix,iy);  {name}
(*
    pComp^.getNode(1,n1,err);    {upstream node}
    pComp^.getNode(2,n2,err);    {downstream node}
*)
    pComp^.GetPar(gImped_R,rr);  {get value of resistor}
    pComp^.GetPar(gImped_X,x1);  {get value of reactance}
    {reactance may also be capacitive}
    if (x1 >= 0) then begin
      xx := x1;
      cc := 0;                   {no capacitances}
    end else begin
      xx := 0;
      cc := -1.e6/x1;            {admittance, in uS}
    end;
    {add explanatory text}
    pPar^.pIlist^.add('*  impedance  '+lbl+'  - - - - - - - - - - - - - ');
    {node names}
(*
    NodeNames(n1,namea,nameb,namec);  {upstream node}
    NodeNames(n2,namer,names,namet);  {downstream node}
*)
    PhaseNames(pComp,1, pPar, n1,namea,nameb,namec);
    PhaseNames(pComp,2, pPar, n2,namer,names,namet);
    ImpedanceLines(namea, nameb, namec, namer, names, namet, first,rr, xx, cc,
                   linea, lineb, linec);
    {add the lines to the output list for impedances}
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
  end;
end;

procedure CapacitorEMTP(ipPar : longint);
{add the lines for an impedance in the EMTP *.dat format}
var
  cc    : complex;        {admittance of the capacitor}
{ err   : integer;       {error flag}
  first : boolean;       {flag for first impedance}
  freq  : single;        {frequency}
  ix,iy : integer;       {temporary}
  lbl   : string;        {label (name) of the component}
  linea : string;        {EMTP data for cap. in phase a}
  lineb : string;
  linec : string;
  namea : string;        {upstream node in phase a}
  nameb : string;        {upstream node in phase b}
  namec : string;        {upstream node in phase c}
  namer : string;        {downstream node in phase a}
  names : string;        {downstream node in phase b}
  namet : string;        {downstream node in phase c}
  n1,n2 : integer;       {nodes}
  pComp : TpComp;        {pointer to the component}
  pPar  : TpEdat;        {pointer to the data}
  pSet  : TpCompSet;     {pointer to the common}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);       {pointer to data}
    pSet := TpCompSet(pPar^.ipSet);     {pointer to the common}
    freq := pSet^.appl_Hz;       {frequency}
    first := (pPar^.pIlist.count <= 1); {empty, when first impedance}
    pComp := pPar^.pComp;        {pointer to the cap.}
    pComp^.GetLabel(lbl,ix,iy);  {name}
(*
    pComp^.getNode(1,n1,err);    {upstream node}
    pComp^.getNode(2,n2,err);    {downstream node}
*)
    AdmCap(pComp, freq, cc);     {admittance of the cap.}
    {add explanatory text}
    pPar^.pIlist^.add('*  capacitor  '+lbl+'  - - - - - - - - - - - - - ');
    {node names}
(*
    NodeNames(n1,namea,nameb,namec);  {upstream node}
    NodeNames(n2,namer,names,namet);  {downstream node}
*)
    PhaseNames(pComp,1, pPar, n1,namea,nameb,namec);
    PhaseNames(pComp,2, pPar, n2,namer,names,namet);
    ImpedanceLines(namea, nameb, namec, namer, names, namet, first,0,0, cc.y*1e6,
                   linea, lineb, linec);       {admittance in uS}
    {add the lines to the output list for impedances}
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
  end;
end;

procedure LoadEMTP(ipPar : longint);
{add the lines for a load in the EMTP *.dat format
 The load is described as an impedance. The size of that impedance
 is calculated using the voltage and current of the load}
var
  cc    : single;        {capacitance}
  cpu   : complex;       {temporary}
  err   : integer;       {error flag}
  first : boolean;       {flag for first impedance}
  Icx   : complex;       {load current}
  ix,iy : integer;       {temporary}
  lbl   : string;        {label (name) of the component}
  linea : string;        {EMTP data for  phase a}
  lineb : string;
  linec : string;
  namea : string;        {upstream node in phase a}
  nameb : string;        {upstream node in phase b}
  namec : string;        {upstream node in phase c}
  namey : string;        {downstream node = neutral node}
  n1,n2 : integer;       {nodes}
  pComp : TpComp;        {pointer to the impedance}
  pPar  : TpEdat;        {pointer to the data}
  Vcx   : complex;       {load voltage}
  rr    : single;        {resistance}
  textb : string;        {temporary}
  textc : string;
  x1    : single;        {temp.reactance}
  xx    : single;        {reactance}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);       {pointer to data}
    first := (pPar^.pIlist.count <= 1); {empty, when first impedance}
    pComp := pPar^.pComp;        {pointer to the impedance}
    pComp^.GetLabel(lbl,ix,iy);  {name}
    pComp^.getNode(1,n1,err);    {upstream node}
    pComp^.getNode(3,n2,err);    {neutral node}
    {get the voltage and current of the load}
    pComp^.GetVar(gLoad_Vr,Vcx.x); {real part of voltage to comp.}
    pComp^.GetVar(gLoad_Vi,Vcx.y); {imag.part of voltage}
    pComp^.GetVar(gLoad_Ir,Icx.x); {real part of current}
    pComp^.GetVar(gLoad_Ii,Icx.y); {imag.part of current}
    {calculate impedance}
    if ((Vcx.x <> 0) or (Vcx.y <> 0)) and ((Icx.x <> 0) or (Icx.y <> 0)) then begin
      cpu := Cdiv(Vcx,Icx);
      if (cpu.x >= 0) then rr := cpu.x    {resistance}
      else                 rr := 0.001;   {some value}
      x1 := cpu.y;                        {reactance, checked later}
    end else begin         {voltage or current zero, must use nominal values}
      NominalLoadPar(pComp, rr,x1);
    end;
    {reactance might also be capacitive}
    if (x1 >= 0) then begin
      xx := x1;
      cc := 0;                   {no capacitances}
    end else begin
      xx := 0;
      cc := -1.e6/x1;            {admittance, in uS}
    end;
    {add explanatory text}
    pPar^.pIlist^.add('*  Load  '+lbl+'  - - - - - - - - - - - - - ');
    {node names}
    NodeNames(n1,namea,nameb,namec);  {upstream node}
    NodeNames(n2,namey,textb,textc);  {downstream node}
    ImpedanceLines(namea, nameb, namec, namey, namey, namey, first,rr, xx, cc,
                   linea, lineb, linec);
    {add the lines to the output list for impedances}
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
  end;
end;

procedure CheckedImedance(rr,xx: single; var r1,x1,c1: single);
{check the impedance and output the inductive and capacitive reactances separately
 rr, xx    resistance, reactance in ohms (in)
 r1,x1,c1  resistance, inductive reactance (ohm), and capacitive admittance (uS)}
begin
  {both cannot be zero}
  if (rr = 0) and (xx = 0) then begin
    r1 := 0.001;
    x1 := 0;
    c1 := 0;
  end else begin
    r1 := rr;
    {reactance might also be capacitive}
    if (xx >= 0) then begin
      x1 := xx;
      c1 := 0;                   {no capacitances}
    end else begin
      x1 := 0;
      c1 := -1.e6/xx;            {admittance, in uS}
    end;
  end;
end;

procedure UnsymmLoadEMTP(ipPar : longint);
{add the lines for an unsymmetrcial load = three different impedances
 in the EMTP *.dat format}
var
  c1,c2,c3 : single;        {capacitive admittance, in uS}
  cpu      : complex;       {temporary}
  err      : integer;       {error flag}
  first    : boolean;       {flag for first impedance}
  ix,iy    : integer;       {temporary}
  lbl      : string;        {label (name) of the component}
  linea    : string;        {EMTP data for  phase a}
  lineb    : string;
  linec    : string;
  namea    : string;        {upstream node in phase a}
  nameb    : string;        {upstream node in phase b}
  namec    : string;        {upstream node in phase c}
  namey    : string;        {downstream node = neutral node}
  n1       : integer;       {nodes}
  pComp    : TpComp;        {pointer to the component}
  pPar     : TpEdat;        {pointer to the data}
  rr       : single;        {temporary resistance}
  r1,r2,r3 : single;        {resistance}
  textb    : string;        {temporary}
  textc    : string;
  xx       : single;        {temporary reactance}
  x1,x2,x3 : single;        {inductive reactance, ohms}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);       {pointer to data}
    first := (pPar^.pIlist.count <= 1); {empty, when first impedance}
    pComp := pPar^.pComp;        {pointer to the impedance}
    pComp^.GetLabel(lbl,ix,iy);  {name}
    pComp^.getNode(1,n1,err);    {upstream node}
    {get the first impedance}
    pComp^.GetPar(1,rr);
    pComp^.GetPar(2,xx);
    CheckedImedance(rr,xx, r1,x1,c1);  {get resist., induct.react., and cap. sucept.}
    {get the second impedance}
    pComp^.GetPar(3,rr);
    pComp^.GetPar(4,xx);
    CheckedImedance(rr,xx, r2,x2,c2);  {get resist., induct.react., and cap. sucept.}
    {get the second impedance}
    pComp^.GetPar(5,rr);
    pComp^.GetPar(6,xx);
    CheckedImedance(rr,xx, r3,x3,c3);  {get resist., induct.react., and cap. sucept.}
    {add explanatory text}
    pPar^.pIlist^.add('*  Unsymmetrical Load  '+lbl+'  - - - - - - - - - - - - - ');
    {node names}
    NodeNames(n1,namea,nameb,namec);  {upstream node}
    NodeNamez(n1,namey,textb,textc);  {downstream node = neutral node}
    ThreeImpedLines(namea, nameb, namec, namey, namey, namey, first,r1,x1,c1,
                    r2,x2,c2,r3,x3,c3,linea, lineb, linec);
    {add the lines to the output list for impedances}
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
  end;
end;

procedure LineEMTP(ipPar : longint);
{add the lines for a transmission line in the EMTP *.dat format
 The line is modeled with a pi-circuit, with no coupling between line}
var
  cap     : single;    {susceptance/km of the capacitances}
  cc      : single;    {checked susceptance in uS}
  cpu     : complex;   {temporary}
{ err     : integer;   {error flag}
  ix,iy   : integer;   {temporary}
  lbl     : string;    {label (name) of the component}
  len     : single;    {length of tr.line (km)}
  linea   : string;    {EMTP data for tr.line in phase a}
  lineb   : string;
  linec   : string;
  namea   : string;    {upstream node in phase a}
  nameb   : string;    {upstream node in phase b}
  namec   : string;    {upstream node in phase c}
  namer   : string;    {downstream node in phase a}
  names   : string;    {downstream node in phase b}
  namet   : string;    {downstream node in phase c}
  n1,n2   : integer;   {nodes}
  pComp   : TpComp;    {pointer to the impedance}
  pPar    : TpEdat;    {pointer to the data}
  par     : single;    {number of parallel lines}
  r1,r0   : single;    {pos., zero seq resistance}
  rii,rij : single;    {real part of diag. and off-diag elements}
  x1,x0   : single;    {pos., zero seq. reactance}
  xii,xij : single;    {imag.part of diag. and off diag. elements}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);       {pointer to data}
    pComp := pPar^.pComp;        {pointer to the tr.line}
    pComp^.GetLabel(lbl,ix,iy);  {name}
    pComp^.GetPar(gTrliS_len,len);   {length of the line (m or km)}
    LineImped(pComp, len, 1.0, cpu); {impedance of the line}
    r1 := cpu.x;
    x1 := cpu.y;
    {capacitance between lines and neutral (ground)}
    pComp^.GetPar(gTrliS_S,par);       {number of parallel lines}
    pComp^.GetPar(gTrliS_S,cap);       {get value of susceptance /km}
    if (par > 0) then cap := cap*par;
    if (cap < 0.0) then cap := 0;
    cc  := Cap*len*1e6;                {total suxc. of capacitances in uS}
    LineZeroImped(pComp, len,cpu);     {zero seq.impedance}
    r0 := cpu.x;
    x0 := cpu.y;
    {convert to matrix elements, user pos.seq. and zero seq.}
    rii := (r0+2*r1)/3.0;
    xii := (x0+2*x1)/3.0;
    rij := (r0-r1)/3.0;
    xij := (x0-x1)/3.0;
    {add explanatory text}
    pPar^.pIlist^.add('*  transmission line   '+lbl+' - - - - - - - - - - - - ');
    {node names}
    PhaseNames(pComp,1, pPar, n1,namea,nameb,namec);
    PhaseNames(pComp,2, pPar, n2,namer,names,namet);
    {generate the lines of text for the EMTP *.dat file}
    inc(pPar^.branch);                 {increment branch counter}
    if (pPar^.branch < 48) then begin  {at most 50 transmission lines (!)}
      linea := ' 1'+namea+namer+'      '+'      '+
               SixChar(rii)+SixChar(xii)+SixChar(cc);
      inc(pPar^.branch);
      lineb := ' 2'+nameb+names+'      '+'      '+
               SixSignedChar(rij)+SixSignedChar(xij)+'      '+
               SixChar(rii)+SixChar(xii)+SixChar(cc);
      inc(pPar^.branch);
      linec :=' 3'+namec+namet+'      '+'      '+
               SixSignedChar(rij)+SixSignedChar(xij)+'      '+
               SixSignedChar(rij)+SixSignedChar(xij)+'      '+
               SixChar(rii)+SixChar(xii)+SixChar(cc);
    end;
    {add the lines to the output list for impedances}
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
  end;
end;

procedure GeneratorEMTP(ipPar : longint);
{add the lines for a generator in the EMTP *.dat format}
const
  BigRes = 1.0e6;        {big resistance, or impedance}
var
  err   : integer;       {error flag}
  freq  : single;        {frequency (50 or 60 Hz)}
  gPar  : GMset;         {generator parameters}
  ipSet : longint;       {pointer to the "common" as integer}
  ix,iy : integer;       {temporary}
  lbl   : string;        {label (name) of the component}
  linea : string;        {one line of data}
  lineb : string;        {one line of data}
  linec : string;        {one line of data}
  name  : string;        {name of component}
  namea : string;        {name of node in phase a}
  nameb : string;        {name of node in phase b}
  namec : string;        {name of node in phase c}
  namey : string;        {name of neutral node}
  nd,nn : integer;       {downstream node, neutral nodes}
  ngp   : single;        {number of parallel generators}
  pComp : TpComp;        {pointer to the component}
  pImped: TpComp;        {pointer to the (eventual) earthing impedance}
  pPar  : TpEdat;        {pointer to the data}
  pSet  : TpCompSet;     {pointer to the "common"}
  Rg    : single;        {the earthing resistance, in series with Xg}
  texta : string;        {temporary}
  textb : string;
  textc : string;
  Xg    : single;        {the earthing reactance, in series with Rg}
  X0    : single;        {zero sequence reactance}
  xs,ys : integer;       {coordinates of the neutral point}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);        {pointer to data}
    ipSet := pPar^.ipSet;         {pointer to the common, as integer}
    pSet := TpCompSet(ipSet);     {pointer to the common}
    pComp := pPar^.pComp;         {pointer to the impedance}
    pComp^.GetLabel(lbl,ix,iy);   {name}
(*  pComp^.getNode(1,nd,err);     {downstream node}*)
    pComp^.getNode(3,nn,err);     {downstream node}
    freq := pSet^.appl_Hz;        {net frequency}
    pComp^.GetName(name);
    if (name = gGen) then begin   {generator}
      GetGenPar(pComp,false,gPar);
    end else begin                {synch. motor}
      GetSMotPar(pComp,false,gPar);
    end;
    ngp := nGenP(pComp);             {number of parallel generators}
    pComp^.GetPar(gGen_X0,X0);       {get zero seq. reactance}
    X0 := X0/ngp;
    if (X0 <= 0) then X0 := 0.001;   {EMTP requires a value > 0}
    {the earthing impedance (if there is one) is a separate component}
    pComp^.getNxy(3,xs,ys);          {coordinates of neutral point}
    FindZg(ipSet, {nn,}xs,ys, pImped);       {find the eventual earthing impedance}
    if (pImped <> nil) then begin    {impedance found}
      pImped^.GetPar(gGrImped_R,Rg); {grounding resistance}
      pImped^.GetPar(gGrImped_X,Xg); {grounding reactance}
    end else begin
      Rg := BigRes;                  {must have some resistance}
      Xg := BigRes;
    end;
    pPar^.pGlist^.add('*  synchronous machine '+lbl+' - - - - - - - - - - - ');
    {node names as text}
    PhaseNames(pComp,1, pPar, nd,namea,nameb,namec);
(*
    NodeNames(nd,namea,nameb,namec);    {gen. output node}
*)
    if (Rg > 0) or (Xg > 0) then begin  {there is an earthing impedance}
      NodeNames(nn,namey,textb,textc);  {name of star point (textb,textc not needed)}
    end else begin
      namey := '      ';                {directly to ground = spaces}
    end;
    {comment line, to mark generator data}
    if (name = gGen) then begin         {generator}
      pPar^.pGlist^.add('*  .   .   .   . Generator, constant speed assumed');
    end else begin
      pPar^.pGlist^.add('*  .   . Synchronous motor, constant speed assumed');
    end;
    {construct first line}
    linea := '50'+namea+namey+TenChar(freq)+TenChar(gPar.Vpp/sqrt(3))+
             TenChar(gPar.fii)+' 0.001    ';
    {second line, base impedance is set to 1 ohm}
    lineb := '  '+nameb+namey+' 1.0      '+' 1.0      ';
    {third line}
    linec := '  '+namec+namey+TenChar(gPar.Rs)+TenChar(gPar.Xd);
    {add these lines to output}
    pPar^.pGlist^.add(linea);
    pPar^.pGlist^.add(lineb);
    pPar^.pGlist^.add(linec);
    {the generator parameters (card 4)}
    texta := TenChar(gPar.Rs)+' 0.001    '+TenChar(gPar.Xd)+TenChar(gPar.Xd)+
             TenChar(gPar.Xdd)+TenChar(gPar.Xqd)+TenChar(gPar.Xd2d)+
             TenChar(gPar.Xq2d);
    textb := '* Ra      Xl        Xd        Xq        X''d       X''q       X"d       X"q';
    pPar^.pGlist^.add(textb);
    pPar^.pGlist^.add(texta);
    {time constants, etc (card 5)}
    texta := TenChar(-gPar.Tdd)+TenChar(-gPar.Tqd)+TenChar(-gPar.Td2d)+
             TenChar(-gPar.Tq2d)+TenChar(X0)+
             ' 0        '+' 0        '+' 0        ';
    textb := '* -T''d    -T''q      -T"d      -T"q      Xo        Ifo       Re        Xc';
    pPar^.pGlist^.add(textb);
    pPar^.pGlist^.add(texta);
    {mechanical data, large inertia}
    pPar^.pGlist^.add('11000000000000');        {voltage and current out}
    pPar^.pGlist^.add('00002000000000000000');  {2 poles}
    pPar^.pGlist^.add('$  blank line to terminate data for this generator');
    {if there is an impedance from the star point to earth, add it to the imped.list}
    if (Rg > 0) or (Xg > 0) then begin  {there is an earthing impedance}
      pPar^.pIlist^.add('*   earthing impedance of generator '+lbl +' - - - - - - - ');
      texta := '  '+namey+'      '+'      '+'      '+SixChar(Rg)+SixChar(Xg);
      pPar^.pIlist^.add(texta);
    end;
  end;
end;

procedure AsynchMotEMTP(ipPar : longint);
{add the lines for an asynchronous motor in the EMTP *.dat format}
var
  apu,apv  : single;     {temporary}
  apw,apx  : single;     {temporary}
  err      : integer;    {error flag}
  errMess  : string;     {error message}
  freq     : single;     {frequency (50 or 60 Hz)}
  ipSet    : longint;    {pointer to the "common" as integer}
  ix,iy    : integer;    {temporary}
  lbl      : string;     {label (name) of the component}
  linea    : string;     {one line of data}
  lineb    : string;     {one line of data}
  linec    : string;     {one line of data}
  namea    : string;     {name of node in phase a}
  nameb    : string;     {name of node in phase b}
  namec    : string;     {name of node in phase c}
  namey    : string;     {name of neutral node}
  nu,nn    : integer;    {upstream node, neutral nodes}
  Pow      : single;     {power}
  pComp    : TpComp;     {pointer to the component}
  pPar     : TpEdat;     {pointer to the data}
  pSet     : TpCompSet;  {pointer to the "common"}
  RL       : single;     {resistance corresponds to load}
  Rs,Rr    : single;     {stastor, rotor resistance}
  slip     : single;     {slip of the motor}
  sn       : single;     {nominla slip}
  texta    : string;     {temporary}
  textb    : string;
  textc    : string;
  Vn       : single;     {nominal phase to phase voltage}
  X1,X2,X3 : single;     {motor impedances}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);        {pointer to data}
    ipSet := pPar^.ipSet;         {pointer to the common, as integer}
    pSet := TpCompSet(ipSet);     {pointer to the common}
    pComp := pPar^.pComp;         {pointer to the impedance}
    pComp^.GetLabel(lbl,ix,iy);   {name}
    pComp^.getNode(1,nu,err);     {upstream node}
    pComp^.getNode(3,nn,err);     {downstream node}
    freq := pSet^.appl_Hz;        {net frequency}
    AsMotPar(pComp, Rs,Rr,X1,X2,X3,Vn,Pow,apu,apv,apw,apx,errMess); {get and calculate parameters}
    {do not care about an eventual error message, it has been handled in Elplex.pas}
    {calculate the  slip}
    pComp^.GetPar(gASmot_sn,sn);           {nominal slip}
    AsMotLoad(Vn,Pow,sn,cplx(Rs,X1),cplx(Rr,X2),cplx(0,X3), RL,slip, err);
    pPar^.pGlist^.add('*  asynchronous machine  '+lbl+'  - - - - - - - - - - - ');
    {node names as text}
    NodeNames(nu,namea,nameb,namec);  {motor input node}
    NodeNames(nn,namey,textb,textc);  {name of star point (textb,textc not needed)}
    {comment line, asynch.motor}
    pPar^.pGlist^.add('*  .   .   .   . Asynchronous motor');
    {construct three first lines}
    linea := '50'+namea+namey+TenChar(freq);
    lineb := '  '+nameb+namey+' 1.0      '+' 1.0      '+
             '                                       +1';  {option: imped. in ohms}
    linec := '  '+namec+namey;
    {add these lines to output}
    pPar^.pGlist^.add(linea);
    pPar^.pGlist^.add(lineb);
    pPar^.pGlist^.add(linec);
    {d-axis impedances}
    texta := TenChar(X1+X3)+TenChar(X3)+' 0        '+' 0        '+TenChar(X1+X3)+
             ' 3.0      '+TenChar(Rr)+' 0        ';
    pPar^.pGlist^.add(texta);     {d-axis, card 4}
    pPar^.pGlist^.add(texta);     {exactly the same for q-axis,card 5}
    {more impedances, extra card after card 5}
    texta := TenChar(Rs)+' 0.001    '+' 0.001    ';
    pPar^.pGlist^.add(texta);
    pPar^.pGlist^.add(' 1');      {card 6, current out}
    texta := '    2    0 1    1   '+TenChar(100*slip)+' 1'; {card 7, design parameters}
    pPar^.pGlist^.add(texta);
    {mass data, output of mechanical var.}
    texta := '            1.0'+'               '+'1.0e+6         '+
             '               '+'               '+'1  1 ';
    pPar^.pGlist^.add(texta);
    {add subtransient reactance, for comparisons}
    apu := X2+X3;
    if (apu > 0) then apv := X1+X2*X3/apu
    else              apv := X1+10;           {just some value}
    texta := '*   subtransient impedance  '+TenChar(Rs)+' +j'+TenChar(apv);
    pPar^.pGlist^.add(texta);
    pPar^.pGlist^.add('$  end of induction motor data');


  end;
end;

procedure FaultEMTP(ipPar : longint);
{add the lines for a fault = switch in the EMTP *.dat format}
var
  err   : integer;       {error flag}
  linea : string;        {EMTP data for impedance in phase a}
  lineb : string;
  linec : string;
  namea : string;        {name of node in phase a}
  nameb : string;        {name of node in phase b}
  namec : string;        {name of node in phase c}
  n1    : integer;       {node}
  pComp : TpComp;        {pointer to the fault}
  pPar  : TpEdat;        {pointer to the data}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);           {pointer to data}
    pComp := pPar^.pComp;            {pointer to the impedance}
    pComp^.getNode(1,n1,err);        {node where fault connected}
    NodeNames(n1,namea,nameb,namec); {node of the fault}
    {generate lines for the fault, assumed from node 1 to earth}
    {closing time set to 1 sec, opening time = 1000 s}
    linea := '  '+namea+'      '+' 1.0      '+' 1000.0   ';
    lineb := '  '+nameb+'      '+' 1.0      '+' 1000.0   ';
    linec := '  '+namec+'      '+' 1.0      '+' 1000.0   ';
    {current margin = 1 (?), switch resistance = 0}
    linea := linea+' 1        '+'          '+' 0        ';
    lineb := lineb+' 1        '+'          '+' 0        ';
    linec := linec+' 1        '+'          '+' 0        ';
    {currents to output}
    linea := linea+'               1';
    lineb := lineb+'               1';
    linec := linec+'               1';
    {write to list of lines}
    pPar^.pSlist^.add('*  three phase fault as switches- - - - - - - - - - - - ');
    pPar^.pSlist^.add(linea);
    pPar^.pSlist^.add(lineb);
    pPar^.pSlist^.add(linec);
    {other faults "commented" away}
    linea := '*'+'  '+namea+'      '+' 1.0      '+' 1000.0   ';
    linea := linea+' 1        '+'          '+' 0        ';
    linea := linea+'               1';
    pPar^.pSlist^.add('*  line to earth fault as a switch- - - - - - - - - - - - ');
    pPar^.pSlist^.add(linea);
    linea := '*'+'  '+namea+nameb+' 1.0      '+' 1000.0   ';
    linea := linea+' 1        '+'          '+' 0        ';
    linea := linea+'               1';
    pPar^.pSlist^.add('*  line to line fault as a switch- - - - - - - - - - - - ');
    pPar^.pSlist^.add(linea);
  end;
end;

procedure OpenLineEMTP(ipPar : longint);
{add the lines for a fault = switch in the EMTP *.dat format}
var
  err   : integer;       {error flag}
  linea : string;        {EMTP data for impedance in phase a}
  lineb : string;
  linec : string;
  namea : string;        {name of node in phase a}
  nameb : string;        {name of node in phase b}
  namec : string;        {name of node in phase c}
  namex : string;        {name of node in phase a}
  namey : string;        {name of node in phase b}
  namez : string;        {name of node in phase c}
  n1,n2 : integer;       {node}
  pComp : TpComp;        {pointer to the fault}
  pPar  : TpEdat;        {pointer to the data}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);           {pointer to data}
    pComp := pPar^.pComp;            {pointer to the impedance}
    pComp^.getNode(1,n1,err);        {node where fault connected}
    NodeNames(n1,namea,nameb,namec); {node of the fault}
    pComp^.getNode(2,n2,err);        {node where fault connected}
    NodeNames(n2,namex,namey,namez); {node of the fault}
    {generate lines for the fault, assumed from node 1 to earth}
    {closing time set to -1 sec, opening time = 1s (phase c) or 1000s (other phases)}
    linea := '  '+namea+namex+'-1.0      '+' 1000.0   ';
    lineb := '  '+nameb+namey+'-1.0      '+' 1000.0   ';
    linec := '  '+namec+namez+'-1.0      '+' 1.0      ';
    {current margin = 1 (?), switch resistance = 0}
    linea := linea+' 1        '+'          '+' 0        ';
    lineb := lineb+' 1        '+'          '+' 0        ';
    linec := linec+' 1        '+'          '+' 0        ';
    {currents to output}
    linea := linea+'               1';
    lineb := lineb+'               1';
    linec := linec+'               1';
    {write to list of lines}
    pPar^.pSlist^.add('*  three phase fault as switches- - - - - - - - - - - - ');
    pPar^.pSlist^.add(linea);
    pPar^.pSlist^.add(lineb);
    pPar^.pSlist^.add(linec);
  end;
end;

procedure SourceEMTP(ipPar : longint);
{add the lines for a voltage source, eventually with some internal impedance.
 Note: The earthing impedance is not included (not possible in EMTP)}
var
  cc    : single;        {capacitance}
  cpu   : complex;
  err   : integer;       {error flag}
  fii   : single;        {phase}
  first : boolean;       {flag for first impedance}
  freq  : single;        {frequency}
  ipset : integer;       {pointer to the common, as integer}
  ix,iy : integer;       {temporary}
  lbl   : string;        {label (name) of the component}
  linea : string;        {EMTP data for impedance in phase a}
  lineb : string;
  linec : string;
  namea : string;        {name of node in phase a}
  nameb : string;        {name of node in phase b}
  namec : string;        {name of node in phase c}
  namer : string;        {name of source node in phase a}
  names : string;        {name of source node in phase b}
  namet : string;        {name of source node in phase c}
  n1,n2 : integer;       {input node, current "node"}
  pComp : TpComp;        {pointer to the component}
  pPar  : TpEdat;        {pointer to the data}
  pSet  : TpCompSet;     {pointer to the common}
  rr    : single;        {internal resistance}

  Vpn   : single;        {phase to neutral voltage}
  xx    : single;        {internal reactance}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);           {pointer to data}
    ipSet := pPar^.ipSet;            {pointer to the common, as integer}
    pSet := TpCompSet(ipSet);        {pointer to the common}
    pComp := pPar^.pComp;            {pointer to the impedance}
    pComp^.GetLabel(lbl,ix,iy);      {name}
    pComp^.getNode(1,n1,err);        {node where source connected}
    NodeNames(n1,namea,nameb,namec); {node of the source}
    {get parameters}
    pComp^.GetVar(gFeed_V1r,cpu.x);
    pComp^.GetVar(gFeed_V1i,cpu.y);
    Vpn := Cabs(cpu);
    fii := 180*Cangle(cpu)/pi;
    pComp^.GetPar(gACg_R,rr);        {get internal resistance}
    pComp^.GetPar(gACg_X,xx);        {get internal reactance}
    freq := pSet^.Appl_Hz;           {frequency}
    cc := 0;
    {if the internal impedance is nonzero, must add additional nodes}
    if (xx > 0) or (rr > 0) then begin
      pComp^.getNode(2,n2,err);        {"node" for current}
      NodeNames(n2,namer,names,namet); {extra node, if nonzero internal impedance }
     {add the impecance to the network}
      first := (pPar^.pIlist.count <= 1); {empty, when first impedance}
      ImpedanceLines(namer,names,namet,namea,nameb,namec, first, rr,xx,cc,
                     linea, lineb,linec);
      {add the lines to the output list for impedances}
      pPar^.pIlist^.add(linea);
      pPar^.pIlist^.add(lineb);
      pPar^.pIlist^.add(linec);
    end else begin
      namer := namea;
      names := nameb;
      namet := namec;
    end;
    {generate lines for source}
    pPar^.pVList^.add('* node  V ampl.     freq.     phase     ph=degr.  '+
                'not used  Tstart    Tstop');   {comment = explanations}
    linea := '14'+namer+'  '+TenChar(Vpn)+TenChar(freq)+TenChar(fii)+
             '          '+'          '+' 0.0      '+' 32.0     ';   {long enough ?}

    lineb := '14'+names+'  '+TenChar(Vpn)+TenChar(freq)+TenChar(fii-120.0)+
             '          '+'          '+' 0.0      '+' 32.0     ';
    linec := '14'+namet+'  '+TenChar(Vpn)+TenChar(freq)+TenChar(fii+120.0)+
             '          '+'          '+' 0.0      '+' 32.0     ';
    pPar^.pVlist^.add('*  feeder  '+lbl+'  - - - - - - - - - - - ');
    pPar^.pVList^.add(linea);
    pPar^.pVList^.add(lineb);
    pPar^.pVlist^.add(linec);
  end;
end;

procedure TrafoEMTP(ipPar : longint);
{add the lines for a transformator of any  type.
 Par.text contains the type description  (YY, YD, DY, DD)
 Note: EMTP does not accept a zero reactance (X) }
var
  apu,apv : single;    {temporary}
  apw     : single;    {temporary}
  cc11    : complex;   {prim.impedance, R11+jwL11}
  cc12    : complex;   {mutual impedance, R12+jwL12 (R12 = 0)}
  cc22    : complex;   {sec.impedance, R22+jwL22}
  err     : integer;   {error flag}
  first   : boolean;   {first line in impedance list}
  ipset   : integer;   {pointer to the common, as integer}
  ix,iy   : integer;   {temporary}
  lbl     : string;    {label of the component}
  linea   : string;    {one line of EMTP data}
  lineb   : string;
  linec   : string;
  L11,L22 : single;    {prim. and sec. inductances}
  L12     : single;    {mutual inductance}
  namea   : string;    {name of prim node in phase a}
  nameb   : string;    {name of prim node in phase b}
  namec   : string;    {name of prim node in phase c}
  namer   : string;    {name of sec node in phase a}
  names   : string;    {name of sec node in phase b}
  namet   : string;    {name of sec node in phase c}
  namea1  : string;    {name of first  node in prim. a-winding}
  namea2  : string;    {name of second node in prim. a-winding}
  nameb1  : string;    {name of first  node in prim. b-winding}
  nameb2  : string;    {name of second node in prim. b-winding}
  namec1  : string;    {name of first  node in prim. c-winding}
  namec2  : string;    {name of second node in prim. c-winding}
  namer1  : string;    {name of first  node in prim. a-winding}
  namer2  : string;    {name of second node in sec. a-winding}
  names1  : string;    {name of first  node in sec. b-winding}
  names2  : string;    {name of second node in sec. b-winding}
  namet1  : string;    {name of first  node in sec. c-winding}
  namet2  : string;    {name of second node in sec. c-winding}
  namyp   : string;    {name of star point node in prim.side}
  namys   : string;    {name of star point node in sec. side}
  n1,n2   : integer;   {node}
  omega   : single;    {angular frequency}
  pComp   : TpComp;    {pointer to the component}
  pPar    : TpEdat;    {pointer to the data}
  pSet    : TpCompSet; {pointer to the common}
//r01     : single;    {zero seq. impedance / pos.seq. impedance}
  R0      : single;    {zero seq. resistance}
  R1e     : single;    {R1 for emtp}
  R1,R2   : single;    {prim. and sec. resistance}
  R1g,R2g : single;    {prim and sec. resistances to ground, in series with X1g,X2g}
  texta   : string;
  textb   : string;
  textc   : string;
  trType  : string;    {trafo type YY, YD, DY, DD}
  V1,V2   : single;    {prim, sec. phase to phase voltages}
  XL      : single;    {(total)leakage reactance =x1+a*a*x2}
  X0      : single;    {zero seq. reactance}
  X1g,X2g : single;    {prim and sec. reactances to ground, in series with R1g,R2g}
  X11,X22 : single;    {prim., sec. reactances}
  X12     : single;    {mutual reactance}
begin
  if (ipPar <> 0) then begin
    pPar := TpEdat(ipPar);           {pointer to data}
    trType := pPar^.text;            {type of trafo}
    ipSet := pPar^.ipSet;            {pointer to the common, as integer}
    pSet := TpCompSet(ipSet);        {pointer to the common}
    omega := 2*pi*pSet^.appl_Hz;     {angular freq.}
    pComp := pPar^.pComp;            {pointer to the component}
    pComp^.GetLabel(lbl,ix,iy);      {get the label (name) of the component}
    first := (pPar^.pIlist.count <= 1);  {needed for (eventual) output flag}
(*
    pComp^.getNode(1,n1,err);        {primary node}
    NodeNames(n1,namea,nameb,namec); {names of prim.nodes}
    pComp^.getNode(2,n1,err);        {sec. node}
    NodeNames(n1,namer,names,namet); {names of sec.nodes}
*)
    PhaseNames(pComp,1, pPar, n1,namea,nameb,namec);
    PhaseNames(pComp,2, pPar, n2,namer,names,namet);
    {get impedances}
    Trafoimp(pComp, omega,trType,true,false,false,false, false, false,
             cc11,cc12,cc22,R1,R2,L11,L12,L22,XL,V1,V2);
    pPar^.pIlist^.add('*  Transformer  '+lbl+' - - - - - - - - - - ');
    {get grounding impedance = read parameters. First, the primary side}
    if (trType = 'YY') or (trType = 'YD') then begin
      pComp^.GetPar(gTrafo_R1g,R1g);   {prim. resistance to ground}
      pComp^.GetPar(gTrafo_X1g,X1g);   {prim. reactance to ground}
//    pComp^.GetPar(gTrafo_01,r01);    {zero seq. imped / pos.seq.imped}
      pComp^.GetPar(gTrafo_R0,R0);     {zero seq. resistance}
      pComp^.GetPar(gTrafo_X0,X0);     {zero seq. reactance}
//    if (r01 <= 0) then r01 := 1;
      {if the grounding impedance is > 0, add the grounding branch}
      {zero seq. impedance included in grounding impedance, if <> pos.seq.impedance}
      if (R1g > 0) or (X1g > 0) or (R0 > 0) or (X0 > 0) then begin {there is some impedance}
        pComp^.getNode(5,n1,err);          {neutral node on primary side}
        NodeNames(n1,namyp,textb,textc);   {name of prim.neutral node}
        {produce the line for the impedance (lines for phases b and c not needed)}
        apu := R1g+(R0-R1)/3;           {include zero seq. imped. into grounding imped}
        apv := X1g+(X0-XL)/3;
        ImpedanceLines(namyp,namyp,namyp,'      ','      ','      ',
                       first, apu,apv{R1g,X1g},0,texta, textb,textc);
        pPar^.pIlist^.add(texta);
      end else begin
        {zero grounding impedance => node has no name}
        namyp := '      ';               {connected directly to ground}
      end;
    end else begin
      R1g := 1.e6;
      X1g := 1.e6;
    end;
    {Then, the secondary side}
    if (trType = 'YY') or (trType = 'DY') then begin
      pComp^.GetPar(gTrafo_R2g,R2g);     {sec.  resistance to ground}
      pComp^.GetPar(gTrafo_X2g,X2g);     {sec.  reactance to ground}
      if (trType = 'DY') then begin      {YY checked on the primary side, above}
        pComp^.GetPar(gTrafo_R0,R0);     {zero seq. resistance}
        pComp^.GetPar(gTrafo_X0,X0);     {zero seq. reactance}
//      pComp^.GetPar(gTrafo_01,r01);    {zero seq. imped / pos.seq.imped}
//      if (r01 <= 0) then r01 := 1;
//    end else begin
//      r01 := 1;
      end;
//    if (R2g > 0) or (X2g > 0) or (r01 <> 1) then begin
      if ((trType = 'YY') and ((X2g > 0) or (R2g > 0))) or
         ((trType = 'YY') and ((X2g > 0) or (R2g > 0) or (R0 > 0) or (X0 > 0))) then begin
        pComp^.getNode(6,n1,err);        {neutral node on sec. side}
        NodeNames(n1,namys,textb,textc); {name of sec. neutral node}
        {produce the line for the impedance (lines for phases b and c not needed)}
        apw := V2/V1; apw := apw*apw;
        if (trType = 'YY') then begin
          apu := R2g;
          apv := X2g;
        end else begin
//        apu := R2g+(r01-1)*R1*apw/3;        {include zero seq. imped. into grounding imped}
//        apv := X2g+(r01-1)*XL*apw/3;
          apu := R2g+(R0-R1)*apw/3;           {include zero seq. imped. into grounding imped}
          apv := X2g+(X0-XL)*apw/3;
        end;
        ImpedanceLines(namys,namys,namys,'      ','      ','      ',
                       first, apu,apv{R2g,X2g},0,texta, textb,textc);
        pPar^.pIlist^.add(texta);
      end else begin
        {drectly to groung => no node}
        namys := '      ';
      end;
    end else begin
      R2g := 1.e6;
      X2g := 1.e6;
    end;
    {if the leakage reactance is > 0 (or > a small number?), the reactances can
     be used as such. But if the leakage reactance is zero (or very small?),
     the mutual reactance must be reduced slightly, because EMTP does not
     accept a zero leakage reactance}
    X11 := cc11.y;                     {prim.reactance, use as such}
    X22 := cc22.y;                     {sec. reactance, use as such}
    if (XL > 0) then X12 := cc12.y
    else             X12 := cc12.y*0.9999;      {enough non-ideality ?}
    {the names of the nodes at the ends of the windings depend on the type of trafo}
    {primary windings}
    if (trType = 'YY') or (trType = 'YD') then begin
      namea1 := namea;
      namea2 := namyp;
      nameb1 := nameb;
      nameb2 := namyp;
      namec1 := namec;
      namec2 := namyp;
    end else if (trType = 'DY') then begin
      namea1 := namea;
      namea2 := namec;
      nameb1 := nameb;
      nameb2 := namea;
      namec1 := namec;
      namec2 := nameb;
    end else begin               {'DD'}
      namea1 := namea;
      namea2 := nameb;
      nameb1 := nameb;
      nameb2 := namec;
      namec1 := namec;
      namec2 := namea;
    end;
    {secondary windings}
    if (trType = 'YY') or (trType = 'DY') then begin
      namer1 := namer;
      namer2 := namys;
      names1 := names;
      names2 := namys;
      namet1 := namet;
      namet2 := namys;
    end else begin      {YD or DD}
      namer1 := namer;
      namer2 := names;
      names1 := names;
      names2 := namet;
      namet1 := namet;
      namet2 := namer;
    end;
    {must change R1 to R of D-coupled winding}
    if ((trType = 'DD') or (trType = 'DY')) then R1e := 3*R1 else R1e := R1;
    {generate the lines for the trafo. Use the "higher input accuracy"-option}
    {phase a}
    linea := '51'+namea1+namea2 +'      '+'      '+FifteenChar(R1e)+'  '+
             FifteenChar(X11)+'  ';
    lineb := '52'+namer1+namer2 +'      '+'      '+'                 '+
             FifteenChar(X12)+'  ';
    linec := '  '+'      '+'      '+'      '+'      '+FifteenChar(R2)+'  '+
             FifteenChar(X22)+'  ';
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
    {phase b similarly}
    linea := '51'+nameb1+nameb2 +'      '+'      '+FifteenChar(R1e)+'  '+
             FifteenChar(X11)+'  ';
    lineb := '52'+names1+names2 +'      '+'      '+'                 '+
             FifteenChar(X12)+'  ';
    linec := '  '+'      '+'      '+'      '+'      '+FifteenChar(R2)+'  '+
             FifteenChar(X22)+'  ';
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
    {phase c similarly}
    linea := '51'+namec1+namec2 +'      '+'      '+FifteenChar(R1e)+'  '+
             FifteenChar(X11)+'  ';
    lineb := '52'+namet1+namet2 +'      '+'      '+'                 '+
             FifteenChar(X12)+'  ';
    linec := '  '+'      '+'      '+'      '+'      '+FifteenChar(R2)+'  '+
             FifteenChar(X22)+'  ';
    pPar^.pIlist^.add(linea);
    pPar^.pIlist^.add(lineb);
    pPar^.pIlist^.add(linec);
    {add a large impedance to the D side, to be sure. Trafo must not be floating}
    if (trType = 'DY') or (trType = 'DD') then begin
      ImpedanceLines(namea,namea,namea,'      ','      ','      ',
                     first, 1.e6,0,0,texta, textb,textc); {textb,textc not needed}
        pPar^.pIlist^.add(texta);
    end;
    if (trType = 'YD') or (trType = 'DD') then begin
      ImpedanceLines(namer,namer,namer,'      ','      ','      ',
                     first, 1.e6,0,0,texta, textb,textc); {textb,textc not needed}
        pPar^.pIlist^.add(texta);
    end;
  end;
end;

procedure TrafoYYEMTP(ipPar : longint);
{add the lines for a transformator of YY-type.
 It is modelled as three single phase transformators connected in stars (YY)
 Note: 1) EMTP does not accept a zero reactance (X)
       2) There is a finite impedance from the star point to earth}
var
  pPar    : TpEdat;    {pointer to the data}
begin
  pPar := TpEdat(ipPar);
  pPar^.text := 'YY';
  TrafoEMTP(ipPar);
end;

procedure TrafoYDEMTP(ipPar : longint);
{add the lines for a transformator of YD-type.}
var
  pPar    : TpEdat;    {pointer to the data}
begin
  pPar := TpEdat(ipPar);
  pPar^.text := 'YD';
  TrafoEMTP(ipPar);
end;

procedure TrafoDYEMTP(ipPar : longint);
{add the lines for a transformator of DY-type.}
var
  pPar    : TpEdat;    {pointer to the data}
begin
  pPar := TpEdat(ipPar);
  pPar^.text := 'DY';
  TrafoEMTP(ipPar);
end;

procedure TrafoDDEMTP(ipPar : longint);
{add the lines for a transformator of DD-type.}
var
  pPar    : TpEdat;    {pointer to the data}
begin
  pPar := TpEdat(ipPar);
  pPar^.text := 'DD';
  TrafoEMTP(ipPar);
end;

procedure FindStar(pComp : TpComp; StarList : TList);
{find eventual star (or wye) termination component connected to this component
 pComp    pointer to the component
 StarList list of pointers to the star-components}
var
  errn      : integer;   {error flag}
  ii,jj     : integer;   {index}
  nnc       : integer;   {(max) no of input / output nodes}
  nnod      : integer;   {number of nodes in this component}
  node      : integer;   {node number (ID) in the network}
  nodst     : integer;   {node of the star}
  nStar     : integer;   {no of stars}
  pStar     : TpComp;    {pointer to a star}
  StarFound : boolean;   {true, when a connected star found}
begin
  pComp^.GetNnod(nnod);        {no of nodes in this component}
  {check at most 3 nodes}
  if (nnod < 3) then nnc := nnod else nnc := 3;
  if (nnc > 0) then begin      {check nodes, if there are any}
    nStar := StarList.count;   {no of stars}
    StarFound := false;
    for ii := 1 to nnc do begin
      pComp^.GetNode(ii, node, errn);
      for jj := 0 to nStar-1 do begin   {stars in the list}
        pStar := StarList[jj];
        pStar^.GetNode(1,nodst,errn);   {first node}
        if (node = nodst) then begin    {comp. connected to this star}
          if      (ii = 1) then pComp^.int1_s := jj
          else if (ii = 2) then pComp^.int2_s := jj
          else if (ii = 3) then pComp^.int3_s := jj;
          StarFound := true;
          break;                        {one star is enough}
        end else begin
          StarFound := false;
        end;
      end;
      if (StarFound) then begin         {one star for one comp. considered}
        break;
      end;
    end;
  end;
end;

procedure EmtpFile(ipSet : longint; FileName : string);
{produce the EMTP-file of the diagram
 ipSet     pointer to the "common" as integer
 FileName  name of file where data is written}
var
  EMTPdat   : TEdat;         {for transferring the parameters}
  EmtpPro   : TCompProc;     {procedure for the *.dat file of each type of comp}
  errc      : integer;       {error flag}
  EarthList : TList;         {list of earthing components}
  FaultList : TStringList;   {lines for the fault(s) = swithc(es) to the *.dat file}
  GenerList : TStringList;   {lines for the *.dat file from generators}
  ii        : integer;       {index}
  ImpedList : TStringList;   {lines for the *.dat file from impedances}
  jcomp     : integer;       {counts components}
  len       : integer;       {length of text}
  line      : string;
  name      : string;        {name (type) of the component}
  nStar     : integer;       {no of terminating stars (wyes)}
  Outfile   : TextFile;      {file where written}
  pComp     : TpComp;        {pointer to a component}
  pSet      : TpCompSet;     {pointer to the "common"}
  SourcList : TStringList;   {lines for sources}
  StarList  : TList;         {list of wye-termination comp}
  text      : string;
begin
  pSet := TpCompSet(ipSet); {pointer from integer}
//if (pSet^.p1SchComp <> nil) then begin   {there are components}
  if (pSet^.pCompList.count > 0) then begin {there are components}
    GenerList := TStringList.create;
    ImpedList := TStringList.create;
    FaultList := TStringList.create;
    SourcList := TStringList.create;
    StarList  := TList.Create;       {list of wye-termination components}
    EarthList := TList.Create;       {list oe earthing impedances}
    {determine nodes (to be sure) = "Click" button "NodeBut"on form "Imagoo"}
(*  Imagoo.NodeButClick(Imagoo.NodeBut); {TURHA, koska laskettava ensin kuitenkin !} *)
    {collect data for the *.dat procedure}
    EmtpDat.pIlist := @ImpedList;
    EmtpDat.pGlist := @GenerList;
    EmtpDat.pSlist := @FaultList;
    EmtpDat.pVlist := @SourcList;
    EmtpDat.StarList := StarList;
    EmtpDat.EarthList := EarthList;
    EmtpDat.ipSet := ipSet;              {pointer to the "common"}
    EmtpDat.branch := 0;                 {initialize for transmission lines}
    {scan all components and search for stars and earthing impedances}
    pComp := nil;
    errc := 0;
    jcomp := 0;
    while(errc = 0) do begin                 {scan all components}
      pSet^.NextComp(pComp,errc,jcomp,true); {get pointer to next component}
      if (errc = 0) then begin
(*
        EmtpDat.pComp := pComp;      {copy also the pointer to the component}
        pComp^.GetEMTPro(EmtpPro);   {get procedure for constructing *dat file}
        EmtpPro(longint(@EmtpDat));  {*.dat lines of this component}
*)
        {fill lists of Stars and earthing imp.}
        pComp^.GetName(name);
        if (name = gStar) then begin
          StarList.add(pComp);
        end else if (name = gGrImped) then begin
          EarthList.add(pComp);
          pComp^.int1_s := 0;         {reset, used as a flag}
        end else begin
          pComp^.int1_s := -1;        {initialize "pointes" to the star comp.}
          pComp^.int2_s := -1;
          pComp^.int3_s := -1;
        end;
      end;
    end;
    {find connections to the star components}
    nStar := StarList.Count;         {no of stars}
{   nEarth := EarthList.Count;       {no of earthing imp.}
    {scan all components, check connection to stars and write *.dat lines}
    pComp := nil;
    errc := 0;
    jcomp := 0;
    while(errc = 0) do begin                 {scan all components}
      pSet^.NextComp(pComp,errc,jcomp,true); {get pointer to next component}
      if (errc = 0) then begin
        if (nStar > 0) then begin     {are there stars}
          FindStar(pComp, StarList);  {connected to this component ?}
        end;
        EmtpDat.pComp := pComp;      {copy also the pointer to the component}
        pComp^.GetEMTPro(EmtpPro);   {get procedure for constructing *dat file}
        EmtpPro(longint(@EmtpDat));  {*.dat lines of this component}
      end;
    end;
    {write the collected data to file}
    {open the given file,BUT ERRORS ?}
    AssignFile(Outfile, FileName{'C:\Temp\emtp.txt'});
    rewrite(outfile);
    {the title, but at most 66 char. long}
    text := 'Title: '+Imagoo.NetTitle.text;
    len := length(text);
    if (len <  66) then begin         {too short, must add spaces}
      line := text;
      for ii := len+1 to 66 do begin
        line := line+' ';
      end;
    end else if (len = 66) then begin {exactly right length}
      line := text;
    end else begin                    {too long, must cut}
      line := '';
      for ii := 1 to 66 do begin
        line := line+text[ii];
      end;
    end;
    {option: save for plotting}
    line := line+' 0';
    {option: inductances as impedances in ohms and capacitances in uS}
    line := line+SixChar(pSet^.appl_hz)+SixChar(pSet^.appl_hz);
    writeln(outfile,line);            {write the title of the network+options}
    {time step, time, saving options. Just some reasonable values}
    writeln(outfile,'* step    max.time  out''h plot''h');
    writeln(outfile,'50.0e-6   2.00      10    5');
    writeln(outfile,'* impedances');
    {impedances}
    if (ImpedList.count > 1) then begin
      for ii := 0 to ImpedList.count-1 do begin
        writeln(outfile,ImpedList.Strings[ii]);
      end;
    end;
    { a 'blank' line terminates branch data}
    writeln(outfile,'$  end of branch data ');
    {swithc(es)}
    {start with an explanatory text}
    if (FaultList.count > 1) then begin
      line := '* node  node  tclose    topen     curr.marg empty     R on      '+
              '               o';
      writeln(outfile,line);
      for ii := 0 to FaultList.count-1 do begin
        writeln(outfile,FaultList.Strings[ii]);
      end;
    end;
    {"empty" line, in any case, to indicate end of switch data}
    writeln(outfile,'$ end of switch data');
    {sources}
    writeln(outfile,'*   Sources ');
    if (SourcList.count > 1) then begin
      for ii := 0 to SourcList.count-1 do begin
        writeln(outfile,SourcList.Strings[ii]);
      end;
    end;
    {generators}
    if (GenerList.count > 1) then begin
      for ii := 0 to GenerList.count-1 do begin
        writeln(outfile,GenerList.Strings[ii]);
      end;
    end;
    writeln(outfile,'$ end of machine data - - - - - - - - - - - - ');
    {voltage output}
    writeln(outfile,'* voltage saved in following nodes');
    writeln(outfile,'* node1 node2 node3 etc.  six charaters/node ');
    writeln(outfile,'  ');
    writeln(outfile,'$ end of voltage output');
    {end of data}
    writeln(outfile,'$ end of data');
    CloseFile(Outfile);
    {finally, free the lists}
    GenerList.free;
    ImpedList.free;
    FaultList.free;
    SourcList.free;
    StarList.free;
    EarthList.free;
  end;
end;


end.
