unit GenProc;
{ Diverse procedures for the generator = GenMot class}
interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls,
  Shortcir, Utildef,matmul;

procedure PutFourvar(ii,nn: integer; pFour : pFourvar;  x,y,z,t : single);
procedure GetFourvar(ii,nn: integer; pFour : pFourvar; var x,y,z,t : single);
procedure ClearEqu(paa,pbb,pww : pSingle; pIndx : pInteger; ngm : integer);
procedure ClearDequ(paa,pbb,pww : pDouble; pIndx : pInteger; ngm : integer);
procedure getequ(var paa,pbb,pww : pSingle; var pIndx : pInteger; ngm : integer);
procedure getDequ(var paa,pbb,pww : pDouble; var pIndx : pInteger; ngm : integer);
procedure freeEqu(paa,pbb,pww : pSingle; pIndx : pInteger; ngm : integer);
procedure freeDequ(paa,pbb,pww : pDouble; pIndx : pInteger; ngm : integer);
procedure ZeroDiskr(Xd,Xdpp,Rf,Rd,l1,l2: single;
                    var mm1,mm2, a1,a2,b11,b12,b21,b22 : single);
procedure diskriminant(Xd,Xdpp,Rf,Rd,mm,l1,l2: single; var a1,a2,b1,b2,diskr : single);
procedure inductances(Xd,Xdp,Xdpp,Rf,Rd,l1,l2,omega,a1,a2,b1,b2,diskr,mm : single;
                      var Lffp,Lddp,Lfdp,Mfp,Mdp,c1 : single);
procedure SolveInductances(Xd,Xdp,Xdpp,Tdp,Tdpp,Rf,Rd,freq :single;
                          var Lff,Ldd,Lfd,Mf,Md: single; var err : integer);
procedure SolvePhysPar(Xd,Xdp,Xdpp,Tdp,Tdpp,Rf,Rd,MdpMf,freq :single;
                   var sLff,sLdd,sLfd,sMf,sMd: single);
procedure SolveGenPar(Par : GMset; freq : single; var PP : PhysPar);



implementation

procedure PutFourvar(ii,nn: integer; pFour : pFourvar;  x,y,z,t : single);
{save two connected variables
 ii   index of the variables
 nn   dimension = max no of variables
 pTwo pointer to first set of variables
 x,y,z,t the variables}
var
  pT   : pFourvar;     {pointer to three variables}
begin
  if (ii>0) and (ii<=nn) and (nn>0) then begin  {reasonable index}
    pT := pFour;                                 {pointer to first variable}
    inc(pT,ii-1);                               {pointer to variable no ii}
    pT^.x := x;
    pT^.y := y;
    pT^.z := z;
    pT^.t := t;
  end;
end;

procedure GetFourvar(ii,nn: integer; pFour : pFourvar; var x,y,z,t : single);
{get three connected variables
 ii   index of the variables
 nn   dimension = max no of variables
 pTre pointer to first set of variables
 x,y,z,t the variables}
var
  pT   : pFourvar;     {pointer to two variables}
  Tv   : Fourvar;      {two variables}
begin
  if (ii>0) and (ii<=nn) and (nn>0) then begin  {reasonable index}
    pT := pFour;                                {pointer to first variable}
    inc(pT,ii-1);                               {pointer to variable no ii}
    Tv := pT^;                                  {get the variables}
    x := Tv.x;
    y := Tv.y;
    z := Tv.z;
    t := Tv.t;
  end else begin                                {something wrong}
    x := 0;
    y := 0;
    z := 0;
    t := 0;
  end;
end;

procedure ClearEqu(paa,pbb,pww : pSingle; pIndx : pInteger; ngm : integer);
{clear the matrices etc. for the equations}
var
  i,j : integer;
begin
  if (paa <> nil) and (pbb <> nil) and (pww <> nil) and (pIndx <> nil) and
     (ngm > 0) then begin
     for i:=1 to ngm do begin
       for j:=1 to ngm do begin
         putmat(paa,i,j,ngm,ngm,0.0);
       end;
       putvec(pbb,i,ngm,0.0);
       putvec(pww,i,ngm,0.0);
       putImat(pIndx,i,1,ngm,1,0);
     end;
  end;
end;

procedure ClearDequ(paa,pbb,pww : pDouble; pIndx : pInteger; ngm : integer);
{clear the matrices etc. for the equations}
var
  i,j : integer;
begin
  if (paa <> nil) and (pbb <> nil) and (pww <> nil) and (pIndx <> nil) and
     (ngm > 0) then begin
     for i:=1 to ngm do begin
       for j:=1 to ngm do begin
         putDmat(paa,i,j,ngm,ngm,0.0);
       end;
       putDvec(pbb,i,ngm,0.0);
       putDvec(pww,i,ngm,0.0);
       putImat(pIndx,i,1,ngm,1,0);
     end;
  end;
end;

procedure getequ(var paa,pbb,pww : pSingle; var pIndx : pInteger; ngm : integer);
{reserve memory for a set of equations, and initialize to zero
 paa    pointer to coeff. matrix
 pbb    pointer to right hand side
 pww    pointer to working space
 pIndx  pointer to indices of row exchanges
 ngm    number  of equations
 }
 var
   nn2   : integer;   { no of equations squared}
   sofs  : integer;   {size of single}
 begin
   if (ngm > 0) then begin
     try begin
       nn2 := ngm*ngm;
       sofs := sizeof(single);
       getmem(paa,nn2*sofs);              {coeff. matrix}
       getmem(pbb,ngm*sofs);              {right hand side}
       getmem(pww,ngm*sofs);              {working space}
       getmem(pIndx,ngm*sizeOf(integer)); {index for row exchanges}
       {clear all initially}
       ClearEqu(paa,pbb,pww,pIndx,ngm);
     end
       except begin
         paa := nil;
         pbb := nil;
         pww := nil;
         pIndx := nil;
       end;
     end;
   end else begin
     paa := nil;
     pbb := nil;
     pww := nil;
     pIndx := nil;
   end;
 end;

procedure getDequ(var paa,pbb,pww : pDouble; var pIndx : pInteger; ngm : integer);
{reserve memory for a set of equations, and initialize to zero
 paa    pointer to coeff. matrix
 pbb    pointer to right hand side
 pww    pointer to working space
 pIndx  pointer to indices of row exchanges
 ngm    number  of equations
 }
 var
   nn2   : integer;   { no of equations squared}
   sofd  : integer;   {size of double}
 begin
   if (ngm > 0) then begin
     try begin
       nn2 := ngm*ngm;
       sofd := sizeof(double);
       getmem(paa,nn2*sofd);              {coeff. matrix}
       getmem(pbb,ngm*sofd);              {right hand side}
       getmem(pww,ngm*sofd);              {working space}
       getmem(pIndx,ngm*sizeOf(integer)); {index for row exchanges}
       {clear all initially}
       ClearDequ(paa,pbb,pww,pIndx,ngm);
     end
       except begin
         paa := nil;
         pbb := nil;
         pww := nil;
         pIndx := nil;
       end;
     end;
   end else begin
     paa := nil;
     pbb := nil;
     pww := nil;
     pIndx := nil;
   end;
 end;

procedure freeEqu(paa,pbb,pww : pSingle; pIndx : pInteger; ngm : integer);
{frees the memory reserved for equations}
var
  nn2  : integer;
  sofs : integer;
begin
  sofs := sizeof(single);
  nn2 := ngm*ngm;
  if (paa   <> nil) then freemem(paa,nn2*sofs);
  if (pbb   <> nil) then freemem(pbb,ngm*sofs);
  if (pww   <> nil) then freemem(pww,ngm*sofs);
  if (pIndx <> nil) then freemem(pIndx,ngm*SizeOf(integer));
  paa := nil;
  pbb := nil;
  pww := nil;
  pIndx := nil;
end;

procedure freeDequ(paa,pbb,pww : pDouble; pIndx : pInteger; ngm : integer);
{frees the memory reserved for equations}
var
  nn2  : integer;
  sofd : integer;
begin
  sofd := sizeof(double);
  nn2 := ngm*ngm;
  if (paa   <> nil) then freemem(paa,nn2*sofd);
  if (pbb   <> nil) then freemem(pbb,ngm*sofd);
  if (pww   <> nil) then freemem(pww,ngm*sofd);
  if (pIndx <> nil) then freemem(pIndx,ngm*SizeOf(integer));
end;

procedure ZeroDiskr(Xd,Xdpp,Rf,Rd,l1,l2: single;
                    var mm1,mm2, a1,a2,b11,b12,b21,b22 : single);
{calculate mm for which discriminant is zero
  Xd    reactance
  Xdpp  subtransient reactance
  Rf,Rd resistances of field and damper windings
  l1,l2 time constants
  mm1,mm2  related to mutual inductances, giving zero discriminant
  a1,a2,b11,b12,b21,b22 coefficients for A and C from B}
var
  aa,bb,cc  : single; {coefficients for eq.}
  apu, apv  : single; {temporary   }
  apw       : single; {temporary   }
  bet       : single; {coefficient }
  gam       : single; {coefficient }
  kk1,kk2   : single; {short hand notation for x-x"* .... /...}
begin
  if (Rf <> Rd) and (Rf <> 0) and (Rd <> 0) then begin
    a1 := 2*Rf/(Rf-Rd);
    a2 := 2*Rd/(Rd-Rf);
    bet := Rf*Rd*(l1+l2)/(Rf-Rd);
    apu := Rf+Rd;
    apv := Rf-Rd;
    gam := 4*apu*apu*l1*l2*Rf*Rd/(apv*apv);
    apv := a1+a2;
    aa := a1*a2;
    bb := 2*bet*(a1-a2);
    cc := bet*bet*(apv*apv-4)-gam;
    {discriminant}
    apw := bb*bb-4*aa*cc;
    if (apw >= 0) then begin
      apw := sqrt(apw);
      kk1 := (-bb+apw)/(2*aa);
      kk2 := (-bb-apw)/(2*aa);
      if (kk1 <> 0) then begin
        mm1 := (xd-Xdpp)*l1*l2*Rf*Rd/(kk1*Xdpp);
        b11 := 0.5*kk1*a1-bet;
        b12 := 0.5*kk1*a2+bet;
      end else begin
        mm1 := 0;
        b11 := 0;
        b12 := 0
      end;
      if (kk2 <> 0) then begin
        mm2 := (Xd-Xdpp)*l1*l2*Rf*Rd/(kk2*Xdpp);
        b21 := 0.5*kk2*a1-bet;
        b22 := 0.5*kk2*a2+bet;
      end else begin
        mm2 := 0;
        b21 := 0;
        b22 := 0;
      end;
    end else begin
      mm1 := 0;
      mm2 := 0;
      b11 := 0;
      b12 := 0;
      b21 := 0;
      b22 := 0;
    end;
  end else begin  {resistances not correct}
    mm1 := 0;
    mm2 := 0;
    b11 := 0;
    b12 := 0;
    b21 := 0;
    b22 := 0;
    a1 := 0;
    a2 := 0;
  end;
end;

procedure diskriminant(Xd,Xdpp,Rf,Rd,mm,l1,l2: single; var a1,a2,b1,b2,diskr : single);
{calculate discriminant, and other coefficients}
var
  apu, apv : single; {temporary}
begin
  a1 := 2*Rf/(Rf-Rd);
  a2 := 2*Rd/(Rd-Rf);
  apu := (Xd-Xdpp)*l1*l2*Rf*Rd/(Xdpp*mm);
  b1 := (apu*Rf-Rf*Rd*(l1+l2))/(Rf-Rd);
  b2 := (apu*Rd-Rf*Rd*(l1+l2))/(Rd-Rf);
  apu := a1*b2+b1*a2;
  apv := (b1*b2-l1*l2*Rf*Rd)*(a1*a2-1);
  diskr := apu*apu-4*apv;
end;

procedure inductances(Xd,Xdp,Xdpp,Rf,Rd,l1,l2,omega,a1,a2,b1,b2,diskr,mm : single;
                      var Lffp,Lddp,Lfdp,Mfp,Mdp,c1 : single);
{calculate the inductances, when the mutual inductance Mf = Md is given
 as  mm = 1.5*omega*Mf*Mf/Xd}
var
  apu      : single;  {temporary                      }
  {bp,}bm  : single;  {for solving coeff.             }
  aa,bb,cc : single;  {coefficients of diff.equations }
  ifo,ido  : single;  {initial currents (less voltage)}
  k1,k2    : single;  {coeff. for dagonalization      }
  y1o{,y2o}: single;  {initial diagonalized currents  }
begin
  apu := a1*b2+b1*a2;
  {bp := 0.5*(-apu+sqrt(diskr))/(a1*a2-1);}
  bm := 0.5*(-apu-sqrt(diskr))/(a1*a2-1);
  {preliminary inductances}
  Lfdp := bm+mm;
  Lffp := a1*bm+b1+mm;
  Lddp := a2*bm+b2+mm;
  Mfp := sqrt(Xd*mm/(1.5*omega));
  Mdp := Mfp;
  {check equation for X' }
  aa := Lffp-1.5*omega*Mfp*Mfp/Xd;
  bb := Lfdp-1.5*omega*Mfp*Mdp/Xd;
  cc := Lddp-1.5*omega*Mdp*Mdp/Xd;
  apu := aa*Rd-cc*Rf;
  k1 := (-apu+sqrt(apu*apu+4*bb*bb*Rf*Rd))/(2*bb*Rd);
  k2 := (-apu-sqrt(apu*apu+4*bb*bb*Rf*Rd))/(2*bb*Rd);
  {initial currents, less voltage }
  apu := Lffp*Lddp-Lfdp*Lfdp;
  ifo := 1.5*(-Lddp*Mfp+Lfdp*Mdp)/(Xdpp*apu);
  ido := 1.5*(-Lffp*Mdp+Lfdp*Mfp)/(Xdpp*apu);
  {initial values of diagonalized currents}
  y1o := Rf*ifo+k1*Rd*ido;
{  y2o := Rf*ifo+k2*Rd*ido;}
  {function value: coefficients for the current-theor.coefficient}
  apu := -omega/(k2-k1);
  c1 := apu*y1o*( k2*Mfp/Rf-Mdp/Rd)/Xd-(1/Xdp-1/Xd);
end;

procedure SolveInductances(Xd,Xdp,Xdpp,Tdp,Tdpp,Rf,Rd,freq :single;
                          var Lff,Ldd,Lfd,Mf,Md: single; var err : integer);
{solves the physical inductances and mutual inductances from the
 customary generator parameters X, X', X'' etc from the condition
 that the current for a stand alone generator in the case of a
 symmetric fault is proportional to
    1/X + (1/X' - 1/X) exp(-t/T1) + (1/X" - 1/X')exp(-t/T2)
It is assumed that Mf = Md
 (can be used for both d- and q- parameters)
 Xd    pos.seq. reactance
 Xdp   transient reactance
 Xdpp  subtransient reactance
 Tdp   transient short circuit time constant
 Tdpp  subtransient short circuit time constant
 Rf,Rd resistances of the windings
 freq  frequency
 Lff   inductance of one winding
 Ldd   inductance of the second winding
 Lfd   mutual inductance
 Mf    mutual inductance between one winding and stator winding (amplitude)
 Md    mutual inductance between 2nd winding and stator winding (amplitude)
 err   error flag
}
const
  niter = 10;  {max no of iterations}
var
 a1,a2    : single;  {coeff. for solving parameters }
 b1,b2    : single;  {coeff. for solving parameters }
 a01,a02  : single;  {coeff. for solving parameters }
 b11,b12  : single;  {coeff. for solving parameters }
 b21,b22  : single;  {coeff. for solving parameters }
 apu,apv  : single;  {temporary                     }
 c1       : single;  {coefficients for the voltage  }
 c11      : single;  {coefficients for the voltage  }
 c21      : single;  {coefficients for the voltage  }
 Dd       : single;  {temporary                     }
 diskr    : single;  {discriminant for solving par. }
 fp,fm    : single;  {pos. and neg. function value  }
 ii       : integer; {index                         }
 Ld       : single;  {inductance Xd = omega * Ld    }
 Ldpp     : single;  {subtransient inductance       }
 Lffp,Lddp: single;  {preliminary inductances       }
 Lfdp     : single;  {preliminary mut. inductance   }
 Mfp,Mdp  : single;  {preliminary mut. inductances  }
 mm       : single;  {1.5*omega*Mf**2/X             }
 mm1,mm2  : single;  {values of mm giving zero diskr}
 omega    : single;  {angular frequency             }
 xp,xm    : single;  {mm corresponding to pos and neg function values}
 begin
  err := 0;
  omega := 2*3.14159265*freq;
  Ld := Xd/omega;
  Ldpp := Xdpp/omega;
 {calculate initial guess for Mf (and Mf), by assuming that Mf/Md = 1 }
  Dd := (Tdp - Tdpp) * Xd / Xdp + Tdpp * Xd / Xdpp;
  apu := 1.5*(1/Rf+1/Rd)/Ld;
  apv := (Dd+Tdp*Tdpp*Ld/(Dd*Ldpp))-(Tdp+Tdpp);
  if ((apu > 0) and (apv > 0)) then begin
    Mf{p} := sqrt(apv/apu);         {Mf, not Mfp  1.6.2002 ?}
  end else begin
    Mf := 0.001;                            {just some value ?? MIETI !!}
  end;
  if (Rf <> Rd) then begin
    {get mm giving zero discriminant}
    ZeroDiskr(Xd,Xdpp,Rf,Rd,Tdp,Tdpp,mm1,mm2,a01,a02,b11,b12,b21,b22);
    {zero mm indicates problems}
    if (mm1 > 0) and (mm2 > 0) then begin
      {inductances and function values with these}
      inductances(Xd,Xdp,Xdpp,Rf,Rd,Tdp,Tdpp,omega,a01,a02,b11,b12,0.0,mm1,
                  Lffp,Lddp,Lfdp,Mfp,Mdp,c11);
      inductances(Xd,Xdp,Xdpp,Rf,Rd,Tdp,Tdpp,omega,a01,a02,b21,b22,0.0,mm2,
                  Lffp,Lddp,Lfdp,Mfp,Mdp,c21);
      {continue only, if mm1 and mm2 can be used as brackets}
      if ((c11 < 0) and (c21 > 0)) or ((c11 > 0) and (c21 < 0)) then begin
        {initial gues from "standard" solution }
        Mfp := Mf;    {initial guess}
        mm := 1.5*omega*Mfp*Mfp/Xd;
        {should be between mm1,mm2 }
        if ((mm1 <= mm) and (mm <= mm2)) or ((mm1 >= mm) and (mm >= mm1)) then begin
          {calculate discriminant and other coefficients}
          {the previous check should guarantee reasonable diskr.}
          diskriminant(Xd,Xdpp,Rf,Rd,mm,Tdp,Tdpp, a1,a2,b1,b2,diskr);
          inductances(Xd,Xdp,Xdpp,Rf,Rd,Tdp,Tdpp,omega,a1,a2,b1,b2,diskr,mm,
                      Lffp,Lddp,Lfdp,Mfp,Mdp,c1);
          {c1 is deviation from 1/X' - 1/X }
          {select brackets}
          if (c1 > 0) then begin
            fp := c1;
            xp := mm;
            if (c11 < 0) then begin
              fm := c11;
              xm := mm1;
            end else begin
              fm := c21;
              xm := mm2;
            end;
          end else begin
            fm := c1;
            xm := mm;
            if (c11 > 0) then begin
              fp := c11;
              xp := mm1;
            end else begin
              fp := c21;
              xp := mm2;
            end;
          end;
        end else begin {initial guess not reasonable, use mm1, and mm2}
          if (c11 > 0) then begin
            fp := c11;
            xp := mm1;
            fm := c21;
            xm := mm2;
          end else begin
            fp := c11;
            xp := mm1;
            fm := c21;
            xm := mm2;
          end;
        end;
        {iterate}
        for ii := 1 to niter do begin
          {interpolate new mm}
          apu := fp-fm;
          if (apu <> 0) then begin
            mm := (fp*xm-fm*xp)/apu;
          end else begin
            mm := 0.5*(xp+xm);
          end;
          {calculate inductances and new function value}
          diskriminant(Xd,Xdpp,Rf,Rd,mm,Tdp,Tdpp, a1,a2,b1,b2,diskr);
          if (diskr < 0) then diskr := 0;     {should not be, because of checks}
          inductances(Xd,Xdp,Xdpp,Rf,Rd,Tdp,Tdpp,omega,a1,a2,b1,b2,diskr,mm,
                      Lffp,Lddp,Lfdp,Mfp,Mdp,c1);
          {new brackets}
          if (c1 > 0) then begin
            fp := c1;
            xp := mm;
          end else begin
            fm := c1;
            xm := mm;
          end;
          {converged ?}
          if ( abs(xm-xp) < 1.e-5*mm) then break
        end;
        {copy local variables to output}
        Lff := Lffp;
        Ldd := Lddp;
        Lfd := Lfdp;
        Mf := Mfp;
        Md := Mdp;
        err := 0;
      end else begin   {cannot find brackets, something wrong }
        err := 1;
      end;
    end else begin    {Resistances or smth. else wrong, cannot calculate}
      err := 1;
    end;
  end else begin
    err := 1;
  end;
end;

procedure SolvePhysPar(Xd,Xdp,Xdpp,Tdp,Tdpp,Rf,Rd,MdpMf,freq :single;
                   var sLff,sLdd,sLfd,sMf,sMd: single);
{                  var Lff,Ldd,Lfd,Mf,Md: single);}
{solves the physical inductances and mutual inductances from the
 customary generator parameters X, X', X'' etc
 (can be used for both d- and q- parameters)
 Xd    pos.seq. reactance
 Xdp   transient reactance
 Xdpp  subtransient reactance
 Tdp   transient short circuit time constant
 Tdpp  subtransient short circuit time constant
 Rf,Rd resistances of the windings
 MdpMf the guessed value of the ratio Md/Mf
 freq  frequency
 Lff   inductance of one winding
 Ldd   inductance of the second winding
 Lfd   mutual inductance
 Mf    mutual inductance between one winding and stator winding (amplitude)
 Md    mutual inductance between 2nd winding and stator winding (amplitude)
}
var
  a,b,c,E   : double;{single;  {short hand notation}
  a1,a2     : double;{single;  {coefficients}
  a1a2m     : double;{single;  {short hand notation}
  alf,ald   : double;{single;  {short hand notations}
  apu,apv   : double;{single;  {temporary}
  apw,apx   : double;{single;  {temporary}
  b1,b2     : double;{single;  {coefficients}
  Cc,Dd     : double;{single;  {short hand notation}
  disk      : double;{single;  {diskriminant}
  det,det2  : double;{single;  {determinant}
  iro,ido   : double;{single;  {initial rotor currents}
  Ld, Ldpp  : double;{single;  {temporary inductances}
  Xdsb      : double;{single;  {what Xd' should be}
  {local, double precision parameters}
  Lff : double;  {inductance of one winding}
  Ldd : double;  {inductance of the second winding}
  Lfd : double;  {mutual inductance}
  Mf  : double;  {mutual inductance between one winding and stator winding (amplitude)}
  Md  : double;  {mutual inductance between 2nd winding and stator winding (amplitude)}
begin
  { get inductances from reactances}
  Ld := Xd / (2 * freq * pi);
  Ldpp := Xdpp / (2 * freq * pi);
  {calculate Md and Mf, by assuming that Mf/Md is given }
  Dd := (Tdp - Tdpp) * Xd / Xdp + Tdpp * Xd / Xdpp;
  apu := 1.5*(1/Rf+MdpMf*MdpMf/Rd)/Ld;
  apv := (Dd+Tdp*Tdpp*Ld/(Dd*Ldpp))-(Tdp+Tdpp);
  if ((apu > 0) and (apv > 0)) then begin
    Mf := sqrt(apv/apu);
    Md := Mf*MdpMf;
  end else begin
    showmessage('not reasonable Mf**2');
    Mf := 0.001;                            {just some value ?? MIETI !!}
    Md := MdpMf*Mf;
  end;
  {next, calculate mutual inductance Lfd}
  Cc := 2 * (Ld - Ldpp) * Rf * Rd * Tdp * Tdpp * Ld / (3 * Ldpp);
  {determinant for solving Lff and Ldd as function of Lfd}
  det := Md * Md / Rd - Mf * Mf / Rf;
  if (det <> 0) then begin                  {determinant must not be zero}
    a1 := 2 * Mf * Md / (Rd * det);
    apu := Dd + Tdp * Tdpp * Ld / (Dd * Ldpp);
    b1 := Cc / (Rd * det) - apu * Mf * Mf / det;
    a2 := -2 * Mf * Md / (Rf * det);
    b2 := -Cc / (Rf * det) + apu * Md * Md / det;
    apv := a1 * b2 + a2 * b1;
    apw := 4 * (b1 * b2 - Rf * Rd * Tdp * Tdpp * Xd / Xdpp) * (a1 * a2 - 1);
    disk := apv * apv - apw;
    a1a2m := a1 * a2 - 1;
    if (a1a2m <> 0) then begin              {to avoid division by zero}
      if (disk >= 0) then begin             {discriminant must be nonnegative}
        {it seems that the minus-sign is the correct one}
        Lfd := 0.5 * (-apv - sqrt(disk)) / a1a2m;
        {next the inductances}
        Lff := a1 * Lfd + b1;
        Ldd := a2 * Lfd + b2;
      end else begin                        {negative discriminant, round off ?}
        {showmessage('negative discriminant! X" or Xq" maybe too small'); }
        Lfd := 0.5 * (-apv ) / a1a2m;       {neglect discriminant, almost zero ?}
        {next the inductances}
        Lff := a1 * Lfd + b1;
        Ldd := a2 * Lfd + b2;

{       Lfd := 1.8*Mf; }                     {just some values....}
{        Lff := 2*Mf;  }                      {try to get something...}
{        Ldd := 2*Mf;  }                      {...better !!!}

      end;
    end else begin
      showmessage('zero divisor!');
      Lfd := 1.8*Mf;
      Lff := 2*Mf;
      Ldd := 2*Mf;
    end;
  end else begin
    showmessage('zero determinant');
    Lfd := 1.8*Mf;
    Lff := 2*Mf;
    Ldd := 2*Mf;
  end;
  {seuraavassa erilaisia testej}
  {check the expression of X',  10.5.2000}
  apu := 0.5*(Xdpp-Xd)/(2*pi*freq)*((tdp-tdpp)+1.5*Mf*Mf/(Ld*Rf)+1.5*Md*Md/(Ld*Rd));
  apv := 0.75*(-Ldd*Mf*Mf+Lff*Md*Md)*(-Lff/Rf+Ldd/Rd)/(Lff*Ldd-Lfd*Lfd);
  apw := (Md*Lfd/Rd)*1.5*(Ldd*Mf-Lfd*Md)/(Lff*Ldd-Lfd*Lfd)+
         (Mf*Lfd/Rf)*1.5*(Lff*Md-Lfd*Mf)/(Lff*Ldd-Lfd*Lfd);
  alf := 1.5*(-Lff*Md+Lfd*Mf)/(Lff*Ldd-Lfd*Lfd);
  ald := 1.5*(-Ldd*Mf+Lfd*Md)/(Lff*Ldd-Lfd*Lfd);
  det2 := 1+alf*Md/Ld+ald*Mf/Ld;
  {Xd' should be...}
  apx := (apu+apv+apw )/(det2*Ld*Ld*(Tdpp-Tdp)){-1/Ld};
  xdsb := 2*freq*pi/apx;
  {tuossa on jotain vrin, aloitetaan aikaisemmin}
  a := Lff-1.5*Mf*Mf/Ld;
  b := Lfd-1.5*Mf*Md/Ld;
  c := Ldd-1.5*Md*Md/Ld;
  {olkoon E = 1/sqrt(3) eli noin 0.57735}
  E := 0.5773503;
  iro := -ald*E/(det2*Xd);
  ido := -alf*E/(det2*Xd);
  {Ik' nitten avulla}
(*
{showmessage(floattostr((Rf*c-Rf*Rd*tdp)*iro)+'  '+floattostr(Rd*b*ido));}
{  apu := -(Mf/Ld)*((Rf*c-Rf*Rd*tdp)*iro-Rd*b*ido)/(Rf*Rd*(Tdpp-Tdp)){;
{  apv :=} -(Md/Ld)*((Rd*a-Rf*Rd*tdp)*ido-Rf*b*iro)/(Rf*Rd*(tdpp-tdp));
{         -E/Xd;}
{showmessage(floattostr(apu+apv));}
*)

  {ent X", eli nopeammin vaimeneva komponentti}
(*  apv := -(Mf/Ld)*(Rd*b*ido+iro*(tdpp*Rf*Rd-c*Rf))/((tdpp-tdp)*Rf*Rd){;
  apv :=} -(Md/Ld)*(Rf*b*iro+ido*(tdpp*Rf*Rd-a*Rd))/((tdpp-tdp)*Rf*Rd);*)
{  apw := -E/Xd;}
{ apx := -(Mf/Ld)*iro-(Md/Ld)*ido-E/Xd;}
  {virta annetulla hetkell}


  {copy local variables to output (temporary ?}
  sLff := Lff;
  sLdd := Ldd;
  sLfd := Lfd;
  sMf  := Mf;
  sMd  := Md;
end;

procedure SolveGenPar(Par : GMset; freq : single; var PP : PhysPar);
{solves the generator parameters Lff, Ldd, Lfd, Md for a generator
 with a D-damping coil,
 when X, X', X", Td', Td" are given
 and   Rf,Rd ja Mf have been guessed
 Par     the set of parameters that were read in (X,X' etc)
 freq    the frequency
 PP      the physical parameters (inducgtances, resistances)
 }
const
  MdpMf = 0.5; {the guessed value of the ratio Md/Mf and Mp/Mq}
  Rfo   = 0.3; {the guessed value of rotor resistance}
  Rdo   = 0.1; {the guessed value of resistance of damping coil}
  Rqo   = 0.2; {the guessed value of resistance of damping Q-winding}
  Rpo   = 0.1; {the guessed value of resistance of damping p winding}
var
  {local variables for parameters (physical and input)}
{ dXq   : single; {Xq-Xq''}
  err   : integer;{error flag }
  Ld    : single; {stator (combined) inductance}
  Lff   : single; {the self inductance of the field winding (exitation winding)}
  Ldd   : single; {the self inductance of the damping D-coil}
  Lfd   : single; {the mutual inductance between the field and damping windings}
  Lqq   : single; {inductance of q-winding}
  Lpp   : single; {inductance of p-winding (second q-winding)}
  Lqp   : single; {mutual inductance between q- and p-windings}
  Md    : single; {the amplitude of the damping winding and the stator winding}
                  {the mutual inductance is assumed to be Md cos(wt)}
  Mf    : single; {the amplitude of the damping winding and the stator winding}
                  {the mutual inductance is assumed to be Mf cos(wt)}
  Mq    : single; {the mutual inductance of the q- and stator windings}
  Mp    : single; {the mutual inductance of the p- and stator windings}
  Rf,Rd : single; {the resistances of the field and damping winding}
  Rq    : single; {the resistance of the q-winding}
  Rp    : single; {the resistance of the p-winding}
  Xd    : single; {the steady state reactance}
  Xdp   : single; {the transient reactance}
  Xdpp  : single; {the subtransient reactance}
  Xqd   : single; {the transient reactance of q-winding, Xq'}
  Xq2d  : single; {the subtransient reactance of q-windings, Xq''}
  Tdp   : single; {the transient time constant (short circuit)}
  Tdpp  : single; {the subtransient time constant (short circuit)}
  Tqd   : single; {transient time constant of q-windings}
  Tq2d  : single; {subtransient time constant of q-windings}
begin
  {copy the input parameters to locals}
   Xd := Par.Xd;
   Xdp := Par.Xdd;
   Xdpp := Par.Xd2d;
   Tdp := Par.Tdd;
   Tdpp := Par.Td2d;
   Xqd := Par.Xqd;
   Xq2d := Par.Xq2d;
{  dXq := Par.Xd-Par.Xq2d;  {assumed Xd = Xq}
   Tqd := Par.Tqd;
   Tq2d := Par.Tq2d;
  {guess the values of resistances}
  Rf := Rfo;
  Rd := Rdo;
  { get stator total inductance Ls+Ms from reactance }
  Ld := Xd / (2 * freq * pi);
  {solve the physical inductances from the "normal" generator parameters}
  SolveInductances(Xd,Xdp,Xdpp,Tdp,Tdpp,Rf,Rd,freq,Lff,Ldd,Lfd,Mf,Md, err);
  if (err <> 0) then   {if not successful, solve similarly as dommel}
    SolvePhysPar(Xd,Xdp,Xdpp,Tdp,Tdpp,Rf,Rd,MdpMf,freq,   Lff,Ldd,Lfd,Mf,Md);
  {solve parameters for the q and p windings, use f and d-parameters, so far}
  Rq := Rqo;           {guessed value for the resistance}
  Rp := Rpo;
  SolveInductances(Xd,Xqd,Xq2d,Tqd,Tq2d,Rq,Rp,freq,Lqq,Lpp,Lqp,Mq,Mp, err);
  if (err <> 0) then   {if error, solve more like dommel}
    SolvePhysPar(Xd,Xqd,Xq2d,Tqd,Tq2d,Rq,Rp,MdpMf,freq, Lqq,Lpp,Lqp,Mq,Mp);
  {copy local results (parameters) to output}
  PP.Lff := Lff;
  PP.Ldd := Ldd;
  PP.Lfd := Lfd;
  PP.Mf  := Mf;
  PP.Md  := Md;
  PP.Ld  := Ld;
  PP.Rs  := Par.Rs;     {copy stator resistance as such}
  PP.Rf  := Rf;
  PP.Rd  := Rd;
  PP.Omega := 2*pi*freq;
  PP.Lqq := Lqq;
  PP.Mq  := Mq;
  PP.Rq  := Rq;
  PP.Rp  := Rp;
  PP.Lpp := Lpp;
  PP.Mp  := Mp;
  PP.Lpq := Lqp;
{
  showmessage('Lff   '+floattostr(Lff));
  showmessage('Ldd   '+floattostr(Ldd));
  showmessage('Lfd   '+floattostr(Lfd));
  showmessage('Mf   '+floattostr(Mf));
  showmessage('Md   '+floattostr(Md));
  showmessage('Ld   '+floattostr(Ld));
  showmessage('Rs   '+floattostr(PP.Rs));
  showmessage('Rf   '+floattostr(Rf));
  showmessage('Rd   '+floattostr(Rd));
  showmessage('omega   '+floattostr(PP.omega));
}{
  showmessage('Lqq   '+floattostr(Lqq));
  showmessage('Lpp   '+floattostr(Lpp));
  showmessage('Lqp   '+floattostr(Lqp));
  showmessage('Mq   '+floattostr(Mq));
  showmessage('Mp   '+floattostr(Mp));
  showmessage('Rq   '+floattostr(Rq));
  showmessage('Rp   '+floattostr(Rp));
}
end;

end.
 