{tarkista FindParallelGenerator, ehdot ?}
unit GenVolt;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Skemadef, Utildef, Buttons,
  Nodes,Elplex,Compdef, Imagodef, Skemproc, {Menus,} Cmatmul;

procedure SolveNetWithGivenOutputVoltages(ipSet: longint; probl : Solve); {solve the net}

type
{class for performing the network calculations}
  TpSetVoltage = ^TSetVoltage;    {define pointer type first !}
  TSetVoltage = class(Tobject)    {derive the class from general object}
public
  constructor Create(ipSet : longint; var success : boolean);
  destructor  Destroy; override;
  procedure   FreeGeneratorArrays;
  procedure   FindGenerators;
  procedure   FindFeeders;
  procedure   ReserveConstructAndDecompose(probl: Solve; var success : boolean);
  procedure   FindNominalSolution;
  procedure   CalculateGeneratorVoltageDerivative;
  procedure   PickDerivative(var ppComp: TppComp; NodeNum : integer;
                             var dd : complex);
  procedure   FindParallelGenerator;
  procedure   FindParallelFeeder;
  procedure   FindInternalVoltages;
  procedure   SolveTheNetWithNewVoltages(probl: Solve);
  function    GetpGen(ii:integer): TpComp;    {get pointer to generator no ii}
  function    IsDesiredPowerPositive:boolean; {check, are desired powers > 0}
protected
  Feeders_gv : TList;       {pointers to feeders}
  freq_gv    : single;      {net frequency }
  nGener_gv  : integer;     {number of generators}
  nnod_gv    : integer;     {number of nodes}
  n1_gv      : integer;     {temporary}
  n2_gv      : integer;     {temporary}
  n3_gv      : integer;     {temporary}
  pdGUdGU_gv : pComplex;    {pointer to d(gener.V out)/d(gener internal Volt.)}
  pdGIdGU_gv : pComplex;    {pointer to d(gener.curr.)/d(gener internal volt.)}
  ppGener_gv : TppComp;     {pointer to an array of generator pointers}
  pGenVo_gv  : pComplex;    {pointer to (desired) generator output voltages}
  pGenVex_gv : pComplex;    {pointer to (unadjusted) generator output voltages}
  pGenVnew_gv: pComplex;    {pointer to new, adjusted gener.internal voltages}
  pNomSol_gv : pComplex;    {pointer to solution with nominal values}
  pParal_gv  : pInteger;    {pointer to indices of eventual parallel generators}
  pSet       : TpCompSet;   {pointer to the set of components, etc}

end;

var
  SetVolt : TSetVoltage;   {object for performing the calculations}

implementation

uses
  imagopas;

constructor TSetVoltage.Create(ipSet : longint; var success : boolean);
{reserve space, initialize
 ipSet   pointer to the set of components etc, as integer}
begin
  inherited create;
  pSet := TpCompSet(ipSet);           {pointer to set of components. etc}
  {determine number of nodes (if any)}
  Imagoo.NodeButClick(Imagoo.NodeBut);
  nnod_gv := pSet^.appl_nn;           {number of nodes}
  if (nnod_gv > 0) then begin
    getmem(pNomSol_gv,nnod_gv*SizeOf(complex));
  end else begin
    pNomSol_gv := nil;
  end;
  freq_gv := pSet^.appl_hz;           {frequency}
  pSet^.CountGivenComponents(gGen,gCurr,gFeed,gLoad,nGener_gv,n1_gv,n2_gv,n3_gv);
  if (n2_gv > 0) then begin           {are there feeders}
    Feeders_gv := TList.create;       {create list of pointers to feeders}
    FindFeeders;                      {pointers to feeders to list}
    FindParallelFeeder;               {if any}
  end;
  if (nGener_gv > 0) then begin
    {get memory for pointers to generators}
    getmem(ppGener_gv,nGener_gv*SizeOf(TpComp));
    {get memory for (desired) generator output voltages}
    getmem(pGenVo_gv,nGener_gv*SizeOf(complex));
    {get memory for old, unadjusted generator output voltages}
    getmem(pGenVex_gv,nGener_gv*SizeOf(complex));
    {get memory for new, adjusted internal generator voltages}
    getmem(pGenVnew_gv,nGener_gv*SizeOf(complex));
    {get memory for indices of eventual parallel generators}
    getmem(pParal_gv,nGener_gv*SizeOf(integer));
    FindGenerators;                {save pointers to generators}
    {get memory for the derivatives of generator output voltages and currents
     with respect to internal generator voltages}
    getmem(pdGUdGU_gv,nGener_gv*nGener_gv*SizeOf(complex));
    getmem(pdGIdGU_gv,nGener_gv*nGener_gv*SizeOf(complex));
    success := true;
  end else begin
    pGenVo_gv := nil;
    pGenVex_gv := nil;
    pGenVnew_gv := nil;
    nGener_gv := 0;
    pdGUdGU_gv := nil;
    pdGIdGU_gv := nil;
    ppGener_gv := nil;
    success := false;
  end;
end;

procedure TSetVoltage.FreeGeneratorArrays;
{free the memory reserved for the generator arrays}
begin
  freemem(pNomSol_gv , nnod_gv  *SizeOf(complex));
  freemem(pGenVo_gv  , nGener_gv*SizeOf(complex));
  freemem(pGenVex_gv , nGener_gv*SizeOf(complex));
  freemem(pGenVnew_gv, nGener_gv*SizeOf(complex));
  freemem(ppGener_gv , nGener_gv*SizeOf(TpComp));
  freemem(pParal_gv  , nGener_gv*SizeOf(integer));
  freemem(pdGUdGU_gv , nGener_gv*nGener_gv*SizeOf(complex));
  freemem(pdGIdGU_gv , nGener_gv*nGener_gv*SizeOf(complex));
end;

destructor TSetVoltage.Destroy;
{free the space reserved, free the object}
begin
  FreeGeneratorArrays;
  inherited destroy;
end;

procedure TSetVoltage.FindGenerators;
{find the pointers to the generators, put them in the arrays for pointers
}
var
  errc    : integer;  {error flag, indicates end of components}
  jcomp   : integer;  {counts components}
  ModNam  : string;   {name of a component}
  pComp   : TpComp;   {pointer to a component}
  ppGener : TppComp;  {pointer to generator poionter}
begin
  {scan all components in order to find generators}
  errc := 0;
  pComp := nil;
  jcomp := 0;
  ppGener := ppGener_gv;
  while (errc = 0) do begin                {scan all components}
    pSet^.NextComp(pComp,errc,jcomp,true); {get pointer to next component}
    if (errc = 0) then begin               {component found}
      pComp^.GetModName(ModNam);           {get the name (type) of the component}
      if (ModNam = gGen) then begin        {is it generator}
        ppGener^:= pComp;                  {save pointer}
        inc(ppGener);
      end;
    end;
  end;
end;

procedure TSetVoltage.FindFeeders;
{find the pointers to the feeders, put them in the arrays for pointers}
var
  errc    : integer;  {error flag, indicates end of components}
  jcomp   : integer;  {counts components}
  ModNam  : string;   {name of a component}
  pComp   : TpComp;   {pointer to a component}
  ppGener : TppComp;  {pointer to generator poionter}
begin
  {scan all components in order to find generators}
  errc := 0;
  pComp := nil;
  jcomp := 0;
  while (errc = 0) do begin                {scan all components}
    pSet^.NextComp(pComp,errc,jcomp,true); {get pointer to next component}
    if (errc = 0) then begin               {component found}
      pComp^.GetModName(ModNam);           {get the name (type) of the component}
      if (ModNam = gFeed) then begin       {is it a feeders}
        Feeders_gv.add(pComp);             {save pointer}
      end;
    end;
  end;
end;


procedure TSetVoltage.ReserveConstructAndDecompose(probl:solve;
                                               var success : boolean);
{reserve the matrices for the network equations, construct the equations,
 and decompose, but do not yet solve
 Use pointers to matrices (pAdmat, pRhs, pWs, pIndex) defined in unit Elplex
 success  true, if successful (there are nodes, no unknown components, etc)}
var
  Dodd   : single;      {keeps track on permutations}
  errcom  : boolean;   {indicates unknown components}
  Sngl   : Integer;     {flag for singular matrix}
  Tout   : Complex;     {test output from matrix decomposition}
begin
  if (nnod_gv > 0) then begin      {there are nodes}
    {reserve and initialize memory for matrices}
    ReserveMat(nnod_gv,pAdmat,pWs,pRhs,pIndex);
    {construct the extended admittance matrix, normal freq, no shorts}
    ConstrMatr(pSet, pAdmat, pRhs, nnod_gv, freq_gv, Steady, probl, false, errcom);
    {decompose, but only if no unknown components}
    if (not(errcom)) then begin
      try
        CLUdecomp(pAdmat,pWs,nnod_gv,pIndex,Dodd,Sngl,Tout); {decompose }
        success := true;
      except
        success := false;
      end;
    end else begin                   {there was an unknown component}
      success := false;
    end;
    (*
    {as a test, complete the calculations}
     CLUbacksub(pAdmat,nnod_gv,pIndex,pRhs);               {backsubstitute}
     {save voltages and currents to components}
     ResultsToComp(pSet, nnod_gv, freq, pRhs,0);
     {calculate voltages, currents for other phases, assuming symmetrical case}
     OtherPhasesSymmetrically(pSet);
     {relays use polar coordinates = amplitude and phase angle}
     PolarToRelaysAndBuses(pSet);
     {update the memo boxes for results, if visible}
     UpdateResultBoxes(pSet);
     {show the currents through the current trafos, if desired}
     ShowCurrOrNot(pSet, Imagoo.ShowCurrBut.down);
     {endtest}
     *)
  end else begin
    success := false;
  end;
end;

procedure TSetVoltage.FindNominalSolution;
{solve the network using the given internal generator voltages,
 save it, and save the internal and external (output) generator voltages.
 It is assumed that the ciefficient matrix has been decomposed}
var
  cpu      : complex;  {temporary}
  err      : integer;  {error flag}
  ii,jj    : integer;  {index}
  ppGener  : TppComp;  {pointer to generator pointers}
  pGen     : TpComp;   {pointer to a generator}
  Vo       : complex;  {present internal voltage}
  VoltEq   : integer;  {index of voltage equation}
begin
  {copy rhs to local solution, note, solution overwrites it!}
  copyCvec(pRhs,pNomSol_gv,nnod_gv);
  CLUbacksub(pAdmat,nnod_gv,pIndex,pNomSol_gv);{get nominal solution}
  {first, save generator output voltages and internal voltages, for later use}
  if (nGener_gv > 0) then begin
    ppGener := ppGener_gv;                        {p. to array of gen.pointers}
    for ii := 1 to nGener_gv do begin             {scan all generators}
      pGen := ppGener^;                           {pointer to this generator}
      inc(ppGener);                               {for next round}
      pGen^.GetNode(2,VoltEq,err);                {get voltage equation number}
      getCvec(pRhs,VoltEq,nnod_gv,Vo);            {get present internal voltage}
      putCvec(pGenVo_gv,ii,nGener_gv,Vo);         {save it for later use}
      pGen^.GetNode(1,jj,err);                    {voltage node}
      getCvec(pNomSol_gv,jj,nnod_gv,cpu);         {get output voltage}
      putCvec(pGenVex_gv,ii,nGener_gv,cpu);       {save it for later use}
    end;
  end;
end;

procedure TSetVoltage.PickDerivative(var ppComp: TppComp; NodeNum : integer;
                                     var dd : complex);
{pick the entry in the solution vector corresponding to the desired
 derivative
  ppComp   LOCAL pointer to an array of component pointers
  NodeNum  the node number in the component, that corresponds to the
           variable to be derived (mostly 1 or 2)
  dd        the derivative}
var
  err   : integer; {error flag}
  Node  : integer; {the node number in the net of the given component node}
  pComp : TpComp;  {pointer to the component}
begin
  pComp := ppComp^;
  inc(ppComp);
  {derivative of output voltage wrt internal voltage}
  pComp^.GetNode(NodeNum,Node,err);          {get node}
  if (Node > 0) and (Node <= nnod_gv) then begin
    getCvec(pWs,Node,nnod_gv,dd);           {derivative}
  end else begin                            {perhaps ground}
    dd.x := 0;
    dd.y := 0;
  end;
end;

procedure TSetVoltage.CalculateGeneratorVoltageDerivative;
{calculate the derivatives of generator output voltages
 with respect to internal generator voltage.
 When entering, the set of equations has been decomposed, but not
 yet backsubstituted}
var
  cpu      : complex;  {temporary}
  err      : integer;  {error flag}
  ii,jj    : integer;  {indices}
  pGen     : TpComp;   {pointer to a generator}
  ppComp   : TppComp;  {pointer to component pointers}
  ppGener  : TppComp;  {pointer to an array of pointers}
  VoltEq   : integer;  {number of the voltage equation = current "node"}
begin
  {calculate the derivatives}
  ppGener := ppGener_gv;                        {p. to array of gen.pointers}
  for ii := 1 to nGener_gv do begin             {scan all generators}
    pGen := ppGener^;                           {pointer to this generator}
    inc(ppGener);
    pGen^.GetNode(2,VoltEq,err);                {get voltage equation number}
    {for derivatives, just put one to the rhs}
    cpu.x := 0;
    cpu.y := 0;                     {clear first}
    for jj:=1 to nnod_gv do putCvec(pWs,jj,nnod_gv,cpu);
    cpu.x := 1.0;
    cpu.y := 0.0;
    putCvec(pWs,VoltEq,nnod_gv,cpu);            {put 1 to the "deriver"}
    CLUbacksub(pAdmat,nnod_gv,pIndex,pWs);      {get "solution" }
    {find change in voltage  for all generators}
    ppComp := ppGener_gv;                       {pointer to first generator}
    for jj:=1 to nGener_gv do begin
      PickDerivative(ppComp,1,cpu);
      putCmat(pdGUdGU_gv,jj,ii,nGener_gv,nGener_gv,cpu); {save it}
    end;
  end;
end;

procedure TSetVoltage.FindParallelGenerator;
{Scan all generators. For each generator, find the first generator,
 that seems to be parallel with the generator under study.
 The criterium for paralleliness is that the derivative
 d(output voltage of this generator)/d(internal voltage of THIS generator)
  is (about) the same as
 d(output voltage of this generator)/d(internal voltage of THAT generator)
 The index of the generator is saved.}
var
  cpu    : complex; {temporary}
  dUdE   : complex; {derivative of output voltage wrt internal voltage}
  ii,jj  : integer; {indices}
begin
  {initialize indices}
  for ii:=1 to nGener_gv do putIvec(pParal_gv,ii,nGener_gv,0);
  {scan all generators}
  for ii:=1 to nGener_gv do begin
    {get derivative of Vout wrt. internal voltage}
    getCmat(pdGUdGU_gv,ii,ii,nGener_gv,nGener_gv,dUdE);
{ONKO TM TARKISTUS TARPEEN ?}
    if (Cabs(dUdE) > 0.001) then begin          {not zero}
      {check eventual parallel generators}
      if (ii > 1) then begin
        for jj:=1 to ii-1 do begin             {scan "earlier" generators}
          getCmat(pdGUdGU_gv,jj,ii,nGener_gv,nGener_gv,cpu);
{RIITTK TM EHTO ??}
          if (Cabs(Csub(dUdE,cpu)) < 0.001) then begin
            {(almost) same derivatives => (probably) parallel generators}
            putIvec(pParal_gv,ii,nGener_gv,jj);
            break;
          end;
        end;
      end;
    end;
  end;
end;

procedure TSetVoltage.FindParallelFeeder;
{Scan all feeders. For each generator, find the first feeder (if any),
 that is parallel with the feeder under study.
 The criterium for paralleliness is a common node (this is sufficient, mostly)
 The pointer of the feeder under study is saved to the parallel feeder}
var
  er     : integer; {erro flag}
  ii,jj  : integer; {indices}
  id,jd  : integer; {node}
  pComp  : TpComp;  {pointer to a feeder}
  pFeed  : TpComp;  {pointer to the feeder under study}
begin
  {initialize the pointers}
  for ii:= 0 to Feeders_gv.count-1 do begin
    pComp := Feeders_gv.items[ii];
    pComp^.PutParal(nil);
  end;
  {scan all feeders, but only if there are more than one}
  if (Feeders_gv.count > 1) then begin
    for ii:= 0 to Feeders_gv.count-1 do begin
      pFeed := Feeders_gv.items[ii];     {pointer to the feeder under study}
      pFeed^.GetNode(1,id,er);           {downstream (voltage) node}
      {scan the other feeders}
      for jj:= ii+1 to Feeders_gv.count-1 do begin   {start from the next feeder}
        pComp := Feeders_gv.items[jj];     {pointer to another feeder}
        pComp^.GetNode(1,jd,er);           {downstream node}
        if (id = jd) then begin            {same downstrean node}
          {RIITTK TM EHTO ??}
          pComp^.PutParal(pFeed);          {the feeder under study is parall.with this one}
          break;                           {only one needed (a chain is constructed)}
        end;
      end;
    end;
  end;
end;

procedure TSetVoltage.FindInternalVoltages;
{solve the internal voltages that give the desired output voltages.
 "Desired" = the voltage and phase that are given in the generator parameters.
 When entering this procedure, the network has been solved once, and
 the derivatives dUout/dUint have been calculated}
var
  cpu,cpv : complex;    {temporary}
  dUdE    : complex;    {d(output voltage)/d(internal voltage)}
  ii      : integer;    {indices}
  ip      : integer;    {index of eventual parallel generator}
  Dodd    : single;     {keeps track on permutations}
  pCaa    : pComplex;   {pointer to equations}
  pCbb    : pComplex;   {pointer to right hand side}
  pCww    : pComplex;   {pointer to working space}
  pind    : pInteger;   {pointer to row exchanges}
  Sngl    : integer;    {flag for singular matrix}
  Tout    : complex;    {extra output}
  Vout    : complex;    {old voltage out }
  Vint     : complex;    {old internal voltage}
begin
  if (nGener_gv > 0) then begin
    ReserveMat(nGener_gv,pCaa,pCww,pCbb,pind);
    {construct equations = derivatives of ext.voltage wrt to int. voltage}
    {check eventual obviously singular matrix: zero on diagonal}
    {check also parallel generators = about equal derivatives}
    for ii:=1 to nGener_gv do begin
      getCvec(pGenVo_gv,ii,nGener_gv,Vint);         {get internal voltage}
      getCvec(pGenVex_gv,ii,nGener_gv,Vout);        {present external voltage}
      if (Cabs(Vint) >= 0.1) then begin             {reasonable voltage}
        {get derivative of Vout wrt. internal voltage}
        getCmat(pdGUdGU_gv,ii,ii,nGener_gv,nGener_gv,dUdE);
        if (Cabs(dUdE) > 0.001) then begin          {not zero}
          {check eventual parallel generators}
          if (ii > 1) then begin
            getIvec(pParal_gv,ii,nGener_gv,ip);   {index of eventual paraller g}
            if (ip > 0) and (ip <= nGener_gv) then begin  {found parallel}
              {set voltages to equal values}
              cpu.x := 1;
              cpu.y := 0;
              putCmat(pCaa,ii,ii,nGener_gv,nGener_gv,cpu);
              cpu.x := -1;
              putCmat(pCaa,ii,ip,nGener_gv,nGener_gv,cpu);
              {right hand side: goal = to get finally the same internal
               voltage as in the parallel generator}
              getCvec(pGenVo_gv,ip,nGener_gv,cpu); {internal volt.of paral.g}
              putCvec(pCbb,ii,nGener_gv,Csub(cpu,Vint));
            end else begin                               {no paral.generator}
              CopyCrow(pdGUdGU_gv,pCaa,ii,nGener_gv);    {copy this row}
              putCvec(pCbb,ii,nGener_gv,Csub(Vint,Vout));{rhs}
            end;
          end else begin                                {first generator}
            CopyCrow(pdGUdGU_gv,pCaa,ii,nGener_gv);     {copy this row}
            putCvec(pCbb,ii,nGener_gv,Csub(Vint,Vout)); {right hand side = diff.}
          end;
        end else begin                             {practically zero = singular}
          {keep old voltage = put one to diagonal}
          cpu.x := 1;
          cpu.y := 0;
          putCmat(pCaa,ii,ii,nGener_gv,nGener_gv,cpu);
        end;
      end else begin
        {very small voltage, keep it, put one to diagonal}
        cpu.x := 1;
        cpu.y := 0;
        putCmat(pCaa,ii,ii,nGener_gv,nGener_gv,cpu);
      end;
    end;
    {solve change in iternal voltage}
    CLUdecomp(pCaa,pCww,nGener_gv,pind,Dodd,Sngl,Tout); {decompose }
    CLUbacksub(pCaa,nGener_gv,pind,pCbb);               {backsubstitute}
    {the new, adjusted voltages}
    for ii:=1 to nGener_gv do begin
      getCvec(pCbb,ii,nGener_gv,cpu);                   {needed change}
      getCvec(pGenVo_gv,ii,nGener_gv,cpv);              {present internal V}
      cpu := Cadd(cpu,cpv);                             {new internal voltage}
      putCvec(pGenVnew_gv,ii,nGener_gv,cpu);            {save the voltage}
(*    show the voltages
      {change to phase-to-phase voltage, as rms and angle}
      apu := sqrt(3)*Cabs(cpu);
      apv := 180*Cangle(cpu)/pi;
      showmessage(floattostr(apu)+' V  '+floattostr(apv)+' deg');
*)
    end;
    {finally, release the memory of the equations}
    freemem(pCaa,nGener_gv*nGener_gv*SizeOf(complex));
    freemem(pCbb,nGener_gv*SizeOf(complex));
    freemem(pCww,nGener_gv*SizeOf(complex));
    freemem(pind,nGener_gv*SizeOf(integer));
  end;
end;

procedure TSetVoltage.SolveTheNetWithNewVoltages(probl: solve);
{solve the network using the new,adjusted internal generator voltages
 and save and show the results in the usual way}
var
  err     : integer;    {error flag}
  ii      : integer;    {index}
  ppGener : TppComp;    {pointer to array of generator pointers}
  pGen    : TpComp;     {pointer to a generator}
  Vint    : complex;    {the internal generator voltage}
  VoltEq  : integer;    {index of the voltage equation}
begin
  ppGener := ppGener_gv;                        {p. to array of gen.pointers}
  for ii := 1 to nGener_gv do begin             {scan all generators}
    pGen := ppGener^;                           {pointer to this generator}
    inc(ppGener);
    pGen^.GetNode(2,VoltEq,err);                {get voltage equation number}
    getCvec(pGenVnew_gv,ii,nGener_gv,Vint);     {get the new internal voltage}
    putCvec(pRhs,VoltEq,nnod_gv,Vint);          {put it to rhs of network eq.}
  end;
  {solve the network}
  CLUbacksub(pAdmat,nnod_gv,pIndex,pRhs);               {backsubstitute}
  SaveAndDisplayResults(pSet,nnod_gv,freq_gv);          {in Elplex}
  {save the results for later use, when calculating short circuit currents}
  CopyPosSeqResults(pSet);       {in elplex.pas}
  {save the result as such, needed for superposition, in SolveNet of elplex}
  pSet^.List_Probl := probl;                            {save the problem type..}
  pSet^.SaveSolution(pSet^.Volx_List,pSet^.Voly_List);  {..and the solution}
end;

function    TSetVoltage.GetpGen(ii:integer): TpComp;
{get pointer to generator no ii }
var
  ppGen : TppComp;  {pointer to array of generator pointers}
begin
  if (ii > 0) and (ii <= nGener_gv) then begin
    ppGen := ppGener_gv;
    inc(ppGen,ii-1);
    GetpGen := ppGen^;
  end else begin
    GetpGen := nil;
  end;
end;

function TSetVoltage.IsDesiredPowerPositive: boolean;
{check, are the desired powers of all generators > 0}
var
  ii      : integer;     {temporary}
  PGgoal  : single;      {desired power}
  posit   : boolean;     {true, if desired powers positive}
  pGen    : TpComp;      {pointer to a generator}
  ppGener : TppComp;     {pointer to array of generator pointers}

begin
  posit := false;                            {initialize 1.10.2002}
  if (nGener_gv > 0) then begin
    ppGener := ppGener_gv;                   {p. to array of gen.pointers}
    for ii := 1 to nGener_gv do begin        {scan all generators}
      pGen := ppGener^;                      {pointer to this generator}
      inc(ppGener);                          {for next round}
      pGen^.GetPar(gGen_W,PGgoal);           {desired power}
      if (PGgoal > 0) then begin
        posit := true;
      end else begin
        posit := false;
        break;                               {one nonpositive is enough}
      end;
    end;
  end else begin
    posit := false;                          {no generators ?}
  end;
  IsDesiredPowerPositive := posit;           {copy to output}
end;

procedure SolveNetWithGivenOutputVoltages(ipSet : longint; probl : Solve);
{solve the load flow problem for the net
 ipSet   pointer to the set of components etc, as integer}
var
  isDecomposed : boolean;   {true, if the admittance matrix has been decomposed}
  pSet         : TpCompSet; {pointer to the "common"}
  success      : boolean;   {flags that load flow can be started}
begin
  pSet := TpCompSet(ipSet);
  {create obj.for load flow calculations}
  SetVolt := TSetVoltage.Create(ipSet,success);
  {continue, if there are generator(s) etc, i.e. if successful}
  if (success) then begin
    pSet^.GetMemForCurrTrAndRelays;    {meory for curr.tr and relays (if needed)}
    {equations for the network}
    SetVolt.ReserveConstructAndDecompose(probl,isDecomposed);
    {continue only, if managed to decompose the equation matrix}
    if (isDecomposed) then begin
      {solve the network using the given generator voltages}
      SetVolt.FindNominalSolution;
      {calculate derivative of generator output voltage wrt. internal voltage,
       that is needed in the solution process}
      SetVolt.CalculateGeneratorVoltageDerivative;
      {find eventual parallel generators}
      SetVolt.FindParallelGenerator;
      {solve the generator internal voltages}
      SetVolt.FindInternalVoltages;
      {solve the net using the new, adjusted internal generator voltages}
      SetVolt.SolveTheNetWithNewVoltages(probl);
    end else begin
      show_message('error in load flow: coeff. matrix ?');
    end;
    {free the reserved memory from matrices}
    FreeMat(SetVolt.nnod_gv);
    pSet^.FreeMemForCurrTrAndRelays;    {meory for curr.tr and relays (if needed)}
  end else begin               {carry out a normal cacluation, if was not successuful}
    {show_message('No generators (or some error). Carry out a normal calculation');}
    SolveNet(ipset,false,probl);
    pSet^.List_Probl := probl;                            {save the problem type..}
  end;
  pSet^.SavePrefaultVoltages('pref'); {save for buses, voltmeters, shorts}
  {finally, free the object for load flow calculations}
  SetVolt.Feeders_gv.free;     {free list of feeders}
  SetVolt.free;                {free the object for l-f calculations}
end;

end.
