{VOISIKO nimet generoida automaattisesti ?}
unit Imagodef;
{definitions for project imago}
interface

uses
  Graphics;

type
  TPhase = (start, progress, ready);   {phases of a task}

type
  TSeq = (Pos, Neg,Zero,None);

type
  {solved problem type: no fault, dynamical, standard,
   fault in all buses with stadards, with superposition}
  Tcalc = (Nofault, Dyn, std, AllStd, AllSup, AllZ012, Nocalc, SuperpOne);

{define a type that tells, which phase of calculation. Needed when short...}
{..NoFaultTheven needed when calculating Thevenin impedance}
{..TransientTheven needed, when calculating Thevenin impedance with transient impedances}
type FaultPhase = (Steady, Derivates, Transient, NoFaultTheven,TransientTheven);

{define a type that tells which calculation in progress}
type Solve = (Normal, AdjustVolt, SolvingLoadFlow, ThreePhaseShort, TwoPhaseShort,
              EarthShort,TwoPhaseEarth, IECFault, IECPeak, IECEarth,
              TapDeriv, TapLoad, TapVolt, Superpos, Z012Th,OpenLine);

{base power type of 3-winding trafo}
type
  T3wBasePowerFlag = (Primary, PrimaryFan, Secondary, SecondaryFan, Tertiary, TertiaryFan); 

type
  Ind3 = array[1..3] of integer;     {3 indices for PGQ-relays}
  Ind6 = array[1..6] of integer;     {6 indices for recloser relays}
const
  {some constants}
  Igrid_g = 16;          {the default grid spacing, in pixels}
  {flags for data type in *.sld file (input file), used in Skemproc.pas}
  Notes_g     = '&';     {flag for note-line in *.sld file}
  StrtComp_g  = ';';     {flag for indicating new component in input file}
  StrtFreq_g  = '#';     {flag: next item is the frequency in the input file}
  StrtTitle_g = '!';     {flag : next item is the title}
  StrtInfo_g  = '?';     {flag : next item contains some info}
  StrtList_g  = '}';     {flag for coordination data: list of relays, etc}
  StrtNote_g  = '%';     {flag : next lines are notes}
  StrtDist_g  = '';     {flag for distance relay list/graph}
  StrtPoly_g  = '';     {flag for polyg.distance relay list/graph}
  RelGrup_g   = ']';     {flag : data for automatic change of param.group}
  LoadMult_g  = ')';     {flag : next item is the load multiplier}
  ScenInfo_g  = '';     {flag : next item is description for load scenario}
  GrafInfo_g  = '^';     {flag : this line is data for graphical elements}
  EquInfo_g   = '';     {flag : this line is data for equations for PQ-relays}
  NegThree_g  = '';     {flag : next item is flag for 3*I2 or I2 in relays (I2 = neg. seq. current)}
  Sqrt3Volt_g = ' 3';   {sqrt(3) times phase neutral-voltages displayed}
  PhaseNeutral_g = ' n';{phase neutral voltages displayed}
  PhasePhase_g = ' ';  {phase-phase voltages displayed}
  DistanceSeq_g = '+-';  {use sequence variables for distance relays at zero volt}
  DistancePhase_g ='';  {use phase difference for distance relays at zero volt}
  ColorFlag_g = '}])';   {flag for coding font color, in relcurr and skemproc}
  CTIdent_g = '$';       {separator for identifiers, in specialCT.pas and skemadef.pas}
  gMaxWidth = 608{600};       {max. width of wires and buses}
  gMaxHeight = 448{450};      {max height of wires and buses}
  {x- and y-coordinates of conn.point of combined relay (PQ, EM, RC, UD, distance
   for connecting to a current trafo...}
  gRelCTx = 0;           {...used in FindCTcurr of Elplex.pas and ...}
  gRelCTy = 16;          {...FindCurrentTrafo of skemadef.pas}
  {existing colors of (some) components}
  clr_g  : array[0..5] of Tcolor = (clBlack,clBlue,clGreen,clRed,clAqua,clFuchsia);
  gLightGray : TColor = $00E7E7E7;           {color of components that are not in use}
  gLightGrayR : TColor = 0; {when light gray read from a picture, see FormCreate}
  crClickStart = 1;                          {index of cursor}
  gssmin = 1.0e-3;         {smallest relative length of a transmission line section}
  gssmax : double = 0.999; {largest -"- ...}
  {..gssmax must(?) be defined as double, else the compilation of RelCalc.pas will ...}
  {..change its value !? (But has the error disappeared, or only moved to somewhere...? 7.3.2013}
{names of (allowed) components}
{MUST be identical with the first item in the hint of the actual component
 in the Imagoo-form
 Could they be extracted automatically from the hint??}
  gCap    = 'Capacitor';
  gImped  = 'Impedance';
  gGrImped= 'Grounding Impedance'; {grounding impedance (for generators etc)}
  gGrcurrT= 'Ground current trafo';{grounding current trafo (for generator, star(Y)}
  gIsolImp= 'Isolating impedance'; {isolating impedance}
  gFeed   = 'Feeder';
  gTrafoYY= 'Transformator';       {Y-Y transformator}
  gTrafoYD= 'Transformator Y D';   {Y Delta transformator}
  gTrafoDY= 'Transformator D Y';   {Delta Y transformator}
  gTrafoDD= 'Transformator D D';   {Delta Delta transformator}
  gTrafo3W= 'Three-winding transformer'; {3-winding transformer}
  gIdealYD= 'Ideal 1:1 YD transformer';  {ideal 1:1 YD-transformer}
  gGrnd   = 'Ground';              {ground, for completeness}
  gWire   = 'wire';                {a wire, no impedance}
  gNoWire = 'nowire';              {a wire, that is not in use}
  gBus    = 'bus';                 {a bus, no impedance}
  gLoad   = 'Load';                {an impedance, connected to ground (or neutral)}
  gLoadOr = 'Load or PQ-source (or bus)';  {gLoad changed to this in CreModCom of Skemadef.pas}
  gGen    = 'Generator';           {AC generator}
  gSmot   = 'Synch.Motor';         {synchronous motor}
  gASmot  = 'Asynchronous Motor or Generator'; {asynchronous motor, or generator}
  gBrkr   = 'Breaker';             {open breaker}
  gTrliS  = 'Transmission line/km';{transmission line, with susceptance}
  gCurrT  = 'Current Transformer';
  gSpecCT = 'Summing Current Transformer';
  gSpecCTd = 'CTspec_d_img_m';
  gSpecCTu = 'CTspec_u_img_m';
  gSpecCTr = 'CTspec_r_img_m';
  gSpecCTl = 'CTspec_l_img_m';
  gSumCT  = 'Sum Current Transformer';
  gShort  = 'Short Circuit';
  gRelay  = 'relay';
  gRelay_inv = 'relay (normal inverse)';
  gRelay_vinv = 'relay (very inverse)';
  gRelay_einv = 'relay (extremely inverse)';
  grelay_Ltinv = 'relay (long time inverse)';
  gStar   = 'Terminating Star';     {phases connected in wye}
  gmeasZ  = 'Mho distance relay';   {measuring (apparent) impedance}
  gPolyg  = 'Polygonal distance relay';
  gPQrel  = 'PGQ Overcurrent relay';
  gEMRel  = 'Electromechanical relay';
  gRCRel  = 'Recloser relay';
  gUDRel  = 'User defined relay';
  gText   = 'Text';                    {the "text"-component}
  gVolt   = 'Voltmeter';
  gSymbol = 'breaker-2';               {diverse breaker symbols}
  gUnsymm = 'Unsymmetrical load';
  gOpenL  = 'Open line fault';
  gAtrafo = 'Autotrafo';
{old names, for downwards compability, and..}
  gACg    = 'AC-gen';
  gASmot0 = 'Asynch.Motor';            {old name of asynch.motor}
  gCurr   = 'Load (current drain)';    {constant current, as load}
  gMutual = 'Mutual impedance';        {mutual impedance, between transmission lines}
  gZ012   = 'Sequence Impedance';      {impedance, with different values in +,-,0 net}
{indices for parameters of components. They MUST match with the definitions
 in the hint-text of the images}
  gRes_R     = 1;      {resistance}
  gCap_C     = 1;      {capacitance}
  gCap_Np    = 2;      {number in parallel}
  gCap_Ns    = 3;      {number in series}
  gCap_V     = 4;      {reference voltage}
  gCap_Q     = 5;      {reference (apparent) power}
  gCap_A1    = 6;      {ampacity limit}
  gCap_A2    = 7;
  gCap_A3    = 8;
  gCap_A4    = 9;
  gInd_L     = 1;      {inductance}
  gACg_V     = 1;      {nominal voltage of AC generator}
  gACg_R     = 2;      {internal resistance}
  gACg_X     = 3;      {internal reactance}
  gACg_fi    = 4;      {phase angle (degrees)}
  gTrafo_V1  = 1;      {primary voltage of transformator}
  gTrafo_V2  = 2;      {secondary voltage}
  gTrafo_X   = 3;      {leakage reactance}
  gTrafo_R1  = 4{5};   {primary resistance}
  gTrafo_Io  = 5{4};   {open circuit prim. current}
  gTrafo_01t  = 6;      {Z0/Z1}
  gTrafo_X0  = 6;      {zero seq. reactance, version 0.9696,  26.5.08}
  gTrafo_R0  = 7;      {zero seq. resistance}
  gTrafo_R1g = 8{7};      {prim. resistance to ground}
  gTrafo_X1g = 9{8};      {sec.  resistance to ground}
  gTrafo_R2g = 10{9};      {prim. reactance to ground}
  gTrafo_X2g = 11{10};      {sec.  reactance to ground}
  gYDTrf_R1g = 8{7};      {R from neutral to ground for YD-trafo}
  gYDTrf_X1g = 9{8};      {X from neutral to ground for YD-trafo}
  gDYTrf_R2g = 8{7};      {R from neutral to ground for DY-trafo}
  gDYTrf_X2g = 9{8};      {X from neutral to ground for DY-trafo}
  gYY_tap    = 12{11};     {tap changer of a YY transformer}
  gYD_tap    = 10{9};      {tap changer of a YD transformer}
  gYD_phase  = 11{10};      {phase shift -30 or +30 degrees}
  gDY_tap    = 10{9};      {tap changer of a DY transformer}
  gDY_phase  = 11{10};      {phase shift -30 or +30 degrees}
  gDD_tap    = 7;      {tap changer of a DD transformer}
{ gYY_dtap   = 12;     {max tap change of a YY transformer}
{ gYD_dtap   = 10;     {max tap change of a YD transformer}
{ gDY_dtap   = 10;     {max tap change of a DY transformer}
{ gDD_dtap   = 8;      {max tap change of a DD transformer}
  gYY_tmin   = 17{16};     {min tap value of a YY transformer}
  gYY_tmax   = 18{17};     {max tap value of a YY transformer}
  gYY_parall = 19;     {number of parallel trafos}
  gYD_tmin   = 15{14};     {min tap value of a YD transformer}
  gYD_tmax   = 16{15};     {max tap value of a YD transformer}
  gYD_parall = 17;         {number of parallel trafors}
  gDY_tmin   = 15{14};     {min tap value of a DY transformer}
  gDY_tmax   = 16{15};     {max tap value of a DY transformer}
  gDY_parall = 17;         {number of parallel trafors}
  gDD_tmin   = 12;     {min tap value of a DD transformer}
  gDD_tmax   = 13;     {max tap value of a DD transformer}
  gDD_parall = 14;         {number of parallel trafors}
  gYY_VAn    = 14{13};     {nominal apparent power of a YY transformer}
  gYD_VAn    = 12{11};     {nominal apparent power of a YD transformer}
  gDY_VAn    = 12{11};     {nominal apparent power of a DY transformer}
  gDD_VAn    = 9;      {nominal apparent power of a DD transformer}
  gYY_Vs1    = 15{14};     {primary system voltage}
  gYD_Vs1    = 13{12};
  gDY_Vs1    = 13{12};
  gDD_Vs1    = 10;
  gYY_Vs2    = 16{15};     {secondary system voltage}
  gYD_Vs2    = 14{13};
  gDY_Vs2    = 14{13};
  gDD_Vs2    = 11;
  gIdeal_ph  = 3;          {phase shift, +30, or -30, or something else}
  gTrafo_A1  = 20;         {first ampacity limit}
  gTrafo_A2  = 21;
  gTrafo_A3  = 22;
  gTrafo_A4  = 23;
  gTrafo_dgc = 24;         {first damage curve parameter}
  gTrafo_disp= 24;         {damage curve displayed}
  gTrafo_OorA= 25;         {flag for oil/air cooling}
  gTrafo_SorU= 26;         {standard or user defined curve}
  gTrafo_cm  = 27;         {current multiplier}
  gTrafo_tm  = 28;         {time multiplier}
  gTrafo_name= 29;         {name of the curve}
  gTrafo_Clr = 30;         {color of the damage curve}
  gTrafo_thck= 31;         {thickness of the curve}
{ gTrafo_xxx = 32;          spare}
  gTrafo_nC  = 33;         {coord. of the name of damage curve in terms of current}
  gTrafo_nT  = 34;         {coord. of the name of damage curve in terms of time}
  {35..56 the same for the second and third damage curve}
  gTrafo_VAnf= 57;         {nominal fan-cooled power}
  gTrafo_bpf = 58;         {base power flag for Z%, 0 = VAn, 1 = VAnf}
  gTrafo_ncp = 11;         {number of curve parameters}
  gAtrafo_V1 = 1;          {autotrafo, primary voltage}
  gAtrafo_V2 = 2;
  gAtrafo_X1 = 3;
  gAtrafo_R1 = 4;
  gAtrafo_Rg = 5;
  gAtrafo_Xg = 6;
  gAtrafo_V3 = 7;
  gAtrafo_D  = 8;          {flag for D-tertiary winding (instead for Y}
  gAtrafo_X2 = 9;
  gAtrafo_R2 = 10;
  gAtrafo_X3 = 11;
  gAtrafo_R3 = 12;
{}
  g3W_V1     = 1;
  g3W_V2     = 2;
  g3W_V3     = 3;
  g3W_X1     = 4;
  g3W_X3     = 5;
  g3W_X2     = 6;
  g3W_R1     = 7;
  g3W_R2     = 8;
  g3W_R3     = 9;
  g3W_VA     = 10;
  g3W_type   = 11;     {2=YYD. NOTE: some parameters squeezed in, but 11 is correct!}
  g3W_phase  = 12;     {flag phase shift of D-winding, + or - 30 degrees}
  g3W_Vs1    = 13;
  g3W_Vs2    = 14;
  g3W_Vs3    = 15;
  g3W_R1g    = 16;
  g3W_X1g    = 17;
  g3W_R2g    = 18;
  g3W_X2g    = 19;
  g3W_R3g    = 20;
  g3W_X3g    = 21;
  g3W_zpp12  = 22;     {zero sequence impedance/pos.seq.impedance for w1-> w2}
  g3W_zpp13  = 23;     {zero sequence impedance/pos.seq.impedance for w1-> w3}
  g3W_zpp23  = 24;     {zero sequence impedance/pos.seq.impedance for w2-> w3}
  g3W_VAs    = 25;     {secondary power}
  g3W_VAt    = 26;     {tertiary power}
  g3W_VAfp   = 27;     {primary fan cooled power}
  g3W_VAfs   = 28;     {secondary fan cooled power}
  g3W_VAft   = 29;     {tertiary fan cooled power}
  g3W_bpf    = 30;     {base power flag for Z%}
  g3w_VApUD  = 31;     {user def. prim. base power}
  g3w_VAsUD  = 32;     {user def. sec. base power}
  g3w_VAtUD  = 33;     {user def. tert. base power}
  g3w_A1     = 34;     {ampacity limit 1}
  g3w_A2     = 35;     {ampacity limit 2}
  g3w_A3     = 36;     {ampacity limit 3}
  g3w_A4     = 37;     {ampacity limit 4}
  {}
  gImped_R   = 1;      {resistance of impedance}
  gImped_X   = 2;      {reactance}
  gTline_R   = 1;      {resistance/m of a transmission line}
  gTline_C   = 2;      {capacitance/m }
  gTline_L   = 3;      {inductance/m }
  gTline_len = 4;      {length (m)}
  gFeed_V    = 1;      {voltage of feeder}
  gFeed_R    = 2;      {internal resistance}
  gFeed_X    = 3;      {reactance}
  gFeed_fi   = 4;      {phase angle (degrees)}
  gFeed_Rg   = 5;      {grounding resistance}
  gFeed_Xg   = 6;      {grounding reactance}
  gFeed_R0   = 7;      {zero sequence resistance}
  gFeed_X0   = 8;      {zero sequence reactance}
  gFeed_R2   = 9;      {neg. sequence resistance}
  gFeed_X2   = 10;     {neg. sequence reactance}
//  gTrf3_L11  = 1;      {primary inductance of (3-phase)transformator}
//  gTrf3_L22  = 2;      {secondary inductance}
//  gTrf3_L12  = 3;      {mutual inductance}
//  gTrf3_R1   = 4;      {primary resistance}
//  gTrf3_R2   = 5;      {secondary resistance}
//  gTrf3_R1g  = 6;      {prim. resistance to ground}
//  gTrf3_R2g  = 7;      {sec.  resistance to ground}
  gLoad_VA   = 1;      {VA-rating (power) at nominal voltage}
  gLoad_Vn   = 2;      {nominal phase to phase voltage}
  gLoad_pf   = 3;      {power factor}
  gLoad_m    = 4;      {multiplier, or scaling factor}
  gUnsym_Ra  = 1;      {unsymmetrical load, resistance in phase a}
  gUnsym_Xa  = 2;      {unsymmetrical load, reactance in phase a (in series with Ra)}
  gUnsym_Rb  = 3;      {unsymmetrical load, resistance in phase b}
  gUnsym_Xb  = 4;      {unsymmetrical load, reactance in phase b}
  gUnsym_Rc  = 5;      {unsymmetrical load, resistance in phase c}
  gUnsym_Xc  = 6;      {unsymmetrical load, reactance in phase c}
  gUnsym_Rg  = 7;      {unsymmetrical load, earthing resistance}
  gUnsym_Xg  = 8;      {unsymmetrical load, earthing reactance (in series with Rg)}
  gUnsym_Vn  = 9;      {unsymmetrical load, nominal voltage}
  gUnsym_m   =10;      {unsymmetrical load, multiplier (no of parallel loads)}
  gGen_Vn    = 1;      {nominal voltage of generator (phase to phase)}
  gGen_Rs    = 2;      {stator resistance (of one phase)}
  gGen_Xd    = 3;      {stator reactance (of one phase)}
  gGen_fi    = 4;      {phase angle}
  gGen_Xdd   = 5;      {transient reactance}
  gGen_Xd2d  = 6;      {subtransient reactance}
  gGen_Tdd   = 7;      {rotor transient (short circuit) time constant}
  gGen_Td2d  = 8;      {subtransient (short circuit) time constant}
  gGen_X0    = 9;      {zero seq. reactance}
  gGen_X2    =10;      {neg.seq. reactance}
  gGen_Xqd   =11;      {transient q-axis reactance Xq'}
  gGen_Xq2d  =12;      {subtransient q-axis reactance Xq"}
  gGen_Tqd   =13;      {transient q-axis short circuit time constant}
  gGen_Tq2d  =14;      {subtransient q-axis short circuit time constant}
  gGen_W     =15;      {desired generator power}
  gGen_VAn   = 16;     {nominal apparent power}
  gGen_pf    = 17;     {power factor = cos(fii)}
  gGen_Vs    = 18;     {system voltage}
  gGen_paral = 19;     {number of parallel generators}
  gGen_A1    = 20;     {first ampacity limit}
  gGen_A2    = 21;
  gGen_A3    = 22;
  gGen_A4    = 23;
  gSmot_Vn   = 1;      {nominal voltage of synch.motor (phase to phase)}
  gSmot_Rs   = 2;      {stator resistance (of one phase)}
  gSmot_Xd   = 3;      {stator reactance (of one phase)}
  gSmot_fi   = 4;      {phase angle}
  gSmot_Xdd  = 5;      {transient reactance}
  gSmot_Xd2d = 6;      {subtransient reactance}
  gSmot_Tdd  = 7;      {rotor short circuit time constant}
  gSmot_Td2d = 8;      {subtransient short circuit time constant}
  gSmot_ef   = 9;      {efficiency}
  gSmot_Load =10;      {load}
  gSmot_Xqd  =11;      {transient q-axis reactance}
  gSmot_Xq2d =12;      {subtransient q-axis reactance}
  gSmot_Tqd  =13;      {transient q-axis time constant}
  gSmot_Tq2d =14;      {subtransient q-axis time constant}
  gSmot_VAn  = 15;     {nominal apparent power}
  gSmot_Pn   = 15;     {nominal power}
  gASmot_Vn  = 1;      {nominal voltage}
  gASmot_Pn  = 2;      {nominal power}
  gASmot_sn  = 3;      {slip at nominal power}
  gASmot_pf  = 4;      {power factor at nominal power}
  gASmot_eff = 5;      {efficiency at nominal power}
  gASmot_IsIn= 6;      {ratio of locked rotor current to nominal current}
  gASmot_L   = 7;      {actual power}
  gASmot_m   = 8;      {no of pole pairs}
  gASmot_Tstrt= 9;     {starting time}
  gASmot_disp= 10;     {display starting curve}
  gBus_Vn     = 1;     {nominal voltage of bus}
  gBus_puVmin = 2;     {minimum per unit voltage}
  gBus_puVmax = 3;     {maximum per unit voltage}
  gCT_prim    = 1;     {primary current}
  gCT_sec     = 2;     {secondary current}
  gCT_Conn    = 3;     {connection, Y or D}
  gCT_ratio   = 4;     {transformation ratio, = prim/sec current}
  gCT_mode    = 5;     {output mode = prim, sec. or relative sec. current}
                           {0 = use primary current,
                            1 = use secondary current,
                            2 = use relative sec. current = Isec/Inom,sec}
  gCT_mult    = 6;     {multiplier for current}                          
  gSCT_summode= 24;    {summing mode: 0 = prim, <> 0 = secondary}
  gSCT_prim   = 25;    {primary current}
  gSCT_sec    = 26;    {secondary current}
  gSCT_Conn   = 27;    {connection, Y or D}
  gSCT_ratio  = 28;    {transformation ratio, = prim/sec current}
  gSCT_mode   = 29;    {output mode = prim, sec. or relative sec. current}
  gBrkr_st   = 1;      {breaker status, '0' =open,  '<> 0' = closed}
  gTrliS_R   = 1;      {resistance/km of a transmission line}
  gTrliS_S   = 2;      {susceptance/km  (omega*C)}
  gTrliS_X   = 3;      {reactance/km    (omega*L)}
  gTrliS_len = 4;      {length (km)}
  gTrliS_R0  = 5;      {zero sequence resistance}
  gTrliS_X0  = 6;      {zero sequence reactance}
  gTrliS_s1  = 7;      {relative length of the first section}
  gTrliS_A1  = 8;      {first ampacity limit}
  gTrliS_A2  = 9;
  gTrliS_A3  = 10;
  gTrliS_A4  = 11;
  gTrliS_Curr  = 12;   {damage current}
  gTrliS_Time  = 13;   {damage time}
  gTrliS_MinT  = 14;   {minimum time for the curve}
  gTrliS_MaxT  = 15;   {max.time for the curve}
  gTrliS_Name  = 16;   {name = title of the curve}
  gTrliS_Color = 17;   {color index of the curve}
  gTrliS_Thick = 18;   {thickness of the curve}
  gTrliS_Volt  = 19;   {voltage}
  gTrliS_InUse = 20;   {flag for in use/or not}
  {21}
  gTrliS_nC    = 22;   {coordinate of damage curve label as current}
  gTrliS_nT    = 23;   {coordinate of damage curve label as time}
  gTrliS_paral = 24;   {number of parallel lines}
  gCurrT_I   = 1;      {current at which relay reacts}
  gCurrT_t   = 2;      {time delay after which relay reacts}
  gRelay_I   = 1;      {activation current}
  gRelay_t   = 2;      {activation time}
  gInv_I     = 1;      {starting current for inverse time relay}
  gInv_k     = 2;      {time multiplier for inv. time realy}
  gGrImped_R = 1;      {grounding resistance, for generator}
  gGrImped_X = 2;      {grounding reactance, for generator}
  gCurr_Ir   = 1;      {real part of current}
  gCurr_Ii   = 2;      {imag. part of current}
  gCurr_VA   = 3;      {(apparent) power}
  gCurr_cosf = 4;      {cos fii}
  gMutu_nOne = 5;      {number of nodes for one line in the mutual impedance component}
  gMutu_Rm0  = 1;      {zero seq. resistance/length of mutual impedance}
  gMutu_Xm0  = 2;      {zero seq. reactance/length of mutual impedance}
  gMutu_L    = 3;      {length of line with mutual impedance}
  gPQ_Group : Ind3 = ( 1, 2, 3);  {curve family}
  gPQ_Curve : Ind3 = ( 4, 5, 6);  {curve}
  gPQ_Iset  : Ind3 = ( 7, 8, 9);  {setup current}
  gPQ_Fact  : Ind3 = (10,11,12);  {multiplier for setup current}
  {}
  gPQ_Tmult : Ind3 = (16,17,18);  {time multiplier}
  gPQ_Tadd  : Ind3 = (19,20,21);  {time adder}
  gPQ_Tmin  : Ind3 = (22,23,24);  {minimum time}
  {}
  gPQ_I1    : Ind3 = (28,29,30);  {activation current of def.time module 1}
  gPQ_t1    : Ind3 = (31,32,33);  {activation time of def.time module 1}
  gPQ_I2    : Ind3 = (34,35,36);  {activation current of def.time module 2}
  gPQ_t2    : Ind3 = (37,38,39);  {activation time of def.time module 2}
  gPQ_I3    : Ind3 = (40,41,42);  {activation current of def.time module 3}
  gPQ_t3    : Ind3 = (43,44,45);  {activation time of def.time module 3}
  gPQ_I4    : Ind3 = (46,47,48);  {activation current of def.time module 3}
  gPQ_t4    : Ind3 = (49,50,51);  {activation time of def.time module 3}
  {}
  gPQ_Imax  : Ind3 = (55,56,57);  {max.current}
  gPQ_dir   : Ind3 = (58,59,60);  {directional/not}
  gPQ_Z     : Ind3 = (61,62,63);  {line impedance}
  gPQ_angle : Ind3 = (64,65,66);  {angle of line impedance}
  gPQ_show  : Ind3 = (67,68,69);  {show/not curve}
  gPQ_width : Ind3 = (70,71,72);  {line thickness}
  gPQ_color : Ind3 = (73,74,75);  {line color}
  gPQ_set   = 88;                 {parameter set (or group) in use}
  gPQ_auto  = 89;                 {auto/man change of the parameter set}
  {recloser relay}
  gRC_curve : Ind6 = (  1,  2,  3,  4,  5,  6); {index for the char. curve}
  gRC_Iset  : Ind6 = (  7,  8,  9, 10, 11, 12); {setup current}
  gRC_Fact  : Ind6 = ( 13, 14, 15, 16, 17, 18); {multiplier for min.current}
  gRC_ix    : Ind6 = ( 19, 20, 21, 22, 23, 24); {x-coord. of curve identifier}
  gRC_Tmult : Ind6 = ( 25, 26, 27, 28, 29, 30); {time multiplier}
  gRC_Tadd  : Ind6 = ( 31, 32, 33, 34, 35, 36); {time adder}
  gRC_Tmin  : Ind6 = ( 37, 38, 39, 40, 41, 42); {minimum time}
  gRC_iy    : Ind6 = ( 43, 44, 45, 46, 47, 48); {y-coord. of curve identifier}
  gRC_I1    : Ind6 = ( 49, 50, 51, 52, 53, 54); {activation current of def.time module 1}
  gRC_t1    : Ind6 = ( 55, 56, 57, 58, 59, 60); {activation time of def.time module 1}
  gRC_I2    : Ind6 = ( 61, 62, 63, 64, 65, 66); {activation current of def.time module 2}
  gRC_t2    : Ind6 = ( 67, 68, 69, 70, 71, 72); {activation time of def.time module 2}
  gRC_I3    : Ind6 = ( 73, 74, 75, 76, 77, 78); {activation current of def.time module 3}
  gRC_t3    : Ind6 = ( 79, 80, 81, 82, 83, 84); {activation time of def.time module 3}
  gRC_I4    : Ind6 = ( 85, 86, 87, 88, 89, 90); {activation current of def.time module 3}
  gRC_t4    : Ind6 = ( 91, 92, 93, 94, 95, 96); {activation time of def.time module 3}
  gRC_Imax  : Ind6 = (103,104,105,106,107,108); {maximum current}
  gRC_dir   : Ind6 = (109,110,111,112,113,114); {directional/not}
  gRC_Z     : Ind6 = (115,116,117,118,119,120); {line impedance}
  gRC_angle : Ind6 = (121,122,123,124,125,126); {angle of line impedance}
  gRC_show  : Ind6 = (127,128,129,130,131,132); {show/not curve}
  gRC_width : Ind6 = (133,134,135,136,137,138); {line thickness}
  gRC_color : Ind6 = (139,140,141,142,143,144); {line color}
  gRC_spare : Ind6 = (145,146,147,148,149,150); {spare}
  gRC_set    = 169;
  gRC_auto   = 170;
  {it seems that there are some 30 spare places}
{!}gUD_1 = {151}{163}175;                       {first data for user defined curve}
  gUD_2 = 415{403}{391}{231};       {gUD_2 - gUD_1 = 2*no of data points for UD-char...}
  {...must be equal to (CharForm.CharGrid.RowCount-1)*4 }
  gUD_3 = gUD_2+(gUD_2-gUD_1);
  gUD_4 = gUD_3+(gUD_2-gUD_1);
  gUD_5 = gUD_4+(gUD_2-gUD_1);
  gUD_6 = gUD_5+(gUD_2-gUD_1);
{ gRC_char  : Ind6 = (gRC_1,gRC_2,311,391,471,551); {points for user def.char start here}
  gUD_char  : Ind6 = (gUD_1,gUD_2,gUD_3,gUD_4,gUD_5,gUD_6);
  gUD_last = gUD_6+(gUD_2-gUD_1);               {for saving last (largest) current and (smallest) time}
  gPoly_alf  = 3;      {polygonal relay, angle alpha}
  gPoly_bet  = 4;      {angle beta}
  gPoly_gam  = 5;      {gamma}
  gPoly_RpR  = {6}7;      {ratio of left and right resistance}
//  gPoly_Z    = 30;     {impedance of transm.line}
//  gPoly_fi   = 31;     {phase angle of transm.line}
  {indices for results from components. They MUST match with the definitions - - - - - - - - }
  gTwo_V1r   = 1;      {upstream voltage, real part, of two-node component}
  gTwo_V1i   = 2;      {upstream voltage, imag.part}
  gTwo_V2r   = 3;      {downstream voltage, real part}
  gTwo_V2i   = 4;      {downstream voltage, imag.part}
  gTwo_Ir    = 5;      {current, real part}
  gTwo_Ii    = 6;      {current, imag.part}
  gTwo_P     = 7;      {real power (of capacitor)}
  gTwo_Q     = 8;      {apparent power (of capacitor)}
  gOne_Vr    = 1;      {voltage, real part, of one-node component}
  gOne_Vi    = 2;      {voltage, imag.part}
  gOne_Ir    = 3;      {current, real part}
  gOne_Ii    = 4;      {current, imag.part}
  gOne_P     = 5;      {power}
  gOne_Q     = 6;      {reactive power}
  gACg_V1r   = 1;      {upstream (internal) voltage, real part, of AC generator}
  gACg_V1i   = 2;      {upstream (internal) voltage, imag.part}
  gACg_V2r   = 3;      {downstream (external)voltage, real part}
  gACg_V2i   = 4;      {downstream (external)voltage, imag.part}
  gACg_Ir    = 5;      {current, real part}
  gACg_Ii    = 6;      {current, imag.part}
  gTrafo_V1r = 1;      {primary voltage, real part, of transformator}
  gTrafo_V1i = 2;      {primary voltage, imag.part}
  gTrafo_V2r = 3;      {secondary voltage, real part}
  gTrafo_V2i = 4;      {secondary voltage, imag.part}
  gTrafo_I1r = 5;      {prim. current, real part}
  gTrafo_I1i = 6;      {prim. current, imag.part}
  gTrafo_I2r = 7;      {sec. current, real part}
  gTrafo_I2i = 8;      {sec. current, imag.part}
  g3W_V1r   = 1;
  g3W_V1i   = 2;
  g3W_V2r   = 3;
  g3W_V2i   = 4;
  g3W_V3r   = 5;
  g3W_V3i   = 6;
  g3W_I1r   = 7;
  g3W_I1i   = 8;
  g3W_I2r   = 9;
  g3W_I2i   = 10;
  g3W_I3r   = 11;
  g3W_I3i   = 12;
  gImped_V1r = 1;      {upstream voltage, real part, of impedance}
  gImped_V1i = 2;      {upstream voltage, imag.part}
  gImped_V2r = 3;      {downstream voltage, real part}
  gImped_V2i = 4;      {downstream voltage, imag.part}
  gImped_Ir  = 5;      {current, real part}
  gImped_Ii  = 6;      {current, imag.part}
  gTline_V1r = 1;      {upstream voltage, real part, of transmission line}
  gTline_V1i = 2;      {upstream voltage, imag.part}
  gTline_V2r = 3;      {downstream voltage, real part}
  gTline_V2i = 4;      {downstream voltage, imag.part}
  gTline_Ir  = 5;      {(inlet)current, real part}
  gTline_Ii  = 6;      {(inlet)current, imag.part}
  gFeed_V1r  = 1;      {internal voltage, real part}
  gFeed_V1i  = 2;      {internal voltage, imag.part}
  gFeed_V2r  = 3;      {downstream (external)voltage, real part}
  gFeed_V2i  = 4;      {downstream (external)voltage, imag.part}
  gFeed_Ir   = 5;      {current, real part}
  gFeed_Ii   = 6;      {current, imag.part}
  gFeed_P    = 7;      {power}
  gFeed_Q    = 8;      {reactive power}
  gFeed_Vnr  = 9;      {neutral voltage, real part}
  gFeed_Vni  =10;      {neutral voltage, imag.part}
//  V1r,V1i,V2r,V2i,I1r,I1i,I2r,I2i,Vn1r,Vn1i
  gAtrafo_V1r = 1;
  gAtrafo_V1i = 2;
  gAtrafo_V2r = 3;
  gAtrafo_V2i = 4;
  gAtrafo_I1r = 5;
  gAtrafo_I1i = 6;
  gAtrafo_I2r = 7;
  gAtrafo_I2i = 8;
  gAtrafo_Vnr = 9;
  gAtrafo_Vni = 10;
  gAtrafo_V3r = 11;
  gAtrafo_V3i = 12;
  gAtrafo_I3r = 13;
  gAtrafo_I3i = 14;

// YY-trafo
  gTrafoYY_Vn1r = 9;   {neutral voltage, real part}
  gTrafoYY_Vn1i = 10;  {neutral voltage, imag.part}
  gTrafoYY_Vn2r = 11;  {neutral voltage, real part}
  gTrafoYY_Vn2i = 12;  {neutral voltage, imag.part}
  gTrafoYD_Vnr = 9;    {neutral voltage, real part}
  gTrafoYD_Vni = 10;   {neutral voltage, imag.part}
//gTrafoYD_Iwr = 11;   {winding current, real part}
//gTrafoYD_Iwi = 12;   {winding current, imag part}
  gTrafoDY_Vnr = 9;    {neutral voltage, real part}
  gTrafoDY_Vni = 10;   {neutral voltage, imag.part}
  gTrf3_V1r  = 1;      {primary voltage, real part, of (3-phase)transformator}
  gTrf3_V1i  = 2;      {primary voltage, imag.part}
  gTrf3_V2r  = 3;      {secondary voltage, real part}
  gTrf3_V2i  = 4;      {secondary voltage, imag.part}
  gTrf3_I1r  = 5;      {prim. current, real part}
  gTrf3_I1i  = 6;      {prim. current, imag.part}
  gTrf3_I2r  = 7;      {sec. current, real part}
  gTrf3_I2i  = 8;      {sec. current, imag.part}
  gLoad_Vr   = 1;      {voltage, real part, of load}
  gLoad_Vi   = 2;      {voltage, imag.part}
  gLoad_Ir   = 3;      {current, real part}
  gLoad_Ii   = 4;      {current, imag.part}
  gGen_V1r   = 1;      {upstream (internal) voltage, real part, of generator}
  gGen_V1i   = 2;      {upstream (internal) voltage, imag.part}
  gGen_V2r   = 3;      {downstream (external)voltage, real part}
  gGen_V2i   = 4;      {downstream (external)voltage, imag.part}
  gGen_Ir    = 5;      {current, real part}
  gGen_Ii    = 6;      {current, imag.part}
  gGen_P     = 7;      {power}
  gGen_Q     = 8;      {reactive power}
  gSMot_V1r  = 1;      {internal voltage, real part, of synchr.motor}
  gSMot_V1i  = 2;      {internal voltage, imag.part}
  gSmot_Vr   = 3;      {voltage, real part, of synchr. motor}
  gSmot_Vi   = 4;      {voltage, imag.part}
  gSmot_Ir   = 5;      {current, real part}
  gSmot_Ii   = 6;      {current, imag.part}
  gSmot_P    = 7;      {real power}
  gSmot_Q    = 8;      {reactive power}
  gASmot_Vr  = 1;      {voltage, real part, of asynchr. motor}
  gASmot_Vi  = 2;      {voltage, imag.part}
  gASmot_Ir  = 3;      {current, real part}
  gASmot_Ii  = 4;      {current, imag.part}
  gASmot_P   = 5;      {power}
  gASmot_Q   = 6;      {reactive power}
{ gBrkr_                breaker, ntohing shown}
  gTrliS_V1r = 1;      {upstream voltage, real part, of transmission line}
  gTrliS_V1i = 2;      {upstream voltage, imag.part}
  gTrliS_V2r = 3;      {downstream voltage, real part}
  gTrliS_V2i = 4;      {downstream voltage, imag.part}
  gTrliS_I1r = 5;      {inlet current, real part}
  gTrliS_I1i = 6;      {inlet current, imag.part}
  gTrliS_I2r = 7;      {output current, real part}
  gTrliS_I2i = 8;      {output current, imag.part}
  gCurrT_Ir  = 1;      {current, real part, of current trafo (also relay)}
  gCurrT_Ii  = 2;      {current, imag.part}
  gCurrT_P   = 3;      {real power}
  gCurrT_Q   = 4;      {imag. power}
  gSpecCT_Ir = 1;      {processed, real current of special current trafo}
  gSpecCT_Ii = 2;      {processed, imag.current of special current trafo}
  gSpecNP_Ir = 7;      {Special current trafo: real current, non-processed value}
  gSpecNP_Ii = 8;      {Special current trafo: imag.current, non-processed value}
  gShort_Vr  = 1;      {prefault  voltage, real part, of short circuit}
  gShort_Vi  = 2;      {prefault voltage, imag.part}
  gShort_Ir  = 3;      {fault current, real part}
  gShort_Ii  = 4;      {fault current, imag.part}
  gShort_Ip  = 5;      {peak short circuit current}
  gShort_Ib  = 6;      {breaking current}
  gBus_Vr    = 1;      {voltage at a bus, real part}
  gBus_Vi    = 2;      {imag. part}
  gBus_puV   = 3;      {per unit voltage (magnitude)}
  gRelay_Ik  = 1;      {initial transient current}
  gRelay_fii = 2;      {phase of relay current}
  g2Short_Vr = 1;      {voltage, real part, of phase to phase short circuit}
  g2Short_Vi = 2;      {voltage, imag.part}
  gCurr_Vr   = 1;      {real part of voltage}
  gCurr_Vi   = 2;      {imag. part of voltage}
  gCurr_Irout= 3;      {real part of current}
  gCurr_Iiout= 4;      {imag.part of current}
  gCurr_P    = 5;      {power}
  gGrImped_Vr= 1;      {real part of neutral voltage}
  gGrImped_Vi= 2;      {imag part of neutral voltage}
  gGrImped_Ir= 3;      {real part of neutral current}
  gGrImped_Ii= 4;      {imag part of neutral current}
  gGrcurrT_Ir= 1;      {real comp. of neutral current in gr.curr CT}
  gGrcurrT_Ii= 2;      {imag.comp. of neutral current in gr.curr CT}
  gmeasZ_Z   = 1;      {impedance}
  gmeasZ_f   = 2;      {phase}
  gPQ_Vr     = 1;      {real comp. of voltage of PQR-relay}
  gPQ_Vi     = 2;      {imag.comp. of voltage}
  gPQ_Ir     = 3;      {real comp. of current}
  gPQ_Ii     = 4;      {imag.comp. of current}
  gUnsymm_Vr = 1;      {unsymmetrical load, voltage}
  gUnsymm_Vi = 2;
  gUnsymm_Ir = 3;
  gUnsymm_Ii = 4;
  gUnsymm_Vnr= 5;      {unsymmetrical load, voltage of the neutral point}
  gUnsymm_Vni= 6;
  gOL_Ir     = 1;      {open line, real part of current}
  gOL_Ii     = 2;      {open line, imag.part of current}
  gOLd_Ir     = 3;     {open line, real part of downstream current}
  gOLd_Ii     = 4;     {open line, imag.part of downstream current}
  {the following are not yet used (?)}
  gMutu_V1ur   = 1;     {upstream voltage, real part of line 1 of mutual impedance}
  gMutu_V1ui   = 2;     {upstream voltage, imag.part  of line 1 of mutual impedance}
  gMutu_V1dr   = 3;     {downstream voltage, real part of line 1 of mutual impedance}
  gMutu_V1di   = 4;     {downstream voltage, imag.part  of line 1 of mutual impedance}
  gMutu_V2ur   = 5;     {upstream voltage, real part of line 2 of mutual impedance}
  gMutu_V2ui   = 6;     {upstream voltage, imag.part  of line 2 of mutual impedance}
  gMutu_V2dr   = 7;     {downstream voltage, real part of line 2 of mutual impedance}
  gMutu_V2di   = 8;     {downstream voltage, imag.part  of line 2 of mutual impedance}
  gMutu_I1r    = 9;     {current, real part of line 1 of mutual impedance}
  gMutu_I1i    = 10;    {current, imag part of line 1 of mutual impedance}
  gMutu_I2r    = 11;    {current, real part of line 2 of mutual impedance}
  gMutu_I2i    = 12;    {current, imag part of line 2 of mutual impedance}

implementation

(*procedure InvTimePar(ModNam : string; var alpha: single;
                                      var beta : single);
{gives the parameters alpha and beta for different types of
 inverse-time relays}
begin
  if          (ModNam = gRelay_inv ) then begin  {normal inverse}
    alpha := 0.02;
    beta  := 0.14;
  end else if (ModNam = gRelay_vinv) then begin  {very inverse}
    alpha := 1.0;
    beta  := 13.5;
  end else if (ModNam = gRelay_einv) then begin  {extremely inverse}
    alpha := 2.0;
    beta  := 80.0;
  end else if (ModNam = gRelay_Ltinv) then begin {long time inverse}
    alpha := 1.0;
    beta  := 120.0;
  end else begin    {unknown}
    alpha := 0.02;
    beta  := 0.14;
  end;
end;  *)

{end of unit *******************}
end.
