unit morefuses;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, ComCtrls, ExtCtrls, Routines, Utildef, Menus, Buttons;

type
  TFuseForm = class(TForm)
    ClosedDirImage: TImage;
    OpenDirImage: TImage;
    OpenDirImg16: TImage;
    ExcelImage: TImage;
    Graph: TImage;
    DirLabel: TLabel;
    FileLabel: TLabel;
    CurveLabel: TLabel;
    DirectoryTree: TTreeView;
    FileTree: TTreeView;
    DirMarker: TEdit;
    OKBut: TButton;
    CurveGrid: TStringGrid;
    FileMarker: TEdit;
    ShowBut: TButton;
    ClearBut: TButton;
    DirImageList: TImageList;
    FileImageList: TImageList;
    CancelBut: TButton;
    CurveInfoMemo: TMemo;
    TopMenu: TMainMenu;
    FileMenu: TMenuItem;
    exit1: TMenuItem;
    ViewMenu: TMenuItem;
    InfoItem: TMenuItem;
    SeparMenu: TMenuItem;
    SemicolItem: TMenuItem;
    CommaItem: TMenuItem;
    CloseBut: TBitBtn;
    HelpMemo: TMemo;
    HelpMenu: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DirectoryTreeCollapsed(Sender: TObject; Node: TTreeNode);
    procedure DirectoryTreeExpanded(Sender: TObject; Node: TTreeNode);
    procedure DirectoryTreeKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DirectoryTreeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FileTreeCollapsed(Sender: TObject; Node: TTreeNode);
    procedure FileTreeExpanded(Sender: TObject; Node: TTreeNode);
    procedure FileTreeKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileTreeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CurveGridSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure OKButClick(Sender: TObject);
    procedure ShowButClick(Sender: TObject);
    procedure ClearButClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure CancelButClick(Sender: TObject);
    procedure exit1Click(Sender: TObject);
    procedure InfoItemClick(Sender: TObject);
    procedure CurveInfoMemoMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure SemicolItemClick(Sender: TObject);
    procedure CommaItemClick(Sender: TObject);
    procedure CloseButClick(Sender: TObject);
    procedure HelpMemoMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure HelpMemoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure HelpMenuClick(Sender: TObject);

  private
    { Private declarations }
    CurveFileName : string;      {name with path of the file of curves}
    CurrTimeNo    : integer;     {index of the line with the text 'Current;Time...}
    isSemiColon   : boolean;     {lines start with a semicolon}
    CurveRow      : integer;     {selected row}
  public
    { Public declarations }
    procedure TreeOfFiles(FileNode: TTreeNode);
    procedure NewTreeOfFiles(FileNode: TTreeNode);
    procedure ShowFilePath(FileNode: TTreeNode);
    procedure ExtractCurveNames(line: string);
    procedure ClearCurveGrid;
    procedure ReadCurveData(iCurve: integer);
    procedure DrawGraph;
    procedure Coordinates(sCurr,sTime: string; var ix,iy: integer);
    procedure GetInvCurrTime(ii: integer; var curr, temps: string);
    function  CurrListCount: integer;
    function  FullCurveName: string;
    function  GetCurveRow: integer;
    procedure ShowCurveInfo;
    function  GetSeparator: Char;
  end;

var
  FuseForm: TFuseForm;

implementation

uses userdef, recloser, imagopas;

{$R *.DFM}

var
  AllPaths   : TStringList; {pathes to all nodes (directories)}
  FilePaths  : TStringList; {pathes to all files in a given directory}
  CurveInfo  : TStringList; {info about the curves, from the file}
  CurveNames : TStringList; {names of the curves}
  CurrList   : TStringList; {values of current, in the curves}
  TimeList   : TStringList; {values of times, in the curves}
  CurveClr   : array[1..14] of TColor = (clBlack , clAqua, clBlue , clOlive , clYellow,
                                         clMaroon, clRed , clGreen, clPurple, clLime,
                                         clNavy  , clTeal, clGray , clFuchsia);

function TFuseForm.CurrListCount: integer;
begin
  CurrListCount := CurrList.count;
end;

function  TFuseForm.FullCurveName: string;
{get the path + file name + curve name of the selected curve
Not good, row not always up to date !!???}
begin
  FullCurveName := CurveFileName+'\'+CurveGrid.cells[0,CurveGrid.row];
end;

function TFuseForm.GetCurveRow: integer;
begin
  GetCurveRow := CurveRow;
end;

function  TFuseForm.GetSeparator: Char;
{check is the separator for data a semicolon or a comma}
begin
  if (SemicolItem.checked) then GetSeparator := ';'
  else GetSeparator := ',';
end;

procedure TFuseForm.ShowCurveInfo;
{the information of the curve has been read from a file to CurveInfo
 Process the info = remove useless parts, copy useful to CurveInfoMemo}
var
  ii,jj : integer;  {index}
  len   : integer;  {length of line}
  line  : string;   {one line of text}
  spr   : char;     {separator}
  strt  : boolean;  {start to copy, after first non-space and non-semicolon char.}
  txt   : string;   {one more line}
begin
  spr := GetSeparator;                          {separator between data, ; or ,}
  CurveInfoMemo.clear;                          {clear initially}
  if (CurveInfo.count > 0) then  begin
    for ii := 0 to CurveInfo.count-1 do begin   {all lines}
      line := CurveInfo[ii];              {copy to local}
      len := length(line);
      if (len > 0) then begin
        txt := '';
        {there might be a semicolon in the beginning of the line}
        strt := false;
        for jj := 1 to len do begin
          if (line[jj] = spr{';'}) then begin
            if (not strt) then strt := true       {start reading at first ;}
            else break;                           {stop reading at second ; (or..)}
          end else if (line[jj] = ' ') then begin {initial spaces omitted}
            if (strt) then
              txt := txt+line[jj];
          end else begin                          {not space, not semicolon = a "proper" char}
            if (not strt) then strt := true;      {start at first "proper" char, even without a ;}
            txt := txt+line[jj];
          end;
        end;
        if (length(txt) > 0) then CurveInfoMemo.lines.add(txt)
        else break;                 {empty line = end of information, probably}
      end;
    end;
  end;
end;

procedure TFuseForm.GetInvCurrTime(ii: integer; var curr, temps: string);
{get the ii'th current and time FROM the END in the CurrList and TimeList
 ii =1 => the last in the list}
var
  nn : integer;
begin
  nn := CurrList.count;
  if (ii <= nn) and (nn > 0) then begin   {in list}
    curr := CurrList[nn-ii];
    temps := TimeList[nn-ii];
  end else begin             {index outside list}
    curr := '';
    temps := '';
  end;
end;

procedure TFuseForm.TreeOfFiles(FileNode: TTreeNode);
{construct a tree of files (of given type) in directory defined in FileNode}
var
  iit       : integer;     {index of item}
  iRes      : integer;     {flag for file found/not}
  ll        : longint;
  NodePath  : string;      {path of the directory of this node}
  SearchRec : TSearchRec;  {needed, when searching files and directories}
  WholePath : string;      {the whole path (from C:\ etc), including the file name}
begin
  ll := longint(FileNode.data)-1;       {index to path, saved as a pointer!}
  NodePath := AllPaths.strings[ll];
  FilePaths.clear;
  {search for files in this directory}
  iRes := FindFirst(NodePath+'*.csv', faAnyFile, SearchRec); {find first}
  {find next, until end}
  iit := 0;
  while iRes = 0 do begin                  {iRes <> 0 = no files, or error}
    {search for files of given type}
    WholePath := NodePath+SearchRec.name;
    {create node in the tree for this file}
    FileTree.Items.AddChild(FileTree.items[0],SearchRec.name);
    inc(iit);                                 {count items}
    FileTree.Items[iit].ImageIndex := 1;     {image of this new item}
    FileTree.Items[iit].SelectedIndex := 1;  {same image, when selected}
    FilePaths.add(WholePath);                 {save path to this file}
    {find next }
    iRes := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;

procedure TFuseForm.NewTreeOfFiles(FileNode: TTreeNode);
{clear old branches and add new ones, for the files
 FileNode  the node in the main tree (of directories) for which files are searched}
var
  ii,jj : integer;  {index}
  nn    : integer;  {number of items}
begin
  {clear the tree}
  nn := FileTree.items.count;
  if (nn > 1) then begin
    for ii := 1 to nn - 1 do begin  {not the first one}
      jj := nn-ii;                  {start from the end}
      FileTree.items[jj].delete;
    end;
  end;
  {construct the tree = list of files}
  FileTree.items[0].text := FileNode.text{+'  '+Inttostr(longint(FileNode.data))};
  TreeOfFiles(FileNode);                        {fill tree with file names}
  FileTree.FullExpand;
  ClearCurveGrid;       {do not show curve names yet}
  CurveInfoMemo.clear;  {clear the info, because file not yet selected}
  FileMarker.visible := false;
end;

procedure TFuseForm.ShowFilePath(FileNode: TTreeNode);
{show the path to the file, when node selected}
var
  CanSelect: boolean;
  filename : string;      {name of the file, with path}
  ii       : integer;     {index}
  ind      : integer;     {index}
  InFile   : TextFile;    {file of curves (data)}
  len      : integer;     {length of line}
  line     : string;      {one line from the file}
  linecnt  : integer;
  prevline : string;      {previous line}
  spr      : char;        {separator}
  txt      : string;      {temporary}
begin
  ClearCurveGrid;         {crear initially, mainly for the case of some errors}
  spr := GetSeparator;                          {separator between data, ; or ,}
  {use image index to distinguish between top node = directory, and the files}
  if (FileNode.ImageIndex > 0) then begin       {is a file, not a directory}
    ind := FileNode.index;                      {index of node in this branch}
    if (FilePaths.count > ind) then begin       {paths saved in a list}
      filename := FilePaths.strings[ind];       {name of the file saved in a list}
    end else begin
      filename := '';
    end;
    isSemicolon := false;             {initialize, to some value}
    CurrTimeNo := 0;                  {initialize}
    {open the file, and read ...}
    if (filename <> '') then begin
      CurveInfo.clear;                                                     {????}
      AssignFile(InFile,filename);    {open the file}
      Reset(InFile);
      line := '';
      prevline := '';
      linecnt := 0;
      while (not(EOF(Infile)) and (linecnt < 12)) do begin
        inc(linecnt);                             {should be enough to check some first lines}
        readln(Infile,line);
        len := length(line);
        {search for line with texts: current time current time...}
        if (len >= 12) then begin                   {12 = length of 'current;time'}
          {is the line 'Current;time'...}
          if (line[1] = 'C') or (line[1] = 'c') then begin
            txt:= copy(line,1,8);                   {copy to compare with 'current'}
//          if (txt = 'Current;') or (txt = 'current;') then begin
            if (txt = 'Current'+spr) or (txt = 'current'+spr) then begin
              isSemicolon := false;                 {line does not start with ;}
              CurrTimeNo := linecnt;                {line of Current;Time}
              break;
            end;
          {is the line ';Current;time' etc}
          end else if (line[1] = spr{';'}) and ((line[2] = 'C') or (line[2] = 'c')) then begin
            txt := copy(line,2,8);
//          if (txt = 'Current;') or (txt = 'current;') then begin
            if (txt = 'Current'+spr) or (txt = 'current'+spr) then begin
              isSemicolon := true;                 {line starts with ;}
              CurrTimeNo := linecnt;               {line of Current;Time}
              break;
            end else begin                         {might be an info line}
              CurveInfo.add(line);
            end;
          end else begin                           {might be an info line}
            CurveInfo.add(line);
          end;
        end else begin                             {might be an info line}
          CurveInfo.add(line);
        end;
        prevline := line;     {save previous line, may contain curve names}
      end;
      CloseFile(Infile);
      {analyze the prevline. It should contain the names (captions) of the curves}
      ExtractCurveNames(prevline);
      {show the information from the file}
      ShowCurveInfo;
    end else begin            {wrong index}
      ClearCurveGrid;         {clear the grid of curve names}
    end;
  end else begin              {not a file}
    ClearCurveGrid;           {clear the grid of curve names}
    filename := '';
  end;
  CurveFileName := filename;  {save for reading the selected curve later}
//{test...test...}
//if (filename <> '') then begin
//{if changing file, should change the curve}
//CanSelect := true;
//CurveGridSelectCell(nil,0, CurveRow, CanSelect);
//end;
end;

procedure TFuseForm.ExtractCurveNames(line: string);
{extract the curve names from a line, and save them to CurveList
 The line must be like: name1;;name2;;name3
 The line may start with a semicolon: ;name1;name2;;...}
var
  ii  : integer;   {index}
  len : integer;   {length of line}
  spr : char;      {separator}
  txt : string;    {temporary}
begin
  spr := GetSeparator;                          {separator between data, ; or ,}
  CurveNames.clear;   {clear list initially}
  txt := '';
  len := length(line);
  if (len > 0) then begin
    for ii := 1 to len do begin
      if (line[ii] <> spr{';'}) then begin    {semicolon(s) separate names}
        txt := txt+line[ii];
      end else begin
        if (length(txt) > 0) then begin  {there is some name}
          CurveNames.add(txt);           {save name}
          txt := '';
        end;
      end;
    end;
  end;
  {write names to grid}
  if (CurveNames.count > 0) then begin
    CurveGrid.RowCount := CurveNames.count;
    for ii := 0 to CurveNames.count-1 do begin
      CurveGrid.cells[0,ii] := CurveNames.strings[ii];   {???}
    end;

  end else begin

  end;
end;

procedure TFuseForm.ClearCurveGrid;
var
  ii : integer;
begin
  {clear the grid of curve names}
  if (CurveGrid.rowcount > 0) then begin
    for ii := 0 to CurveGrid.rowcount-1 do CurveGrid.cells[0,ii] := '';
  end;
end;

procedure TFuseForm.ReadCurveData(iCurve: integer);
{read the curve data from the selected file to CurrList and TimeList
 iCurve index of the curve}
var
  curr     : string;      {current}
  ii       : integer;     {index}
  InFile   : TextFile;    {file of curves (data)}
  isc      : integer;     {flag for semicolon}
  len      : integer;     {length of line}
  line     : string;      {one line of text}
  nc,nt    : integer;     {counts characters}
  nsc      : integer;     {counts semicolons}
  spr      : char;        {separator}
  temps    : string;      {time}
begin
  spr := GetSeparator;                   {separator between data, ; or ,}
  if (CurveFileName <> '') then begin
    CurrList.clear;
    TimeList.clear;
    AssignFile(InFile,CurveFileName);    {open the file}
    Reset(InFile);
    line := '';
    if (isSemicolon) then isc := 1 else isc := 0;  {a semicolon or not in the beginning of the line}
    {first, read the lines with text, no current-time data yet}
    for ii := 1 to CurrTimeNo do readln(Infile,line);
    {next, read the actual data}
    while not(EOF(Infile)) do begin
      readln(Infile,line);
      {find the current and time, several pairs on the line, data separated by a ';' or a ','}
      {there may be one (or more?) ';'s or ','s in the beginning of the line}
      len := length(line);
      if (len > 2) then begin
        curr := '';
        temps := '';
        nsc := 0;                          {initialize no of semicolons}
        nc := 0;
        nt := 0;
        for ii := 1 to len do begin
          if (line[ii] = spr{';'}) then begin   {separator between data items}
            inc(nsc);                      {count semicolons}
            if (nsc >= 2*iCurve+isc) then break;
          end else begin                   {one char of data}
            if (nsc = 2*(iCurve-1)+isc) then begin     {current of selected curve}
              if (line[ii] <> ' ') then inc(nc);       {count characters}
              if (nc <= 6) then curr := curr+line[ii]; {not too many characters}
            end else if (nsc = 2*(iCurve-1)+1+isc) then begin {time of selected curve}
              if (line[ii] <>' ') then inc(nt);        {count characters}
              if (nt <= 6) or ((nt <= 7) and (temps[1] = '0')) then
                temps := temps+line[ii];
            end;
          end;
        end;
        if (curr = '' ) or (temps = '' ) or
           (curr = ' ') or (temps = ' ') then begin
          break;   {no more data}
        end else begin
          CurrList.add(curr);
          TimeList.add(temps);
        end;
      end;
    end;
    CloseFile(InFile);
  end;
end;

procedure TFuseForm.DrawGraph;
{draw a log-log graph of CurrList and TimeList}
const
  iclr : integer = 1;
var
  ii      : integer;
  ix,iy   : integer;  {coordinates}
begin
  if (CurrList.count > 1) then begin {there is data}
    Graph.canvas.pen.color := CurveClr[iclr];
    inc(iclr);
    if (iclr > high(CurveClr)) then iclr := 1;
    {initialize coordinates}
    ix := 0;
    iy := Graph.height;
    {first point}
    Coordinates(CurrList[0],TimeList[0],ix,iy);
    Graph.Canvas.MoveTo(ix,iy);
    {draw the curve}
    for ii := 1 to CurrList.count-1 do begin
      Coordinates(CurrList[ii],TimeList[ii],ix,iy);
      Graph.Canvas.LineTo(ix,iy);
    end;
  end;
end;

procedure TFuseForm.Coordinates(sCurr,sTime: string; var ix,iy: integer);
{convert current and time as strings to coordinates in the graph
 sCurr  current as string
 sTime  time as string
 ix,iy  current and time coordinates. Must have old values when entering (for error cases)}
const
 ln10  = 2.302585;
 LgMinCur = 0;      {log of min.current}
 LgMinTim = -2;     {log of min time}
 nCdec = 5;         {number of current decades}
 nTdec = 5;         {number of time decades}
var
  curr    : single;   {current}
  er1,er2 : integer;  {eror flag}
  LgCur   : single;   {log of time}
  LgTim   : single;   {log of current}
  temps   : single;   {time}
  txx,txy : string;   {temporary}
  Wcurr   : single;   {width of one current decade}
  Wtime   : single;   {width of one time decade}
begin
  Wcurr := graph.width/nCdec;        {width of one current decade}
  Wtime := graph.height/nTdec;       {width of one time decade}
  {starting point = first data}
  chkfloat(sCurr,er1,txx);
  chkfloat(sTime,er2,txy);
  if (er1 = 0) and (er2 = 0) then begin
    curr := strtofloat(txx);             {current}
    temps := strtofloat(txy);            {time}
    if (curr > 0) and (temps > 0) then begin
      {find the coordinate corresponding to the value of the current}
      LgCur := ln(curr)/ln10;            {log of current}
      if (LgCur <= LgMinCur) then begin  {too small a current}
        ix := 0;
      end else if (LgCur <= LgMinCur+nCdec) then begin  {in range}
        ix := round(LgCur*Wcurr);        {x-coordinate = current}
      end else begin                     {too large a current}
        ix := graph.width;
      end;
      {find the coordinate corresponding to the value of time}
      LgTim := ln(temps)/ln10;           {log of time}
      if (LgTim <= LgMinTim) then begin  {too small a time}
        iy := Graph.height;
      end else if (LgTim <= LgMinTim+nTdec) then begin  {in range}
        iy := Graph.height-round((LgTim-LgMinTim)*WTime); {y-coordinate = time}
      end else begin                     {too large a time}
        iy := 0;
      end;
    end else begin                       {negative time or current}
      {keep old values}
    end;
  end else begin                         {error in converting string to value}
    {keep old values}
  end;
end;

procedure GetLowestDirectory(DirPath: string; var DirName: string);
{extract the lowest directory name from a path. For example
 if DirPath = C:\sofware\programs\testprog\ then DirName = testprog}
var
  ii,jj : integer;  {indices}
  istr  : integer;  {start}
  len   : integer;  {length of line}
  txt   : string;   {temporary}
begin
  len := length(DirPath);
  if (len > 2) then begin
    {last char. may be \, it is not counted}
    if (DirPath[len] = '\') then istr := 1 else istr := 0; {start-index}
    txt := '';
    for ii := istr to len do begin
      jj := len-ii;                       {from the end}
      if (DirPath[jj] = '\') then break   {separator between dir. found}
      else txt := txt+DirPath[jj];
    end;
    {directory name is in txt, but backwards, turn it}
    len := length(txt);
    if (len > 0) then begin
      DirName := '';
      for ii := 0 to len-1 do begin
        jj := len-ii;
        DirName := DirName+txt[jj];
      end;
    end else begin
      DirName := '';
    end;
  end else begin
    DirName := '';
  end;
end;

procedure TFuseForm.FormCreate(Sender: TObject);
var
  DirFound  : boolean;     {true, when a directory found}
  DirList   : TStringList; {list of directories (with pathes) found}
  DirName   : string;      {name of the directory where this program is}
  iAll      : integer;     {index for strings in AllPath}
  iDir,jDir : integer;     {index}
  ii,jj     : integer;
  iLevel    : integer;     {search level = level of subdirectories}
  jLevel    : integer;     {level of saved dir (or path)}
  iNod      : integer;     {index of node}
  iRes      : integer;     {flag for file or dir found/not}
  ll        : longint;     {temporary}
  nDir      : integer;     {number of directories (pathes)}
  PathList  : TStringList; {list of pathes}
  PathName  : string;      {name of the path under search}
  pNode     : TpTreeNode;  {pointer to a node in the tree}
  SearchRec : TSearchRec;  {needed, when searching files and directories}
  ThisPath  : string;      {path to the dir, where this file is (like C:\temp)}
  txt       : string;      {temporary}
  WholePath : string;      {the whole path (from C:\ etc), including the dir.name}
begin
  {fill the image list for the tree view-component}
  DirImageList.add(ClosedDirImage.picture.bitmap,nil);
  DirImageList.add(OpenDirImage.picture.bitmap,nil);
  FileImageList.add(OpenDirImg16.picture.bitmap,nil);
  FileImageList.add(ExcelImage.picture.bitmap,nil);
  {(try to..) generate the tree of directories - - - - - - - - - - - }
  AllPaths := TStringList.create;                  {paths to all directories}
  FilePaths := TStringList.create;                 {paths to files}
  CurveNames := TStringList.create;                {names of the curves}
  PathList := TStringList.create;                  {search directories in this path}
  DirList := TStringList.create;                   {save here the directories found}
  CurrList := TStringList.create;                  {values of current}
  TimeList := TStringList.create;                  {values of time}
  CurveInfo := TStringList.create;                 {info from the files}
  ThisPath := ExtractFilePath(paramStr(0));        {path to this *.exe file}
  GetLowestDirectory(ThisPath, DirName);
  DirectoryTree.items[0].text := DirName;          {directory at the top of the tree}
  PathList.add(ThisPath);                          {initialize list of pathes}
  AllPaths.add(ThisPath);                          {first item to list of all pathes}
  iAll := 1;                                       {index: first string in AllPaths}
  DirectoryTree.items[0].data := pInteger(iAll);   {change integer to a pointer, to fool the compiler}
  for iLevel := 0 to 10 do begin                   {search level by level (should be enough?)}
    DirFound := false;                             {initialize, no Dir found yet}
    {search all pathes (directories) in list}
    nDir := PathList.count;                        {number of directories in list}
    if (nDir > 0) then begin
      for iDir := 0 to nDir-1 do begin
        PathName := PathList[iDir];
        {find index in treeview by comparing path with pathes stored in AllPaths}
        iNod := -1;                                   {initialize}
        for jj := 0 to DirectoryTree.items.count-1 do begin
          ll := longint(DirectoryTree.items[jj].data)-1;  {the index is stored as a pointer!}
          if (PathName = AllPaths.strings[ll]) then begin
            iNod := jj;
            break;
          end;
        end;
        jDir := -1;
        {scan this directory, search for the first directory}
        iRes := FindFirst(PathName+'*.', faAnyFile, SearchRec); {find first}
        {find next, until end}
        while iRes = 0 do begin                  {iRes <> 0 = no files, or error}
          {search for directories, but skip '.' and '..' (DOS?)}
          if (SearchRec.name <> '.') and (SearchRec.name <> '..') then begin
            {is it a directory ?}
            if ((SearchRec.attr and faDirectory) = faDirectory) then begin
              WholePath := PathName+SearchRec.name+'\';
              {add to the list of found directories}
              DirList.add(WholePath);
              DirFound := true;                  {set flag: directory found}
              inc(jDir);
              {create node in the tree for this directory}
              if (iNod >= 0) then begin
                DirectoryTree.Items.AddChild(DirectoryTree.items[iNod],SearchRec.name);
                AllPaths.add(WholePath);
                inc(iAll);                         {index of this path as string, in AllPaths}
                DirectoryTree.Items[iNod+jDir+1].data := pInteger(iAll);
                DirectoryTree.Items[iNod+jDir+1].SelectedIndex := 1; {picture, when selected}
              end;
            end;
          end;
          {find next }
          iRes := FindNext(SearchRec);
        end;
        FindClose(SearchRec);
      end;
    end;
    {when ready with this level, copy list of directories to search list}
    PathList.Clear;
    if (DirList.count > 0) then begin
      for ii := 0 to DirList.count-1 do begin
        PathList.add(DirList[ii]);
      end;
    end;
    DirList.clear;
    if (not DirFound) then break;        {stop, when no more directories found}
  end;
  PathList.free;
  DirList.free;
  CurveRow := 0;
//Graph.canvas.rectangle(0,0,Graph.width,Graph.height);
  {initialize the memo for additional info}
  CurveInfoMemo.top := 50;
  CurveInfoMemo.left := 0;
  CurveInfoMemo.width := width-8;
  CurveInfoMemo.height := 100;
  {set data separator, read from *.ini-file in imagopas}
  if (data_separ = ';') then begin          {in imagopas}
    SemicolItem.checked := true;
    SeparMenu.caption := '&Separator ( ; )';
  end else begin
    CommaItem.checked := true;
    SeparMenu.caption := '&Separator ( , )';
  end;
  {add flag that form created}
  Imagoo.ReadyEdit.text := Imagoo.ReadyEdit.text+'f';
end;



procedure TFuseForm.FormDestroy(Sender: TObject);
begin
  AllPaths.free;
  FilePaths.free;
  CurveNames.free;
  CurrList.free;
  TimeList.free;
  CurveInfo.free;
end;

procedure TFuseForm.DirectoryTreeCollapsed(Sender: TObject;
  Node: TTreeNode);
begin
  DirMarker.visible := false;
end;

procedure TFuseForm.DirectoryTreeExpanded(Sender: TObject;
  Node: TTreeNode);
begin
  DirMarker.visible := false;
end;

procedure TFuseForm.DirectoryTreeKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{a node may be selected also with the up- and down- arrow keys}
var
  ii       : integer;
  NodeRect : TRect;     {rectangle of the node}
  nn       : integer;
begin
  if (key = 38) or (key = 40) then begin
    CloseBut.enabled := false;
    {check is some node selected = focused}
    nn := DirectoryTree.items.count;   {number of nodes}
    for ii := 0 to nn-1 do begin   {scan all nodes}
      if (DirectoryTree.items[ii].selected) then begin
//      Edit1.text := DirectoryTree.items[ii].text;
        NewTreeOfFiles(DirectoryTree.items[ii]);               {update the tree of files}
        NodeRect := DirectoryTree.items[ii].DisplayRect(true); {get rectangle around the node text}
        {mark this node as selected}
        DirMarker.top := DirectoryTree.top+NodeRect.top+2;
        DirMarker.left := DirectoryTree.left+NodeRect.right+4;
        DirMarker.visible := true;
        DirMarker.BringToFront;
CharForm.ClearCharGrid;
        break;
      end;
    end;
  end;

end;

procedure TFuseForm.DirectoryTreeMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{update the tree for files = display files under the clicked directory}
var
  FileNode : TTreeNode;
  NodeRect : TRect;     {rectangle of the node}
begin
  CloseBut.enabled := false;
  FileNode := DirectoryTree.GetNodeAt(X,Y);             {get node at x,y}
  if (FileNode <> nil) then begin                   {node found}
    NodeRect := FileNode.DisplayRect(true);         {get rectangle around the node text}
    {clicking at the text of the node? or the image}
    if (NodeRect.left-DirImageList.width-4 < X) and (X < NodeRect.right) then begin
//    Edit1.text := DirectoryTree.GetNodeAt(X,Y).Text;
      {clear old files, add files of this dir.}
      NewTreeOfFiles(FileNode);
      {mark this node as selected}
      DirMarker.top := DirectoryTree.top+NodeRect.top+2;
      DirMarker.left := DirectoryTree.left+NodeRect.right+4;
      DirMarker.visible := true;
      DirMarker.BringToFront;
CharForm.ClearCharGrid;
    end else begin
//    Edit1.text := '';
    end;
  end;

end;

procedure TFuseForm.FileTreeCollapsed(Sender: TObject; Node: TTreeNode);
begin
  FileMarker.visible := false;
  ClearCurveGrid;
end;

procedure TFuseForm.FileTreeExpanded(Sender: TObject; Node: TTreeNode);
begin
  FileMarker.visible := false;
  ClearCurveGrid;
end;

procedure TFuseForm.FileTreeKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{a node may be selected also with the up- and down- arrow keys}
var
  ii       : integer;
  NodeRect : TRect;     {rectangle of the node}
  nn       : integer;
begin
  if (key = 38) or (key = 40) then begin
    CloseBut.enabled := false;               {must select a curve before closing, or cancel}
    {check is some node selected = focused}
    nn := FileTree.items.count;   {number of nodes}
    for ii := 0 to nn-1 do begin   {scan all nodes}
      if (FileTree.items[ii].selected) then begin
        ShowFilePath(FileTree.items[ii]);         {show the path to the file}
        {mark this node as selected}
        NodeRect := FileTree.items[ii].DisplayRect(true); {get rectangle around the node text}
        FileMarker.top := FileTree.top+NodeRect.top+2;
        FileMarker.left := FileTree.left+NodeRect.right+4;
        FileMarker.visible := true;
        FileMarker.BringToFront;
        CharForm.ClearCharGrid;
        break;
      end;
    end;
  end;


end;

procedure TFuseForm.FileTreeMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{show some parameters of the selected file}
var
  FileNode : TTreeNode;
  ind      : integer;   {index}
  NodeRect : TRect;     {rectangle of the node}
begin
  CloseBut.enabled := false;
  FileNode := FileTree.GetNodeAt(X,Y);              {get node at x,y}
  if (FileNode <> nil) then begin                   {node found}
    NodeRect := FileNode.DisplayRect(true);         {get rectangle around the node text}
    {clicking at the text of the node? or the image}
    if (NodeRect.left-FileImageList.width-4 < X) and (X < NodeRect.right) then begin
      ShowFilePath(FileNode);                       {show path to file}
      {mark this node as selected}
      FileMarker.top := FileTree.top+NodeRect.top+2;
      FileMarker.left := FileTree.left+NodeRect.right+4;
      FileMarker.visible := true;
      FileMarker.BringToFront;
      CharForm.ClearCharGrid;
    end else begin
//    Edit1.text := '';
    end;
  end;
end;

procedure TFuseForm.CurveGridSelectCell(Sender: TObject; Col, Row: Longint;
  var CanSelect: Boolean);
{select and draw curve}
var
  ii   : integer;
  len  : integer;
  line : string;
begin
  CloseBut.enabled := true;
  if (row < CurveGrid.rowcount) then begin
    if (CurveGrid.cells[col,row] <> '') then begin   {some curve defined}
      CurveRow := row;     {save row}
      OKButClick(sender);
      {save the info of the location of the curve to the relay}
      len := length(CurveFileName);
      line := '';
      if (len > 0) then begin
        for ii := 1 to len do begin
          {add spaces so that long lines can be automatically split in the memo}
          if (CurveFileName[ii] = '\') then line := line+'\ '
          else line := line+CurveFileName[ii];
        end;
      end;
      line := line+'\ '+CurveGrid.cells[0,row];
      CharForm.ReplaceInfoLine(line);
      {if (RecloserForm.InfoMemo.visible) then} RecloserForm.InfoMemo.text := line;
      CharForm.SetCurveIndex(5000);    {index, and in grid (below)}
      RecloserForm.ValueGrid.cells[Recloser.GetActCol,Recloser.GetCurveRow] :=
                    inttostr(5000);
    end;
    end else begin
  end;
end;

procedure TFuseForm.OKButClick(Sender: TObject);
{get the data of the selected curve}
begin
  ReadCurveData(CurveRow+1);   {read data from file to CurrList, TimeList}
  {copy the data from the lists to the grid of the user defined relay}
  CharForm.CurveFromMoreFuses(CurveRow);
end;

procedure TFuseForm.ShowButClick(Sender: TObject);
begin
//DrawGraph;
end;

procedure TFuseForm.ClearButClick(Sender: TObject);
begin
  Graph.picture := nil;
  Graph.canvas.rectangle(0,0,Graph.width,Graph.height);
end;

procedure TFuseForm.FormHide(Sender: TObject);
  {Collapse and hide, in order to avoid confusion next time the form is displayed.
   The curve of this window is not necessarily the curve shown in the coordination graph,
   when the form becomes visible again, when a different relay is selected}
var
  ii : integer;
begin
  FileTree.FullCollapse;
  FileMarker.visible := false;
  for ii := 0 to CurveGrid.rowcount-1 do CurveGrid.cells[0,ii] := '';
  CurveInfoMemo.visible := false;
  InfoItem.checked := false;
  CurveInfoMemo.clear;
end;

procedure TFuseForm.CloseButtonClick(Sender: TObject);
begin
  FuseForm.visible := false;
end;

procedure TFuseForm.CancelButClick(Sender: TObject);
begin
  CharForm.CancelButtonClick(nil);
  {return the original values to the grid for the user def.curve}
  ParamToCharGrid(Userdef.GetpActComp, Recloser.GetActCol);
end;

procedure TFuseForm.exit1Click(Sender: TObject);
begin
  FuseForm.visible := false;
end;

procedure TFuseForm.InfoItemClick(Sender: TObject);
begin
  InfoItem.checked := not InfoItem.Checked;
  CurveInfoMemo.visible := InfoItem.Checked;
  if (InfoItem.Checked) then CurveInfoMemo.BringToFront;
end;

procedure TFuseForm.CurveInfoMemoMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
{move the memo}
const
  x0  : integer = 0;    {old coordinates}
  y0  : integer = 0;
begin
  if (ssLeft in Shift) then begin        {left button pressed}
    CurveInfoMemo.left := CurveInfoMemo.left+X-x0;
    CurveInfoMemo.top  := CurveInfoMemo.top +Y-y0;
  end else begin
    x0 := X;
    y0 := Y;
  end;
end;

procedure TFuseForm.FormShow(Sender: TObject);
begin
  if (CurveInfoMemo.top < 0) then CurveInfoMemo.top := 0
  else if (CurveInfoMemo.top > FuseForm.height-80) then
    CurveInfoMemo.top := FuseForm.height-80;
  CloseBut.enabled := false;  
end;

procedure TFuseForm.SemicolItemClick(Sender: TObject);
begin
  SemicolItem.checked := true;
  SeparMenu.caption := '&Separator ( ; )';
end;

procedure TFuseForm.CommaItemClick(Sender: TObject);
begin
  CommaItem.checked := true;
  SeparMenu.caption := '&Separator ( , )';
end;

procedure TFuseForm.CloseButClick(Sender: TObject);
begin
  FuseForm.visible := false;
end;

procedure TFuseForm.HelpMemoMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
{move the memo}
const
  x0  : integer = 0;    {old coordinates}
  y0  : integer = 0;
begin
  if (ssLeft in Shift) then begin        {left button pressed}
    HelpMemo.left := HelpMemo.left+X-x0;
    HelpMemo.top  := HelpMemo.top +Y-y0;
  end else begin
    x0 := X;
    y0 := Y;
  end;
end;


procedure TFuseForm.HelpMemoMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (x < 40) and (y < 22) then HelpMemo.visible := false;
end;

procedure TFuseForm.HelpMenuClick(Sender: TObject);
begin
  HelpMemo.top := DirectoryTree.top;
  HelpMemo.left := DirectoryTree.left;
  HelpMemo.width := 250;
  HelpMemo.height := 280;
  HelpMemo.visible := true;
  HelpMemo.BringToFront;
end;

end.
