unit phasor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, ExtCtrls, Compdef, Skemadef, Imagodef, Cmatmul, Utildef, StdCtrls,
  Menus ;

procedure ShowCurrPhasors(ipComp : longint); far;  {show currents using phasors}
procedure RemoveCTphasor (ipComp : longint); far;  {remove the curr.trafoand phasor}
procedure RemoveAllPhasors;
procedure UpdatePhasors;
procedure DrawPhasors;

type
  TCurrPhasorForm = class(TForm)
    CurrPhasorImg: TImage;
    CurrPhasorGrid: TStringGrid;
    OnOffPanel: TPanel;
    OnOffBox: TCheckBox;
    DeleteBut: TButton;
    ColorPanel: TPanel;
    ColorImage: TImage;
    IDimage: TImage;
    PointEdit: TEdit;
    IncBut: TButton;
    DecBut: TButton;
    ResetBut: TButton;
    TopMenu: TMainMenu;
    FileItem: TMenuItem;
    ExitItem: TMenuItem;
    ViewItem: TMenuItem;
    ButItem: TMenuItem;
    PhasorDialog: TSaveDialog;
    BitmapItem: TMenuItem;
    CurrentLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure CurrPhasorGridSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure OnOffBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OnOffBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DeleteButClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure ColorImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CurrPhasorGridDrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure CurrPhasorGridExit(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure CurrPhasorImgMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CurrPhasorImgMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CurrPhasorImgMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure IncButClick(Sender: TObject);
    procedure DecButClick(Sender: TObject);
    procedure ResetButClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure ButItemClick(Sender: TObject);
    procedure BitmapItemClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  CurrPhasorForm: TCurrPhasorForm;

implementation

uses
  Imagopas, PQRelay;
{$R *.DFM}


const
  {indices of columns}
  ColorCol  = 0;   {column for colors}
  OnOffCol  = 1;   {column for on/off (display/not) switch}
  IdentCol  = 2;   {column for phasor identifier}
  TrafoCol  = 3;   {column for the data of the current trafo (current, phase)}
  {widths of columns}
  ColorWidth = 16;
  OnOffWidth = 24;
  IdentWidth = 32;
  TrafoWidth = 128;
  {height of the label rows}
  LabelHeight = 24;
  OldWidth : integer = 1;     {old width of form}
  OldHeight : integer = 1;    {old height of form}
  CgridMarg : integer = 1;    {right margin of current grid}
  CimagMarg : integer = 1;    {margin for the image}
  nCurrTrafo : integer = 0;   {no of current trafos in grid}
  maxCT = 5;                  {max. no of current trafos}
  CTrows = 4;                 {no of rows for one current trafo}
  pCgrid  : TpGrid  = nil;    {pointer to the current grid}
  pCimage : TpImage = nil;    {pointer to the phasor image}
  actrow  : integer = 0;      {selected row}
  actcol  : integer = 0;      {selected column}
  downx0  : integer = 0;      {x of mouse down}
  downy0  : integer = 0;      {y of mouse down}
  downkk  : integer = 0;      {row index of identifier at which mouse down}
  modScale : integer = 0;     {modification index for scale}
var
  CRwidth  : integer;         {width (and height) of a small color rectangle}
  CRspace  : integer;         {width of space between color rectangles}
  diamax   : integer;         {diameter of the lagest circle}
  maxScale : single;          {current corresponding the largest circle}
  CTpoint  : array[1..maxCT] of longint;   {pointers to current trafos, as integers}
  CTident  : array[1..maxCT] of integer;   {keeps track of the identifiers for the phasors}
  IDxx     : array[0..maxCT*CTrows-1] of integer; {unmodified x-coord. of the identifier}
  IDyy     : array[0..maxCT*CTrows-1] of integer; {unmodified y-coord. of the identifier}
  Relxx    : array[0..maxCT*CTrows-1] of integer; {relative x-coord. of the identifier}
  Relyy    : array[0..maxCT*CTrows-1] of integer; {relative y-coord. of the identifier}

procedure SetGridHeight;
{set the heigth of the grid according to the no of lines}
var
  ii,ipu : integer;
begin
{the total height}
  ipu := 0;
  for ii := 0 to pCgrid^.rowCount-1 do begin
    ipu := ipu+pCgrid^.RowHeights[ii];
  end;
  ipu := ipu+(pCgrid^.RowCount+2)*pCgrid^.GridLineWidth;
  pCgrid^.height := ipu;
end;

procedure TCurrPhasorForm.FormCreate(Sender: TObject);
{set the size etc of the grind and image}
var
  ii      : integer;   {index}
  ipu,ipv : integer;   {temporary}
begin
  pCgrid := @CurrPhasorGrid;
  pCimage := @CurrPhasorImg;
  OldWidth := width;             {old width of form}
  OldHeight := height;           {old height of form}
  diamax   := 100;               {initialize diameter of the lagest circle}
  maxScale := 100;               {current corresponding the largest circle}
  CgridMarg := width-pCgrid^.left-pCgrid^.width;
  CimagMarg := pCimage^.left;     {margin around the image}
  {set the dimensions of the grid for current trafos}
  CurrPhasorGrid.ColWidths[ColorCol] := ColorWidth;
  CurrPhasorGrid.ColWidths[OnOffCol] := OnOffWidth;
  CurrPhasorGrid.ColWidths[IdentCol] := IdentWidth;
  CurrPhasorGrid.ColWidths[TrafoCol] := TrafoWidth;
  {the total width of the grid}
  ipu := 0;
  for ii := 0 to CurrPhasorGrid.ColCount-1 do begin
    ipu := ipu+CurrPhasorGrid.ColWidths[ii];
  end;
  CurrPhasorGrid.width := ipu+(CurrPhasorGrid.ColCount+2)*CurrPhasorGrid.GridLineWidth;
  SetGridHeight;         {the total height of the grid}
  left := Imagoo.width-width-8;
  {a rectangle around the phasor image}
  CurrPhasorImg.canvas.pen.color := clBlack;
  CurrPhasorImg.canvas.pen.width := 1;
  CurrPhasorImg.canvas.brush.style := bsClear;
  CurrPhasorImg.canvas.rectangle(0,0,CurrPhasorImg.width,CurrPhasorImg.height);
  {set the (original) size of the form}
  CurrPhasorForm.width := CurrPhasorGrid.left+CurrPhasorGrid.width+24;
  ipu := CurrPhasorGrid.top+CurrPhasorGrid.height+32;
  ipv := CurrPhasorImg.top+CurrPhasorImg.height+PointEdit.height+32+16;
  if (ipu > ipv) then  CurrPhasorForm.height := ipu
  else  CurrPhasorForm.height := ipv;
  {clear the pointer array}
  for ii := 1 to maxCT do CTpoint[ii] := 0;
  for ii := 1 to maxCT do CTident[ii] := ii;
  {initialize the color selection image}
  PQForm.ColorInitialize(@ColorImage);
  PQForm.GetCRsize(CRwidth,CRspace);
  ColorPanel.width := ColorImage.width+2*CRspace;
  ColorPanel.height := ColorPanel.width;
  ipu := pCgrid^.RowHeights[0]+pCgrid^.RowHeights[1];
  ColorPanel.top := pCgrid^.top+ipu+2*pCgrid^.GridLineWidth;
  ColorPanel.left := pCgrid^.left+pCgrid^.ColWidths[0]+2*pCgrid^.GridLineWidth;
  {initialize = clear the position-arrays of the identifier}
  ipu := high(IDxx);     {dimension, same for all}
  for ii := 0 to ipu do begin
    IDxx[ii] := 0;
    IDyy[ii] := 0;
    Relxx[ii] := 0;
    Relxx[ii] := 0;
  end;
end;

function isThere(ipComp : longint; var row : integer) : boolean;
{check that current trafo is in the grid
ipComp  pointer to the trafo, as integer
row     the row where the identifier is}
var
  ii : integer;  {index}
begin
  if (nCurrTrafo = 0) then begin     {no trafos yet}
    isThere := false;
    row := 0;
  end else begin
    isThere := false;                       {initialize}
    row := 0;
    for ii := 1 to nCurrTrafo do begin      {check all}
      if (ipComp = CTpoint[ii]) then begin  {identical pointers (as integers)}
        isThere := true;
        row := (ii-1)*CTrows;               {the row for the identifier}
        break;
      end;
    end;
  end;
end;

procedure GetCurrents(ipComp : longint; var CurrA,AngleA,CurrB,AngleB,CurrC,AngleC : single);
{get the currents of a current trafo in polar form
 ipComp  pointer to the trafo as integer
 CurrA, AngleA   magnitude and current of phase a (phase in degrees)
 CurrB, AngleB   magnitude and current of phase b (phase in degrees)
 CurrC, AngleC   magnitude and current of phase c (phase in degrees)}
var
  Ia,Ib,Ic : complex;  {the currents of phases a,b,c}
  nvar  : integer;  {no of output variables}
  pComp : TpComp;   {pointer to the trafo}
begin
  if (ipComp <> 0) then begin
    pComp := TpComp(ipComp);
    pComp^.GetNvar(nvar);                 {no of variables}
    pComp^.GetVar(gCurrT_Ir,Ia.x);        {real part of current, phase a}
    pComp^.GetVar(gCurrT_Ii,Ia.y);        {imag.part of current}
    pComp^.GetVar(gCurrT_Ir+nvar,Ib.x);   {real part of current, phase b}
    pComp^.GetVar(gCurrT_Ii+nvar,Ib.y);   {imag.part of current, phase b}
    pComp^.GetVar(gCurrT_Ir+2*nvar,Ic.x); {real part of current, phase c}
    pComp^.GetVar(gCurrT_Ii+2*nvar,Ic.y); {imag.part of current, phase c}
    {get magnitude and phase}
    CurrA  := Cabs(Ia);
    AngleA := Cangle(Ia)*180/pi;
    CurrB  := Cabs(Ib);
    AngleB := Cangle(Ib)*180/pi;
    CurrC  := Cabs(Ic);
    AngleC := Cangle(Ic)*180/pi;
  end else begin                          {error case}
    CurrA  := 0;
    AngleA := 0;
    CurrB  := 0;
    AngleB := 0;
    CurrC  := 0;
    AngleC := 0;
  end;
end;

procedure CurrentsToGrid(CurrA,AngleA,CurrB,AngleB,CurrC,AngleC : single;
                         iCT : integer);
{write the currents (magnitude and angle) to the grid
 CurrA, AngleA   magnitude and current of phase a (phase in degrees)
 CurrB, AngleB   magnitude and current of phase b (phase in degrees)
 CurrC, AngleC   magnitude and current of phase c (phase in degrees)
 iCt             index of the current trafo 1..CTmax}
var
  lblRow    : integer; {row of the label etc}
  txta,txtb : string;  {temporary}
begin
  {current magnitude and phase angle}
  if (iCT > 0) and (iCT <= maxCT) then begin
    lblRow := (iCT-1)*CTrows;            {row for the labes of this trafo}
    txta := Format('%8.1f',[CurrA]);
    txtb := Format('%5.1f',[AngleA]);
    pCgrid^.Cells[TrafoCol,lblRow+1] := txta+' A  < '+txtb+' ';
    txta := Format('%8.1f',[CurrB]);
    txtb := Format('%5.1f',[AngleB]);
    pCgrid^.Cells[TrafoCol,lblRow+2] := txta+' A  < '+txtb+' ';
    txta := Format('%8.1f',[CurrC]);
    txtb := Format('%5.1f',[AngleC]);
    pCgrid^.Cells[TrafoCol,lblRow+3] := txta+' A  < '+txtb+' ';
  end;
end;

procedure ShowCurrPhasors(ipComp : longint);
{show the current(s) using phasors, if allowed (button down)
 ipComp  pointer to the current trafo as integer}
const
  iclr : integer = 1;   {color index}
var
  Aa,Ab,Ac  : single;   {phase angle of currents}
  Ca,Cb,Cc  : single;   {magnitudes of currents}
  ipu       : integer;  {temporary}
  ii        : integer;  {index}
  ix,iy     : integer;  {coordinates}
  lbl       : string;   {label of the trafo}
  lblRow    : integer;  {row for the label}
  pComp     : TpComp;   {pointer to the current trafo}
  txta,txtb : string;   {temporary text}
begin
  if (Imagoo.CurrPhasorBut.down) then begin   {display of phasors allowed}
    {add curr.trafo, if there is room, and the trafo is not yet there}
    if (nCurrTrafo < maxCT) and not(isThere(ipComp,ipu)) then begin {room & not there}
      inc(nCurrTrafo);                 {one more}
      pCgrid^.rowcount := nCurrTrafo*CTrows;  {add rows for this trafo}
      pComp := TpComp(ipComp);                {pointer to the current trafo}
      pComp^.GetLabel(lbl,ix,iy);             {label of the current trafo}
      GetCurrents(ipComp,Ca,Aa,Cb,Ab,Cc,Ac);  {currents in polar form}
      {add to the grid}
      lblRow := (nCurrTrafo-1)*CTrows;          {row for the labes of this trafo}
      pCgrid^.Cells[TrafoCol,lblRow] := lbl;    {label to the grid}
      {default colors, as a first guess}
      for ii := 1 to 3 do begin
        pCgrid^.Cells[ColorCol,lblRow+ii] := inttostr(iclr);
        inc(iclr);
        if (iclr > high(gColor)) then iclr := low(gColor);
      end;  
      {flag for the display of phasor}
      for ii := 1 to 3 do pCgrid^.Cells[OnOffCol,lblRow+ii] := '1';
      {identifiers for the phasors}
      ipu := CTident[nCurrTrafo];
      pCgrid^.Cells[IdentCol,lblRow+1] := inttostr(ipu)+'A';
      pCgrid^.Cells[IdentCol,lblRow+2] := inttostr(ipu)+'B';
      pCgrid^.Cells[IdentCol,lblRow+3] := inttostr(ipu)+'C';
      {magnitude and angle to the grid}
      CurrentsToGrid(Ca,Aa,Cb,Ab,Cc,Ac,nCurrTrafo);
      { "X" for delete}
      pCgrid^.Cells[OnOffCol,lblRow] := '(X)';
      {pointer, as integer}
      CTpoint[nCurrTrafo] := ipComp;             {save pointer, as integer}
      pComp^.SetPWR(true);                       {set flag}
      DrawPhasors;
    end;
    SetGridHeight;
    CurrPhasorForm.visible := true;
  end;

end;

procedure ExtractCurrent(line : string; var Curr, angle : single);
{extract the magnitude and angle (in degrees) of current from text in line
 assumed line is written in CurrentsToGrid, and the line is like  3,456 A < 12,3 
 line   the text with the values
 Curr   the magnitude
 angle  the phase angle}
var
  ii    : integer;
  istop : integer;
  len   : integer;  {length of text}
  txt   : string;   {temporary}
begin
  len := length(line);
  {first, extract the magnitude}
  txt := '';
  istop := 1;
  for ii := 1 to len do begin
    if (line[ii] <> 'A') then begin
      txt := txt+line[ii];           {copy up to the 'A'}
    end else begin
      istop := ii;
      break;
    end;
  end;
  Curr := strtofloat(txt);        {the magnitude}
  {next, extract the angle}
  if (istop < len-1) then begin
    txt := '';
    for ii := istop+1 to len do begin
      if (line[ii] <> '<') and (line[ii] <> '') then txt := txt+line[ii];
    end;
    angle := strtofloat(txt);
  end else begin
    angle := 0;
  end;
end;

procedure TCurrPhasorForm.CurrPhasorGridSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
var
  apu,apv : single;
  ii     : integer;
  ipu    : integer;  {temporary}
  bpLeft : integer;  {left coordinate of additional panel or button}
  bpTop  : integer;  {top coord. of additional panel or button}
  line   : string;
begin
  {if on/off column, display button for selecting show current or not}
  actrow := row;                  {save so that other procedures can use}
  actcol := col;
  if (col = OnOffCol) then begin
    {coordinates of "command" components}
    ipu := 0;
    for ii := 0 to row do ipu := ipu+pCgrid^.RowHeights[ii];
    bpTop := ipu+(row+2)*pCgrid^.GridLineWidth+pCgrid^.top;
    ipu := 0;
    for ii := 0 to col do ipu := ipu+pCgrid^.ColWidths[ii];
    bpLeft := ipu+(col+2)*pCgrid^.GridLineWidth+pCgrid^.left;
    {not label row}
    if (row <> CTrows*(row div CTrows)) then begin
      OnOffPanel.top := {pCgrid^.top+ipu}bpTop;        {put panel below the selected cell..}
      OnOffPanel.left := {pCgrid^.left+pCgrid^.ColWidths[0]}bpLeft; {..on the OnOff-column}
      OnOffBox.checked := (pCgrid^.Cells[col,row] <> '0');  {copy "show" status"}
      OnOffPanel.visible := true;
      OnOffPanel.BringToFront;
      DeleteBut.visible := false;
    end else begin                       {label row, show delete button}
      DeleteBut.top := bpTop;
      DeleteBut.left := bpLeft;
      DeleteBut.visible := true;
      OnOffPanel.visible := false;
    end;
    ColorPanel.visible := false;
  end else if (col = ColorCol) then begin
    ipu := (row div CTrows)*CTrows;      {rows of the labels}
    if (row <> ipu) then begin           {not label row}
      {y-coordinate of the color panel}
      ipu := 0;
      for ii := 0 to row do ipu := ipu+pCgrid^.RowHeights[ii]+pCgrid^.GridLineWidth;
      ColorPanel.top := pCgrid^.top+ipu+pCgrid^.GridLineWidth;
      ColorPanel.visible := true;
    end else begin
      ColorPanel.visible := false;
    end;
    OnOffPanel.visible := false;
    DeleteBut.visible := false;
(*
  end else if (col = TrafoCol) then begin
  {test...}
    if (row > 0) and (row < 4) then begin
      line := pCgrid^.Cells[col,row];
      ExtractCurrent(line,apu,apv);
      showmessage(floattostr(apu)+'  '+floattostr(apv));
    end;
    OnOffPanel.visible := false;
    DeleteBut.visible := false;
    ColorPanel.visible := false;
*)
  end else begin
    OnOffPanel.visible := false;
    DeleteBut.visible := false;
    ColorPanel.visible := false;
  end;
end;

procedure TCurrPhasorForm.OnOffBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{change the status of the show/not flag of the selected variable}
begin
(*
  OnOffBox.checked := not OnOffBox.checked;
  if (OnOffBox.checked) then pCgrid^.Cells[actcol,actrow] := '1'
  else pCgrid^.Cells[actcol,actrow] := '0';
*)
  OnOffPanel.visible := false;
  pCgrid^.SetFocus;
  DrawPhasors;
end;

procedure TCurrPhasorForm.OnOffBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{change the status of the show/not flag of the selected variable}
begin
  OnOffBox.checked := not OnOffBox.checked;
  if (OnOffBox.checked) then pCgrid^.Cells[actcol,actrow] := '1'
  else pCgrid^.Cells[actcol,actrow] := '0';
(*
  OnOffPanel.visible := false;
  pCgrid^.SetFocus;
*)
end;


procedure DeleteCTphasor(row : integer);
{delete the current trafo + data in the grid for curr.trafos, update the graph
 row  the row of the label}
var
  ii,jj  : integer;  {index}
  ipComp : longint;  {pointer to the trafo as integer}
  ipu    : integer;  {temporary}
  irow   : integer;  {index of row}
  mTrafo : integer;  {index of the trafo, in the list, 1..CTmax}
  mident : integer;  {identifier of the deleted trafo}
  pComp  : TpComp;   {pointer to the trafo}
begin
  if (row < CTrows*nCurrTrafo) then begin
    mTrafo := row div CTrows+1;            {index of trafo}
    ipComp := CTpoint[mTrafo];
    if (ipComp <> 0) then begin
      pComp := TpComp(ipComp);                {pointer to the trafo}
      pComp^.setPWR(false);                   {clear flag }
      mident := CTident[mTrafo];              {save the idenifier, needed = released later}
      if (nCurrTrafo > 0) then begin          {there are trafos}
        if (nCurrTrafo > 1 ) and              {more than one, must drop, but...}
           (mTrafo < nCurrTrafo) then begin   {no dropping for the last one}
          for ii := mTrafo to nCurrTrafo-1 do begin
            irow := (ii-1)*CTrows;            {index of the label row = first row of this trafo}
            for jj := 0 to CTrows-1 do begin  {copy the next = "drop"}
              pCgrid^.Cells[ColorCol,irow+jj] := pCgrid^.Cells[ColorCol,irow+jj+CTrows];
              pCgrid^.Cells[OnOffCol,irow+jj] := pCgrid^.Cells[OnOffCol,irow+jj+CTrows];
              pCgrid^.Cells[IdentCol,irow+jj] := pCgrid^.Cells[IdentCol,irow+jj+CTrows];
              pCgrid^.Cells[TrafoCol,irow+jj] := pCgrid^.Cells[TrafoCol,irow+jj+CTrows];
              IDxx[irow+jj] := IDxx[irow+jj+CTrows];  {coordinates of the identifiers}
              IDyy[irow+jj] := IDyy[irow+jj+CTrows];
              Relxx[irow+jj] := Relxx[irow+jj+CTrows];
              Relyy[irow+jj] := Relyy[irow+jj+CTrows];
            end;
            CTpoint[ii] := CTpoint[ii+1];     {copy also the pointer}
            CTident[ii] := CTident[ii+1];     {update identifiers}
          end;
        end;
        {clear the last entry}
        CTpoint[nCurrTrafo] := 0;       {clear pointer}
        CTident[nCurrTrafo] := mident;  {identifier of deleted trafo can be reused}
        {clear the grid}
        ipu := (nCurrTrafo-1)*CTrows;
        for ii := 0 to CTrows-1 do begin
          pCgrid^.Cells[ColorCol,ipu+ii] := '';
          pCgrid^.Cells[OnOffCol,ipu+ii] := '';
          pCgrid^.Cells[IdentCol,ipu+ii] := '';
          pCgrid^.Cells[TrafoCol,ipu+ii] := '';
          IDxx[ipu+ii] := 0;           {coordinates of the identifiers}
          IDyy[ipu+ii] := 0;
          Relxx[ipu+ii] := 0;
          Relyy[ipu+ii] := 0;
        end;
        dec(nCurrTrafo);
        if (nCurrTrafo <= 0) then begin  {reset the identifiers}
          for ii := 1 to maxCT do CTident[ii] := ii;
        end;
      end;
    end;
  end;
  {remove rows that are not needed}
  if (nCurrTrafo > 0) then pCgrid^.rowcount := nCurrTrafo*CTrows
  else pCgrid^.RowCount := CTrows;
  SetGridHeight;
  DrawPhasors;
end;

procedure TCurrPhasorForm.DeleteButClick(Sender: TObject);
{delete the "active" current trafo in the list, update phasor graph}
begin
  DeleteCTphasor(actrow);
  DeleteBut.visible := false;
  OnOffPanel.visible := false;
  ColorPanel.visible := false;
  pCgrid^.SetFocus;
  DrawPhasors;
end;

procedure RemoveCTphasor(ipComp : longint);
{remove (delete) a current trafo from the grid of CT:s for phasor display
 ipComp  pointer to the current trafo, as integer}
var
  row : integer;  {the row of the identifier in the grid}
begin
  if (isThere(ipComp,row)) then begin
    DeleteCTphasor(row);
    DrawPhasors;
  end;
end;

procedure RemoveAllPhasors;
{clear the grid, tables and phasor picture}
var
  ii,jj : integer;
  ipu   : integer;
begin
  {the grid itself}
  for ii := 0 to pCgrid^.rowcount-1 do begin
    for jj := 0 to pCgrid^.colcount-1 do begin
      pCgrid^.Cells[jj,ii] := '';
    end;
  end;
  {array of pointers and identifiers}
  for ii := 1 to maxCT do begin
    CTpoint[ii] := 0;      {pointers to current trafos, as integers}
    CTident[ii] := ii;     {keeps track of the identifiers for the phasors}
  end;
  {coordinates of identifiers}
  ipu := high(IDxx);     {dimension, same for all}
  for ii := 0 to ipu do begin
    IDxx[ii] := 0;
    IDyy[ii] := 0;
    Relxx[ii] := 0;
    Relxx[ii] := 0;
  end;
  nCurrTrafo := 0;
  CurrPhasorForm.visible := false;
  pCgrid^.rowcount := CTrows;
  SetGridHeight;
  DrawPhasors;
end;


procedure TCurrPhasorForm.FormHide(Sender: TObject);
begin
  Imagoo.CurrPhasorBut.down := false;
end;

procedure TCurrPhasorForm.ColorImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{get color index from X and Y}
var
  ii,jj,kk : integer;
begin
  ii := (X div (CRwidth+CRspace))+1;
  jj := (Y div (CRwidth+CRspace))+1;
  kk := (ii-1)*4+jj;                 {color index}
  pCgrid^.cells[ColorCol,actrow] := inttostr(kk);
  ColorPanel.visible := false;
  pCgrid^.Repaint;
  DrawPhasors;
end;

procedure TCurrPhasorForm.CurrPhasorGridDrawCell(Sender: TObject; Col,
  Row: Longint; Rect: TRect; State: TGridDrawState);
var
  iclr : integer;   {color index}
  ipu  : integer;   {temporary}
  txt  : string;    {temporary}
  txta : string;    {temporary}
begin
  {column for the selection of the color}
  if (col = ColorCol) then begin
    ipu := (row div CTrows)*CTrows;              {label row}
    if (row <> ipu) then begin                   {not label row}
      txt := pCgrid.cells[col,row];
      if (txt <> '') then begin
        iclr := strtoint(txt);
        if (iclr < 1) then iclr := 1;
        if (iclr > 16) then iclr := 16;
        pCgrid^.canvas.pen.color := gColor[iclr];
        pCgrid^.canvas.brush.color := gColor[iclr];
        {a rectangle indicates the color}
        pCgrid^.canvas.rectangle(rect.left+pCgrid^.GridLineWidth,
                                 rect.top +pCgrid^.GridLineWidth,
                       rect.left+pCgrid^.ColWidths[col]-pCgrid^.GridLineWidth,
                       rect.top+pCgrid^.RowHeights[row]-pCgrid^.GridLineWidth);
      end;
    end;
  {column for the identifier of the phasor}
  end else if (col = identCol) then begin
    ipu := (row div CTrows)*CTrows;              {label row}
    if (row <> ipu) then begin                   {not label row}
      txta := pCgrid.cells[col,row];             {identifier}
      txt := pCgrid.cells[ColorCol,row];         {color index}
      if (txt <> '') and (txta <> '') then begin {color and identifier specified}
        iclr := strtoint(txt);
        if (iclr < 1) then iclr := 1;
        if (iclr > 16) then iclr := 16;
        {write the identifier in colors}
        pCgrid^.canvas.font.style := [fsBold];
        pCgrid^.canvas.font.name := 'Arial';
        pCgrid^.canvas.font.size := 8;
        pCgrid^.canvas.font.color := gColor[iclr];
        pCgrid^.canvas.brush.style := bsSolid;
        pCgrid^.canvas.TextOut(2+rect.left,rect.top,txta);
      end;
    end;
  end;
end;

procedure TCurrPhasorForm.CurrPhasorGridExit(Sender: TObject);
begin
{  OnOffPanel.visible := false;
  DeleteBut.visible := false;
  ColorPanel.visible := false;  }
end;

procedure TCurrPhasorForm.FormDeactivate(Sender: TObject);
begin
  OnOffPanel.visible := false;
  DeleteBut.visible := false;
  ColorPanel.visible := false;
end;

procedure UpdatePhasors;
{update the phasors (after some new calculation}
var
  Aa,Ab,Ac : single;   {phase of currents, in degrees}
  Ca,Cb,Cc : single;   {magnitude of currents}
  ii     : integer;    {index}
  ipComp : longint;    {pointer to a current trafo as integer}
begin
  if (nCurrTrafo > 0) then begin
    for ii := 1 to nCurrTrafo do begin
      ipComp := CTpoint[ii];                  {pointers to the current trafos}
      GetCurrents(ipComp,Ca,Aa,Cb,Ab,Cc,Ac);  {currents in polar form}
      CurrentsToGrid(Ca,Aa,Cb,Ab,Cc,Ac,ii);   {write currents to the grid}
    end;
    DrawPhasors;
  end;
end;

procedure DrawPhasors;
{determine the scale and draw the phasors}
type
  Tscale = array[1..11] of single;
const
  ln10 = 2.302585;
  marg = 12;       {image margins}
  {max.scale}
  scales : Tscale = (10.0,12.0,15.0,20.0,25.0,30.0,40.0,50.0,60.0,80.0,100.0);
  {markers = differences between "ticks"}
  steps  : Tscale = ( 2.0, 2.0, 5.0, 5.0, 5.0, 5.0, 5.0,10.0,10.0,20.0, 20.0);
  dimscale = high(scales);                                {dimension}
var
  alpha       : single;           {angle}
  apu,apv     : single;           {temporary}
  cx,cy       : integer;          {cooridnates of origo = center of the image}
  decad       : integer;          {decade of max current}
  dimSc       : integer;          {dim of scales}
  diami       : integer;          {diameter of circle i}
  ii,jj,kk    : integer;          {index}
  ipu,ipv     : integer;          {temporary}
  irow        : integer;          {row index}
  iscale      : integer;          {index of scale}
  ix,iy       : integer;          {coordinates}
  ixp,iyp     : integer;          {coordinates}
  ixm,iym     : integer;          {coordinates}
  ix180,iy180 : integer;          {text position for 180 deg.}
  jscale      : integer;          {modified index of scale}
  lgmax       : single;           {lg(10) of max current}
  line        : string;           {temporary}
  ll          : longint;          {temporary}
  maxCur      : single;           {the maximum current}
  ncirc       : integer;          {no of circles}
  normcur     : single;           {normalized max.current}
  n15         : integer;          {multiply of 15, for marking the angles}
  scalfact    : single;           {multiplies scales[.] to get proper scale}
  txh,txw     : integer;          {height, width of text}
  txt         : string;           {temporary}
  tx180       : string;           {text for 180 deg.}
  Angle       : array[0..maxCT*CTrows-1] of single; {array of the angle of the currents}
  Curr        : array[0..maxCT*CTrows-1] of single; {array of the magnitude of the currents}
begin
  if (nCurrTrafo > 0) then begin     {there are currents to be drawn}
    dimSc := high(scales);           {dimension of scale array}
    {extract the currents from the text in the grid}
    for ii := 0 to high(Curr) do begin
      Curr[ii] := 0;                   {initialize}
      Angle[ii] := 0;
    end;
    for ii := 1 to nCurrTrafo do begin
      irow := (ii-1)*CTrows;         {index of the first row of this current trafo}
      for jj := 1 to 3 do begin
        line := pCgrid^.Cells[TrafoCol,irow+jj];
        ExtractCurrent(line,apu,apv);
        Angle[(ii-1)*CTrows+jj] := apv*pi/180;    {save angle in radians}
        Curr[(ii-1)*CTrows+jj] := apu;            {save magnitude}
      end;
    end;
    {find max.current}
    maxCur := 0;
    for ii := 1 to CTrows*nCurrTrafo do begin
      if (Curr[ii] > maxCur) then maxCur := Curr[ii];
    end;
    {determine scale}
    if (maxCur > 0) then begin
      lgmax := ln(maxCur)/ln10;      {lg(max current)}
      {the decade: 0.1 .. 1 = decade 0, 1..10 = decade 1, etc}
      if (lgmax > 0) then decad := trunc(lgmax)+1
      else decad := trunc(lgmax);    {decade of current}
      {normalize between 10 and 100}
      apu := exp(decad*ln10);        {ten to decade = 1, 10, 100, etc}
      normcur := maxCur*100/apu;     {normalized max current}
      scalfact := 0.01*apu;           {multiplier for scale[.]}
      {find scale, a simple search}
      iscale := 1;                   {initialize, just in case}
      for ii := 1 to dimscale-1 do begin
        jj := dimscale-ii;           {downwards}
        if (normcur > scales[jj]) then begin
          iscale := jj+1;
          break;
        end;
      end;
    end else begin
      iscale := 1;
      scalfact := 0.1;
    end;
    {draw the circles, if image large enough}
    {clear old image first}
    pCimage^.canvas.pen.color := clWhite;
    pCimage^.canvas.brush.color := clWhite;
    pCimage^.canvas.brush.style := bsSolid;
    pCimage^.canvas.rectangle(0,0,pCimage^.width,pCimage.height);
(*
    apu := scales[iscale]/steps[iscale];
    ncirc := round(apu);                      {no of circles}
*)
    pCimage^.canvas.pen.width := 1;
    pCimage^.canvas.pen.color := clSilver{clGray};
    pCimage^.canvas.brush.color := clWhite;
    pCimage^.canvas.brush.style := bsClear;
    {set font for markings}
    pCimage^.canvas.font.name := 'arial';
    pCimage^.canvas.font.size := 8;
    pCimage^.canvas.font.color := clBlack;
    pCimage^.canvas.font.style := [];
    if (pCimage^.width > 2*marg) then begin
      diamax := pCimage^.width-2*marg;        {diameter of the largest circle}
      ix180 := 0;                             {initialize, to keep...}
      iy180 := 0;                             {..the compiler happy}
      tx180 := '';
      {draw angled lines, every 15 degrees}
      for ii := 0 to 11 do begin
        alpha := ii*0.2617994;                     {ii* 15 degrees}
        ipu := round((diamax div 2)*cos(alpha));
        ipv := round((diamax div 2)*sin(alpha));
        ixp := (pCimage^.width div 2)+ipu;
        iym := (pCimage^.width div 2)-ipv;
        ixm := ixp-2*ipu;
        iyp := iym+2*ipv;
        pCimage^.canvas.moveto(ixp,iym);
        pCimage^.canvas.lineto(ixm,iyp);
        {write the angles, 0, 45, 90,.. etc}
        if (ii div 3 = ii/3.0) then begin
          n15 := ii*15;                           {the angle}
          txt := inttostr(n15)+'';
          txh := pCimage^.canvas.textheight(txt);         {height of text}
          txw := pCimage^.canvas.textwidth(txt);          {width of text}
          ipv := round(0.5*txh*( 1+sin(alpha)));
          ipu := round(-0.5*txw+(0.5*txw+2)*cos(alpha));  {shift of text in x-direction}
          pCimage^.canvas.textout(ixp+ipu,iym-ipv,txt);
          {texts to the other side}
          txt := inttostr(n15+180)+'';
          txh := pCimage^.canvas.textheight(txt);         {height of text}
          txw := pCimage^.canvas.textwidth(txt);          {width of text}
          ipv := round(0.5*txh*( 1+sin(alpha+pi)));
          ipu := round(-0.5*txw+(0.5*txw+2)*cos(alpha+pi));  {shift of text in x-direction}
          {180 differently, later, in order to save drawing area. Save parameters}
          if (ii = 0) then begin
            ix180 := ixm+ipu+txw;
            iy180 := iyp-ipv;
            tx180 := txt;
            {draw later}
          end else begin
            pCimage^.canvas.textout(ixm+ipu,iyp-ipv,txt);
          end;
        end;
      end;
(*
      {draw the circles}
      apu := scales[iscale]/steps[iscale];
      ncirc := round(apu);                         {no of circles}
      for ii := 1 to ncirc do begin                {draw ncirc circles}
        {clear the inside of the smallest circle, because it is getting crowded}
        if (ii = 1) then pCimage^.canvas.brush.style := bsSolid
        else pCimage^.canvas.brush.style := bsClear;
        diami := round(ii*diamax/ncirc);
        ipu := ((pCimage^.width-diami)div 2);      {upper left corner}
        pCimage^.canvas.ellipse(ipu,ipu,ipu+diami,ipu+diami); {the circle}
      end;
      {mark 180 degrees}
      pCimage^.canvas.brush.style := bsSolid;
      pCimage^.canvas.font.style := [];
      pCimage^.canvas.textout(ix180,iy180,tx180);
*)
      {check the eventual change in scales}
      if (modScale <> 0) then begin
        if (iscale+modScale > 1) then begin
          ipu := (iscale+modScale) div (dimSc-1);        {decades}
          jscale := iscale+modScale-ipu*(dimSc-1);       {modified scale}
          if (jscale = 0) then begin
            jscale := dimSc-1;
            ipu := ipu-1;
          end;
          apu := exp(ipu*ln10);                      {decade as number}
          maxScale := apu*scalfact*scales[jscale];
        end else if (iscale+modScale = 1) then begin
          jscale := 1;
          maxScale := scalfact*scales[jscale];
        end else begin   {downwards, below default decade}
          ipu := ((iscale+modScale) div (dimSc-1))-1;        {decades}
          jscale := iscale+modScale-ipu*(dimSc-1);       {modified scale}
          if (jscale = 0) then begin
            jscale := dimSc-1;
            ipu := ipu-1;
          end;
          apu := exp(ipu*ln10);                      {decade as number}
          maxScale := apu*scalfact*scales[jscale]; if (maxScale < 1) then maxScale := 1;
        end;
      end else begin
        maxScale := scalfact*scales[iscale];       {current of the max.circle}
        jscale := iscale;
      end;

      {draw the circles}
      apu := scales[jscale]/steps[jscale];
      ncirc := round(apu);                         {no of circles}
      for ii := 1 to ncirc do begin                {draw ncirc circles}
        {clear the inside of the smallest circle, because it is getting crowded}
        if (ii = 1) then pCimage^.canvas.brush.style := bsSolid
        else pCimage^.canvas.brush.style := bsClear;
        diami := round(ii*diamax/ncirc);
        ipu := ((pCimage^.width-diami)div 2);      {upper left corner}
        pCimage^.canvas.ellipse(ipu,ipu,ipu+diami,ipu+diami); {the circle}
      end;
      {mark 180 degrees}
      pCimage^.canvas.brush.style := bsSolid;
      pCimage^.canvas.font.style := [];
      pCimage^.canvas.textout(ix180,iy180,tx180);

      {first, mark the scale (max.value) (when maxScale calculated)}
      ll := round(maxScale);
      txt := inttostr(ll)+' A';
      txh := pCimage^.canvas.textheight(txt);         {height of text}
      txw := pCimage^.canvas.textwidth(txt);          {width of text}
      ipu := (pCimage^.width-txw) div 2;
      ipv := (pCimage^.height-diamax{-txh}) div 2;
      pCimage^.canvas.textout(ipu,ipv,txt);
      {draw the phasors - - - - - - - - - - - - - - - - - - - -}
      for ii := 1 to nCurrTrafo do begin           {all currents}
        for jj := 1 to 3 do begin
          kk := (ii-1)*CTrows+jj;
          if (pCgrid^.Cells[OnOffCol,kk] <> '0') then begin    {allowed to draw}
            txt := pCgrid^.cells[ColorCol,kk];
            ipu := strtoint(txt);
            pCimage^.canvas.pen.color := gColor[ipu];  {color}
            pCimage^.canvas.pen.width := 2;
            apu := 0.5*diamax*Curr[kk]/maxScale;   {magnitude in graph}
            ix := round(apu*cos(Angle[kk]));       {relative x of endpoint}
            iy := round(apu*sin(Angle[kk]));       {relative y of endpoint}
            cx := pCimage^.width div 2;
            cy := pCimage^.height div 2;
            pCimage^.canvas.moveto(cx,cy);       {start from origo}
            {must be within the image}
            if (cy-iy < 0) then ipv := 0
            else if (cy-iy < pCimage^.height) then ipv := cy-iy
            else ipv := pCimage^.height;
            {then y-coordinate}
            if (cx+ix < 0) then ipu := 0
            else if (cx+ix < pCimage^.width) then ipu := cx+ix
            else ipu := pCimage^.width;
            pCimage^.canvas.lineto(cx+ix,cy-iy); {draw phasor}
            {identifier of the current}
            txt := pCgrid^.Cells[identCol,kk];                  {identifier}
            txh := pCimage^.canvas.textheight(txt);             {height of text}
            txw := pCimage^.canvas.textwidth(txt);              {width of text}
            ipv := round(0.5*txh*( 1+sin(Angle[kk])));
            ipu := round(-0.5*txw+(0.5*txw+2)*cos(Angle[kk]));  {shift of text in x-direction}
            pCimage^.canvas.font.color := pCimage^.canvas.pen.color; {color of text = color of line}
            pCimage^.canvas.font.style := [fsBold];
            pCimage^.canvas.textout(cx+ix+ipu+Relxx[kk],cy-iy-ipv+Relyy[kk],txt);   {write the text}
            {save the basic position of the identifier}
            IDxx[kk] := cx+ix+ipu;
            IDyy[kk] := cy-iy-ipv;

          end;
        end;
      end;
    end;
  end else begin    {no trafos selected, clear the image}
    pCimage^.canvas.pen.color := clWhite;
    pCimage^.canvas.brush.color := clWhite;
    pCimage^.canvas.brush.style := bsSolid;
    pCimage^.canvas.rectangle(0,0,pCimage^.width,pCimage.height);
  end;
end;

{ - - - - - - - - - - - - - - - }
procedure TCurrPhasorForm.FormResize(Sender: TObject);
{resize the image and move the grid correspondingly}
const
  botmarg = 32;
var
  ipu,ipv,ipw : integer; {temporary}
begin
  pCgrid^.left := width-CgridMarg-pCgrid^.width;  {move the grid}
  {the size of the image}
  pCimage^.picture := nil;          {clear image first}
  ipu := pCgrid^.left-2*CimagMarg;
  ipv := {Client}Height-PointEdit.height-botmarg;
  if (ipu < ipv) then ipw := ipu else ipw := ipv;
  if (ipw > 64) then pCimage^.width := ipw
  else pCimage^.width := 64;
  pCimage^.height := pCimage^.width;             {square image}
  PointEdit.top := {Client}Height-PointEdit.height-botmarg+4-20;
  CurrentLabel.left := width div 2;              {the label (or caption) in the middle}
  DrawPhasors;                                   {update phasors}
  {save presentsize as old}
  OldWidth := width;             {old width of form}
  OldHeight := height;           {old height of form}
end;

procedure TCurrPhasorForm.CurrPhasorImgMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ii,jj : integer; {indices}
  ix,iy : integer; {coordinates of the identifiers}
  kk    : integer;
begin
  {is the cursor at an identifier?}
  if (nCurrTrafo > 0) then begin
    for ii := 1 to nCurrTrafo do begin
      for jj := 1 to 3 do begin
        kk := (ii-1)*CTrows+jj;                           {"row" = index of the current}
        if (pCgrid^.Cells[IdentCol,kk] <> '0') then begin {phasor visible}
          ix := IDxx[kk]+Relxx[kk];                       {x of the identifier}
          iy := IDyy[kk]+Relyy[kk];                       {y of the ID}
          if (x > ix-2) and (x < ix+14) and               {mouse at identifier}
             (y > iy-2) and (y < iy+14) then begin
            IDimage.left := CurrPhasorImg.left+ix;
            IDimage.top := CurrPhasorImg.top+iy;
            IDimage.visible := true;
            downx0 := x;                          {save down-coordinates}
            downy0 := y;
            downkk := kk;                         {save index}
            break;
          end;
        end;
      end;
    end;
  end;
end;

procedure TCurrPhasorForm.CurrPhasorImgMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{move the identifier to the mouse-up-point}
var
ipu : integer;
begin
  if IDimage.visible then begin
    {move the identifier}
    if (downkk >= 0) and (downkk <= maxCT*CTrows-1) then begin {reasonable index}
      {check that not too far to the left or right}
      ipu := Relxx[downkk]+x-downx0;              {shift in x-direction}
      if (ipu+IDxx[downkk] > 0) and               {not too far to the left..}
         (ipu+IDxx[downkk] < pCimage^.width-12) then begin  {..or right}
        Relxx[downkk] := ipu;                     {move with the mouse}
      end else begin
        if (ipu+IDxx[downkk] <= 0) then Relxx[downkk] := -IDxx[downkk] {left}
        else Relxx[downkk] := pCimage^.width-12-IDxx[downkk];
      end;
      {check that not too far up or down}
      ipu := Relyy[downkk]+y-downy0;              {shift in y-direction}
      if (ipu+IDyy[downkk] > 0) and               {not too far up..}
         (ipu+IDyy[downkk] < pCimage^.height-12) then begin  {..or down}
        Relyy[downkk] := ipu;                     {move with the mouse}
      end else begin
        if (ipu+IDyy[downkk] <= 0) then Relyy[downkk] := -IDyy[downkk] {up}
        else Relyy[downkk] := pCimage^.height-12-IDyy[downkk];
      end;
      DrawPhasors;                                {update phasors with the new ID position}
    end;
    IDimage.visible := false;
  end;
end;

procedure TCurrPhasorForm.CurrPhasorImgMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  curx,cury : single;     {the values of current at the cursor position}
  deg       : single;     {phase of current in degrees}
  ii        : integer;
  ix,iy     : integer;    {coordinates in the phasor diagram}
  len       : integer;    {length of string}
  magn      : single;     {magnitude of current}
  txta,txtb : string;     {temporary text}
  txtc      : string;
begin
  {show the current corresponding to the cursor position}
  if (nCurrTrafo > 0) then begin     {but only, if there are phasors}
    ix := x-(pCimage^.width div 2);
    iy := (pCimage^.height div 2)-y;
    if (diamax > 0) then begin
      curx := 2*maxScale*ix/diamax;       {real current}
      cury := 2*maxScale*iy/diamax;       {imag.current}
      txta := format('%9.4g',[curx])+' ';
      if (cury >= 0) then txtb := '+' else txtb := '';
      txtb := txtb+format('%9.4g',[cury]);
      len := length(txtb);
      {remove spaces}
      txtc := '';
      for ii := 1 to len do begin
        if (txtb[ii] <> ' ') then txtc := txtc+txtb[ii];
      end;
      txta := txta+txtc+'j A ';           {current in rect.form}
      {display also in polar form}
      magn := sqrt(curx*curx+cury*cury);        {magnitude}
      deg := 180*Cangle(cplx(curx,cury))/pi;    {phase angle}
      PointEdit.text := txta+                   {real + imag.}
        ' = '+format('%9.4g',[magn])+' A  < '+format('%6.1f',[deg])+' '; {magnitude + angle}
    end else begin
      PointEdit.text := '';
    end;
  end else begin
    PointEdit.text := '';
  end;
  {show a rectangle around ID-number, if moving}
  if (IDimage.visible) then begin
    IDimage.left := CurrPhasorImg.left+x-6;
    IDimage.top := CurrPhasorImg.top+y-6;
  end;
end;

procedure TCurrPhasorForm.IncButClick(Sender: TObject);
{increase the scale by one step}
begin
  inc(modScale);
  DrawPhasors;
end;

procedure TCurrPhasorForm.DecButClick(Sender: TObject);
{decrease the scale by one step}
begin
  if (maxScale > 1.5) then dec(modScale);
  DrawPhasors;
end;

procedure TCurrPhasorForm.ResetButClick(Sender: TObject);
{reset the scale}
begin
  modScale := 0;
  DrawPhasors;
end;

procedure TCurrPhasorForm.ExitItemClick(Sender: TObject);
begin
  visible := false;
end;

procedure TCurrPhasorForm.ButItemClick(Sender: TObject);
begin
  ButItem.checked := not ButItem.checked;
  IncBut.visible := ButItem.checked;
  DecBut.visible := ButItem.checked;
  ResetBut.visible := ButItem.checked;
  PointEdit.visible := ButItem.checked;
end;

procedure TCurrPhasorForm.BitmapItemClick(Sender: TObject);
{save diagram as bitmap}
var
  DirName     : string;
  FormImage   : TBitmap;    {for saving the whole form as a bitmap}
begin
{ TitleEdit.visible := false;}
  if (PhasorDialog.execute) then begin
    FormImage := CurrPhasorForm.getFormImage;
    FormImage.savetofile(PhasorDialog.Filename);
    {save the directory for next time}
    DirName := ExtractFilePath(PhasorDialog.FileName);
    PhasorDialog.InitialDir := DirName;    {save the directory}
    FormImage.free;
  end;
end;

end.
