{contains the object "short"}
{activation time calculated here !!!}
unit Shortcmp;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls,Compdef,Cmatmul,Skemadef,
  Imagodef,Utildef,Skemproc,Elplex,Relcurr,ReclData;

{define an object for handling (some operations and variables of) short circuits}
type
  TpShort = ^TShort;         {define pointer to this object}
  TShort  = class(TObject)   {define the object itself}
  {constructor = reserve memory, initialize}
  constructor Constr(ipSet : longint; nShort : integer);
  procedure destr;           {release memory}
  procedure PutSCpoint(ii : integer; pComp : TpComp);     {save pointer to a short}
  procedure GetSCpoint(ii : integer; var pComp : TpComp); {recall pointer}
  {save and recall pre-fault voltage at the location of the fault}
  procedure PutVolt;                                      {save voltages for all shorts}
  procedure GetVolt(ii : integer; var Volt : complex);    {recall voltage}
  procedure PutVnode(ii, node : integer);                 {save the voltage node}
  procedure GetVnode(ii : integer; var node : integer);   {recall the voltage node}
  procedure PutCnode(ii, node : integer);                 {save the current node}
  procedure GetCnode(ii : integer; var node : integer);   {recall the current node}
  {put or get a complex variable}
  procedure PutComplex(ii : integer; pC1  : pComplex; cpu : complex; var errc : integer);
  procedure GetComplex(ii : integer; pC1 : pComplex;
                             var cpu : complex; var errc : integer);
private
  number_sc : integer;       {number of short circuits}
  pVnode_sc : pInteger;      {pointer to the voltage nodes}
  pCnode_sc : pInteger;      {pointer to the current nodes}
  ppComp_sc : TppComp;       {pointer to an array of pointers to the shorts}
  pVolt_sc  : pComplex;      {pointer to prefault voltages}
end;

{define an object for handling current transformators}
type
  TpCurrT = ^TCurrT;         {define pointer to this object}
  TCurrT  = class(TObject)   {define the object itself}
  {constructor = reserve memory, initialize}
  constructor Constr(ipSet : longint; nCurrT, nSteps : integer);
  procedure destr;           {release memory}
  procedure PutCTpoint(ii : integer; pComp : TpComp);     {save pointer to a relay}
  procedure GetCTpoint(ii : integer; var pComp : TpComp); {recall pointer}
  procedure SaveCurr(it : integer; pWs : pComplex; nnod : integer); {pick up and save current}
  procedure PhaseMaxOrSum(it : integer; pSet: TpCompSet); {calc. phase currents from seq., save abs(Ia)}
  procedure SaveComplex(pWs : pComplex; nnod, ic,it : integer);  {save (temporarily) complex numbers}
  procedure ClearTempVar;    {creal the variables var1_s...var6_s for all relays}
  procedure SumToOutVar;     {for sum current trafos, calculate the sum to output variables}
  function  GetNumber : integer; {get no of current trafos}
(*procedure SumCurr(pCurrT : TpComp; var curr : complex); {get current in curr.trafo} *)
private
  number_ct : integer;        {number of current trafos}
  steps_ct  : integer;        {number of calculations = steps}
  ppComp_ct : TppComp;        {pointer to array for pointers to current trafos}
  pSet_ct   : TpCompSet;      {pointer to the set of components etc}
end;

{define an object for a group of relays, connected to same current trafo}
type
  TpRelGroup = ^TRelGroup;         {define pointer to this object}
  TRelGroup  = class(TObject)      {define the object itself}
  {constructor = reserve memory, initialize}
  constructor Constr(pSet: TpCompSet; nrg : integer; PQRel,RCrel : boolean);
  procedure destr;           {release memory}
  procedure AddRelay(pComp :TpComp);
  procedure PutNextRpoint(pComp : TpComp); {save pointer to a relay}
  procedure PutBackwRpoint(pComp : TpComp); {save pointer to a inv.time relay}
  procedure GetRpoint(ii : integer; var pComp : TpComp); {get pointer to a relay}
  procedure PutRpoint(ii : integer; pComp :TpComp);      {save pointer }
  procedure GetInvTpoint(var pComp: TpComp);  {get the pointer to the inverse-time relay}
  procedure FindRightmost(var rimo : integer); {find the rightmost relay}
  procedure FindAndShowActTime;                {find and show the activation time of this group}
  procedure GetMinActCurr(var mincur: single); {get smallest activation current}
  procedure SortActCurrents;               {sort the relays after the activation current}
  procedure ActCurrents(isPQ: boolean);    {"sort" the modules of a PGQ-relay}
  {get the activation current and delay time of relay ii}
  procedure GetCurrTime(ii,ir :integer; var curr , time : single);
  function  ActivationTime(ir : integer): single;  {check, is some relay in the group activated}
  procedure GetNdtrel(var ndtrel : integer); {get number of def.time relays}
  procedure FindCurrTrafos(pCurrT : TpCurrT); {find the current trafos}
  function  isPQrelay : boolean;   {true, if relay group = PQ-relay or electromech.relay}
  function  isRCrelay : boolean;   {true, if relay group = recloser relay}
  procedure GetActTime(ir : integer; var actime : single);
  function isForwrd(ir : integer): boolean; {true, when fault is in forward direction (PQ-relays)}
  procedure PQMinMaxCur(cur1, cur2, cur3,cur4 : single; ii: integer; var Imin: single);
  procedure GetIECcurrent(pCTrafo: TpComp; rr: integer; var curr : single);
  procedure GetIEC_CTcurrent(pCTrafo: TpComp; rr: integer;
                             conn, Isec, mode, ratio: single; var curr : single);
  function  pTimeShown: TpComp;
private
  backw_rg   : integer;        {temporary, index of next relay, when filling backwards}
  mincur_rg  : single;         {smallest activation current > 0}
  next_rg    : integer;        {temporary, index of next relay}
  number_rg  : integer;        {number of relays in the group}
  ndeft_rg   : integer;        {number of definite time relays in the group}
  node_rg    : integer;        {node to which relays are connected}
  ppComp_rg  : TppComp;        {pointer to array for pointers to relays in this group}
  pInv_rg    : TpComp;         {pointer to the eventual inverse-time relay}
  pCurrT_rg  : TpComp;         {pointer to the current trafo group is connected to}
  pSet_rg    : TpCompSet;      {pointer to the "common"}
  right_rg   : integer;        {index of the rightmost relay }
  actime_rg  : array [1..6] of single;  {time when first relay in the group activated}
  PQminc_rg  : array [1..6] of integer; {index of PGQ-relay module with min. activation curr.}
  PQlowc_rg  : array [1..6] of integer; {index of PGQ-relay module with third largest activation curr.}
  PQhigc_rg  : array [1..6] of integer; {index of PGQ-relay module with second largest activation curr.}
  PQmaxc_rg  : array [1..6] of integer; {index of PGQ-relay module with max. activation curr.}
  PQRelay_rg : boolean;        {true, when group = PGQ-relay}
  RCrelay_rg : boolean;        {true, when group = recloser relay}
  forwrd_rg  : array [1..6] of boolean; {PQ-relays: true, when fault in forward direction}
end;

type
  TppRelGroup = ^TpRelGroup;  {pointer to an array for pointers to relay groups}

{define an object for handling relays}
type
  TpRelay = ^TRelay;         {define pointer to this object}
  TRelay  = class(TObject)   {define the object itself}
  {constructor = reserve memory, initialize}
  constructor Constr(ipSet : longint; nRelay, nQrel : integer);
  procedure destr;           {release memory}
  procedure PutRpoint(ii : integer; pComp : TpComp);     {save pointer to a relay}
  procedure GetRpoint(ii : integer; var pComp : TpComp); {recall pointer}
(*  procedure CheckActivation; {check the activation of the relay}*)
  procedure GroupActivation; {check activation of the relay groups}
  procedure FindGroups(pNodar,pNumbg : pInteger); {find the relay groups}
  procedure CreateGroups;    {find and create relay groups}
  procedure GetGpoint(ii : integer; var pGroup : TpRelGroup); {get pointer to relay group}
  procedure PutGpoint(ii : integer; pGroup : TpRelGroup);     {save pointer to relay group}
  procedure FindCurrTrafos(pCurrT: TpCurrT);  {find current trafos for the groups}
  procedure CheckSumCurr;  {calc. sum current for relays connected to sum trafo}
  function  GetNumber : integer;  {get number of relays}
private
  group_r  : integer;        {index of group this relay belongs to}
  number_r : integer;        {number of relays}
  nQrel_r  : integer;        {number of PQ-, electromech. and recloser relays}
  ppComp_r : TppComp;        {pointer to array for pointers to relays}
  pSet_r   : TpCompSet;      {pointer to the set of components etc}
  ngroup_r : integer;        {number of relay groups}
  ppRelG_r : TppRelGroup;    {pointer to array of pointers to relay groups (welll)}
end;

procedure SumCurrents(pComp: TpComp);

implementation

uses
  shortcir, twoshort,imagopas, userdef, recloser, Skemobj, routines ;

constructor TShort.Constr(ipSet : longint; nShort : integer);
 {construct=initialize
  ipSet    pointer to the set of components etc, as integer
  nShort   number of short circuits
  }
 var
   errn    : integer;    {error flag}
   errc    : integer;    {error flag}
   isc     : integer;    {index for short circuits}
   jcomp   : integer;    {counts components}
   ModNam  : string;     {name of a component}
   node    : integer;    {(voltage) node of the short}
   pComp   : TpComp;     {pointer to components}
   pSet    : TpCompSet;  {pointer to the set of components etc}
 begin
   pSet := TpCompSet(ipSet);
   number_sc := nShort;                             {number of short circuits}
   {initialize: get room for pointers to shorts, etc..}
   if (number_sc > 0) then begin
     getmem(pVolt_sc,number_sc*SizeOf(complex));       {room for prefault voltages}
     getmem(ppComp_sc,number_sc*SizeOf(TpComp));       {reserve memory for pointers}
     getmem(pVnode_sc,number_sc*SizeOf(integer));      {room for voltage nodes}
     getmem(pCnode_sc,number_sc*SizeOf(integer));      {room for current nodes}
     for isc := 1 to number_sc do PutSCpoint(isc,nil); {pointers to nil, initially}
     {fill the arrays of pointers}
     errc := 0;
     pComp := nil;
     jcomp := 0;
     isc := 0;
     while (errc = 0) do begin                 {scan all components}
       pSet^.NextComp(pComp,errc,jcomp,true);  {get pointer to next component}
       if (errc = 0) then begin           {component found}
         pComp^.GetModName(ModNam);       {get the name (type) of the component}
         if (ModNam = gShort) then begin  {is it a short circuit}
           inc(isc);                      {increment index}
           PutSCpoint(isc,pComp);         {save the pointer to the array of pointers}
           pComp^.GetNode(1,node,errn);   {get first node of component}
           if (errn = 0) then begin
             putVnode(isc,node);          {save voltage node into array of nodes}
           end else begin
             putVnode(isc,-1);            {if error, node = unknown}
           end;
           pComp^.GetNode(2,node,errn);   {get second node of component}
           if (errn = 0) then begin
             putCnode(isc,node);          {save current node into array of nodes}
           end else begin
             putCnode(isc,-1);            {if error, node = unknown}
           end;
         end;
       end;
     end;
   end else begin                         {no short circuits}
     ppComp_sc := nil;
     pVolt_sc := nil;
     pVnode_sc := nil;
     pCnode_sc := nil;
   end;
 end;

procedure TShort.Destr;
 {releases memory reserved for short circuit}
 begin
   if (number_sc > 0) then begin
     try
       freemem(pVolt_sc,number_sc*SizeOf(complex));  {release prefault voltages}
       freemem(ppComp_sc,number_sc*SizeOf(TpComp));  {release pointers}
       freemem(pVnode_sc,number_sc*SizeOf(integer)); {release voltage nodes}
       freemem(pCnode_sc,number_sc*SizeOf(integer)); {release current nodes}
     finally
     end;
   end;
 end;

procedure TShort.PutSCpoint(ii : integer; pComp : TpComp);
{saves the pointer to short number ii into the array for pointers
 ii     index of the generator
 pComp  pointer to the short circuit
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   try
     if (ii > 0) and (ii <= number_sc) then begin   {reasonable index}
       ppComp := ppComp_sc;                         {pointer to first pointer}
       inc(ppComp,ii-1);                            {pointer to pointer no ii}
       ppComp^ := pComp;                            {save the pointer}
     end;
   finally
   end;
 end;

procedure TShort.GetSCpoint(ii : integer; var pComp : TpComp);
{recalls the pointer to short number ii into the array for pointers
 ii     index of the generator
 pComp  pointer to the short
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   pComp := nil;                                   {initialize, just in case}
   try
     if (ii > 0) and (ii <= number_sc) then begin   {reasonable index}
       ppComp := ppComp_sc;                         {pointer to first pointer}
       inc(ppComp,ii-1);                            {pointer to pointer no ii}
       pComp := ppComp^;                            {get the pointer}
     end else begin
       pComp := nil;
     end;
   finally
   end;
 end;

procedure TShort.PutVolt;
{save the prefault voltage at the location of the short circuits
 It is assumed that the voltages have been calculated, i.e.
 the network has been solved without the short circuit}
var
  isc    : integer;  {index}
  pComp  : TpComp;   {pointer to the short as component}
  Volt   : complex;  {prefault voltage}
  pVolt  : pComplex; {pointer to elements in the array for voltages}
begin
  try
    if (number_sc > 0) then begin        {are there short circuits}
      pVolt := pVolt_sc;                 {pointer to the first voltage}
      for isc := 1 to number_sc do begin {all short circuits}
        getSCpoint(isc,pComp);           {pointer to the short circuit}
        pComp^.GetVar(gShort_Vr,Volt.x); {real  part of the voltage}
        pComp^.GetVar(gShort_Vi,Volt.y); {imag. part of the voltage}
        pVolt^ := Volt;                  {save the voltage}
        inc(pVolt);                      {pointer to the next voltage}
      end;
    end;
  finally
  end;
end;

procedure TShort.GetVolt(ii : integer; var Volt : complex);
{recall the prefault voltage at the location of short number ii}
var
  pVolt  : pComplex;   {pointer to the short, as component}
begin
  try
    if (ii > 0) and (ii <= number_sc) then begin
      pVolt := pVolt_sc;     {pointer to first short circuit}
      inc(pVolt,ii-1);       {pointer to voltage of ii'th short circuit}
      Volt := pVolt^;        {prefault voltage}
    end else begin
      Volt.x := 0.0;         {give some value}
      Volt.y := 0.0;
    end;
  finally
  end
end;

procedure TShort.PutVnode(ii : integer; node : integer);
{saves the node of the short number ii into the array for nodes
 ii       index of the generator
 node     the node
 }
 var
   pNode : pInteger;    {pointer to nodes}
 begin
   try
     if (ii > 0) and (ii <= number_sc) then begin  {reasonable index}
       pNode := pVNode_sc;                         {pointer to first node}
       inc(pNode,ii-1);                            {pointer to node ii}
       pNode^ := node;                             {save the node}
     end;
   finally
   end;
 end;

procedure TShort.GetVnode(ii : integer; var node : integer);
{gets the (voltage) node  of the short number ii
 ii     index of the generator
 node     the node
 }
 var
   pNode : pInteger;    {pointer to nodes}
 begin
   try
     if (ii > 0) and (ii <= number_sc) then begin  {reasonable index}
       pNode := pVnode_sc;                         {pointer to first node}
       inc(pNode,ii-1);                            {pointer to node ii}
       node := pNode^;                             {get the node}
     end;
   finally
   end;
 end;

procedure TShort.PutCnode(ii : integer; node : integer);
{saves the node of the short number ii into the array for current nodes
 ii       index of the generator
 node     the node
 }
 var
   pNode : pInteger;    {pointer to nodes}
 begin
   try
     if (ii > 0) and (ii <= number_sc) then begin  {reasonable index}
       pNode := pCNode_sc;                         {pointer to first node}
       inc(pNode,ii-1);                            {pointer to node ii}
       pNode^ := node;                             {save the node}
     end;
   finally
   end;
 end;

procedure TShort.GetCnode(ii : integer; var node : integer);
{gets the (current) node  of the short number ii
 ii     index of the generator
 node     the node
 }
 var
   pNode : pInteger;    {pointer to nodes}
 begin
   try
     if (ii > 0) and (ii <= number_sc) then begin  {reasonable index}
       pNode := pCnode_sc;                         {pointer to first node}
       inc(pNode,ii-1);                            {pointer to node ii}
       node := pNode^;                             {get the node}
     end;
   finally
   end;
 end;

procedure TShort.GetComplex(ii : integer; pC1 : pComplex;
                             var cpu : complex; var errc : integer);
{get a complex variable, from an array of complex variables
ii      ii'th variable recalled
pC1     pointer to the first variable in the array
cpu     the variable from the array
errc    error flag
}
var
  pC   : pComplex;     {pointer to a complex variable}
begin
  try
    if (ii > 0) and (ii <= number_sc) then begin  {reasonable index}
      pC := pC1;                                  {pointer to first variable}
      inc(pC,ii-1);                               {pointer to variable no ii}
      cpu := pC^;                                 {get the variable}
      errc := 0;                                  {no error}
    end else begin                                {something wrong}
      errc := 1;                                  {set error flag}
    end;
  except
    errc := 2;                                     {in case of some other error}
  end;
end;

procedure TShort.PutComplex(ii : integer; pC1  : pComplex;
                            cpu : complex; var errc : integer);
{put a complex variable, to an array of complex variables
ii      ii'th variable saved
pC1     pointer to the first variable in the array
cpu     the variable to the array
errc    error flag
}
var
  pC   : pComplex;     {pointer to a complex variable}
begin
  try
    if (ii > 0) and (ii <= number_sc) then begin  {reasonable index}
      pC := pC1;                                  {pointer to first variable}
      inc(pC,ii-1);                               {pointer to variable no ii}
      pC^ := cpu;                                 {save the variable}
      errc := 0;                                  {no error}
    end else begin                                {something wrong}
      errc := 1;                                  {set error flag}
    end;
  except
    errc := 2;                                    {in case of some other error}
  end;
end;

{ current transformer - - - - - - - - - - - - - - - - - - - - - - - - - - - }
procedure TCurrT.SaveComplex(pWs : pComplex; nnod, ic, it : integer);
{save the calculated currents to the current trafo- components
 pWs   pointer to the solution = array of voltages and currents
 nnod  number of nodes = voltages and currents
 ic    determines pair of singles (1...3) to which complex is saved
 it    time step  }
var
  cpu   : complex;  {temporary}
  cpv   : complex;  {temporary}
  ipu   : integer;  {temporary}
  irl   : integer;  {index}
  name  : string;   {name = type of the component}
  pComp : TpComp;   {pointer to components = curr.trafos}
begin
  if (number_ct > 0) then begin           {if there are relays}
  {currents of other components to comp.nodes, needed for curr.trafos}
  ResultsToComp(pSet_ct, nnod, pSet_ct^.appl_hz, pWs,ic,ForNode);
    try
      for irl := 1 to number_ct do begin   {scal all relays}
        GetCTpoint(irl,pComp);             {get pointer to curr.trafo}
{       pComp^.SumCurr(cpu.x,cpu.y);       {new way! 23.8.2002}
        pComp^.GetName(name);              {name = type of the component}
        if (name = gSpecCT) then begin     {special, summing trafo}
          if (it >= 0) then begin
            {save sequence currents temporarily,
             processed further and saved back in PhaseMaxOrSum, below}
            pComp^.GetCurrOut(2,cpv.x,cpv.y); {unprocessed current}
            if          (ic = 1) then begin
              pComp^.PutMultix(it, 3,cpv.x,cpv.y,0,0,0,0); {save first and second}
            end else if (ic = 2) then begin
              pComp^.PutMultix(it,12,0,0,cpv.x,cpv.y,0,0); {..3rd and 4th}
            end else if (ic = 3) then begin
              pComp^.PutMultix(it,48,0,0,0,0,cpv.x,cpv.y); {..5th and 6th}
            end else begin
              {error case}
            end;
          end;
          {get the processed current. It is used in the same way as the proper current of
           "normal" current trafos (It was calculated in ResultsToComp, above, in Elplex.pas}
          pComp^.GetCurrOut(3,cpu.x,cpu.y); {processed current, calc. in ResultsToComp, above (1.2.06)}
        end else begin
          pComp^.GetCurrOut(2,cpu.x,cpu.y);  {1.2.06}
        end;
        if (ic = 1) then begin             {save the current in one of three sets}
          pComp^.var1_s := cpu.x;
          pComp^.var2_s := cpu.y;
        end else if (ic = 2) then begin
          pComp^.var3_s := cpu.x;
          pComp^.var4_s := cpu.y;
        end else if (ic = 3) then begin
          pComp^.var5_s := cpu.x;
          pComp^.var6_s := cpu.y;
        end;
      end;
    finally
    end;
  end;
end;

procedure TCurrT.PhaseMaxOrSum(it : integer; pSet: TpCompSet);
{calculates the abs value largest current of the phase A,B,C currents,
 or the sum current, depending on the type of the current trafo
 when the pos,neg,and zero seq. currents are saved in var1_s...var6_s
 and saves the abs value to the array pSingle_s
 Called in Earthflt.pas, Twoshort.pas, IECTwophase and IECearthfault of IEC.pas
 it    step = location
 pSet  pointer to the common
}
var
  apu,apv  : single;   {temporary}
  apw,apx  : single;   {temporary}
  apy      : single;   {temporary}
  cp,cn,cz : complex;  {pos,neg,zero seq. currents}
  cpu      : complex;  {temporary}
  Ia,Ib,Ic : complex;  {phase currents}
  irl      : integer;  {index}
  ModNam   : string;   {name of the component}
  pComp    : TpComp;   {pointer to components = relays}
begin
  if (number_ct > 0) then begin      {if there are relays}
    try
      for irl := 1 to number_ct do begin  {scal all relays}
        GetCTpoint(irl,pComp);            {get pointer to curr.trafo}
        pComp^.GetModName(ModNam);        {get name of the component}
        {save "unprocessed" currents to special current trafo}
        if (ModNam = gSpecCT) then begin
          pComp^.GetMultix(it,cp.x,cp.y,cn.x,cn.y,cz.x,cz.y);   {get seq.curr.,Saved in SaveComplex}
          SeqToPhase(cp,cn,cz,Ia,Ib,Ic);                        {change to phase variables}
          pComp^.putMultix(it,63,Ia.x,Ia.y,Ib.x,Ib.y,Ic.x,Ic.y);{put phase curr. back}
        end;
        {process all current trafos. For special CT's use processed current}
        cp.x := pComp^.var1_s;            {get the sequence currents..}
        cp.y := pComp^.var2_s;            {..saved in SaveComplex, above..}
        cn.x := pComp^.var3_s;            {..called in PhaseToPhaseShort..}
        cn.y := pComp^.var4_s;            {..of Twoshort.pas, or ..}
        cz.x := pComp^.var5_s;            {..in EarthFault of ..}
        cz.y := pComp^.var6_s;            {..Earthflt.pas}
        SeqToPhase(cp,cn,cz,Ia,Ib,Ic);    {change to phase variables}
        {save sum to sum current trafo}
        if (ModNam = gSumCT) then begin     {sum current trafo ?}
          apu := 3.0*Cabs(cz);              {sum current}
        end else begin                      {normal current trafo}
          apu := Cabs(Ia);                  {abs value current in phase a}
          apv := Cabs(Ib);                  {abs.value of current in phase b}
          if (apv > apu) then apu := apv;   {select the largest}
          apv := Cabs(Ic);                  {abs.value of current in phase c}
          if (apv > apu) then apu := apv;   {select the largest}
        end;
        pComp^.putSingle(it,apu);           {save the current to location for step it}
        {save the current of all phases, for eventual display}
        cpu := Cadd(Ia,Cadd(Ib,Ic));        {residual current}
        apx := Cabs(cpu);
        apu := Cabs(Ia);
        apv := Cabs(Ib);
        apw := Cabs(Ic);
        {three times neg.seq. current, or neg.seq. current as such}
//{!?}    if (pSet^.neg_three) then
//          apy := 3*Cabs(cn) else apy := Cabs(cn);   {3*neg.seq., or neg.seq}
apy := Cabs(cn);
        pComp^.PutTriple(it,apu,apv,apw,apx,apy);   {save magnitude}
        apx := Cangle(cpu)*180/pi;
        apu := Cangle(Ia)*180/pi;
        apv := Cangle(Ib)*180/pi;
        apw := Cangle(Ic)*180/pi;
        apy := Cangle(cn)*180/pi;
        pComp^.PutPhase(it,apu,apv,apw,apx,apy);   {save phase}
      end;
    finally
    end;
  end;
end;

procedure TCurrT.ClearTempVar;
{clear the temporary variables var1_s...var6_s for all current trafos
}
var
  irl   : integer;  {index for current trafos}
  pComp : TpComp;   {pointer to components = current trafos}
begin
  if (number_ct > 0) then begin          {if there are current trafos}
    try
      for irl := 1 to number_ct do begin  {scal all current trafos}
        GetCTpoint(irl,pComp);            {get pointer to current trafos}
        pComp^.var1_s := 0;
        pComp^.var2_s := 0;
        pComp^.var3_s := 0;
        pComp^.var4_s := 0;
        pComp^.var5_s := 0;
        pComp^.var6_s := 0;
      end;
    finally
    end;
  end;
end;

procedure TCurrT.PutCTpoint(ii : integer; pComp : TpComp);
{saves the pointer to current trafo number ii into the array for pointers
 ii     index of the current trafo
 pComp  pointer to the current trafo
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   try
     if (ii > 0) and (ii <= number_ct) then begin   {reasonable index}
       ppComp := ppComp_ct;                         {pointer to first pointer}
       inc(ppComp,ii-1);                           {pointer to pointer no ii}
       ppComp^ := pComp;                           {save the pointer}
     end;
   finally
   end;
 end;

procedure TCurrT.GetCTpoint(ii : integer; var pComp : TpComp);
{recalls the pointer to current trafo number ii into the array for pointers
 ii     index of the current trafo
 pComp  pointer to the current trafo
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   pComp := nil;                                   {initialize, just in case}
   try
     if (ii > 0) and (ii <= number_ct) then begin   {reasonable index}
       ppComp := ppComp_ct;                         {pointer to first pointer}
       inc(ppComp,ii-1);                            {pointer to pointer no ii}
       pComp := ppComp^;                            {get the pointer}
     end else begin
       pComp := nil;
     end;
   finally
   end;
 end;
(*
procedure TCurrT.SumCurr(pCurrT : TpComp; var curr : complex);
{calculate the current through the current trafo as sum of currents
 from or to components xonnected to this current trafo
 pCurrT   pointer to the current trafo
 curr     the calculated current}
var
  curx,cury : single;      {components of current}
  ii        : integer;     {index}
  nConComp  : integer;     {number of connected components}
  nodeflg   : integer;     {1 = upstream node, 2 = downstr, of current trafo}
  pList     : TpConList;   {local pointer to connected components}
  pComp     : TpComp;      {pointer to component}
  sumx,sumy : single;      {components of sum currents}
begin
  if (pCurrT <> nil) then begin
    sumx := 0;
    sumy := 0;
    pCurrT^.GetConData(pList, nConComp, nodeflg);   {data for connected comp}
    if (nConComp > 0) and (pList <> nil) then begin
      for ii := 1 to nConComp do begin               {all connected comp}
        pComp := TpComp(pList^.ipComp);             {pointer to this conn.comp}
        if (pComp <> nil) then begin
          pComp^.GetCurrOut(pList^.indx,curx,cury); {get current}
          sumx := sumx+curx;
          sumy := sumy+cury;
        end;
        inc(pList);
      end;
    end;
    if (nodeflg = 1) then           {upstream node}
      curr := cplx(sumx,sumy)
    else if (nodeflg = 2) then  {downstream node}
      curr := cplx(-sumx,-sumy)
    else
      curr := cplx(0,0);            {something wrong}
  end;
end;
*)
procedure TCurrT.SaveCurr(it : integer; pWs : pComplex; nnod : integer);
{save the calculated currents to the current trafo components
 as such, if a "normal" current trafo
 but save zero, if sum current trafo (symmetrical case assumed)
 it    step = location
 pWs   pointer to the solution = array of voltages and currents
 nnod  number of nodes = voltages and currents
}
var
  apu,apv : single;   {temporary}
  cpu,cpv : complex;  {temporary}
  cpw     : complex;  {temporary}
  irl     : integer;  {index for current trafos}
  ModNam  : string;   {name = type of the trafo}
  pComp   : TpComp;   {pointer to components = relays}
begin
  {currents of other components to comp.nodes, needed for curr.trafos}
  ResultsToComp(pSet_ct, nnod, pSet_ct^.appl_hz, pWs,1,ForNode);
  if (number_ct > 0) then begin              {if there are relays}
    try
      for irl := 1 to number_ct do begin     {scal all relays}
        GetCTpoint(irl,pComp);               {get pointer to current trafo}
        if (pComp <> nil) then begin
          pComp^.GetModName(ModNam);         {get name = type of the trafo}
          if (ModNam = gSumCT) then begin    {sum current trafo}
            pComp^.putSingle(it,0.0);        {save zero, assumed sum = 0}
          end else begin                     {normal trafo}
            {pComp^.SumCurr(cpu.x,cpu.y);    {new way! 23.8.2002}
            if (ModNam = gSpecCT) then begin {special CT needs "processed" currents}
              pComp^.GetCurrOut(3,cpu.x,cpu.y); {calc. in ResultsToComp, above (1.2.06)}
            end else begin
              pComp^.GetCurrOut(2,cpu.x,cpu.y); {calc. in ResultsToComp, above (1.2.06)}
            end;
            apu := Cabs(cpu);
            apv := Cangle(cpu)*180/pi;       {phase angle}
            pComp^.putSingle(it,apu);
            {save current of all phases (same current)}
            pComp^.putTriple(it,apu,apu,apu,0,0);
            pComp^.putPhase(it,apv,apv-120,apv+120,0,0);
            {save "unprocessed" current of special current trafos}
            if (ModNam = gSpecCT) then begin
              {same current in all phases, different phase}
              pComp^.GetCurrOut(2,cpu.x,cpu.y); {calc. in ResultsToComp, above (1.2.06)}
              cpv := Cmult(cpu,emifii);         {exp(-i fii), defined in elplex.pas}
              cpw := Cmult(cpu,eifii);          {exp i fii), defined in elplex.pas}
              pComp^.putMultix(it,63,cpu.x,cpu.y,cpv.x,cpv.y,cpw.x,cpw.y);
            end;
          end;
        end;
      end;
    finally
    end;
  end;
end;

procedure SumCurrents(pComp: TpComp);
{sum the phase currents of a sum current trafo, and put the sum to all phases
 pComp  pointer to the sum current trafo}
var
  apu,apv,apw : single;  {temporary}
begin
  pComp^.GetVar(1,apu);            {get real parts of phase currents}
  pComp^.GetVar(3,apv);
  pComp^.GetVar(5,apw);
  apu := apu+apv+apw;
  pComp^.PutVar(1,apu,'SumCurrents');            {put the same sum to all phases}
  pComp^.PutVar(3,apu,'SumCurrents');
  pComp^.PutVar(5,apu,'SumCurrents');
  pComp^.GetVar(2,apu);            {get imag parts of phase currents}
  pComp^.GetVar(4,apv);
  pComp^.GetVar(6,apw);
  apu := apu+apv+apw;
  pComp^.PutVar(2,apu,'SumCurrents');            {put the same sum to all phases}
  pComp^.PutVar(4,apu,'SumCurrents');
  pComp^.PutVar(6,apu,'SumCurrents');
end;

procedure TCurrT.SumToOutVar;
{for sum current trafos, calculate the sum to output variables
 It is assumed that the output variables (1...6) contain the phase currents
 Called in Earthfault in Earthflt.pas, see also PolarToRelay in Elplex.pas
 and in PhaseToPhaseShort of Twoshort.pas (why?)}
var
  apu,apv : single;    {temporary}
  apw     : single;    {temporary}
  ict     : integer;   {index}
  ModNam  : string;    {name of the component}
  pComp   : TpComp;    {pointer to a current trafo}
begin
  if (number_ct > 0) then begin              {if there are relays}
    try
      for ict := 1 to number_ct do begin     {scal all relays}
        GetCTpoint(ict,pComp);               {get pointer to current trafo}
        if (pComp <> nil) then begin
          pComp^.GetModName(ModNam);         {get name = type of the trafo}
          if (ModNam = gSumCT) then begin    {sum current trafo}
            SumCurrents(pComp);
          end;
        end;
      end;
    finally
    end;
  end;
end;

function TCurrT.GetNumber : integer;
{get the number of current transformators}
begin
  GetNumber := number_ct;
end;

constructor TCurrT.Constr(ipSet : longint; nCurrT, nSteps : integer);
 {construct=initialize
  ipSet    pointer to the set of components etc, as integer
  nCurrT   number of current trafos
  nSteps   number of calculations (steps)
  }
 var
   errc    : integer;    {error flag}
   ir      : integer;    {index for trafos}
   jcomp   : integer;    {counts components}
   ModNam  : string;     {name of a component}
   pComp   : TpComp;     {pointer to components}
 begin
   pSet_ct := TpCompSet(ipSet);
   number_ct := nCurrT;
   steps_ct := nSteps;
   {get room for calculated currents, for all trafos, all steps}
   if (number_ct > 0) then begin
     pSet_ct^.GetMemSingle(steps_ct);               {reserve room for time}
     getmem(ppComp_ct,number_ct*SizeOf(TpComp));    {reserve memory for pointers}
     for ir := 1 to number_ct do PutCTpoint(ir,nil); {initialize}
     {fill the arrays of pointers and nodes}
     errc := 0;
     pComp := nil;
     jcomp := 0;
     ir := 0;
     while (errc = 0) do begin                   {scan all components}
       pSet_ct^.NextComp(pComp,errc,jcomp,true); {get pointer to next component}
       if (errc = 0) then begin                  {component found}
         pComp^.GetModName(ModNam);              {get the name (type) of the component}
         if ((ModNam = gCurrT) or (ModNam = gSumCT) or           {is it current trafo..}
             (ModNam = gSpecCT) or                               {summing current trafo}
             (ModNam = gMeasZ) or (ModNam = gPolyg)) then begin  {...or distance relay }
           {save the CT pointer to the array of pointers to cur.trafos, but only CT}
           if (ModNam <> gMeasZ) and (ModNam <> gPolyg) then begin
             inc(ir);                       {increment index}
             PutCTpoint(ir,pComp);
           end;
           pComp^.GetMemSingle(steps_ct); {reserve array for calculations}
           pComp^.GetMemTriple1(steps_ct); {reserve array for saving currents}
           pComp^.GetMemPhase(steps_ct);  {reserve array for saving phases}
           {special current trafo needs space to store the "unprocessed" currents}
           if (ModNam = gSpecCT) then pComp^.GetMemMultix1(steps_ct);
         end;
       end;
     end;
   end else begin                         {no current trafos}
     ppComp_ct := nil;
   end;
 end;

procedure TCurrT.Destr;
 {releases memory reserved for current trafos}
begin
  if (number_ct > 0) then begin
    try
      freemem(ppComp_ct,number_ct*SizeOf(TpComp));
    finally
    end;
  end;
end;

{ relay - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - }
procedure TRelay.PutRpoint(ii : integer; pComp : TpComp);
{saves the pointer to relay number ii into the array for pointers
 ii     index of the generator
 pComp  pointer to the relay
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   try
     if (ii > 0) and (ii <= number_r) then begin   {reasonable index}
       ppComp := ppComp_r;                         {pointer to first pointer}
       inc(ppComp,ii-1);                           {pointer to pointer no ii}
       ppComp^ := pComp;                           {save the pointer}
     end;
   finally
   end;
 end;

procedure TRelay.GetRpoint(ii : integer; var pComp : TpComp);
{recalls the pointer to relay number ii into the array for pointers
 ii     index of the relay
 pComp  pointer to the relay
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   pComp := nil;                                   {initialize, just in case}
   try
     if (ii > 0) and (ii <= number_r) then begin   {reasonable index}
       ppComp := ppComp_r;                         {pointer to first pointer}
       inc(ppComp,ii-1);                           {pointer to pointer no ii}
       pComp := ppComp^;                           {get the pointer}
     end else begin
       pComp := nil;
     end;
   finally
   end;
 end;

procedure TRelay.PutGpoint(ii : integer; pGroup : TpRelGroup);
{saves the pointer to relay group number ii into the array for pointers
 ii      index of the generator
 pGroup  pointer to the relay group
 }
 var
   ppGroup : TppRelGroup;    {pointer to pointers in the array}
 begin
   try
     if (ii > 0) and (ii <= ngroup_r) then begin   {reasonable index}
       ppGroup := ppRelG_r;                        {pointer to first pointer}
       inc(ppGroup,ii-1);                          {pointer to pointer no ii}
       ppGroup^ := pGroup;                         {save the pointer}
     end;
   finally
   end;
 end;

procedure TRelay.GetGpoint(ii : integer; var pGroup : TpRelGroup);
{recalls the pointer to relay group number ii into the array for pointers
 ii      index of the relay
 pGroup  pointer to the relay group
 }
 var
   ppGroup : TppRelGroup;    {pointer to pointers in the array}
 begin
   pGroup := nil;                                  {initialize, just in case}
   try
     if (ii > 0) and (ii <= ngroup_r) then begin   {reasonable index}
       ppGroup := ppRelG_r;                        {pointer to first pointer}
       inc(ppGroup,ii-1);                          {pointer to pointer no ii}
       pGroup := ppGroup^;                         {get the pointer}
     end else begin
       pGroup := nil;
     end;
   finally
   end;
 end;

procedure TRelay.GroupActivation;
{check the activation of relay groups}
var
  actime : single;        {activation time}
  color  : TColor;        {color of the text}
  forwrd : boolean;       {true, when fault in forward direction (PQ-relays)}
  ii,jj  : integer;       {index}
  ir     : integer;       {index for relays}
  imin   : integer;       {index of min.time group}
  ix,iy  : integer;       {coordinates of the label (not needed, actually)}
  lbl    : string;        {label of the relay}
  ll     : longint;       {temporary}
  line   : string;        {text line}
  mintime: single;        {minimum activation time}
  nir    : integer;       {number of relays in one PQ or em or recl.relay}
  pComp  : TpComp;        {pointer to a relay, as a component}
  pCurrT : TpComp;        {pointer to the current trafo of the group}
  pGroup : TpRelGroup;    {pointer to a relay group}
  pRel   : TpComp;        {pointer to a relay}
  Rend   : integer;       {3 if PQ-relay, else 1}
  rmin   : integer;       {index of min.time relay (in PQ-relay)}
  txt    : string;        {P,G, or Q for PQ-relay}
  txx    : string;        {temporary}
begin
  mintime := -1;                      {initialize}
  imin := 0;
  rmin := 0;
  for ii:=1 to ngroup_r do begin      {scan all groups}
    getGpoint(ii,pGroup);             {get pointer to group ii}
    pGroup^.GetRpoint(1,pRel);        {get pointer to the first relay}
    pRel^.GetipComp(ll);              {get pointer to the trafo, as integer}
    if (ll <> 0) then pCurrT := TpComp(ll)  {pointer to the trafo of the group}
    else pCurrT := nil;
    if (pCurrT <> nil) then begin     {clear descriptions, used to save act.time}
      ClearCTactivation(ll);          {clear activation data, saved as descriptions}
{     pCurrT^.PutDescrP(7,'','','');  {see also RelayHide in PQrelay}
{     pCurrT^.PutDescrG(1,'','',''); }
{     pCurrT^.PutDescrQ(7,'','',''); }
      {add flag for PQ-relay, needed when writing activation times to report}
{!}   if      (pGroup^.isPQRelay) then pCurrT^.PutDescrP(4,'','','PQ')
      else if (pGroup^.isRCrelay) then pCurrT^.PutDescrP(4,'','','RC')
      else pCurrT^.PutDescrP(4,'','','');   {empty = not PQ-relay}
      {add labels of relays to descriptions}
      line := '';
      for jj := 1 to pGroup^.number_rg do begin
        pGroup^.GetRpoint(jj,pRel);
        pRel^.getLabel(lbl,ix,iy);     {label of the relay}
        txx := inttostr(longint(pRel));  {pointer to the relay, as integer}
        if (jj = 1) then begin
          {line := lbl;}
          line := txx;  {pointer to the relay, as integer}
        end else begin                 {PQ-relays have only one label}
          if not (pGroup^.isPQRelay or pGroup^.isRCrelay) then line := line+' '+txx{lbl};
        end;
      end;
      pCurrT^.PutDescrP(2,'',line,''); {save pointers (for labels) as descriptions}
    end;
    {scan all relay types}
    if (pGroup^.isPQRelay) then Rend := 3
{!} else if (pGroup^.isRCrelay) then Rend := 6       {3 or 6?}
    else Rend := 1;
    for ir := 1 to Rend do begin
      actime := pGroup^.ActivationTime(ir); {check the activation of this group..., this unit}
//if (ir = 1) then showmessage(floattostr(actime));
     {... and save it to the current trafo, as descriptions}
      {check minimum time, but consider only faults in forward direction (for PQ-relays)}
      if (pGroup^.isForwrd(ir)) then begin
        if (mintime < 0) then begin       {first positive activation time}
          mintime := actime;
          imin := ii;
          rmin := ir;
        end else begin                    {some pos. act.time already found}
          if (actime > 0) and (actime < mintime) then begin {smaller ?, but positive}
            mintime := actime;
            imin := ii;
            rmin := ir;
          end;
        end;
      end;
    end;
  end;
  {show min.time in color, if found}
  if (imin > 0) then begin
    getGpoint(imin,pGroup);                     {get pointer to the group with smallest act.time}
    pGroup^.GetRpoint(pGroup^.right_rg, pComp); {get pointer to the rightmost relay}
   {PQ-relay displays (max) three time values, recloser displays max. six values}
{!} if (pGroup^.isPQrelay) or (pGroup^.isRCrelay) then begin
      if (pGroup^.isPQrelay) then nir := 3 else nir := 6;
      for ir := 1 to nir do begin                 {must also re-show the other times}
        if (ir = 1) or ((ir = 2) and pGroup^.isRCrelay) then begin
          if (ir = rmin) then color := clRed else color := clFuchsia;
          txt := 'P';
        end else if ((ir = 2) and pGroup^.isPQrelay) or
                    (((ir = 3) or (ir = 4)) and pGroup^.isRCrelay) then begin
          if (ir = rmin) then color := clRed else color := clGreen;
          txt := 'G';
        end else begin
          if (ir = rmin) then color := clRed else color := clBlue;
          txt := 'Q';
        end;
        forwrd := pGroup^.isForwrd(ir);
        ShowActime(pComp,pGroup^.actime_rg[ir],color,ir,txt,forwrd); {in shortcir.pas}
      end;
    end else begin                              {normal relay}
      ShowActime(pComp,mintime,clRed,rmin,'',true);  {show the time in red, in shortcir.pas}
    end;
  end;
end;
(*
procedure TRelay.CheckActivation;
 {check is the relay activated = calculate the time delay}
 var
   actcur  : single;   {activation (threshold) current of the relay}
   actime  : single;   {activation time (delay) of the relay}
   apu     : single;   {temporary}
   curr    : single;   {current}
   ii      : integer;  {index}
   ll      : longint;  {temporary}
   mintime : single;   {minimum activation time delay) of all relays}
   pComp   : TpComp;   {pointer to the relay, as component}
   pCmin   : TpComp;   {pointer to the component with smalles activation time}
   pCTrafo : TpCOmp;   {pointer to the current trafo}
   text    : string;   {temporary string}
   time    : single;   {time, when current = given}
 begin
   mintime := 100;                             {initialize min. activation delay}
   pCmin := nil;
   if (number_r > 0) then begin                {are there relays}
     for ii:=1 to number_r do begin            {scan all relays}
       try
         time := -1;                           {initialize time, as flag}
         GetRpoint(ii,pComp);                  {get pointer to the relay}
         pComp^.GetPar(gRelay_I,actcur);       {get value of activation current}
         pComp^.GetPar(gRelay_t,actime);       {get value of activation time}
         pComp^.GetipComp(ll);                 {get pointer to the trafo, as integer}
         if (ll<> 0) then begin
           pCTrafo := TpComp(ll);              {pointer to the trafo}
           if (actcur > 0) and (actime > 0) then begin {parameters defined}
             GetCurr(pComp, actime, curr);       {find current at the given time}
             if (curr >= actcur) then begin      {relay activated ?}
               time := actime;
               if (actime < mintime) then begin  {shortest time, so far}
                 mintime := actime;              {update shortest time}
                 pCmin := pComp;                 {update pointer to comp.}
               end;
             end else begin                      {check also vertical branch, to be sure}
               GetTime(pCTrafo,actcur,actime,100.0,time);{time when current = actcur}
               if (time > 0) then begin          {negative time = not found}
                 if (time < mintime) then begin  {shortest time, so far ?}
                   mintime := time;
                   pCmin := pComp;
                 end;
               end;
             end;
           end;
         end;
        {show activation time, if activated}
         ShowActime(pComp,time,clBlue);
       finally
       end;
     end;
     {indicate relay with shortest activation time}
     ShowActime(pCmin,mintime,clRed);
   end;
 end;
*)
procedure TRelay.CheckSumCurr;
{calculate the sum of phase currents for those relays that are connected
 to a sum current trafo. The sum is calculated to the variables to be displayed
 NOTE: the phase variables must have been calculated.
 NOTE: The current trafo connected to the relay must be known
       (found using the object for the current trafos)}
var
  apu,apv : single;    {temporary}
  ii,jj   : integer;   {index}
  j1,j2   : integer;   {indices}
  ll      : longint;   {temporary}
  ModNam  : string;    {name of a component}
  mult    : integer;   {the multiplicity of output variables (should be 3)}
  pComp   : TpComp;    {pointer to the relay}
  pCTrafo : TpComp;    {pointer to the current trafo}
  si,sr   : single;    {sum of imag., real parts}
begin
  if (number_r > 0) then begin                {are there relays}
    for ii:=1 to number_r do begin            {scan all relays}
      try
        GetRpoint(ii,pComp);                  {get pointer to the relay}
        if (pComp <> nil) then begin          {maybe reasonable pointer}
          pComp^.GetipComp(ll);               {get pointer to the trafo}
          if (ll <> 0) then begin             {maybe reasonable pointer}
            pCTrafo := TpComp(ll);            {pointer to the trafo}
            pCTrafo^.GetModName(ModNam);      {get name of the trafo}
            if (ModNam = gSumCT) then begin   {sum trafo}
              pComp^.GetMult(mult);           {get the multiplicity of the set of variables}
              if (mult >= 3) then begin       {at least 3, for a 3 phase system}
                si := 0;
                sr := 0;
                for jj:=1 to 3 do begin       {all three phases}
                  j2 := 2*jj;
                  j1 := j2-1;
                  pComp^.GetVar(j1,apu);      {get real part}
                  pComp^.GetVar(j2,apv);      {get real part}
                  si := si+apv;
                  sr := sr+apu;
                end;
                {write the sums back}
                for jj:=1 to 3 do begin
                  j2 := 2*jj;                 {index of imag.part}
                  j1 := j2-1;
                  pComp^.PutVar(j1,sr,'CheckSumCurr');       {same sum to all phases}
                  pComp^.PutVar(j2,si,'CheckSumCurr');
                end;
              end;
            end;
          end;
        end;
      finally
      end;
    end;
  end;
end;

constructor TRelay.Constr(ipSet : longint; nRelay, nQrel : integer);
 {construct=initialize
  ipSet    pointer to the set of components etc, as integer
  nRelay   number of relays
  nSteps   number of calculations (steps)
  Called in CreateRelays of shortcir.pas}
 var
   errc    : integer;    {error flag}
   ir      : integer;    {index for relays}
   jcomp   : integer;    {counts components}
   ModNam  : string;     {name of a component}
   pComp   : TpComp;     {pointer to components}
 begin
   pSet_r := TpCompSet(ipSet);
   number_r := nRelay+nQrel;             {all relays}
   nQrel_r := nQrel;                     {PQ relays}
   group_r := 0;                         {group not yet defined}
   {get room for calculated currents, for all relays, all steps}
   if (number_r > 0) then begin
     getmem(ppComp_r,number_r*SizeOf(TpComp));     {reserve memory for pointers}
     for ir := 1 to number_r do PutRpoint(ir,nil); {initialize}
     {fill the arrays of pointers and nodes}
     errc := 0;
     pComp := nil;
     jcomp := 0;
     ir := 0;
     while (errc = 0) do begin                  {scan all components}
       pSet_r^.NextComp(pComp,errc,jcomp,true); {get pointer to next component}
       if (errc = 0) then begin                 {component found}
         pComp^.GetModName(ModNam);             {get the name (type) of the component}
         if ((ModNam = gRelay) or IsInvTimeRelay(ModNam) or
             (ModNam = gPQRel) or (ModNam = gEMRel) or
             (ModNam = gRCrel) or (ModNam = gUDrel)) then begin  {is it some relay type}
           inc(ir);                       {increment index}
           PutRpoint(ir,pComp);           {save the pointer to the array of pointers}
         end;
       end;
     end;
   end else begin                         {no relays}
     ppComp_r := nil;
   end;
 end;

procedure TRelay.Destr;
 {releases memory reserved for relays and relay groups}
var
  ii     : integer;
  pGroup : TpRelGroup;   {pointer to relay group}
begin
  if (number_r > 0) then begin
    try
      freemem(ppComp_r,number_r*SizeOf(TpComp));
    finally
    end;
  end;
  {free memory for array of pointers to groups}
  if (ngroup_r > 0) then begin
    try
      for ii := 1 to ngroup_r do begin
        getGpoint(ii,pGroup);                {get pointer to group}
        pGroup^.Destr;                       {release memory from group}
        pGroup^.free;                        {release memory for the group}
        dispose(pGroup);
        pGroup := nil;
      end;
      freemem(ppRelG_r,ngroup_r*SizeOf(TpRelGroup));
      ppRelG_r := nil;
    finally
    end;
  end;
end;

procedure TRelay.FindGroups(pNodar,pNumbg : pInteger);
{find the number of relay groups and the number of relays in the groups
 pNodar    pointer to an array for working space
           dimension of the array = number of relays (must be !)
           at exit contains the index of the group for each relay
 pNumbg   pointer to an array, that at exit contains the number of
          relays in the groups
 }
var
  err    : integer;   {error flag}
  ii,jj  : integer;   {index}
  name   : string;    {name (type) of the relay}
  namj   : string;    {name (type) of a relay}
  ni,nj  : integer;   {node of relay i, and j}
  nrel   : integer;   {number of relays in a group}
  pComp  : TpComp;    {pointer to a relay}
  pCompj : TpComp;    {pointer to another relay}
begin
  try
   {first, get the nodes of all relays}
    ngroup_r := 0;                             {initialize number of groups}
    for ii:=1 to number_r do begin             {scan all relays}
      GetRpoint(ii,pComp);                     {recall pointer to relay}
      pComp^.GetName(name);
      if (name = gPQRel) or (name = gEMRel) or
         (name = gRCrel) or (name = gUDrel) then begin
if (pComp^.IsInuse) then begin           {include only relays that are in use}
          {one PQ-relay , electromech.relay or recloser relay = one group}
          inc(ngroup_r);                           {new group found}
          putIvec(pNodar,ii,number_r,ngroup_r);    {indicate the group}
          putIvec(pNumbg,ngroup_r,number_r,1);     {one PQ-relay in one group }
end;
      end else begin
        pComp^.GetNode(1,ni,err);                {get the (first) node}
        if (err = 0) and (ni < -1) then begin    {a proper node found}
          putIvec(pNodar,ii,number_r,ni);        {put the node to the array}
        end else begin
          putIvec(pNodar,ii,number_r,0);         {0 indicates "no node"}
        end;
      end;
    end;
    {next, scan the nodes, and find the groups}
    {pNodar contains initially the nodes (< -1). When group found,
     the node is replaced by the group number (> 0)}
    for ii:=1 to number_r do begin
      GetRpoint(ii,pComp);                     {recall pointer to relay}
      pComp^.GetName(name);
      if (name <> gPQRel) and (name <> gEMRel) and
         (name <> gRCrel) and (name <> gUDrel) then begin {PQ-relays already checked}
        getIvec(pNodar,ii,number_r,ni);          {get the node of this relay}
        if (ni < -1) then begin                   {only real nodes considered}
          nrel := 1;                             {at least one relay in this croup}
          inc(ngroup_r);                         {new group found}
          putIvec(pNodar,ii,number_r,{-}ngroup_r); {indicate the group}
          if (ii < number_r) then begin          {not the last relay}
            for jj:=ii+1 to number_r do begin    {scan the remaining relays}
              GetRpoint(jj,pCompj);              {recall pointer to relay}
              pComp^.GetName(namj);
              if (namj <> gPQRel) and (namj <> gEMRel) and
                 (namj <> gRCrel) and (namj <> gUDrel) then begin {PQ-relays do not connect with other relays}
                getIvec(pNodar,jj,number_r,nj);    {node of relay j}
                if (ni=nj) then begin              {same node ?}
                  putIvec(pNodar,jj,number_r,{-}ngroup_r); {indicate: "this relay handled" and the group}
                  inc(nrel);                             {one more relay in this group}
                end;
              end;
            end;
          end;
          putIvec(pNumbg,ngroup_r,number_r,nrel);      {save number of relays}
        end;
      end;
    end;
    {change the sign of the indices back to positive}
{    negIvec(pNodar,number_r);}
  finally
  end;
end;

procedure TRelay.CreateGroups;
{Find the number of relay groups. Group = relays that are connected
 to the same current trafo, i.e. to the same node
 Create the group objects, and initialize them}
var
  err    : integer;    {error flag}
  ii,jj  : integer;    {index}
ix,iy:integer;
lbl:string;
  kk     : integer;    {temporary}
  name   : string;     {name = type of the component}
  ni     : integer;    {node}
  pComp  : TpComp;     {pointer to a relay as a component}
  pNodar : pInteger;   {pointer to an array for the group indices for all relays}
  pNumbg : pInteger;   {pointer to an array for the number of relays in the groups}
  pRelG  : TpRelGroup; {pointer to a relay group}
{  PQrel  : boolean;    {true, when PQ-relay}
  rimo   : integer;    {the index of the rightmost relay}
  Qrel   : boolean;    {true, when PGQ, electromech, or recloser relay}
begin
  pNodar := nil;
  pNumbg := nil;
  try
    getmem(pNodar,number_r*SizeOf(integer));   {get memory for the node array}
    getmem(pNumbg,number_r*SizeOf(integer));   {get memory for the number of relays in the groups}
    {find the groups and their number}
    FindGroups(pNodar,pNumbg);
    {create objects for the groups, if there are any}
    if (ngroup_r > 0) then begin
      {get memory for array of pointers to groups}
      getmem(ppRelG_r,ngroup_r*SizeOf(TpRelGroup));
      {create the groups}
      for ii:=1 to ngroup_r do begin           {each group}
        new(pRelG);                            {pointer to the group}
        pRelG^ := TRelGroup.Create;            {create the group object}
        Qrel := (ii <= nQrel_r);               {PQ-, EM-, RC-relays first}
        if (Qrel) then kk := {3}5              {five (?) modules in a PQ-relay}
        else getIvec(pNumbg,ii,number_r,kk);   {get number of relays in this group}
        pRelG^.Constr(pSet_r,kk,false,false);  {initialize this group, continue below}
        PutGpoint(ii, pRelG);                  {save pointer to relay group}
      end;
      {put pointers to groups}
      for ii:=1 to number_r do begin           {scan again all relays}
        getIvec(pNodar,ii,number_r,kk);        {get index of group this relay belongs to}
        if (kk > 0) then begin                 {reasonable group ?}
          group_r := kk;                       {save group index also to the relay}
          getRpoint(ii,pComp);                 {get pointer to the relay}
          GetGpoint(kk,pRelG);                 {get pointer to the group}
          {add this relay to the list, (inverse time relays to the end )}
          pComp^.GetName(name);
          {set the relay type}
          if (name = gPQrel) or (name = gEMrel) then pRelG^.PQrelay_rg := true
          else if ((name = gRCrel) or (name = gUDrel)) then pRelG^.RCrelay_rg := true;
          {relays may contain several modules}
          if (name = gPQrel) or (name = gEMRel) then begin
{?}         for jj := 1 to {3}{4}5 do pRelG^.AddRelay(pComp);  {five modules in one PQ-relay}
          end else if (name = gRCrel) or (name = gUDrel) then begin
{?}         for jj := 1 to {3}5{4} do pRelG^.AddRelay(pComp);  {five modules in one recloser-relay}
          end else begin
            pRelG^.AddRelay(pComp);
          end;
        end;
      end;
      {find the rightmost relay in each group, and sort after activation current}
      {and save the node to which the relays are connected to}
      for ii:=1 to ngroup_r do begin           {each group}
        GetGpoint(ii, pRelG);                  {get pointer to relay group}
        if      (pRelG^.PQrelay_rg) then pRelG^.ActCurrents(true) {"sort" = set flag when PQRelay}
        else if (pRelG^.RCrelay_rg) then pRelG^.ActCurrents(false)
        else pRelG^.SortActCurrents;           {sort the relays}
        pRelG^.FindRightmost(rimo);            {find the rightmost relay}
        pRelG^.GetRpoint(1,pComp);             {get pointer to first relay}
        pComp^.GetNode(1,ni,err);              {get the (first) node}
        {NOTE, the first node connected to the current trafo only for the "simple" relays,
         not PQ-, EM-, RC- UD-relays. Their first node is for voltage input !!!}
        pRelG^.node_rg := ni;                  {save the node}
      end;
      {find the current trafos for the groups}
    end else begin
      ppRelG_r := nil;                         {no relay groups, pointer = nil}
    end;
  finally
  freemem(pNodar,number_r*SizeOf(integer));    {free memory for the node array}
  freemem(pNumbg,number_r*SizeOf(integer));    {free memory for the number of rel}
  end;
end;

(* see below *)
procedure TRelay.FindCurrTrafos(pCurrT: TpCurrT);
{find the current trafos for the relay groups (and the relays in the groups)
NOTE: works only with "simple" relays, not PGQ-, RC-, EM-, UD-relays}
var
  ii     : integer;     {index}
  pRelGr : TpRelGroup;  {pointer to a relay group}
begin
  if (ngroup_r > 0) then begin      {there are relay groups}
    for ii:=1 to ngroup_r do begin
      GetGpoint(ii,pRelGr);         {get pointer to the relay group ii}
      if (pRelGr <> nil) then pRelGr^.FindCurrTrafos(pCurrT);
    end;
  end;
end;
(**)
(*
{toisella tavalla, use earlier results  14.2.2004}
procedure TRelay.FindCurrTrafos(pCurrT: TpCurrT);
{find the current trafos for the relay groups (and the relays in the groups)}
var
  ii     : integer;     {index}
  ll     : longint;     {temporary}
  pRelay : TpComp;      {pointer to a relay}
  pRelGr : TpRelGroup;  {pointer to a relay group}
begin
  if (ngroup_r > 0) then begin       {there are relay groups}
    for ii:=1 to ngroup_r do begin
      GetGpoint(ii,pRelGr);          {get pointer to the relay group ii}
      GetRpoint(1,pRelay);           {pointer to first relay (=any relay)}
      if (pRelay <> nil) then begin
        pRelay^.GetipComp(ll);       {pointer to current trafo, found in nodes.pas}
        pRelGr^.pCurrT_rg := TpComp(ll);
      end else begin
        pRelGr^.pCurrT_rg := nil;
      end;
    end;
  end;
end;
*)
function  TRelay.GetNumber : integer;
{get number of relays}
begin
  GetNumber := number_r;
end;


{ relay group - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - }
procedure TRelGroup.FindCurrTrafos(pCurrT : TpCurrT);
{ONKO tarpeen?, ainakin PQ-releet tuntuvat toimivan ilman! 5.3.2004}
{find the current trafos connected to the relays of this group.
 NOTE, the object for the current trafos must exist !
 NOTE: works only with "simple" relays, not PGQ-, RC-, EM-, UD-relays !!!!!!
 pCurrT   pointer to the object for current trafos}
var
  errn     : integer;   {error flag}
  found    : boolean;   {found trafo connected to this group}
  ii,jj    : integer;   {indices}
  ipCTrafo : longint;   {pointer to the trafo, as integer}
  kk       : integer;   {temporary}
  nct      : integer;   {number of current trafos}
  pCTrafo  : TpComp;    {pointer to a current trafo}
  pRelay   : TpComp;    {pointer to a relay}
begin
  if (pCurrT <> nil) then begin
    nct := pCurrT^.GetNumber;         {number of current trafos}
    found := false;                   {initialzie, to be sure?}
    for ii := 1 to nct do begin       {scan all current trafos}
      pCurrT^.GetCTpoint(ii,pCTrafo); {get pointer to this current trafo}
      pCTrafo^.GetNode(3,kk,errn);    {get current node}
      if (kk = node_rg) then begin    {connected to same node}
        found := true;
        break;
      end else begin
        found := false;
      end;
    end;
    {if trafo found, save pointer to group, and to all relays}
    if found then begin
      pCurrT_rg := pCTrafo;           {save pointer to the trafo into group}
      {save the pointer to the relays, as integer, to be sure ??}
      ipCTrafo := longint(pCTrafo);
    end else begin
      pCurrT_rg := nil;
      ipCTrafo := 0;
    end;
  end else begin                      {no current trafos, or no object for them}
    ipCTrafo := 0;
    pCurrT_rg := nil;
  end;
  {Now, save the pointer (as integer), or zero, if no trafo found}
  for jj:=1 to number_rg do begin    {all relays in this group}
    GetRpoint(jj,pRelay);            {get pointer to relay number jj}
    if (pRelay <> nil) then pRelay^.PutipComp(ipCtrafo); {save pointer as integer}
  end;
end;

function TRelGroup.isPQrelay : boolean;
{true, if relay group = PQ-relay}
begin
  isPQRelay := PQRelay_rg;
end;

function TRelGroup.isRCrelay : boolean;
{true, if relay group = recloser relay}
begin
  isRCRelay := RCrelay_rg;
end;

function TRelGroup.isForwrd(ir : integer): boolean;
{true, when fault is in forward direction (PQ-relays)}
begin
  if (ir > 0) and (ir <= 6) then isForwrd := forwrd_rg[ir]
  else isForwrd := true;
end;


procedure TRelGroup.GetActTime(ir : integer; var actime : single);
{get the activation time of the group
 ir  index 1..3 of relays, for PQ relays}
begin
  if (ir < 1) then actime := actime_rg[1]
  else if (ir <= 3) then actime := actime_rg[ir]
  else actime := actime_rg[1];
end;

procedure TRelGroup.AddRelay(pComp :TpComp);
{add a relay pointer to the array of pointers
 The definite time relays to the beginning of the list, and
 the inverse  time relays to the end of the list
 pComp   pointer to the relay
 }
var
  ModNam : string;   {name = type of the relay}
 begin
   try
     if (pComp <> nil) then begin                  {does it help anything ??}
       pComp^.GetModName(ModNam);                  {get model name}
     end else begin
       ModNam := ' ';
     end;
     {PGQ-relay, electromech.relay, or recloser relay: three modules in one}
     if (ModNam = gPQRel) or (ModNam = gEMRel) or
        (ModNam = gRCrel) or (ModNam = gUDrel) then begin
       if (pInv_rg = nil) then begin                {inverse time module of PGQ-relay}
         pInv_rg := pComp;
         PutBackwRPoint(pComp);      {inverse time relays to the end of the list}
       end else begin
         PutNextRPoint(pComp);       {save the pointer to the relay, into the relay group}
         inc(ndeft_rg);              {one more definite time relay}
       end;
     end else if IsInvTimeRelay(ModNam) then begin  {is it an inverse-time relay}
       if (pInv_rg = nil) then pInv_rg := pComp     {first inverse time relay}
       else showmessage('too many inverse-time relays');   {only one inverse-time relay allowed}
       PutBackwRPoint(pComp);      {inverse time relays to the end of the list}
     end else begin                {definite-time relay, to the beginning of the list}
       PutNextRPoint(pComp);       {save the pointer to the relay, into the relay group}
       inc(ndeft_rg);              {one more definite time relay}
     end;
   finally
   end;
end;

procedure TRelGroup.PutNextRpoint(pComp : TpComp);
{saves the pointer to next relay  into the array for pointers of this group
 pComp  pointer to the relay
 NOTE   next_rg must be equal to one, when entering this procedure
        for the first time !!!
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   try
     if (next_rg > 0) and (next_rg <= number_rg) then begin  {reasonable index}
       ppComp := ppComp_rg;                        {pointer to first pointer}
       inc(ppComp,next_rg-1);                      {pointer to pointer no next_rg}
       ppComp^ := pComp;                           {save the pointer}
       inc(next_rg);                               {increase index of next relay}
     end;
   finally
   end;
 end;

 procedure TRelGroup.PutBackwRpoint(pComp : TpComp);
{saves the pointer to next relay  into the array for pointers of this group,
 filling backwards from the end of the array
 pComp  pointer to the relay
 NOTE   backw_rg must be equal to number_rg, when entering this procedure
        for the first time !!!
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   try
     if (backw_rg > 0) and (backw_rg <= number_rg) then begin  {reasonable index}
       ppComp := ppComp_rg;                        {pointer to first pointer}
       inc(ppComp,backw_rg-1);                     {pointer to pointer no backw_rg}
       ppComp^ := pComp;                           {save the pointer}
       dec(backw_rg);                              {decrement index of next relay}
     end;
   finally
   end;
 end;

procedure TRelGroup.GetRpoint(ii : integer; var pComp : TpComp);
{recalls the pointer to relay number ii in the array for pointers of this group
 ii     index of the relay
 pComp  pointer to the relay
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   pComp := nil;                                   {initialize, just in case}
   try
     if (ii > 0) and (ii <= number_rg) then begin  {reasonable index}
       ppComp := ppComp_rg;                        {pointer to first pointer}
       inc(ppComp,ii-1);                           {pointer to pointer no ii}
       pComp := ppComp^;                           {get the pointer}
     end else begin
       pComp := nil;
     end;
   finally
   end;
 end;

procedure TRelGroup.PutRpoint(ii : integer; pComp : TpComp);
{saves the pointer to a relay to position number ii in the array for pointers
 ii     index of the relay
 pComp  pointer to the relay
 }
 var
   ppComp : TppComp;    {pointer to pointers in the array}
 begin
   try
     if (ii > 0) and (ii <= number_rg) then begin   {reasonable index}
       ppComp := ppComp_rg;                         {pointer to first pointer}
       inc(ppComp,ii-1);                           {pointer to pointer no ii}
       ppComp^ := pComp;                            {save the pointer}
     end;
   finally
   end;
 end;

procedure TRelGroup.GetInvTpoint(var pComp: TpComp);
{get the pointer to the inverse-time relay}
begin
  pComp := nil;  {initialize, to be sure}
  try
    pComp := pInv_rg;
  finally
  end;
end;


procedure TRelGroup.FindRightmost(var rimo : integer);
{find the index of the rightmost relay in this group,
 i.e. the relay with the largest .left
 rmio  the index (in this group) of the rightmost relay}
var
  ii,jj  : integer;     {index, counter}
  kk     : integer;     {temporary}
  pComp  : TpComp;      {pointer to a relay}
  left   : integer;     {x-coordinate}
begin
  try
    if (number_rg > 0) then begin       {are there relays in this group}
      if (PQrelay_rg) or (RCrelay_rg) then begin
        right_rg := 1;                  {only one relay in PQ-, EM-, and RC-relay}
      end else begin
        jj := 1;                          {initialize index}
        getRpoint(1,pComp);               {pointer to the first relay}
        left := pComp^.left;              {initialize for the search of the rightmost relay}
        if (number_rg > 1) then begin     {any more relays ?}
          for ii:=2 to number_rg do begin {scan all relays in this group}
            getRpoint(ii,pComp);          {get pointer to the relay}
            kk := pComp^.left;            {x-coordinate}
            if (kk > left) then begin     {more right ?}
              left := kk;                 {new rightmost x-coordinate}
              jj := ii;                   {index of the (so far) rightmost relay}
            end;
          end;
        end;
        right_rg := jj;                   {save the index}
      end;
    end;
  finally
  end;
  rimo := right_rg;
end;

function TRelGroup.pTimeShown: TpComp;
{true, if the box of activation time is shown for one relay, at least, in the group}
var
  ii     : integer;  {index}
  pComp  : TpComp;   {pointer to one relay}
  ppComp : TppComp;  {pointer to relay pointers}
  pShown : TpComp;   {pointer to the relay for which time shown}
  shown  : boolean;  {true, when time box shown}
begin
  {only one relay in the group of combined relays}
  pShown := nil;
  if (isPQrelay) or (isRCrelay) then begin
    pComp := ppComp_rg^;            {pointer to the first relay}
    shown := pComp^.ResultShown2;
  end else begin                   {one or more simple relays in the group}
    shown := false;                {initialize}
    ppComp := ppComp_rg;
    for ii := 1 to number_rg do begin
      pComp := ppComp^;
      if (pComp^.ResultShown2) then begin  {results shown for this relay}
        shown := true;
        pShown := pComp;
        break;
      end else begin
        inc(ppComp);
      end;
    end;
  end;
  pTimeShown := pShown;
end;


procedure TRelGroup.FindAndShowActTime;
{find the activation time of this group (if any) and display it}
var
  apu    : single;   {temporary}
  color  : Tcolor;   {color of act.time as text}
  forwrd : boolean;  {true, when fault in forward direction (for PQ-relays)}
  pComp  : TpComp;   {pointer to the rightmost relay}
  pShown : TpComp;   {pointer to the relay whose act.time shown}
  Rend   : integer;  {number of modules in one relay: 6 for RC, UD, 3 for PQ,EM, else 1}
  rimo   : integer;  {the index (in this group) of the rightmost relay }
  rr     : integer;  {index of the modules}
  txt    : string;   {text for activation time}
begin
  if (isRCRelay) then Rend := 6        {RC or UD relay}
  else if (isPQRelay) then Rend := 3   {PQ or EM relay}
  else Rend := 1;                      {simple relay}
  for rr := 1 to Rend do begin
    if (isPQRelay) then begin               {PGQ-relay}
      if (rr = 1) then begin
        color := clFuchsia;
        txt := 'P';
      end else if (rr = 2) then begin
        color := clGreen;
        txt := 'G';
      end else begin
        color := clBlue;
        txt := 'Q';
      end;
    end else if (isRCRelay) then begin      {recloser relay}
      if (rr = 1) or (rr = 2) then begin
        color := clFuchsia;
        txt := 'P';
      end else if (rr = 3) or (rr = 4) then begin
        color := clGreen;
        txt := 'G';
      end else begin
        color := clBlue;
        txt := 'Q';
      end;
    end else begin                          {normal relay}
      color := clBlue;
      txt := '';
    end;
    apu := ActivationTime(rr);       {find the act.time of this group, in Shortcmp.pas}
    FindRightmost(rimo);             {find the rightmost relay}
    GetRpoint(rimo, pComp);          {get pointer to the rightmost relay}
    forwrd := isForwrd(rr);
    ShowActime(pComp, apu, color,rr,txt,forwrd);     {show time right to the rightmost r.}
    pShown := pTimeShown;
    if (pShown <> nil) then pShown^.ShowMoreResults; {update boxes for activation times}
  end;                                               {ShowActime is in Shortcir.pas}
end;

procedure TRelGroup.GetMinActCurr(var mincur : single);
{get the smallest nonzero activation current.
 It was checked, when sorting the activation currents in SortActCurrents}
begin
  mincur := mincur_rg;
end;
(* uusi...*)
procedure TRelGroup.PQMinMaxCur(cur1, cur2, cur3, cur4: single; ii: integer; var Imin: single);
{find the minimum, max and middle currents for the def.time modules of a PQ-relay
 cur1, cur2, cur3, cur4  the currents
 ii    index of the relay unit (P, G, Q - unit)
 Imin  the minimum current}
{PQminc_rg    index of module with the smallest current
 PQlowc_rg    .. next smallest current
 PQhigc_rg    .. next largest current
 PQmaxc_rg    .. largest current
If some of the currents cur1..cur4 are not defined (<= 0), then
 PQlowc_rg ... get the same index as the smallest current}
const
  nn = 4;    {number of currents}
var
  jj,kk   : integer;
  ipu,ipv : integer;    {temporary}
  nonz    : integer;      {index of one nozero current}
  nz      : integer;      {number of zero (or less) currents}
  cur     : array[1..nn] of single;   {local for the currents}
  order   : array[1..nn] of integer;  {the position when arranged}
  zero    : array[1..nn] of boolean;  {flag for currents that are zero or less}
begin
  cur[1] := cur1;   {copy to the local array}
  cur[2] := cur2;
  cur[3] := cur3;
  cur[4] := cur4;
  {check zero currents}
  nz := 0;
  for kk := 1 to nn do begin
    if (cur[kk] <= 0) then  begin
      inc(nz);
      zero[kk] := true;
    end else begin
      zero[kk] := false;
    end;
  end;
  if (nz = nn) then begin   {all zeroes or less}
    Imin := 0;
    PQminc_rg[ii] := 0;     {nonreasonable value, as a flag (?)}
    PQmaxc_rg[ii] := 0;
    PQlowc_rg[ii] := 0;
    PQhigc_rg[ii] := 0;
  end else if (nz = nn-1) then begin {all except for one zero or less}
    {find the index of the one}
    nonz := 0;
    for kk := 1 to nn do begin
      if (not zero[kk]) then begin
        nonz := kk;
        break;
      end;
    end;
    Imin := cur[nonz];
    PQminc_rg[ii] := nonz;                      {same to all..}
    PQmaxc_rg[ii] := nonz;
    PQlowc_rg[ii] := nonz;
    PQhigc_rg[ii] := nonz;
  end else begin                                {more than one nonzero}
    {find how many currents larger than this current}
    for kk := 1 to nn do order[kk] := -1; {initialize}
    for kk := 1 to nn-1 do begin
      if (cur[kk] > 0) then  begin              {zero or negative values not counted}
        if (order[kk] < 0) then order[kk] := 0; {reset flag}
        for jj := kk+1 to nn do begin
          if (cur[jj] > 0) then begin
            if (order[jj] < 0) then order[jj] := 0; {reset flag}
            if (cur[jj] > cur[kk]) then inc(order[kk]) else inc(order[jj]);
          end;
        end;
      end;
    end;
    {order[..] contains the order-index of the currents: 0 largest, < 0 not reasonable}
    if (nz = 2) then begin                      {only largest and smallest}
      for kk := 1 to nn do begin
        if (order[kk] = 0) then begin           {largest}
          PQmaxc_rg[ii] := kk;
        end else if (order[kk] = 1) then begin  {smallest}
          Imin := cur[kk];
          PQminc_rg[ii] := kk;                  {same to all the others}
          PQlowc_rg[ii] := kk;
          PQhigc_rg[ii] := kk;
        end;
      end;
    end else if (nz = 1) then begin             {three nonzero}
      for kk := 1 to nn do begin
        if (order[kk] = 0) then begin           {largest}
          PQmaxc_rg[ii] := kk;
        end else if (order[kk] = 1) then begin  {the middle one}
          PQhigc_rg[ii] := kk;
        end else if (order[kk] = 2) then begin  {the smallest}
          Imin := cur[kk];
          PQminc_rg[ii] := kk;
          PQlowc_rg[ii] := kk;                  {not defined, set same as min}
        end;
      end;
    end else begin                              {all four are nonzero}
      for kk := 1 to nn do begin
        if (order[kk] = 0) then begin           {largest}
          PQmaxc_rg[ii] := kk;
        end else if (order[kk] = 1) then begin  {the middle one}
          PQhigc_rg[ii] := kk;
        end else if (order[kk] = 2) then begin  {the third largest = second smallest}
          PQlowc_rg[ii] := kk;
        end else begin                          {the smallest}
          Imin := cur[kk];
          PQminc_rg[ii] := kk;
        end;
      end;
    end;
  end;
end;
(* *)
(* vanha..
procedure TRelGroup.PQMinMaxCur(cur1, cur2, cur3: single; ii: integer; var Imin: single);
{find the minimum, max and middle current for the def.time modules of a PQ-relay
 cur1, cur2 cur3  the currents
 ii    index of the relay unit (P, G, Q - unit)
 Imin  the minimum current}
begin
  if (cur1 <= 0) and (cur2 <= 0) and (cur3 <= 0) then begin
    Imin := 0;
    PQminc_rg[ii] := 0;     {nonreasonable value, as a flag (?)}
    PQmaxc_rg[ii] := 0;
    PQlowc_rg[ii] := 0;
  end else if (cur1 <= 0) and (cur2 <= 0) then begin
    Imin := cur3;
    PQminc_rg[ii] := 3;
    PQmaxc_rg[ii] := 3;
    PQlowc_rg[ii] := 3;
  end else if (cur1 <= 0) and (cur3 <= 0) then begin
    Imin := cur2;
    PQminc_rg[ii] := 2;
    PQmaxc_rg[ii] := 2;
    PQlowc_rg[ii] := 2;
  end else if (cur2 <= 0) and (cur3 <= 0) then begin
    Imin := cur1;
    PQminc_rg[ii] := 1;
    PQmaxc_rg[ii] := 1;
    PQlowc_rg[ii] := 1;
  end else if (cur3 <= 0) then begin
    if (cur1 >= cur2) then begin
      PQminc_rg[ii] := 2;
      PQmaxc_rg[ii] := 1;
      PQlowc_rg[ii] := 2;     {middle chosen to be the same as min}
      Imin := cur2;
    end else begin
      PQminc_rg[ii] := 1;
      PQmaxc_rg[ii] := 2;
      PQlowc_rg[ii] := 1;
      Imin := cur1;
    end;
  end else if (cur2 <= 0) then begin
    if (cur1 >= cur3) then begin
      PQminc_rg[ii] := 3;
      PQmaxc_rg[ii] := 1;
      PQlowc_rg[ii] := 3;
      Imin := cur3;
    end else begin
      PQminc_rg[ii] := 1;
      PQmaxc_rg[ii] := 3;
      PQlowc_rg[ii] := 1;
      Imin := cur1;
    end;
  end else if (cur1 <= 0) then begin
    if (cur2 >= cur3) then begin
      PQminc_rg[ii] := 3;
      PQmaxc_rg[ii] := 2;
      PQlowc_rg[ii] := 3;
      Imin := cur3;
    end else begin
      PQminc_rg[ii] := 2;
      PQmaxc_rg[ii] := 3;
      PQlowc_rg[ii] := 2;
      Imin := cur2;
    end;
  end else begin                   {all larger than zero}
    if (cur1 >= cur2) then begin   {cur2 < cur1}
      if (cur2 >= cur3) then begin
        PQminc_rg[ii] := 3;
        PQlowc_rg[ii] := 2;
        Imin := cur3;
      end else begin               {cur1 > cur 2, and cur2 < cur 3}
        PQminc_rg[ii] := 2;
        Imin := cur2;
        if (cur1 >= cur3) then PQlowc_rg[ii] := 3 else PQlowc_rg[ii] := 1;
      end;
      if (cur1 >= cur3) then begin
        PQmaxc_rg[ii] := 1;
      end else begin
        PQmaxc_rg[ii] := 3;
      end;
    end else begin                 {cur2 > cur1}
      if (cur1 >= cur3) then begin
        PQminc_rg[ii] := 3;
        Imin := cur3;
        PQlowc_rg[ii] := 1;
      end else begin               {cur2 > cur1, and cur3 > cur1}
        PQminc_rg[ii] := 1;
        Imin := cur1;
        if (cur2 >= cur3) then PQlowc_rg[ii] := 3 else PQlowc_rg[ii] := 2;
      end;
      if (cur2 >= cur3) then begin
        PQmaxc_rg[ii] := 2;
      end else begin
        PQmaxc_rg[ii] := 3;
      end;
    end;
  end;
end;
*)
procedure TRelGroup.ActCurrents(isPQ : boolean);
{"sort" the definite time modules in a PGQ-relay.
 and find the minimum time: smallest of the def.time of the def.time modules,
    and the starting current of the inverset time module
 Check only modules that are displayed!!
 The inverse time relay is the last one
 isPQ  true, when a PGQ or electromech.relay, false for a recloser}
var
  apu,apv   : single;   {temporary}
  cur1,cur2 : single;   {activation currents of definite time modules}
  cur3,cur4 : single;
  ic        : integer;  {index of curve}
  Ifirst    : single;   {first proper current (not normalized)}
  ii        : integer;  {index}
  Imin      : single;   {min.currents for the P, G, Q modules}
  ipu       : integer;  {temporary}
  Iset      : single;   {set current of inverse time module}
  Isml      : single;   {smaller current}
  mincur    : single;   {minimum activation current}
  name      : string;   {name = type of the relay}
  nRel      : integer;  {number of relay functions in one relay}
  pComp     : TpComp;   {pointer to the relay}
  show      : single;   {curve shown, if show > 0}
begin
{MUST still check, is relay in use, joku parametri}
  mincur := 0;                        {initialize}
  if (isPQ) then nRel := 3 else nRel := 6; {relay functions}
  {all three relays (P,G,Q) for PGQ and electromech.rel.,  twice for a recloser}
  GetRpoint(1,pComp);                 {pointer to the relay}
  if (pComp <> nil) then begin
    for ii := 1 to nRel do begin
      if (isPQ) then begin                 {PGQ or electromech.relay}
        pComp^.getPar(gPQ_Iset[ii],Iset);  {set current}
        Ifirst := Iset;                    {first current (first normalized current = 1)}
        pComp^.getPar(gPQ_I1[ii],cur1);    {act.current of def.time module 1}
        pComp^.getPar(gPQ_I2[ii],cur2);    {act.current of def.time module 2}
        pComp^.getPar(gPQ_I3[ii],cur3);    {act.current of def.time module 3}
        pComp^.getPar(gPQ_I4[ii],cur4);    {act.current of def.time module 4}
        pComp^.GetPar(gPQ_show[ii],show);  {flag for displaying curve, or not}
      end else begin                       {recloser}
        pComp^.getPar(gRC_Iset[ii],Iset);  {set current}
        pComp^.getPar(gRC_I1[ii],cur1);    {act.current of def.time module 1}
        pComp^.getPar(gRC_I2[ii],cur2);    {act.current of def.time module 2}
        pComp^.getPar(gRC_I3[ii],cur3);    {act.current of def.time module 3}
        pComp^.getPar(gRC_I4[ii],cur4);    {act.current of def.time module 4}        
        pComp^.GetPar(gRC_show[ii],show);  {flag for displaying curve, or not}
        {for user defined relay must read also frist current in the table}
        pComp^.GetName(name);              {name = type of the relay}
        if (name = gUDRel) then begin      {user defined relay}
          GetUDFirst(pComp, ii, ipu, apu, apv); {get first nozero normalized current}
          Ifirst := Iset*apu;
        end else begin                     {"normal curve" from tables}
          Ifirst := Iset;                  {first normalized = 1}
        end;
      end;
      PQMinMaxCur(cur1, cur2, cur3, cur4, ii, Imin);
      {the inverse time module}
      if (Imin > 0) and (Ifirst > 0) then begin
        if (Imin < Ifirst) then Isml := Imin
        else                  Isml := Ifirst;
      end else if (Imin > 0) then begin
        Isml := Imin;
      end else if (Ifirst{Iset} > 0) then begin
        Isml := Ifirst{Iset};
      end else begin
        Isml := 0;
      end;
      {consider only modules for which the char.is shown}
      if (show <> 0) then begin
        if (mincur > 0) and (Isml > 0) then begin
          if (Isml < mincur) then mincur := Isml;
        end else if (Isml > 0)then begin
          mincur := Isml;
        end;
      end;
    end;
    {save minimum current only if char. shown}
    mincur_rg := mincur;
  end else begin                    {no relay ??}
    mincur_rg := 0;
    for ii := 1 to nRel do PQminc_rg[ii] := 0;
  end;
  {save pointer to def.time modules = same relay!}
  for ii:=1 to ndeft_rg do begin      {copy all pointers for def.time relays}
    putRpoint(ii,pComp);              {same, because several modules in the relay}
  end;
end;

procedure TRelGroup.SortActCurrents;
{sort the definite time relays in this group
 according to the activation current, largest first
 One eventual inverse-time relays will always be the last one
 (if there are more than one inverse-time relay, they are neglected}
const
  hugecurr = 1.e9;
var
  ActCurr : single;   {activation current}
  apu     : single;   {temporary}
  ii,jj   : integer;  {index}
  mincur  : single;   {smallest nonzero activation current}
  pCompi  : TpComp;   {pointer to a relay as a component}
  pCompj  : TpComp;   {pointer to a relay as a component}
  pCurOrd : pInteger; {pointer to a temporary array for sorting the relays}
  ppComp  : TppComp;  {pointer to a temporary array of pointers to relays}
  ppCompi : TppComp;  {pointer to somewhere in the temp. array}
begin
  try
    if (ndeft_rg <= 0) then begin           {no definite time relays in this group}
      mincur := 0;                          {some value (a good one ?)}
    end else if (ndeft_rg = 1) then begin   {only one definite time relay in this group}
      getRpoint(1,pCompi);                  {get pointer to the relay}
      pCompi^.GetPar(gRelay_I,mincur);      {it has also smallest activation current}
    end else begin                          {more than one relay}
      getmem(pCurOrd,ndeft_rg*sizeof(integer)); {temp. array for sorting the relays}
      mincur := 0;                          {initialize smallest act.current}
      for ii:=1 to ndeft_rg do begin
        putIvec(pCurOrd,ii,ndeft_rg,1);     {initialize, 1 = one relay with this...}
      end;                                  {...current, at least}
      {initialize min.activation current}
      mincur := hugecurr;
      for ii:=1 to ndeft_rg{-1} do begin      {scan all relays (but the last.. why?6.3.2004)}
        GetRpoint(ii,pCompi);               {get pointer to this relay}
        pCompi^.GetPar(gRelay_I,ActCurr);   {get value of activation current}
        {check also smallest activation current}
        if ((ActCurr > 0) and (ActCurr < mincur)) then mincur := ActCurr;
        for jj:=ii+1 to ndeft_rg do begin   {scan the other relays}
          GetRpoint(jj,pCompj);             {get pointer to relay}
          pCompj^.GetPar(gRelay_I,apu);     {get activation current of relay jj}
          if (apu > ActCurr) then begin     {larger activation current ?}
            incIvec(pCurOrd,ii,ndeft_rg);  {one more relay with larger current}
          end else begin
            incIvec(pCurOrd,jj,ndeft_rg); {jj found one more relay with larger current}
          end;
        end;
      end;
      {minimum activation current not found ?}
      if (mincur >= hugecurr) then mincur := 0; {give some value}
      {rearrange relays, according to the activation current}
      getmem(ppComp,ndeft_rg*SizeOf(TpComp));  {get memory for a temp. array of pointers}
      {copy pointers to the temporary array}
      ppCompi := ppComp;                        {pointer to first pointer}
      for ii:=1 to ndeft_rg do begin            {copy all pointers for def.time relays}
        GetRpoint(ii,pCompi);
        ppCompi^:= pCompi;                      {copy to the temp. array}
        inc(ppCompi);                           {next location in the array}
      end;
      ppCompi := ppComp;                        {pointer to first pointer}
      {rearrange, smallest time first}
      for ii:=1 to ndeft_rg do begin            {rearrange all def.time pointers}
        getIvec(pCurOrd,ii,ndeft_rg,jj);        {get the ordinal number of this relay}
        putRpoint(jj,ppCompi^);                 {save the pointer according to the order}
        inc(ppCompi);                           {pointer to next pointer}
      end;
      freemem(pCurOrd,ndeft_rg*SizeOf(Integer));
      freemem(ppComp,ndeft_rg*SizeOf(TpComp)); {free the temporary array}
    end;
    {check still the eventual inverse-time relay}
    if (pInv_rg <> nil) then begin             {is there any inv.time relays}
      pInv_rg^.GetPar(gInv_I,apu);             {get the starting current}
      if (mincur > 0) then begin               {min.current was found for def.time relays}
        if (apu < mincur) and (apu > 0) then mincur_rg := apu
        else                   mincur_rg := mincur;
      end else begin                           {no min.current found}
        if (apu > 0) then      mincur_rg := apu{starting current defined ?}
        else                   mincur_rg := 0; {not, use zero (also as a flag)}
      end;
    end else begin                             {no inverse time relay}
      mincur_rg := mincur;                     {use the min.current of def.time relays}
    end;
  finally
  end;
end;
(*  uusi...*)
procedure TRelGroup.GetCurrTime(ii,ir :integer; var curr , time : single);
{get the ii:th largest activation current and the corresponding delay
 of a PGQ- or electromechanical or recloser relay
 ir   one of three relay types in a PQ-relay, P, G, or Q-relay
      or one of six relay types in a recloser relay
 If there are four nonzero currents available, the indices are
    PQmaxc_rg     largest
    PQhigc_rg     second largest
    PQlowc_rg     third largest
    PQminc_rg     smallest
 If there are less than four nonzero currents available, the indices are the following:
 Three nonzero currents:
    PQmaxc_rg     largest
    PQhigc_rg     second largest
    PQlowc_rg     smallest
    PQminc_rg     smallest
 Two nonzero currents:
    PQmaxc_rg     largest
    PQhigc_rg     smallest
    PQlowc_rg     smallest
    PQminc_rg     smallest
 One nonzero current
    All contain the index of the only nonzero current = largest = smallest = ..
 No nonzero currents
      }
var
  ipu         : integer;  {temporary}
  jjI,jjt     : integer;  {index for current, time}
  pComp       : TpComp;   {pointer to a component}
  rr          : integer;  {chekced PQ-relay index}
  jI : array[1..4] of integer;  {indices of currents}
  jt : array[1..4] of integer;  {indices of times}
begin
  {first, check PQ-relay and RC-relay index}
  if (ir < 1) then rr := 1
  else if (ir <= 3) then rr := ir
  else if (ir <= 6) then begin
    if (RCrelay_rg) then rr := ir
    else rr := 1;
  end else rr := 1;
  {indices to locals}
  if (PQRelay_rg) then begin
    jI[1] := gPQ_I1[rr];
    jI[2] := gPQ_I2[rr];
    jI[3] := gPQ_I3[rr];
    jI[4] := gPQ_I4[rr];
    jt[1] := gPQ_t1[rr];
    jt[2] := gPQ_t2[rr];
    jt[3] := gPQ_t3[rr];
    jt[4] := gPQ_t4[rr];
  end else if (RCrelay_rg) then begin
    jI[1] := gRC_I1[rr];
    jI[2] := gRC_I2[rr];
    jI[3] := gRC_I3[rr];
    jI[4] := gRC_I4[rr];
    jt[1] := gRC_t1[rr];
    jt[2] := gRC_t2[rr];
    jt[3] := gRC_t3[rr];
    jt[4] := gRC_t4[rr];
  end else begin           {not a PQ nor a RC relay, actually not needed}
    jI[1] := gRelay_I;
    jI[2] := gRelay_I;
    jI[3] := gRelay_I;
    jI[4] := gRelay_I;
    jt[1] := gRelay_t;
    jt[2] := gRelay_t;
    jt[3] := gRelay_t;
    jt[4] := gRelay_t;
  end;
  curr := 0;  {initialize}
  time := 0;
  try
    if (ii> 0) and (ii<=number_rg) then begin
      GetRpoint(ii, pComp);                           {get pointer to the relay}
      if (pComp <> nil) then begin
        if (PQRelay_rg) or (RCrelay_rg) then begin    {several modules in one relay}
          if (ii = 1) then begin                      {get the largest current}
            if (PQmaxc_rg[rr] > 0) then
              ipu := PQmaxc_rg[rr]
            else ipu := 0;                            {index of module with the largest current}
          end else if (ii = 2) then begin             {the second largest current}
            if (PQhigc_rg[rr] > 0) and (PQhigc_rg[rr] <> PQmaxc_rg[rr]) then {defined?}
              ipu := PQhigc_rg[rr]
            else ipu := 0;
          end else if (ii = 3) then begin             {the third largest current}
            if (PQlowc_rg[rr] > 0) and (PQlowc_rg[rr] <> PQhigc_rg[rr]) then {defined?}
              ipu := PQlowc_rg[rr]
            else ipu := 0;
          end else begin                              {smallest current}
            if (PQminc_rg[rr] > 0) and (PQminc_rg[rr] <> PQlowc_rg[rr]) then {defined}
              ipu := PQminc_rg[rr]
            else ipu := 0;
          end;
          {index found, get current and time, if reasonable index}
          if (ipu >= 1) and (ipu <= 4) then begin
            jjI := jI[ipu];                             {index for largest current}
            jjt := jt[ipu];
            pComp^.GetPar(jjI,curr);                    {get largest current}
            pComp^.GetPar(jjt,time);                    {get corresponding time}
          end else begin
            curr := 0;
            time := 0;
          end;
        end else begin
          pComp^.GetPar(gRelay_I,curr);      {get value of activation current}
          pComp^.GetPar(gRelay_t,time);      {get value of activation time}
        end;
      end else begin
        curr := 0;
        time := 0;
      end;
    end;
  finally
  end;
end;
(**)
(*  vanha..
procedure TRelGroup.GetCurrTime(ii,ir :integer; var curr , time : single);
{get the ii:th largest activation current and the corresponding delay
 of a PGQ- or electromechanical or recloser relay
 ir   one of three relay types in a PQ-relay, P, G, or Q-relay
      or one of six relay types in a recloser relay}
var
  ipu         : integer;  {temporary}
  jjI,jjt     : integer;  {index for current, time}
  pComp       : TpComp;   {pointer to a component}
  rr          : integer;  {chekced PQ-relay index}
  jI : array[1..3] of integer;  {indices of currents}
  jt : array[1..3] of integer;  {indices of times}
begin
  {first, check PQ-relay and RC-relay index}
  if (ir < 1) then rr := 1
  else if (ir <= 3) then rr := ir
  else if (ir <= 6) then begin
    if (RCrelay_rg) then rr := ir
    else rr := 1;
  end else rr := 1;
  {indices to locals}
  if (PQRelay_rg) then begin
    jI[1] := gPQ_I1[rr];
    jI[2] := gPQ_I2[rr];
    jI[3] := gPQ_I3[rr];
    jt[1] := gPQ_t1[rr];
    jt[2] := gPQ_t2[rr];
    jt[3] := gPQ_t3[rr];
  end else if (RCrelay_rg) then begin
    jI[1] := gRC_I1[rr];
    jI[2] := gRC_I2[rr];
    jI[3] := gRC_I3[rr];
    jt[1] := gRC_t1[rr];
    jt[2] := gRC_t2[rr];
    jt[3] := gRC_t3[rr];
  end else begin           {not a PQ nor a RC relay, actually not needed}
    jI[1] := gRelay_I;
    jI[2] := gRelay_I;
    jI[3] := gRelay_I;
    jt[1] := gRelay_t;
    jt[2] := gRelay_t;
    jt[3] := gRelay_t;
  end;
  curr := 0;  {initialize}
  time := 0;
  try
    if (ii> 0) and (ii<=number_rg) then begin
      GetRpoint(ii, pComp);                           {get pointer to the relay}
      if (pComp <> nil) then begin
        if (PQRelay_rg) or (RCrelay_rg) then begin    {several modules in one relay}
          if (ii = 1) then begin                      {get the largest current}
            if (PQmaxc_rg[rr] > 0) then
              ipu := PQmaxc_rg[rr]
            else ipu := 0;                            {index of module with the largest current}
          end else if (ii = 2) then begin             {the second largest current}
            if (PQlowc_rg[rr] > 0) and (PQlowc_rg[rr] <> PQmaxc_rg[rr]) then {defined?}
              ipu := PQlowc_rg[rr]
            else ipu := 0;
          end else begin                              {smallest current}
            if (PQminc_rg[rr] > 0) and (PQminc_rg[rr] <> PQlowc_rg[rr]) then {defined}
              ipu := PQminc_rg[rr]
            else ipu := 0;
          end;
          {index found, get current and time, if reasonable index}
          if (ipu >= 1) and (ipu <= 3) then begin
            jjI := jI[ipu];                             {index for largest current}
            jjt := jt[ipu];
            pComp^.GetPar(jjI,curr);                    {get largest current}
            pComp^.GetPar(jjt,time);                    {get corresponding time}
          end else begin
            curr := 0;
            time := 0;
          end;
        end else begin
          pComp^.GetPar(gRelay_I,curr);      {get value of activation current}
          pComp^.GetPar(gRelay_t,time);      {get value of activation time}
        end;
      end else begin
        curr := 0;
        time := 0;
      end;
    end;
  finally
  end;
end;
*)

procedure TRelGroup.GetIEC_CTcurrent(pCTrafo: TpComp; rr: integer;
                                     conn, Isec, mode, ratio: single; var curr : single);
{get the abs.value of the phase, sum, or neg.seq. current of an IEC-calculation
 from the current trafo, taking the trafo parameters into account
 pCTrafo   pointer to the current transformer
 rr        relay module index (checked)
 conn      connection of the secondary of the trafo, wye or delta
 Isec      nominal secondary current of the current trafo
 mode      flag for the use of the current trafo: prim, sec, or relative sec.current
 ratio     transformation ratio of the current trafo, prim/sec
 curr      the current (abs.value}
var
  apu,apv  : single;  {temporary}
  apw,apx  : single;  {temporary}
  ca,cb,cc : complex; {the phase currents from the current trafo (=after processing)}
  cpu,cpv  : complex; {temporary}
  cpw      : complex; {temporary}
  nvar     : integer; {number of (saved) variables}
  pa,pb,pc : complex; {the primary phase currents from the current trafo (=without processing)}
begin
  if (pCTrafo <> nil) then begin
    pCTrafo^.GetNvar(nvar);           {number of output variables}
    pCTrafo^.GetVar(1,pa.x);          {get the initial current of phase a}
    pCTrafo^.GetVar(2,pa.y);
    pCTrafo^.GetVar(1+nvar,pb.x);     {get the initial current of phase b}
    pCTrafo^.GetVar(2+nvar,pb.y);
    pCTrafo^.GetVar(1+nvar*2,pc.x);   {get the initial current of phase c}
    pCTrafo^.GetVar(2+nvar*2,pc.y);
    {process the currents, using the parameters of the current trafo}
    ProcessCTcurr(pa,pb,pc, mode, ratio, conn, Isec, ca,cb,cc);
    {largest phase current, sum current, or neg.seq.current}
    if (rr = 1) or ((rr = 2) and RCrelay_rg) then begin  {largest phase current}
      apu := ca.x*ca.x+ca.y*ca.y;
      apv := cb.x*cb.x+cb.y*cb.y;
      apw := cc.x*cc.x+cc.y*cc.y;
      if (apu >= apv) then apx := apu else apx := apv;
      if (apw > apx) then apx := apw;
      curr := sqrt(apx);               {largest phase current}
    end else if ((rr = 2) and PQrelay_rg) or  {sum current}
               (((rr = 3) or (rr = 4)) and RCrelay_rg) then begin
      cpu := Cadd(Cadd(ca,cb),cc);     {complex sum}
      curr := Cabs(cpu);
    end else begin                     {neg.seq.current}
      PhaseToSeq(ca,cb,cc,cpu,cpv,cpw);
      curr := Cabs(cpv);
    end;
  end else begin                      {no current trafo}
    curr := 1;                        {just some value}
  end;
end;

procedure TRelGroup.GetIECcurrent(pCTrafo: TpComp; rr: integer; var curr : single);
{get the abs.value of the phase, sum, or neg.seq. current from an IEC-calculation
 pCTrafo   pointer to the current transformer
 rr        relay module index (checked)
 curr      the current (abs.value}
var
  apu,apv  : single;  {temporary}
  apw,apx  : single;  {temporary}
  ca,cb,cc : complex; {the phase currents}
  cpu,cpv  : complex; {temporary}
  cpw      : complex; {temporary}
  nvar     : integer; {number of (saved) variables}
begin
  if (pCTrafo <> nil) then begin
    pCTrafo^.GetNvar(nvar);           {number of output variables}
    pCTrafo^.GetVar(1,ca.x);          {get the initial current of phase a}
    pCTrafo^.GetVar(2,ca.y);
    pCTrafo^.GetVar(1+nvar,cb.x);     {get the initial current of phase b}
    pCTrafo^.GetVar(2+nvar,cb.y);
    pCTrafo^.GetVar(1+nvar*2,cc.x);   {get the initial current of phase c}
    pCTrafo^.GetVar(2+nvar*2,cc.y);
    {largest phase current, sum current, or neg.seq.current}
    if (rr = 1) or ((rr = 2) and RCrelay_rg) then begin  {largest phase current}
      apu := ca.x*ca.x+ca.y*ca.y;
      apv := cb.x*cb.x+cb.y*cb.y;
      apw := cc.x*cc.x+cc.y*cc.y;
      if (apu >= apv) then apx := apu else apx := apv;
      if (apw > apx) then apx := apw;
      curr := sqrt(apx);               {largest phase current}
    end else if ((rr = 2) and PQrelay_rg) or  {sum current}
               (((rr = 3) or (rr = 4)) and RCrelay_rg) then begin
      cpu := Cadd(Cadd(ca,cb),cc);     {complex sum}
      curr := Cabs(cpu);
    end else begin                     {neg.seq.current}
      PhaseToSeq(ca,cb,cc,cpu,cpv,cpw);
      curr := Cabs(cpv);
    end;
  end else begin                      {no current trafo}
    curr := 1;                        {just some value}
  end;
end;

function TRelGroup.ActivationTime(ir : integer): single;
{TM suoritetaan liian monta kertaa ??}
{ check, does the short circuit current activate some relay in the group
  return the activation time
  ir   index of relay, one of three in case of a PQR-relay, else not used}
const
  jI1 : integer = 0;
  jI2 : integer = 0;  {index for current, must define as constant, else fatal error C1217}
  jI3 : integer = 0;
  jI4 : integer = 0;
var
  actime   : single;  {activation time (parameter)}
  actcur   : single;  {activation current (param.)}
  alpha    : single;  {inv.time relay parameter}
  apu,apv  : single;  {temporary}
  apw,apx  : single;  {temporary}
  beta     : single;  {inv.time relay parameter}
  c1       : single;  {first (nonzero) normalized current of electromech.relay}
  ca,cb,cc : complex; {phase currents}
  color    : TColor;  {color of text}
  conn     : single;  {connection of the current trafo: wye or delta}
  cpu      : complex; {temporary}
  curr     : single;  {current through the relay (current trafo)}
  curri    : single;  {current of inv.time module (mostly = curr)}
  curt     : single;  {current at given time}
  curve    : integer; {index of curve within a group}
  ct       : TTrois;  {temporary (primary, unprocessed) currents from the current trafo}
  c3,f3    : Ttrois;  {magnitude and phase (in degrees) of phase currents}
  deftime  : single;  {activation time of a definite time relay}
  fact     : single;  {pickup factor}
  forwrd   : boolean; {fault in forward direction}
  group    : integer; {index of group of curves}
  ii,jj    : integer; {index}
  ipu,ipv  : integer; {temporary}
  Inonz    : single;  {smallest nonzero act.current (if no relay activated)}
  InvNam   : string;  {name = type of the inverse time module}
  invpar   : TIntSing;{dummy, for the procedure call}
  invtime  : single;  {activation time of inverse time relay}
  invt0    : single;  {preliminary activation time}
  Iprim    : single;  {nominal prim.current of the current trafo}
  Isec     : single;  {nominal secondary current of the current trafo}
  Iss      : single;  {set current parameter}
  Ip,Im,Iz : complex; {pos., neg., zero seq. currents}
  Ima      : single;  {abs.value of neg.seq.current}
  Imin     : single;  {min.current in the char. of the em- or PQ-relay}
  ix,iy    : integer; {coordinates}
//jI1,jI2  : integer; {indices for current}
//jI3      : integer;
  jt1,jt2  : integer; {indices for time}
  jt3,jt4  : integer;
  ktm      : single;  {time multiplier}
  line     : string;  {temporary text}
  ll       : longint; {temporary}
  lbl      : string;  {label of the relay}
  mode     : single;  {flag for the use of the CT current: prim, sec, or relative sec}
  ModNam   : string;  {name (type) of the relay}
  nvar     : integer; {number of variables (saved)}
  nxtime   : single;  {activation time of next relay}
  pComp    : TpComp;  {pointer to a relay}
  pCompr   : TpCOmp;  {pointer to the rightmost relay}
  pCTrafo  : TpComp;  {pointer to current trafo}
  pCT      : TpComp;  {temp. pointer to current trafo}
  pCTi     : TpComp;  {pointer to current trafo, of inv.time relay}
  pNext    : TpComp;  {pointer to next relay}
  pSet     : TpCompSet;
  ratio    : single;  {transformation ratio of the CT prim/sec.current}
  rr       : integer; {checked relay index}
  t1       : single;  {first nonzero time of electromech.relay}
  tadd,tmin: single;  {time adder, min.time}
  time     : single;  {time when current = actcurr}
  tnonz    : single;  {time corresponding to Inonz}
  txt      : string;  {indication for time}
begin
  {check index, needed for PQ-, electromechanical and recloser relays}
  if (ir < 1) then rr := 1
  else if (ir <= 3) then rr := ir
  else if (ir <= 6) then begin
    if (RCrelay_rg) then rr := ir else rr := 1;
  end else rr := 1;
  {indices for current and time, for different relays}
  if (PQRelay_rg) then begin
    jI1 := gPQ_I1[rr];
    jI2 := gPQ_I2[rr];
    jI3 := gPQ_I3[rr];
    jI4 := gPQ_I4[rr];
    jt1 := gPQ_t1[rr];
    jt2 := gPQ_t2[rr];
    jt3 := gPQ_t3[rr];
    jt4 := gPQ_t4[rr];
  end else begin
    jI1 := gRC_I1[rr];
    jI2 := gRC_I2[rr];
    jI3 := gRC_I3[rr];
    jI4 := gRC_I4[rr];
    jt1 := gRC_t1[rr];
    jt2 := gRC_t2[rr];
    jt3 := gRC_t3[rr];
    jt4 := gRC_t4[rr];
  end;
  {scan all definite time realys in the group, start with the highest activation current}
  actime_rg[rr] := -1;                           {initialize = not activated}
  pCT := nil;
  pComp := nil;
  try
    deftime := -1;                           {initialize = not activated}
    invtime := -1;
    Inonz := 0;                              {initialize}
    tnonz := 0;
    curr := -1;                              {initialize, and a flag}
    pCTrafo := nil;
    pSet := nil;
    if (ndeft_rg > 0) then begin             {are there def.time relays }
      for ii:=1 to ndeft_rg do begin
        GetRpoint(ii, pComp);                {get pointer to the relay}
        if (pSet = nil) then begin
          pComp^.Getipset(ll);
          pSet := TpCompSet(ll);
        end;
        {pointer to the current trafo, from integer}
        pComp^.GetipComp(ll);                {get pointer to curr.trafo [only once !?]}
        if (ll <> 0) then pCTrafo := TpComp(ll) else pCTrafo := nil;
        if (PQRelay_rg) or (RCrelay_rg) then begin  {one PQ-relay contains three modules}
          {find current}
          if (ii = 1) then begin             {looking for max.current}
            if (PQmaxc_rg[rr] = 1) then begin
              pComp^.GetPar({gPQ_I1[rr]}jI1,actcur);
              pComp^.GetPar({gPQ_t1[rr]}jt1,actime);
            end else if (PQmaxc_rg[rr] = 2) then begin  {but if the same...??}
              pComp^.GetPar({gPQ_I2[rr]}jI2,actcur);
              pComp^.GetPar({gPQ_t2[rr]}jt2,actime);
            end else if (PQmaxc_rg[rr] = 3) then begin  {but if the same...??}
              pComp^.GetPar({gPQ_I3[rr]}jI3,actcur);
              pComp^.GetPar({gPQ_t3[rr]}jt3,actime);
            end else begin
              pComp^.GetPar({gPQ_I4[rr]}jI4,actcur);
              pComp^.GetPar({gPQ_t4[rr]}jt4,actime);
            end;
          end else if (ii = 2) then begin     {looking for the next highest current}
            if (PQhigc_rg[rr] = 1) then begin
              pComp^.GetPar(jI1,actcur);
              pComp^.GetPar(jt1,actime);
            end else if (PQhigc_rg[rr] = 2) then begin  {but if the same...??}
              pComp^.GetPar(jI2,actcur);
              pComp^.GetPar(jt2,actime);
            end else if (PQhigc_rg[rr] = 3) then begin  {but if the same...??}
              pComp^.GetPar(jI3,actcur);
              pComp^.GetPar(jt3,actime);
            end else begin
              pComp^.GetPar(jI4,actcur);
              pComp^.GetPar(jt4,actime);
            end;
          end else if (ii = 3) then begin      {looking for the next smallest current}
            if (PQlowc_rg[rr] = 1) then begin
              pComp^.GetPar(jI1,actcur);
              pComp^.GetPar(jt1,actime);
            end else if (PQlowc_rg[rr] = 2) then begin  {but if the same...??}
              pComp^.GetPar(jI2,actcur);
              pComp^.GetPar(jt2,actime);
            end else if (PQlowc_rg[rr] = 3) then begin  {but if the same...??}
              pComp^.GetPar(jI3,actcur);
              pComp^.GetPar(jt3,actime);
            end else begin
              pComp^.GetPar(jI4,actcur);
              pComp^.GetPar(jt4,actime);
            end;
          end else begin                       {looking for the smallest current}
            if (PQminc_rg[rr] = 1) then begin
              pComp^.GetPar(jI1,actcur);
              pComp^.GetPar(jt1,actime);
            end else if (PQminc_rg[rr] = 2) then begin  {but if the same...??}
              pComp^.GetPar(jI2,actcur);
              pComp^.GetPar(jt2,actime);
            end else if (PQminc_rg[rr] = 3) then begin  {but if the same...??}
              pComp^.GetPar(jI3,actcur);
              pComp^.GetPar(jt3,actime);
            end else begin
              pComp^.GetPar(jI4,actcur);
              pComp^.GetPar(jt4,actime);
            end;
          end;
        end else begin                         {normal relay}
          pComp^.GetPar(gRelay_I,actcur);      {get value of activation current}
          pComp^.GetPar(gRelay_t,actime);      {get value of activation time}
        end;
        {check activation, but only for relays with reasonable parameters}
        if ((actcur > 0) and (actime > 0)) then begin
          Inonz := actcur;                   {save (so far) smallest nonzero act.current}
          tnonz := actime;
          {parameters of the current trafo}
          if (pCTrafo <> nil) then begin
            GetCTmainPar(pCTrafo, ratio, Iprim, Isec, conn, mode);
          end else begin                                  {no current trafo}
            ratio := 1; Isec := 1; conn := 0; mode := 0;  {default values}
          end;
          {currents stored in a different way in dynamical and IEC-calculations}
          if (pSet_rg^.appl_type = dyn) then begin  {"dynamical" calculation . . . }
            {read currents, convert accoriding to CT parameters}
            if (pCTrafo <> nil) then begin
              GetCurr(pComp, actime, apv,ct,f3);   {find current at the given time, in skemproc.pas}
              {get secondary (or..) current of the CT, taking parameters into account}
              SecondaryCTCurrents(apv,ct.x,ct.y,ct.z,ct.r,ct.q,f3.x,f3.y,f3.z,f3.r,f3.q,
                                  conn,Isec,mode,ratio, apu,c3.x,c3.y,c3.z,c3.r,c3.q);

              {phase current, sum current or neg.seq.current}
              if (rr = 1) or ((rr = 2) and RCrelay_rg) then begin
                curr := apu;                       {magnitude of phase current}
              end else if ((rr = 2) and PQrelay_rg) or
                          (((rr = 3) or (rr = 4)) and RCrelay_rg) then begin
                curr := c3.r;                      {magnitude of sum current}
              end else begin                       {neg.seq.current}
                curr := c3.q;
              end;
            end else begin                      {no current trafo}
              curr := 0;
            end;
          end else begin                        {IEC-calculation . . . .}
            if (pCTrafo <> nil) then begin
              GetIEC_CTcurrent(pCTrafo, rr, conn,Isec,mode,ratio,curr);
            end else begin                      {no current trafo}
              curr := 1;                        {just some value}
            end;

          end;
          {now, check the activation}
          if (curr >= actcur) then begin       {relay activated ?}
            deftime := actime;                 {time, when this relay activated}
            break;                             {no need to study any more relays}
          end else begin                       {check also vertical branch}
{b}
            {but time range limited by next relay}
            if (ii < ndeft_rg) then begin      {not the last relay}
              GetRpoint(ii+1,pNext);           {pointer to the next relay}
              if (PQRelay_rg) or (RCrelay_rg) then begin   {one PQ-relay contains four modules}
                if (ii = 1) then begin             {looking for next to max.current}
                  if (PQhigc_rg[rr] = 1) then begin
                    pNext^.GetPar(jI1,apu);
                    pNext^.GetPar(jt1,apv);
                  end else if (PQhigc_rg[rr] = 2) then begin  {but if the same...??}
                    pNext^.GetPar(jI2,apu);
                    pNext^.GetPar(jt2,apv);
                  end else if (PQhigc_rg[rr] = 3) then begin  {but if the same...??}
                    pNext^.GetPar(jI3,apu);
                    pNext^.GetPar(jt3,apv);
                  end else begin
                    pNext^.GetPar(jI4,apu);
                    pNext^.GetPar(jt4,apv);
                  end;
                end else if (ii = 2) then begin             {looking for next to  min. current}
                  if (PQlowc_rg[rr] = 1) then begin
                    pNext^.GetPar(jI1,apu);
                    pNext^.GetPar(jt1,apv);
                  end else if (PQlowc_rg[rr] = 2) then begin  {but if the same...??}
                    pNext^.GetPar(jI2,apu);
                    pNext^.GetPar(jt2,apv);
                  end else if (PQlowc_rg[rr] = 3) then begin  {but if the same...??}
                    pNext^.GetPar(jI3,apu);
                    pNext^.GetPar(jt3,apv);
                  end else begin
                    pNext^.GetPar(jI4,apu);
                    pNext^.GetPar(jt4,apv);
                  end;
                end else if (ii = 3) then begin     {looking for the min. current}
                  if (PQminc_rg[rr] = 1) then begin
                    pNext^.GetPar(jI1,apu);
                    pNext^.GetPar(jt1,apv);
                  end else if (PQminc_rg[rr] = 2) then begin  {but if the same...??}
                    pNext^.GetPar(jI2,apu);
                    pNext^.GetPar(jt2,apv);
                  end else if (PQminc_rg[rr] = 3) then begin  {but if the same...??}
                    pNext^.GetPar(jI3,apu);
                    pNext^.GetPar(jt3,apv);
                  end else begin
                    pNext^.GetPar(jI4,apu);
                    pNext^.GetPar(jt4,apv);
                  end;
                end;
              end else begin                     {normal relay}
                pNext^.GetPar(gRelay_I,apu);     {get value of activation current}
                pNext^.GetPar(gRelay_t,apv);     {get value of activation time}
              end;
              {take this relay into account only if reasonable parameters}
              if ((apu > 0) and (apv > 0)) then nxtime := apv
              else                              nxtime := 100; {some large value}
            end else begin
              nxtime := 100;                   {last relay, use some large value}
            end;
//            pComp^.GetipComp(ll);              {get pointer to curr.trafo}
//            if (ll <> 0) then begin
//              pCTrafo := TpComp(ll);           {pointer from integer}
            if (pCTrafo <> nil) then begin
              GetTime(pCTrafo,actcur,actime,nxtime,rr,time); {time when current = actcur...}
            end else begin                                {...GetTime is in Skemproc.pas}
              time := -1;
            end;
            if (time > 0) then begin           {reasonable time}
              deftime := time;                 {this relay reacts, stop search}
              break;
            end;
{e}
          end;
        end;
      end;
    end;
    {check the eventual inverse-time relay}
    if (pInv_rg <> nil) then begin
      {must iterate time, if "dynamical" calculation, but calculate, if IEC}
      if (pSet_rg^.appl_type = dyn) then begin  {"dynamical" calculation . . . }
        invpar.i := 0; invpar.j := 0; invpar.x := 0; invpar.y := 0; invpar.z := 0;
        GetInvTime(pInv_rg, Inonz, tnonz, rr,{invpar,} invt0);    {in Skemproc.pas}
//showmessage(floattostr(invt0));
      end else begin        {IEC-calculation}
        {get current trafo, and current (maybe known, but not if no def.time relays)}
        pInv_rg^.GetipComp(ll);             {get pointer to curr.trafo [maybe already known]}

        if (ll <> 0) then begin
          pCTi := TpComp(ll);
          GetCTmainPar(pCTi, ratio, Iprim, Isec, conn, mode);
          GetIEC_CTcurrent(pCTi, rr, conn, Isec, mode, ratio, curri);  {actually needed only if no def.time modules, see above}
        end else begin
          curri := 1;                       {some small value}
        end;
        pInv_rg^.GetModName(InvNam);          {name (type) of the inverse time module}
        if (InvNam = gPQrel) or (InvNam = gEMrel) or
           (InvNam = gRCrel) or (InvNam = gUDrel) then begin  {combiner relay}
          GetInvTimeAtCurr(pComp, rr, curri, invt0);   {in Skemproc.pas}
//showmessage(floattostr(invt0));
        end else begin                        {simple relay (module)}
          pInv_rg^.GetPar(gInv_I,Iss);        {get the relay starting current}
          pInv_rg^.GetPar(gInv_k,ktm);        {get the time multiplier}
          InvTimePar(InvNam, alpha, beta);    {get the parameters for this type of relay}
          invt0 := TimeAtInvCurr(curri,alpha,beta,Iss,ktm);
        end;
      end;
      {for PQ-relays, check that current not too small = smaller than fact*Iss}
      if (PQRelay_rg) or (RCrelay_rg) then begin
        if (PQrelay_rg) then begin
          pComp^.GetPar(gPQ_Iset[rr],Iss);       {set current (TUNNETAANKO pComp ???)}
          pComp^.GetPar(gPQ_Tmult[rr],ktm);      {time multiplier}
          pComp^.GetPar(gPQ_Tadd[rr],tadd);      {time adder}
          pComp^.GetPar(gPQ_Tmin[rr],tmin);      {min time}
          pComp^.GetPar(gPQ_fact[rr],fact);      {pickup factor}
          pComp^.GetPar(gPQ_group[rr],apu);      {group of curves}
          group := round(apu);
          pComp^.GetPar(gPQ_curve[rr],apu);      {curve within the group}
          curve := round(apu);
        end else begin
          pComp^.GetPar(gRC_Iset[rr],Iss);       {set current (TUNNETAANKO pComp ???)}
          pComp^.GetPar(gRC_Tmult[rr],ktm);      {time multiplier}
          pComp^.GetPar(gRC_Tadd[rr],tadd);      {time adder}
          pComp^.GetPar(gRC_Tmin[rr],tmin);      {min time}
          pComp^.GetPar(gRC_fact[rr],fact);      {pickup factor}
//        group := round(apu);
          group := 1;                            {group not needed for RC- and UD-relays}
          pComp^.GetPar(gRC_curve[rr],apu);      {curve within the group}
          curve := round(apu);
        end;
        pComp^.GetModName(ModNam);
        if (Iss > 0) and (fact >= 1) and (invt0 > 0) then begin  {oli: fact > 1}
          {min.current defined for the relay}
          if  (ModNam = gPQRel) then begin         {PGQ- relay... GetFactCrit is in Relcurr.pas}
            GetFactCrit(apu);                      {criterion for using fact as min.time}
            if (fact <= apu) then                  {small fact = multiplier}
              Imin := fact*Iss
            else Imin := Iss;                      {large fact = time limit, multiplier = 1}
          end else if (ModNam = gUDrel) then begin {user defined recloser relay}
            GetUDFirst(pComp, rr, ipu, apu, apv);  {get first nozero normalized current}
            Imin := Iss*apu*fact;                  {minimum current}
          end else if (ModNam = gRCrel) then begin {normal, curve from tables}
//          Imin := fact*Iss;
            GetFactCrit(apu);                      {criterion for using fact as min.time}
            if (fact <= apu) then                  {small fact = multiplier}
              Imin := fact*Iss
            else Imin := Iss;                      {large fact = time limit, multiplier = 1}
          end else begin                           {electromech.relay}
            FirstEMTimeAndCurr(group,curve,ktm, c1,t1);
            if (c1 > fact) then Imin := c1*Iss else Imin := fact*Iss;
          end;
          if (ModNam = gPQRel) then begin           {PGQ-relay}
            curt := -1;
            PQTimeOrCurr(Iss,ktm,tadd,fact,group,curve,'curr',false,curt,invt0) {get current at invt0, in relcurr.pas}
          end else if (ModNam = gUDrel) then begin
            GetUDCurrOrTime(pComp, rr, Iss, ktm, tadd, 'curr', invt0, curt); {in relcurr.pas}
          end else if (ModNam = gRCrel) then begin   {"normal curve", from tables}
            RCTimeOrCurr(round(curve), Iss,ktm,tadd,fact,'curr',curt, invt0);     {in relcurr.pas}
          end else begin                             {electromech.relay}
            EMTimeOrCurr(Iss,ktm,tadd,group,curve,'curr',curt,invt0); {get current at invt0} {in relcurr.pas}
          end;
          {negative current = no solution = too small time = very large current, mostly recloser}
          if ((curt < Imin) and (curt > 0)) or       {too small current}
             (invt0 = gMaxTime) then invtime := -1   {or too large a time}
          else invtime := invt0;
        end else begin
          invtime := invt0;
        end;
      end else begin                           {"normal" relay}
        invtime := invt0;
      end;
    end else begin
      invtime := -1;
    end;
//showmessage(floattostr(invtime));
    {select activation time (not necessarily the smallest !!!}
    if (deftime > 0) then begin
      actime_rg[rr] := deftime;         {def. time relay has highest priority}
    end else begin
      if (invtime > 0) then begin       {check inverse time relay}
        actime_rg[rr] := invtime;
      end else begin
        actime_rg[rr] := -1 ;           {no relay activated}
      end;
    end;
    {for PQ-relays, check, is the fault in forward direction}
    {directionality check does not work in IEC-calculations !!!!!}
    if (pSet_rg^.appl_type = dyn) then begin  {"dynamical" calculation . . . }
      if (PQrelay_rg or RCrelay_rg) then begin
        GetRpoint(1, pComp);                 {get pointer to the relay}
        if (PQrelay_rg) then pComp^.getPar(gPQ_dir[rr],apu) {directionality flag}
        else pComp^.getPar(gRC_dir[rr],apu);
        if (apu <> 0) then begin             {must check fault direction}
          SeqZone(pComp,1,1,rr, 1.e6,forwrd,Ima);  {in Elplex.pas}
        end else begin
          forwrd := true;                    {not checked = assume forward direction}
        end;
{       if (forwrd) then showmessage('forwrd') else showmessage('reverse');}
      end else begin           {normal relay}
        forwrd := true;        {actually, not checked, but must assume}
      end;
    end else begin             {IEC-calculation, no forward/reverse check !!?}
      forwrd := true;
      {..but set warning, if directionality set}
      GetRpoint(1, pComp);                                {get pointer to the relay}
      if (PQrelay_rg) then pComp^.getPar(gPQ_dir[rr],apu) {directionality flag}
      else pComp^.getPar(gRC_dir[rr],apu);
      if (apu > 0) then begin
        pComp^.GetLabel(lbl,ix,iy);
         {save label, also as a flag}
        pSet_rg^.dir_warn := AppendTxtIf(lbl, pSet^.dir_warn, ',');  {in routines.pas}
      end;
//    if (apu > 0) then imagopas.dir_warn := true;        {initialized in imagopas}
    end;
    forwrd_rg[rr] := forwrd;  {save to group}
    {show activation time, if activated, show it next to the rightmost relay}
    GetRpoint(right_rg, pCompr);            {get pointer to the rightmost relay}
    {color and indication for time depends on relay type}
    if (isPQRelay) then begin               {PGQ-relay}
      if (rr = 1) then begin
        color := clFuchsia;
        txt := 'P';  {if (forwrd) then txt := 'F' else txt := 'R';}
      end else if (rr = 2) then begin
        color := clGreen;
        txt := 'G';  {if (forwrd) then txt := 'F' else txt := 'R';}
      end else begin
        color := clBlue;
        txt := 'Q';  {if (forwrd) then txt := 'F' else txt := 'R';}
      end;
    end else if (isRCrelay) then begin               {recloser}
      if (rr = 1) or (rr = 2) then begin
        color := clFuchsia;
        txt := 'P';
      end else if (rr = 3) or (rr = 4) then begin
        color := clGreen;
        txt := 'G';
      end else begin
        color := clBlue;
        txt := 'Q';
      end;
    end else begin                          {normal relay}
      color := clBlue;
      txt := '';
    end;
    ShowActime(pCompr,actime_rg[rr],color,rr,txt,forwrd); {in shortcir.pas}
  finally
    if (pComp <> nil) then begin            {definite time relay}
      pComp^.GetipComp(ll);                 {get pointer to curr.trafo}
      if (ll <> 0) then pCT := TpComp(ll);  {pointer from integer}
    end else if (pInv_rg <> nil) then begin {inverse time relay}
      {pointer to connected current trafo}
      pInv_rg^.GetipComp(ll);
      pCT := TpComp(ll);
    end;
{!} if (pCT <> nil) then begin
      if      (rr = 1) then pCT^.putZpar(actime_rg[rr],0,0,7)  {clear the others}
      else if ((rr = 2) and PQrelay_rg) or
              ((rr = 3) and RCrelay_rg) then pCT^.putZpar(0,actime_rg[rr],0,2)
      else if ((rr = 3) and PQrelay_rg) or
              ((rr = 5) and RCrelay_rg) then pCT^.putZpar(0,0,actime_rg[rr],4)
      {the second unit in recloser relay overwrites the first, only if smaller}
      else begin    {rr = 2, 4, or 6, and recloser relay}
        apu := actime_rg[rr];
        ipu := rr-1;              {indices for the modules of the first unit of the recloser rel.}
        ipv := (rr*rr-2*rr+8) div 8;  {binary flag: where to write = 1,2 or 4}
        if (actime_rg[ipu] > 0) and (apu > 0) then begin
          if (apu < actime_rg[ipu]) then pCT^.putZpar(apu,apu,apu,ipv);
        end else if (apu > 0) then begin   {only second activation time reasonable}
          pCT^.putZpar(apu,apu,apu,ipv);
        end;
      end;
{     pCT^.PutCpar(actime_rg[rr],0); {save to current trafo}
      {save activation time to current trafo as text, as description}

      if (isForwrd(rr)) then begin  {fault in forward direction, or normal relay}
        if (actime_rg[rr] > 0) then
          line := format('%7.2f',[actime_rg[rr]]) {activation time as text}
        else
          line := '   ----';                        {not activated}
      end else begin
        if (actime_rg[rr] > 0) then line := '      R'  {fault in reverse direction}
        else                        line := '   ----'; {not activated}
      end;
      if      (rr = 1) then
        PutAcTimeP1(pCT,line)          { in Skemobj.pas}
//      pCT^.PutDescrP(1,line,'','')  {as first descr.}
      else if ((rr = 2) and PQrelay_rg) or ((rr = 3) and RCrelay_rg) then
        PutAcTimeG1(pCT,line)
//      pCT^.PutDescrG(1,line,'','')  {as second descr.}
       else if ((rr = 3) and PQrelay_rg) or (rr = 5) then
         PutAcTimeQ1(pCT,line)
//      pCT^.PutDescrQ(1,line,'','') {as third descr.}
      else if (rr = 2) and (RCrelay_rg) then
//      pCT^.PutDescr4(1,line,'','')  {second phase relay module in RC-relay}
        PutAcTimeP2(pCT,line)
      else if (rr = 4) then
//      pCT^.PutDescr4(2,'',line,'')  {second ground relay module in RC-relay}
        PutAcTimeG2(pCT,line)
      else
//      pCT^.PutDescr4(4,'','',line); {second sequence relay module in RC-relay}
        PutAcTimeQ2(pCT,line)
    end;
//if (rr=1) then showmessage(floattostr(actime_rg[rr]));
    ActivationTime := actime_rg[rr];  {return the activation time}
  end;
end;

procedure TRelGroup.GetNdtrel(var ndtrel : integer);
{get number of def.time relays}
begin
  ndtrel := ndeft_rg;
end;

constructor TRelGroup.Constr(pSet: TpCompSet; nrg : integer; PQRel,RCrel : boolean);
{construct = initialize the object for one relay group
 nrg    number of relays in the group
 PQrel  true, when PGQ-relay
 RCrel  true, when recloser relay}
begin
  {get room for pointers to relays}
  if (nrg > 0) then begin
    getmem(ppComp_rg,nrg*SizeOf(TpComp));
    number_rg := nrg;
    next_rg := 1;
    backw_rg := nrg;
    ndeft_rg := 0;      {initialize no of definite time relays, updated later}
  end else begin
    ppComp_rg := nil;
    number_rg := 0;
    next_rg := 0;
    backw_rg := 0;
    ndeft_rg := 0;
  end;
  mincur_rg := 0;      {initialize smallest activation current}
  pInv_rg := nil;      {initialize pointer to an eventual inverse-time relay}
  pCurrT_rg := nil;    {initialize pointer to the current trafo}
  PQRelay_rg := PQRel; {group = PGQ-relay, or not}
  RCrelay_rg := RCrel; {group = recloser relay, or not}
  pSet_rg := pSet;
{
if (RCrel) then showmessage('RCrel')
else if (PQrel) then showmessage('PQrel')
else showmessage('rel');
}
end;

procedure TRelGroup.Destr;
{release reserved memory}
begin
  if (number_rg > 0) then begin
    freemem(ppComp_rg,number_rg*SizeOf(TpComp));
  end;
end;


end.
